/*
 * Copyright (c) 1991-1994,1997, by Sun Microsystems, Inc.
 * All Rights Reserved
 */

#pragma ident	"@(#)ae.c	1.31	97/09/11 SMI"

/*
 *  Solaris Multithreaded STREAMS DLPI PCnet (Am79C970) PCI Ethernet Driver
 *
 */

#include	<sys/types.h>
#include	<sys/conf.h>
#include	<sys/debug.h>
#include	<sys/stropts.h>
#include	<sys/stream.h>
#include	<sys/strlog.h>
#include	<sys/kmem.h>
#include	<sys/stat.h>
#include	<sys/kstat.h>
#include	"ae_ddict.h"
#include 	<sys/modctl.h>
#include 	<sys/errno.h>
#include	<sys/ddi.h>
#include	<sys/sunddi.h>
#include	"pcnet.h"
#include	"ae.h"

/*
 * PCI Configuration Header offsets
 */
#define	AE_PCI_CONF_COMM	0x4	/* command register, 2 bytes */

/*
 * PCI command register bits
 */
#define	AE_PCI_COMM_IO		0x0001   /* I/O access enable */
#define	AE_PCI_COMM_MAE		0x0002   /* memory access enable */
#define	AE_PCI_COMM_ME		0x0004   /* master enable */

#define	NOCOPYBUFF 1
#define	AEBUFSRESERVED	4

#define	LANCE_CSR_WRITE(addr, value)			\
	ddi_put32(aep->ae_handle, 			\
		(uint32_t *)&aep->ae_regsp->lance_rap,	\
		addr);	\
	ddi_put32(aep->ae_handle,			\
		(uint32_t *)&aep->ae_regsp->lance_rdp,	\
		value)

#define	LANCE_PUT_CSR0(value)				\
	ddi_put32(aep->ae_handle,			\
		(uint32_t *)&aep->ae_regsp->lance_rdp,	\
		value)

#define	LANCE_GET_CSR0()				\
	ddi_get32(aep->ae_handle,			\
		(uint32_t *)&aep->ae_regsp->lance_rdp)

#define	get_ether_type(ptr) (\
	(((uint8_t *)&((struct ether_header *)ptr)->ether_type)[0] << 8) | \
	(((uint8_t *)&((struct ether_header *)ptr)->ether_type)[1]))
#define	put_ether_type(ptr, value) {\
	((uint8_t *)(&((struct ether_header *)ptr)->ether_type))[0] = \
		((u_int)value & 0xff00) >> 8; \
	((uint8_t *)(&((struct ether_header *)ptr)->ether_type))[1] = \
		(value & 0xff); }

/* XXX should this really be ddi_dma_mem_alloc()? */
#define	GETBUF(structure, size)   \
	((structure *) kmem_zalloc((u_int)(size), KM_SLEEP))

/*
 * Translate a kernel virtual address to i/o address.
 */
#define	AEIOPBIOADDR(aep, a) \
	((aep)->ae_iopbiobase + ((uint32_t)(a) - (aep)->ae_iopbkbase))

/*
 * ddi_dma_sync() a buffer.
 */
#define	AESYNCIOPB(aep, a, size, who) \
	(void) ddi_dma_sync((aep)->ae_iopb_hdl, \
		((uint32_t)(a) - (aep)->ae_iopbkbase), (size), (who))


#define	AESAPMATCH(sap, type, flags) ((sap == type)? 1 : \
	((flags & SLALLSAP)? 1 : \
	((sap <= ETHERMTU) && (sap > 0) && (type <= ETHERMTU))? 1 : 0))

/*
 * Function prototypes.
 */
static	int aeprobe(dev_info_t *);
static	int aeattach(dev_info_t *, ddi_attach_cmd_t);
static	int aedetach(dev_info_t *, ddi_detach_cmd_t);
static	void aestatinit(struct ae *);
static	int aeinfo(dev_info_t *, ddi_info_cmd_t, void *, void **);
static	int aealloc_dma_mem(struct ae *);
static 	void aefree_dma_mem(register struct ae *aep, int count);
static	int aeopen(queue_t *, dev_t *, int, int, cred_t *);
static	int aeclose(queue_t *);
static	int aewput(queue_t *, mblk_t *);
static	int aewsrv(queue_t *);
static	void aeproto(queue_t *, mblk_t *);
static	void aeioctl(queue_t *, mblk_t *);
static	void ae_dl_ioc_hdr_info(queue_t *, mblk_t *);
static	void aeareq(queue_t *, mblk_t *);
static	void aedreq(queue_t *, mblk_t *);
static	void aedodetach(struct aestr *);
static	void aebreq(queue_t *, mblk_t *);
static	void aeubreq(queue_t *, mblk_t *);
static	void aeireq(queue_t *, mblk_t *);
static	void aeponreq(queue_t *, mblk_t *);
static	void aepoffreq(queue_t *, mblk_t *);
static	void aeemreq(queue_t *, mblk_t *);
static	void aedmreq(queue_t *, mblk_t *);
static	void aepareq(queue_t *, mblk_t *);
static	void aespareq(queue_t *, mblk_t *);
static	void aeudreq(queue_t *, mblk_t *);
static	int aestart(queue_t *, mblk_t *, struct ae *);
static	u_int aeintr();
static	void aereclaim(struct ae *);
static	void aewenable(struct ae *);
static	struct aestr *aeaccept(struct aestr *, struct ae *, int,
	struct ether_addr *);
static	struct aestr *aepaccept(struct aestr *, struct ae *, int,
	struct	ether_addr *);
static	void aesetipq(struct ae *);
static	void aeread(struct ae *, volatile struct lmd *);
static	void aesendup(struct ae *, mblk_t *, struct aestr *(*)());
static	mblk_t *aeaddudind(struct ae *, mblk_t *, struct ether_addr *,
	struct ether_addr *, int, uint32_t);
static	int aemcmatch(struct aestr *, struct ether_addr *);
static	int aeinit(struct ae *);
static	void aeuninit(struct ae *aep);
static	struct aebuf *aegetbuf(struct ae *, int);
static	void aefreebuf(struct aebuf *);
static	void aermdinit(struct ae *, volatile struct lmd *, struct aebuf *);
static	void ae_rcv_error(struct ae *, struct lmd *);
static	int ae_xmit_error(struct ae *, struct lmd *);
static	int ae_chip_error(struct ae *);
static	void	aeerror();

static	void ae_etheraddr(struct ether_addr *hint, struct ether_addr *result);

static	void ae_merror(queue_t *, mblk_t *, int);
static	void ae_miocack(queue_t *, mblk_t *, int, int);
static	void ae_miocnak(queue_t *, mblk_t *, int, int);
static	mblk_t *ae_mexchange(queue_t *, mblk_t *, int, uint8_t, int32_t);

static	int ae_msgsize(register mblk_t *mp);
static	void ae_mcopymsg(mblk_t *mp, uint8_t *bufp);

static	void ae_dlbindack(queue_t *, mblk_t *, uint32_t, caddr_t, int,
	int,    uint32_t);
static  void ae_dlokack(queue_t *, mblk_t *, uint32_t);
static  void ae_dlerrorack(queue_t *, mblk_t *, uint32_t, uint32_t, uint32_t);
static  void ae_dluderrorind(queue_t *, mblk_t *, uint8_t *, uint32_t,
	uint32_t, uint32_t);
static  void ae_dlphysaddrack(queue_t *, mblk_t *, caddr_t, int);

static	char *ether_sprintf(struct ether_addr *addr);




/* Data access requirements. */
static struct ddi_device_acc_attr dev_attr = {
	DDI_DEVICE_ATTR_V0,
	DDI_STRUCTURE_LE_ACC,
	DDI_STRICTORDER_ACC
};

/* On sparc, the buffers should be native endianness */
#ifdef sparc
static struct ddi_device_acc_attr buf_attr = {
	DDI_DEVICE_ATTR_V0,
	DDI_STRUCTURE_BE_ACC,	/* buffer endianness */
	DDI_STRICTORDER_ACC
};
#else
#define	buf_attr dev_attr
#endif

static	struct module_info aeminfo = {
	AEIDNUM,	/* mi_idnum */
	AENAME,		/* mi_idname */
	AEMINPSZ,	/* mi_minpsz */
	AEMAXPSZ,	/* mi_maxpsz */
	AEHIWAT,	/* mi_hiwat */
	AELOWAT		/* mi_lowat */
};

static	struct qinit aerinit = {
	(int (*)()) NULL, /* qi_putp */
	(int (*)()) NULL, /* qi_srvp */
	aeopen,		/* qi_qopen */
	aeclose,	/* qi_qclose */
	(int (*)()) NULL, /* qi_qadmin */
	&aeminfo,	/* qi_minfo */
	NULL		/* qi_mstat */
};

static	struct qinit aewinit = {
	aewput,		/* qi_putp */
	aewsrv,		/* qi_srvp */
	(int (*)()) NULL, /* qi_qopen */
	(int (*)()) NULL, /* qi_qclose */
	(int (*)()) NULL, /* qi_qadmin */
	&aeminfo,	/* qi_minfo */
	NULL		/* qi_mstat */
};

static struct streamtab	ae_info = {
	&aerinit,	/* st_rdinit */
	&aewinit,	/* st_wrinit */
	NULL,		/* st_muxrinit */
	NULL		/* st_muxwrinit */
};

static	struct cb_ops cb_ae_ops = {
	nulldev,	/* cb_open */
	nulldev,	/* cb_close */
	nodev,		/* cb_strategy */
	nodev,		/* cb_print */
	nodev,		/* cb_dump */
	nodev,		/* cb_read */
	nodev,		/* cb_write */
	nodev,		/* cb_ioctl */
	nodev,		/* cb_devmap */
	nodev,		/* cb_mmap */
	nodev,		/* cb_segmap */
	nochpoll,	/* cb_chpoll */
	ddi_prop_op,	/* cb_prop_op */
	&ae_info,	/* cb_stream */
	D_MP		/* cb_flag */
};

static	struct dev_ops ae_ops = {
	DEVO_REV,	/* devo_rev */
	0,		/* devo_refcnt */
	aeinfo,		/* devo_getinfo */
	nulldev,	/* devo_identify */
	aeprobe,	/* devo_probe */
	aeattach,	/* devo_attach */
	aedetach,	/* devo_detach */
	nodev,		/* devo_reset */
	&cb_ae_ops,	/* devo_cb_ops */
	NULL,		/* devo_bus_ops */
	ddi_power	/* devo_power */
};

static struct modldrv modldrv = {
	&mod_driverops,	/* Type of module.  This one is a driver */
	"PCnet Lance Ethernet Driver v2.0",
	&ae_ops,	/* driver ops */
};

static struct modlinkage modlinkage = {
	MODREV_1, &modldrv, NULL
};

/*
 * Patchable debug flag.
 * Set this to nonzero to enable *all* error messages.
 */
static	int	aedebug = 0;

/*
 * Single private "global" lock for the few rare conditions
 * we want single-threaded.
 */
static	kmutex_t	aelock;
static	krwlock_t	aestruplock;

/*
 * Linked list of "ae" structures - one per device.
 */
static struct ae *aedev = NULL;

/*
 * Linked list of active (inuse) driver Streams.
 */
static	struct	aestr	*aestrup = NULL;

/*
 * Ethernet broadcast address definition.
 */
static	struct ether_addr	etherbroadcastaddr = {
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff
};

/*
 * Resource amounts.
 *	For now, at least, these never change, and resources
 *	are allocated once and for all at aeattach() time.
 */
static int	ae_ntmdp2 = 7;	/* power of 2 Transmit Ring Descriptors */
static int	ae_nrmdp2 = 5;	/* power of 2 Receive Ring Descriptors */
static int	ae_nbufs = 64;	/* # buffers allocated for xmit/recv pool */

/*
 * Our DL_INFO_ACK template.
 */
static	dl_info_ack_t aeinfoack = {
	DL_INFO_ACK,			/* dl_primitive */
	ETHERMTU,			/* dl_max_sdu */
	0,				/* dl_min_sdu */
	AEADDRL,			/* dl_addr_length */
	DL_ETHER,			/* dl_mac_type */
	0,				/* dl_reserved */
	0,				/* dl_current_state */
	-2,				/* dl_sap_length */
	DL_CLDLS,			/* dl_service_mode */
	0,				/* dl_qos_length */
	0,				/* dl_qos_offset */
	0,				/* dl_range_length */
	0,				/* dl_range_offset */
	DL_STYLE2,			/* dl_provider_style */
	sizeof (dl_info_ack_t),		/* dl_addr_offset */
	DL_VERSION_2,			/* dl_version */
	ETHERADDRL,			/* dl_brdcst_addr_length */
	sizeof (dl_info_ack_t) + AEADDRL, /* dl_brdcst_addr_offset */
	0				/* dl_growth */
};

int
_init(void)
{
	int 	status;

	rw_init(&aestruplock, "ae streams list lock", RW_DRIVER, NULL);
	mutex_init(&aelock, "aelock", MUTEX_DRIVER, NULL);
	status = mod_install(&modlinkage);
	if (status != 0) {
		rw_destroy(&aestruplock);
		mutex_destroy(&aelock);
	}
	return (status);
}

int
_fini(void)
{
	int	status;

	if ((status = mod_remove(&modlinkage)) == 0) {
		mutex_destroy(&aelock);
		rw_destroy(&aestruplock);
	}
	return (status);
}

int
_info(struct modinfo *modinfop)
{
	return (mod_info(&modlinkage, modinfop));
}

/*
 * Probe for device.
 */
/*ARGSUSED*/
static int
aeprobe(dev_info_t *dip)
{
	return (DDI_PROBE_SUCCESS);
}

/*
 * Interface exists: make available by filling in network interface
 * record.  System will initialize the interface when it is ready
 * to accept packets.
 */
static int
aeattach(dev_info_t *dip, ddi_attach_cmd_t cmd)
{
	register struct ae *aep;
	struct lanceregs *regsp;
	ddi_iblock_cookie_t	c;
	uint32_t	timestamp;
	uint16_t comm;
	ddi_acc_handle_t conf_handle;

	if (cmd == DDI_ATTACH) {
		/*
		 * Allocate soft data structure
		 */
		aep = (struct ae *)kmem_zalloc(sizeof (*aep), KM_SLEEP);

		/*
		 * Map in the device registers.
		 */
		if (ddi_regs_map_setup(dip, 1, (caddr_t *)&regsp,
			0, 0, &dev_attr, &aep->ae_handle)) {
			kmem_free((caddr_t)aep, sizeof (struct ae));
			aeerror(dip, "ddi_regs_map_setup failed");
			return (DDI_FAILURE);
		}


		/*
		 * Make sure the Master enable and memory access enable
		 * and I/O access enable. bits are set in the config
		 * command register
		 */
		if (pci_config_setup(dip, &conf_handle) != DDI_SUCCESS) {
			kmem_free((caddr_t)aep, sizeof (struct ae));
			aeerror(dip, "ddi_regs_map_setup failed");
			return (DDI_FAILURE);
		}

		comm = pci_config_getw(conf_handle, AE_PCI_CONF_COMM);
		comm |= (AE_PCI_COMM_ME | AE_PCI_COMM_MAE | AE_PCI_COMM_IO);
		pci_config_putw(conf_handle, AE_PCI_CONF_COMM, comm);
		pci_config_teardown(&conf_handle);

		aep->ae_regsp = regsp;

		/*
		 * Stop the chip.
		 */

		LANCE_CSR_WRITE(LANCE_CSR0, LANCE_STOP);

		/*
		 * Add interrupt to system.
		 */
		if (ddi_add_intr(dip, 0, &c, 0, aeintr, (caddr_t)aep)) {
			ddi_regs_map_free(&aep->ae_handle);
			kmem_free((caddr_t)aep, sizeof (struct ae));
			aeerror(dip, "ddi_add_intr failed");
			return (DDI_FAILURE);
		}

		/*
		 * Initialize mutex's for this device.
		 */
		mutex_init(&aep->ae_xmitlock, "ae xmit lock",
			MUTEX_DRIVER, (void *)c);
		mutex_init(&aep->ae_intrlock, "ae intr lock",
			MUTEX_DRIVER, (void *)c);
		mutex_init(&aep->ae_buflock, "ae bufs lock",
			MUTEX_DRIVER, (void *)c);

		/*
		 * At this point, we are *really* here.
		 */
		ddi_set_driver_private(dip, (caddr_t)aep);
		aep->ae_dip = dip;

		/*
		 * Get the local ethernet address.
		 */
		(void) ddi_rep_getb((caddr_t)aep->ae_handle,
			(uchar_t *)&aep->ae_ouraddr, (uchar_t *)regsp,
			6, DDI_DEV_AUTOINCR);

		/* set the hardware ethernet address */
		ae_etheraddr(&aep->ae_ouraddr, (struct ether_addr *)NULL);

		/*
		 * Create the filesystem device node.
		 */
		if (ddi_create_minor_node(dip, "ae", S_IFCHR,
			ddi_get_instance(dip), DDI_NT_NET,
				CLONE_DEV) == DDI_FAILURE) {
			mutex_destroy(&aep->ae_xmitlock);
			mutex_destroy(&aep->ae_intrlock);
			mutex_destroy(&aep->ae_buflock);
			(void) ddi_remove_intr(dip, 0, c);
			ddi_regs_map_free(&aep->ae_handle);
			kmem_free((caddr_t)aep, sizeof (struct ae));
			aeerror(dip, "ddi_create_minor_node failed");
			return (DDI_FAILURE);
		}

		/*
		 * Create power management properties.
		 */
		(void) drv_getparm(TIME, &timestamp);
		if (ddi_prop_update_byte_array(DDI_DEV_T_NONE, dip,
			"pm_timestamp", (uint8_t *)&timestamp,
			sizeof (timestamp)) != DDI_SUCCESS)
			aeerror(dip,
			    "aeattach:  ddi_prop_update_byte_array error");
		timestamp = 1;
		if (ddi_prop_update_byte_array(DDI_DEV_T_NONE, dip,
			"pm_norm_pwr", (uint8_t *)&timestamp,
			sizeof (timestamp)) != DDI_SUCCESS)
			aeerror(dip,
			    "aeattach:  ddi_prop_update_byte_array error");

		/*
		 * Link this per-device structure in with the rest.
		 */
		mutex_enter(&aelock);
		aep->ae_nextp = aedev;
		aedev = aep;
		mutex_exit(&aelock);

		aestatinit(aep);
		ddi_report_dev(dip);
		return (DDI_SUCCESS);
	} else if (cmd == DDI_RESUME) {
		if ((aep = (struct ae *)ddi_get_driver_private(dip)) == NULL)
			return (DDI_FAILURE);
		aep->ae_flags &= ~AESUSPENDED;
		(void) aeinit(aep);
		return (DDI_SUCCESS);
	} else
		return (DDI_FAILURE);
}

static int
aedetach(dev_info_t *dip, ddi_detach_cmd_t cmd)
{
	int	i;
	register struct ae *aep, *aetmp, **prevaep;

	if (cmd == DDI_DETACH) {
		aep = (struct ae *)ddi_get_driver_private(dip);

		/*
		 * stop the device
		 */
		LANCE_CSR_WRITE(LANCE_CSR0, LANCE_STOP);

		kstat_delete(aep->ae_ksp);

		/*
		 * Remove aep from the link list of device structures
		 */
		for (prevaep = &aedev; (aetmp = *prevaep) != NULL;
			prevaep = &aetmp->ae_nextp)
			if (aetmp == aep)
				*prevaep = aep->ae_nextp;

		ddi_prop_remove_all(dip);
		ddi_remove_minor_node(dip, NULL);
		ddi_remove_intr(dip, 0, aep->ae_cookie);

		mutex_destroy(&aep->ae_xmitlock);
		mutex_destroy(&aep->ae_intrlock);
		mutex_destroy(&aep->ae_buflock);

		if (aep->ae_iopb_hdl) {
			/*
			 * Free any pending buffers or mgs.
			 */
			for (i = 0; i < 128; i++) {
				if (aep->ae_tbufp[i]) {
					aefreebuf(aep->ae_tbufp[i]);
				}
				if (aep->ae_tmblkp[i]) {
					freemsg(aep->ae_tmblkp[i]);
				}
				if (aep->ae_rbufp[i]) {
					aefreebuf(aep->ae_rbufp[i]);
				}
			}
			aefree_dma_mem(aep, aep->ae_nbufs);
		}

		ddi_regs_map_free(&aep->ae_handle);
		kmem_free((caddr_t)aep, sizeof (struct ae));

		return (DDI_SUCCESS);
	} else if ((cmd == DDI_SUSPEND) || (cmd == DDI_PM_SUSPEND)) {
		if ((aep = (struct ae *)ddi_get_driver_private(dip)) == NULL)
			return (DDI_FAILURE);
		aep->ae_flags |= AESUSPENDED;
		aeuninit(aep);
		return (DDI_SUCCESS);
	} else
		return (DDI_FAILURE);
}

static int
aestat_kstat_update(kstat_t *ksp, int rw)
{
	struct ae *aep;
	struct aestat *aesp;

	aep = (struct ae *)ksp->ks_private;
	aesp = (struct aestat *)ksp->ks_data;

	if (rw == KSTAT_WRITE) {
#ifdef	kstat
		aep->ae_ipackets	= aesp->aes_ipackets.value.ul;
		aep->ae_ierrors		= aesp->aes_ierrors.value.ul;
		aep->ae_opackets	= aesp->aes_opackets.value.ul;
		aep->ae_oerrors		= aesp->aes_oerrors.value.ul;
		aep->ae_collisions	= aesp->aes_collisions.value.ul;
		aep->ae_defer		= aesp->aes_defer.value.ul;
		aep->ae_fram		= aesp->aes_fram.value.ul;
		aep->ae_crc		= aesp->aes_crc.value.ul;
		aep->ae_oflo		= aesp->aes_oflo.value.ul;
		aep->ae_uflo		= aesp->aes_uflo.value.ul;
		aep->ae_missed		= aesp->aes_missed.value.ul;
		aep->ae_tlcol		= aesp->aes_tlcol.value.ul;
		aep->ae_trtry		= aesp->aes_trtry.value.ul;
		aep->ae_tnocar		= aesp->aes_tnocar.value.ul;
		aep->ae_inits		= aesp->aes_inits.value.ul;
		aep->ae_nocanput	= aesp->aes_nocanput.value.ul;
		aep->ae_allocbfail	= aesp->aes_allocbfail.value.ul;
#else
		return (EACCES);
#endif	kstat
	} else {
		aesp->aes_ipackets.value.ul	= aep->ae_ipackets;
		aesp->aes_ierrors.value.ul	= aep->ae_ierrors;
		aesp->aes_opackets.value.ul	= aep->ae_opackets;
		aesp->aes_oerrors.value.ul	= aep->ae_oerrors;
		aesp->aes_collisions.value.ul	= aep->ae_collisions;
		aesp->aes_defer.value.ul	= aep->ae_defer;
		aesp->aes_fram.value.ul		= aep->ae_fram;
		aesp->aes_crc.value.ul		= aep->ae_crc;
		aesp->aes_oflo.value.ul		= aep->ae_oflo;
		aesp->aes_uflo.value.ul		= aep->ae_uflo;
		aesp->aes_missed.value.ul	= aep->ae_missed;
		aesp->aes_tlcol.value.ul	= aep->ae_tlcol;
		aesp->aes_trtry.value.ul	= aep->ae_trtry;
		aesp->aes_tnocar.value.ul	= aep->ae_tnocar;
		aesp->aes_inits.value.ul	= aep->ae_inits;
		aesp->aes_nocanput.value.ul	= aep->ae_nocanput;
		aesp->aes_allocbfail.value.ul	= aep->ae_allocbfail;
	}
	return (0);
}

static void
aestatinit(struct ae *aep)
{
	kstat_t	*ksp;
	struct aestat *aesp;

#ifdef	kstat
	if ((ksp = kstat_create("ae", ddi_get_instance(aep->ae_dip),
	    NULL, "net", KSTAT_TYPE_NAMED,
	    sizeof (struct aestat) / sizeof (kstat_named_t),
		KSTAT_FLAG_PERSISTENT)) == NULL) {
#else
	if ((ksp = kstat_create("ae", ddi_get_instance(aep->ae_dip),
	    NULL, "net", KSTAT_TYPE_NAMED,
	    sizeof (struct aestat) / sizeof (kstat_named_t), 0)) == NULL) {
#endif	kstat
		aeerror(aep->ae_dip, "kstat_create failed");
		return;
	}
	aep->ae_ksp = ksp;
	aesp = (struct aestat *)(ksp->ks_data);
	kstat_named_init(&aesp->aes_ipackets,		"ipackets",
		KSTAT_DATA_ULONG);
	kstat_named_init(&aesp->aes_ierrors,		"ierrors",
		KSTAT_DATA_ULONG);
	kstat_named_init(&aesp->aes_opackets,		"opackets",
		KSTAT_DATA_ULONG);
	kstat_named_init(&aesp->aes_oerrors,		"oerrors",
		KSTAT_DATA_ULONG);
	kstat_named_init(&aesp->aes_collisions,		"collisions",
		KSTAT_DATA_ULONG);
	kstat_named_init(&aesp->aes_defer,		"defer",
		KSTAT_DATA_ULONG);
	kstat_named_init(&aesp->aes_fram,		"framming",
		KSTAT_DATA_ULONG);
	kstat_named_init(&aesp->aes_crc,		"crc",
		KSTAT_DATA_ULONG);
	kstat_named_init(&aesp->aes_oflo,		"oflo",
		KSTAT_DATA_ULONG);
	kstat_named_init(&aesp->aes_uflo,		"uflo",
		KSTAT_DATA_ULONG);
	kstat_named_init(&aesp->aes_missed,		"missed",
		KSTAT_DATA_ULONG);
	kstat_named_init(&aesp->aes_tlcol,		"late_collisions",
		KSTAT_DATA_ULONG);
	kstat_named_init(&aesp->aes_trtry,		"retry_error",
		KSTAT_DATA_ULONG);
	kstat_named_init(&aesp->aes_tnocar,		"nocarrier",
		KSTAT_DATA_ULONG);
	kstat_named_init(&aesp->aes_inits,		"inits",
		KSTAT_DATA_ULONG);
	kstat_named_init(&aesp->aes_nocanput,		"nocanput",
		KSTAT_DATA_ULONG);
	kstat_named_init(&aesp->aes_allocbfail,		"allocbfail",
		KSTAT_DATA_ULONG);
	ksp->ks_update = aestat_kstat_update;
	ksp->ks_private = (void *) aep;
	kstat_install(ksp);
}

/*
 * Translate "dev_t" to a pointer to the associated "dev_info_t".
 */
/* ARGSUSED */
static int
aeinfo(dev_info_t *dip, ddi_info_cmd_t infocmd, void *arg, void **result)
{
	dev_t	dev = (dev_t)arg;
	int	instance;
	int	rc;
	struct	aestr	*slp;

	instance = getminor(dev);
	rc = DDI_FAILURE;

	switch (infocmd) {
	case DDI_INFO_DEVT2DEVINFO:
		rw_enter(&aestruplock, RW_READER);
		dip = NULL;
		for (slp = aestrup; slp; slp = slp->sl_nextp)
			if (slp->sl_minor == instance)
				break;
		if (slp && slp->sl_aep)
			dip = slp->sl_aep->ae_dip;
		rw_exit(&aestruplock);

		if (dip) {
			*result = (void *) dip;
			rc = DDI_SUCCESS;
		}
		break;

	case DDI_INFO_DEVT2INSTANCE:
		*result = (void *)instance;
		rc = DDI_SUCCESS;
		break;

	default:
		break;
	}
	return (rc);
}

static void
aefree_dma_mem(register struct ae *aep, int count)
{
	int	size;
	int	i;
	struct aebuf *lbp;

	for (i = 0; i < count; i++) {
		lbp = aep->ae_buftab[i];
		ASSERT(lbp);

		(void) ddi_dma_unbind_handle(lbp->lb_buf_hdl);
		(void) ddi_dma_mem_free(&lbp->lb_acchdl);
		ddi_dma_free_handle(&lbp->lb_buf_hdl);
	}

	size = aep->ae_nbufs * sizeof (struct aebuf *);
	kmem_free(aep->ae_buftab, size);
	(void) ddi_dma_unbind_handle(aep->ae_iopb_hdl);
	(void) ddi_dma_mem_free(&aep->ae_acchdl);
	ddi_dma_free_handle(&aep->ae_iopb_hdl);
}


/*
 * Allocate memory for:
 *   - LANCE initialization block
 *   - LANCE transmit descriptor ring
 *   - LANCE receive descriptor ring
 *   - pool of xmit/recv buffers
 *
 * For SLAVE devices, allocate out of device local memory.
 * For DVMA devices, allocate out of main memory.
 *
 * The init block and descriptors are allocated contiguously
 * and the buffers are allocated contiguously.  Each
 * of the iopb and buf areas are described by a DDI DMA handle,
 * (null for slave devices), the base kernel virtual address
 * for driver usage, and the base io virtual address
 * for the chip usage.
 */
static int
aealloc_dma_mem(register struct ae *aep)
{
	struct	aebuf	*lbp;
	uint32_t a;
	int	size;
	int	i;
	ddi_dma_cookie_t	cookie;
	ddi_dma_handle_t buf_hdl;
	ddi_dma_attr_t  ae_dma_attr;
	ddi_acc_handle_t acchdl;
	uint_t	count;
	size_t	rlen;

	aep->ae_nrmdp2 = ae_nrmdp2;
	aep->ae_nrmds = 1 << aep->ae_nrmdp2;
	aep->ae_ntmdp2 = ae_ntmdp2;
	aep->ae_ntmds = 1 << aep->ae_ntmdp2;
	aep->ae_nbufs = ae_nbufs;

	/*
	 * Allocate the chip init block and descriptors
	 * all at one time remembering to allocate extra for
	 * alignments.
	 */
	size = AEROUNDUP((sizeof (struct lance_init_block)), DESCRING_ALIGN) +
		(aep->ae_nrmds * sizeof (struct lmd)) +
		(aep->ae_ntmds * sizeof (struct lmd));
	/*
	 * dynamically create a dma attribute structure
	 */
	ae_dma_attr.dma_attr_version = DMA_ATTR_V0;
	ae_dma_attr.dma_attr_addr_lo = 0;
	ae_dma_attr.dma_attr_addr_hi = 0xffffffff;
	ae_dma_attr.dma_attr_count_max = 0x00ffffff;
	ae_dma_attr.dma_attr_align = DESCRING_ALIGN;
	ae_dma_attr.dma_attr_burstsizes = 1;
	ae_dma_attr.dma_attr_minxfer = 1;
	ae_dma_attr.dma_attr_maxxfer = 0x00ffffff;
	ae_dma_attr.dma_attr_seg = 0xffffffff;
	ae_dma_attr.dma_attr_sgllen = 1;
	ae_dma_attr.dma_attr_granular = size;
	ae_dma_attr.dma_attr_flags = 0;

	if (ddi_dma_alloc_handle(aep->ae_dip, &ae_dma_attr,
	    DDI_DMA_SLEEP, NULL, &aep->ae_iopb_hdl) != DDI_SUCCESS) {
		return (DDI_FAILURE);
	}

	if (ddi_dma_mem_alloc(aep->ae_iopb_hdl, (uint_t)size,
		&dev_attr, DDI_DMA_CONSISTENT, DDI_DMA_SLEEP, NULL,
		(caddr_t *)&aep->ae_iopbkbase, &rlen, &aep->ae_acchdl) !=
		DDI_SUCCESS) {
		ddi_dma_free_handle(&aep->ae_iopb_hdl);
		return (DDI_FAILURE);
	}

	if (ddi_dma_addr_bind_handle(aep->ae_iopb_hdl, (struct as *)0,
			(caddr_t)aep->ae_iopbkbase, (u_int)rlen,
			DDI_DMA_RDWR|DDI_DMA_CONSISTENT, DDI_DMA_SLEEP, 0,
			&cookie, &count) != DDI_SUCCESS) {
		(void) ddi_dma_mem_free(&aep->ae_acchdl);
		ddi_dma_free_handle(&aep->ae_iopb_hdl);
		return (DDI_FAILURE);
	}
	aep->ae_iopbiobase = cookie.dmac_address;

	/*
	 * Allocate buffer pointer stack (fifo).
	 */
	size = aep->ae_nbufs * sizeof (struct aebuf *);
	aep->ae_buftab = kmem_zalloc(size, KM_SLEEP);
	if (aep->ae_buftab == NULL) {
		(void) ddi_dma_unbind_handle(aep->ae_iopb_hdl);
		(void) ddi_dma_mem_free(&aep->ae_acchdl);
		ddi_dma_free_handle(&aep->ae_iopb_hdl);
		return (DDI_FAILURE);
	}

	size = AEROUNDUP((sizeof (struct aebuf)), AEBURSTSIZE);
	/*
	 * dynamically create a dma attribute structure
	 */
	ae_dma_attr.dma_attr_version = DMA_ATTR_V0;
	ae_dma_attr.dma_attr_addr_lo = 0;
	ae_dma_attr.dma_attr_addr_hi = 0xffffffff;
	ae_dma_attr.dma_attr_count_max = 0x00ffffff;
	ae_dma_attr.dma_attr_align = AEBURSTSIZE;
	ae_dma_attr.dma_attr_burstsizes = AEBURSTSIZE;
	ae_dma_attr.dma_attr_minxfer = 1;
	ae_dma_attr.dma_attr_maxxfer = 0x00ffffff;
	ae_dma_attr.dma_attr_seg = 0xffffffff;
	ae_dma_attr.dma_attr_sgllen = 1;
	ae_dma_attr.dma_attr_granular = size;
	ae_dma_attr.dma_attr_flags = 0;

	/*
	 * Initialize buffer pointer stack, buffer-by-buffer.
	 */
	for (i = 0; i < aep->ae_nbufs; i++) {
		/*
		 * IO map the buffers and get a "buffer" dma handle.
		 */
		if (ddi_dma_alloc_handle(aep->ae_dip, &ae_dma_attr,
		    DDI_DMA_SLEEP, NULL, &buf_hdl) != DDI_SUCCESS) {
			aefree_dma_mem(aep, i);
			return (DDI_FAILURE);
		}

		/*
		 * Allocate the buffers burst-aligned.
		 */
		if (ddi_dma_mem_alloc(buf_hdl, (uint_t)size, &buf_attr,
		    DDI_DMA_CONSISTENT, DDI_DMA_SLEEP, NULL,
		    (caddr_t *)&lbp, &rlen, &acchdl) != DDI_SUCCESS) {
			ddi_dma_free_handle(&buf_hdl);
			aefree_dma_mem(aep, i);
			return (DDI_FAILURE);
		}

		if (ddi_dma_addr_bind_handle(buf_hdl, (struct as *)0,
		    (caddr_t)lbp, rlen, DDI_DMA_CONSISTENT|DDI_DMA_RDWR,
		    DDI_DMA_SLEEP, 0, &cookie, &count) != DDI_SUCCESS) {
			(void) ddi_dma_mem_free(&acchdl);
			ddi_dma_free_handle(&buf_hdl);
			aefree_dma_mem(aep, i);
			return (DDI_FAILURE);
		}

		lbp->lb_aep = aep;
		lbp->lb_frtn.free_func = aefreebuf;
		lbp->lb_frtn.free_arg = (char *)lbp;
		lbp->lb_buf_hdl = buf_hdl;
		lbp->lb_acchdl = acchdl;

		lbp->phys_addr = cookie.dmac_address;
		aep->ae_buftab[i] = lbp;
	}
	aep->ae_bufi = aep->ae_nbufs;

	a = aep->ae_iopbkbase;
	aep->ae_ibp = (struct lance_init_block *)a;
	a = a + AEROUNDUP((sizeof (struct lance_init_block)), DESCRING_ALIGN);

	/* ring pointer for receive */
	aep->ae_rmdp = (struct lmd *)a;
	a += aep->ae_nrmds * sizeof (struct lmd);

	/* ring pointer for transmit */
	aep->ae_tmdp = (struct lmd *)a;

	/*
	 * Keep handy limit values for RMD, TMD, and Buffers.
	 */
	aep->ae_rmdlimp = &((aep->ae_rmdp)[aep->ae_nrmds]);
	aep->ae_tmdlimp = &((aep->ae_tmdp)[aep->ae_ntmds]);


	/*
	 * Zero out xmit and rcv holders.
	 */
	bzero((caddr_t)aep->ae_tbufp, sizeof (aep->ae_tbufp));
	bzero((caddr_t)aep->ae_tmblkp, sizeof (aep->ae_tmblkp));
	bzero((caddr_t)aep->ae_rbufp, sizeof (aep->ae_rbufp));

	if (aep->ae_wantw)
		aewenable(aep);

	return (DDI_SUCCESS);
}

/*ARGSUSED*/
static
aeopen(rq, devp, flag, sflag, credp)
queue_t	*rq;
dev_t	*devp;
int	flag;
int	sflag;
cred_t	*credp;
{
	register struct	aestr	*slp;
	register struct	aestr	**prevslp;
	int	minordev;
	int	rc = 0;

	ASSERT(rq);
	ASSERT(sflag != MODOPEN);
	TRACE_1(TR_FAC_AE, TR_AE_OPEN, "aeopen:  rq %X", rq);

	/*
	 * Serialize all driver open and closes.
	 */
	rw_enter(&aestruplock, RW_WRITER);

	/*
	 * Determine minor device number.
	 */
	prevslp = &aestrup;
	if (sflag == CLONEOPEN) {
		minordev = 0;
		for (; (slp = *prevslp) != NULL; prevslp = &slp->sl_nextp) {
			if (minordev < slp->sl_minor)
				break;
			minordev++;
		}
		*devp = makedevice(getmajor(*devp), minordev);
	} else
		minordev = getminor(*devp);

	if (rq->q_ptr)
		goto done;

	slp = (struct aestr *)kmem_zalloc(sizeof (*slp), KM_SLEEP);
	slp->sl_minor = minordev;
	slp->sl_rq = rq;
	slp->sl_state = DL_UNATTACHED;
	slp->sl_sap = 0;
	slp->sl_flags = 0;
	slp->sl_aep = NULL;
	slp->sl_mccount = 0;
	slp->sl_mctab = NULL;
	mutex_init(&slp->sl_lock, "ae stream lock", MUTEX_DRIVER, (void *)0);

	/*
	 * Link new entry into the list of active entries.
	 */
	slp->sl_nextp = *prevslp;
	*prevslp = slp;

	rq->q_ptr = WR(rq)->q_ptr = (char *)slp;

	/*
	 * Disable automatic enabling of our write service procedure.
	 * We control this explicitly.
	 */
	noenable(WR(rq));

done:
	rw_exit(&aestruplock);
	qprocson(rq);
	return (rc);
}

static int
aeclose(queue_t	*rq)
{
	register	struct	aestr	*slp;
	register	struct	aestr	**prevslp;

	TRACE_1(TR_FAC_AE, TR_AE_CLOSE, "aeclose:  rq %X", rq);
	ASSERT(rq);
	ASSERT(rq->q_ptr);

	qprocsoff(rq);

	slp = (struct aestr *)rq->q_ptr;

	/*
	 * Implicit detach Stream from interface.
	 */
	if (slp->sl_aep)
		aedodetach(slp);

	rw_enter(&aestruplock, RW_WRITER);

	/*
	 * Unlink the per-Stream entry from the active list and free it.
	 */
	for (prevslp = &aestrup; (slp = *prevslp) != NULL;
		prevslp = &slp->sl_nextp)
		if (slp == (struct aestr *)rq->q_ptr)
			break;
	ASSERT(slp);
	*prevslp = slp->sl_nextp;

	mutex_destroy(&slp->sl_lock);
	kmem_free((char *)slp, sizeof (struct aestr));

	rq->q_ptr = WR(rq)->q_ptr = NULL;

	rw_exit(&aestruplock);
	return (0);
}

static int
aewput(queue_t *wq, mblk_t *mp)
{
	register	struct	aestr	*slp = (struct aestr *)wq->q_ptr;
	struct	ae	*aep;

	TRACE_2(TR_FAC_AE, TR_AE_WPUT_START,
		"aewput start:  wq %X db_type %o", wq, DB_TYPE(mp));

	switch (DB_TYPE(mp)) {
		case M_DATA:		/* "fastpath" */
			aep = slp->sl_aep;
			if ((slp->sl_flags & (SLFAST|SLRAW) == 0) ||
				(slp->sl_state != DL_IDLE) ||
				(aep == NULL)) {
				ae_merror(wq, mp, EPROTO);
				break;
			}

			/*
			 * If any msgs already enqueued or the interface will
			 * loop back up the message (due to AEPROMISC), then
			 * enqueue the msg.  Otherwise just xmit it directly.
			 */
			if (wq->q_first) {
				(void) putq(wq, mp);
				aep->ae_wantw = 1;
			} else if (aep->ae_flags & AEPROMISC) {
				(void) putq(wq, mp);
				qenable(wq);
			} else
				(void) aestart(wq, mp, aep);

			break;

		case M_PROTO:
		case M_PCPROTO:
			/*
			 * Break the association between the current thread and
			 * the thread that calls aeproto() to resolve the
			 * problem of aeintr() threads which loop back around
			 * to call aeproto and try to recursively acquire
			 * internal locks.
			 */
			(void) putq(wq, mp);
			qenable(wq);
			break;

		case M_IOCTL:
			aeioctl(wq, mp);
			break;

		case M_FLUSH:
			if (*mp->b_rptr & FLUSHW) {
				flushq(wq, FLUSHALL);
				*mp->b_rptr &= ~FLUSHW;
			}
			if (*mp->b_rptr & FLUSHR)
				qreply(wq, mp);
			else
				freemsg(mp);
			break;

		default:
			freemsg(mp);
			break;
	}
	TRACE_1(TR_FAC_AE, TR_AE_WPUT_END, "aewput end:  wq %X", wq);
	return (0);
}

/*
 * Enqueue M_PROTO/M_PCPROTO (always) and M_DATA (sometimes) on the wq.
 *
 * Processing of some of the M_PROTO/M_PCPROTO msgs involves acquiring
 * internal locks that are held across upstream putnext calls.
 * Specifically there's the problem of aeintr() holding ae_intrlock
 * and aestruplock when it calls putnext() and that thread looping
 * back around to call aewput and, eventually, aeinit() to create a
 * recursive lock panic.  There are two obvious ways of solving this
 * problem: (1) have aeintr() do putq instead of putnext which provides
 * the loopback "cutout" right at the rq, or (2) allow aeintr() to putnext
 * and put the loopback "cutout" around aeproto().  We choose the latter
 * for performance reasons.
 *
 * M_DATA messages are enqueued on the wq *only* when the xmit side
 * is out of tbufs or tmds.  Once the xmit resource is available again,
 * wsrv() is enabled and tries to xmit all the messages on the wq.
 */
static
aewsrv(queue_t *wq)
{
	mblk_t	*mp;
	struct	aestr	*slp;
	struct	ae	*aep;

	TRACE_1(TR_FAC_AE, TR_AE_WSRV_START, "aewsrv start:  wq %X", wq);

	slp = (struct aestr *)wq->q_ptr;
	aep = slp->sl_aep;

	while (mp = getq(wq))
		switch (DB_TYPE(mp)) {
			case	M_DATA:
				if (aep) {
					if (aestart(wq, mp, aep))
						goto done;
				} else
					freemsg(mp);
				break;

			case	M_PROTO:
			case	M_PCPROTO:
				aeproto(wq, mp);
				break;

			default:
				ASSERT(0);
				break;
		}

done:
	TRACE_1(TR_FAC_AE, TR_AE_WSRV_END, "aewsrv end:  wq %X", wq);
	return (0);
}

static void
aeproto(queue_t	*wq, mblk_t *mp)
{
	union	DL_primitives	*dlp;
	struct	aestr	*slp;
	uint32_t	prim;

	slp = (struct aestr *)wq->q_ptr;
	dlp = (union DL_primitives *)mp->b_rptr;
	prim = dlp->dl_primitive;

	TRACE_2(TR_FAC_AE, TR_AE_PROTO_START,
		"aeproto start:  wq %X dlprim %X", wq, prim);

	mutex_enter(&slp->sl_lock);

	switch (prim) {
		case	DL_UNITDATA_REQ:
			aeudreq(wq, mp);
			break;

		case	DL_ATTACH_REQ:
			aeareq(wq, mp);
			break;

		case	DL_DETACH_REQ:
			aedreq(wq, mp);
			break;

		case	DL_BIND_REQ:
			aebreq(wq, mp);
			break;

		case	DL_UNBIND_REQ:
			aeubreq(wq, mp);
			break;

		case	DL_INFO_REQ:
			aeireq(wq, mp);
			break;

		case	DL_PROMISCON_REQ:
			aeponreq(wq, mp);
			break;

		case	DL_PROMISCOFF_REQ:
			aepoffreq(wq, mp);
			break;

		case	DL_ENABMULTI_REQ:
			aeemreq(wq, mp);
			break;

		case	DL_DISABMULTI_REQ:
			aedmreq(wq, mp);
			break;

		case	DL_PHYS_ADDR_REQ:
			aepareq(wq, mp);
			break;

		case	DL_SET_PHYS_ADDR_REQ:
			aespareq(wq, mp);
			break;

		default:
			ae_dlerrorack(wq, mp, prim, DL_UNSUPPORTED, 0);
			break;
	}

	TRACE_2(TR_FAC_AE, TR_AE_PROTO_END,
		"aeproto end:  wq %X dlprim %X", wq, prim);

	mutex_exit(&slp->sl_lock);
}

static void
aeioctl(queue_t *wq, mblk_t *mp)
{
	struct	iocblk	*iocp = (struct iocblk *)mp->b_rptr;
	struct	aestr	*slp = (struct aestr *)wq->q_ptr;

	switch (iocp->ioc_cmd) {
	case DLIOCRAW:		/* raw M_DATA mode */
		slp->sl_flags |= SLRAW;
		ae_miocack(wq, mp, 0, 0);
		break;

	case DL_IOC_HDR_INFO:	/* M_DATA "fastpath" info request */
		ae_dl_ioc_hdr_info(wq, mp);
		break;

	default:
		ae_miocnak(wq, mp, 0, EINVAL);
		break;
	}
}

/*
 * M_DATA "fastpath" info request.
 * Following the M_IOCTL mblk should come a DL_UNITDATA_REQ mblk.
 * We ack with an M_IOCACK pointing to the original DL_UNITDATA_REQ mblk
 * followed by an mblk containing the raw ethernet header corresponding
 * to the destination address.  Subsequently, we may receive M_DATA
 * msgs which start with this header and may send up
 * up M_DATA msgs with b_rptr pointing to a (uint32_t) group address
 * indicator followed by the network-layer data (IP packet header).
 * This is all selectable on a per-Stream basis.
 */
static void
ae_dl_ioc_hdr_info(wq, mp)
queue_t	*wq;
mblk_t	*mp;
{
	mblk_t	*nmp;
	struct	aestr	*slp;
	struct	aedladdr	*dlap;
	dl_unitdata_req_t	*dludp;
	struct	ether_header	*headerp;
	struct	ae	*aep;
	int	off, len;
	int	minsize;

	slp = (struct aestr *)wq->q_ptr;
	minsize = sizeof (dl_unitdata_req_t) + AEADDRL;

	/*
	 * Sanity check the request.
	 */
	if ((mp->b_cont == NULL) ||
		(MBLKL(mp->b_cont) < minsize) ||
		(*((uint32_t *)mp->b_cont->b_rptr) != DL_UNITDATA_REQ) ||
		((aep = slp->sl_aep) == NULL)) {
		ae_miocnak(wq, mp, 0, EINVAL);
		return;
	}

	/*
	 * Sanity check the DL_UNITDATA_REQ destination address
	 * offset and length values.
	 */
	dludp = (dl_unitdata_req_t *)mp->b_cont->b_rptr;
	off = dludp->dl_dest_addr_offset;
	len = dludp->dl_dest_addr_length;
	if (!MBLKIN(mp->b_cont, off, len) || (len != AEADDRL)) {
		ae_miocnak(wq, mp, 0, EINVAL);
		return;
	}

	dlap = (struct aedladdr *)(mp->b_cont->b_rptr + off);

	/*
	 * Allocate a new mblk to hold the ether header.
	 */
	if ((nmp = allocb(sizeof (struct ether_header), BPRI_MED)) == NULL) {
		ae_miocnak(wq, mp, 0, ENOMEM);
		return;
	}
	nmp->b_wptr += sizeof (struct ether_header);

	/*
	 * Fill in the ether header.
	 */
	headerp = (struct ether_header *)nmp->b_rptr;
	ether_copy(&dlap->dl_phys, &headerp->ether_dhost);
	ether_copy(&aep->ae_ouraddr, &headerp->ether_shost);
	put_ether_type(headerp, slp->sl_sap);

	/*
	 * Link new mblk in after the "request" mblks.
	 */
	linkb(mp, nmp);

	slp->sl_flags |= SLFAST;

	/*
	 * XXX Don't bother calling aesetipq() here.
	 */

	ae_miocack(wq, mp, ae_msgsize(mp->b_cont), 0);
}

static void
aeareq(queue_t *wq, mblk_t *mp)
{
	struct	aestr	*slp;
	union	DL_primitives	*dlp;
	struct	ae	*aep;
	int	ppa;
	uint32_t	timestamp;

	slp = (struct aestr *)wq->q_ptr;
	dlp = (union DL_primitives *)mp->b_rptr;

	if (MBLKL(mp) < DL_ATTACH_REQ_SIZE) {
		ae_dlerrorack(wq, mp, DL_ATTACH_REQ, DL_BADPRIM, 0);
		return;
	}

	if (slp->sl_state != DL_UNATTACHED) {
		ae_dlerrorack(wq, mp, DL_ATTACH_REQ, DL_OUTSTATE, 0);
		return;
	}

	ppa = dlp->attach_req.dl_ppa;

	/*
	 * Valid ppa?
	 */
	mutex_enter(&aelock);
	for (aep = aedev; aep; aep = aep->ae_nextp) {
		int tmp;
		tmp = ddi_get_instance(aep->ae_dip);
		if (ppa == tmp)  break;
	}
	mutex_exit(&aelock);

	if (aep == NULL) {
		ae_dlerrorack(wq, mp, dlp->dl_primitive, DL_BADPPA, 0);
		return;
	}

	/*
	 * Has device been initialized?  Do so if necessary.
	 * Also update power management property.
	 */

	if ((aep->ae_flags & AERUNNING) == 0) {
		if (aeinit(aep)) {
			ae_dlerrorack(wq, mp, dlp->dl_primitive,
			    DL_INITFAILED, 0);
			return;
		}

		timestamp = 0;	/* Busy */
		if (ddi_prop_update_byte_array(DDI_DEV_T_NONE, aep->ae_dip,
			"pm_timestamp", (uint8_t *)&timestamp,
			sizeof (timestamp)) != DDI_SUCCESS)
			aeerror(aep->ae_dip,
				"aeareq:  ddi_prop_update_byte_array error");
	}

	/*
	 * Set link to device and update our state.
	 */
	slp->sl_aep = aep;
	slp->sl_state = DL_UNBOUND;

	ae_dlokack(wq, mp, DL_ATTACH_REQ);
}

static void
aedreq(queue_t *wq, mblk_t *mp)
{
	struct	aestr	*slp;

	slp = (struct aestr *)wq->q_ptr;

	if (MBLKL(mp) < DL_DETACH_REQ_SIZE) {
		ae_dlerrorack(wq, mp, DL_DETACH_REQ, DL_BADPRIM, 0);
		return;
	}

	if (slp->sl_state != DL_UNBOUND) {
		ae_dlerrorack(wq, mp, DL_DETACH_REQ, DL_OUTSTATE, 0);
		return;
	}

	aedodetach(slp);
	ae_dlokack(wq, mp, DL_DETACH_REQ);
}

/*
 * Detach a Stream from an interface.
 */
static void
aedodetach(struct aestr	*slp)
{
	struct	aestr	*tslp;
	struct	ae	*aep;
	int	reinit = 0;
	uint32_t	timestamp;

	ASSERT(slp->sl_aep);

	aep = slp->sl_aep;
	slp->sl_aep = NULL;

	/*
	 * Disable promiscuous mode if on.
	 */
	if (slp->sl_flags & SLALLPHYS) {
		slp->sl_flags &= ~SLALLPHYS;
		reinit = 1;
	}

	/*
	 * Disable ALLMULTI mode if on.
	 */
	if (slp->sl_flags & SLALLMULTI) {
		slp->sl_flags &= ~SLALLMULTI;
		reinit = 1;
	}

	/*
	 * Disable any Multicast Addresses.
	 */
	slp->sl_mccount = 0;
	if (slp->sl_mctab) {
		kmem_free(slp->sl_mctab, AEMCALLOC);
		slp->sl_mctab = NULL;
		reinit = 1;
	}

	/*
	 * Detach from device structure.
	 * Uninit the device and update power management property
	 * when no other streams are attached to it.
	 */
	for (tslp = aestrup; tslp; tslp = tslp->sl_nextp)
		if (tslp->sl_aep == aep)
			break;
	if (tslp == NULL) {
		aeuninit(aep);

		(void) drv_getparm(TIME, &timestamp);
		if (ddi_prop_update_byte_array(DDI_DEV_T_NONE, aep->ae_dip,
			"pm_timestamp", (uint8_t *)&timestamp,
			sizeof (timestamp)) != DDI_SUCCESS)
			aeerror(aep->ae_dip,
			    "aedodetach:  ddi_prop_update_byte_array error");
	} else if (reinit)
		(void) aeinit(aep);

	slp->sl_state = DL_UNATTACHED;

	aesetipq(aep);
}

static void
aebreq(queue_t *wq, mblk_t *mp)
{
	struct	aestr	*slp;
	union	DL_primitives	*dlp;
	struct	ae	*aep;
	struct	aedladdr	aeaddr;
	uint32_t	sap;
	int	xidtest;

	slp = (struct aestr *)wq->q_ptr;

	if (MBLKL(mp) < DL_BIND_REQ_SIZE) {
		ae_dlerrorack(wq, mp, DL_BIND_REQ, DL_BADPRIM, 0);
		return;
	}

	if (slp->sl_state != DL_UNBOUND) {
		ae_dlerrorack(wq, mp, DL_BIND_REQ, DL_OUTSTATE, 0);
		return;
	}

	dlp = (union DL_primitives *)mp->b_rptr;
	aep = slp->sl_aep;
	sap = dlp->bind_req.dl_sap;
	xidtest = dlp->bind_req.dl_xidtest_flg;

	ASSERT(aep);

	if (xidtest) {
		ae_dlerrorack(wq, mp, DL_BIND_REQ, DL_NOAUTO, 0);
		return;
	}

	if (sap > ETHERTYPE_MAX) {
		ae_dlerrorack(wq, mp, dlp->dl_primitive, DL_BADSAP, 0);
		return;
	}

	/*
	 * Save SAP value for this Stream and change state.
	 */
	slp->sl_sap = sap;
	slp->sl_state = DL_IDLE;

	aeaddr.dl_sap = sap;
	ether_copy(&aep->ae_ouraddr, &aeaddr.dl_phys);
	ae_dlbindack(wq, mp, sap, (caddr_t)&aeaddr, AEADDRL, 0, 0);

	aesetipq(aep);
}

static void
aeubreq(queue_t *wq, mblk_t *mp)
{
	struct	aestr	*slp;

	slp = (struct aestr *)wq->q_ptr;

	if (MBLKL(mp) < DL_UNBIND_REQ_SIZE) {
		ae_dlerrorack(wq, mp, DL_UNBIND_REQ, DL_BADPRIM, 0);
		return;
	}

	if (slp->sl_state != DL_IDLE) {
		ae_dlerrorack(wq, mp, DL_UNBIND_REQ, DL_OUTSTATE, 0);
		return;
	}

	slp->sl_state = DL_UNBOUND;
	slp->sl_sap = 0;

	(void) putnextctl1(RD(wq), M_FLUSH, FLUSHRW);
	ae_dlokack(wq, mp, DL_UNBIND_REQ);

	aesetipq(slp->sl_aep);
}

static void
aeireq(queue_t *wq, mblk_t *mp)
{
	struct	aestr	*slp;
	dl_info_ack_t	*dlip;
	struct	aedladdr	*dlap;
	struct	ether_addr	*ep;
	int	size;

	slp = (struct aestr *)wq->q_ptr;

	if (MBLKL(mp) < DL_INFO_REQ_SIZE) {
		ae_dlerrorack(wq, mp, DL_INFO_REQ, DL_BADPRIM, 0);
		return;
	}

	/*
	 * Exchange current msg for a DL_INFO_ACK.
	 */
	size = sizeof (dl_info_ack_t) + AEADDRL + ETHERADDRL;
	if ((mp = ae_mexchange(wq, mp, size, M_PCPROTO, DL_INFO_ACK)) == NULL)
		return;

	/*
	 * Fill in the DL_INFO_ACK fields and reply.
	 */
	dlip = (dl_info_ack_t *)mp->b_rptr;
	*dlip = aeinfoack;
	dlip->dl_current_state = slp->sl_state;
	dlap = (struct aedladdr *)(mp->b_rptr + dlip->dl_addr_offset);
	dlap->dl_sap = slp->sl_sap;
	if (slp->sl_aep) {
		ether_copy(&slp->sl_aep->ae_ouraddr, &dlap->dl_phys);
	} else {
		bzero((caddr_t)&dlap->dl_phys, ETHERADDRL);
	}
	ep = (struct ether_addr *)(mp->b_rptr + dlip->dl_brdcst_addr_offset);
	ether_copy(&etherbroadcastaddr, ep);

	qreply(wq, mp);
}

static void
aeponreq(queue_t *wq, mblk_t *mp)
{
	struct	aestr	*slp;

	slp = (struct aestr *)wq->q_ptr;

	if (MBLKL(mp) < DL_PROMISCON_REQ_SIZE) {
		ae_dlerrorack(wq, mp, DL_PROMISCON_REQ, DL_BADPRIM, 0);
		return;
	}

	switch (((dl_promiscon_req_t *)mp->b_rptr)->dl_level) {
		case DL_PROMISC_PHYS:
			slp->sl_flags |= SLALLPHYS;
			break;

		case DL_PROMISC_SAP:
			slp->sl_flags |= SLALLSAP;
			break;

		case DL_PROMISC_MULTI:
			slp->sl_flags |= SLALLMULTI;
			break;

		default:
			ae_dlerrorack(wq, mp, DL_PROMISCON_REQ,
				DL_NOTSUPPORTED, 0);
			return;
	}

	if (slp->sl_aep)
		(void) aeinit(slp->sl_aep);

	if (slp->sl_aep)
		aesetipq(slp->sl_aep);

	ae_dlokack(wq, mp, DL_PROMISCON_REQ);
}

static void
aepoffreq(queue_t *wq, mblk_t *mp)
{
	struct	aestr	*slp;
	int	flag;

	slp = (struct aestr *)wq->q_ptr;

	if (MBLKL(mp) < DL_PROMISCOFF_REQ_SIZE) {
		ae_dlerrorack(wq, mp, DL_PROMISCOFF_REQ, DL_BADPRIM, 0);
		return;
	}

	switch (((dl_promiscoff_req_t *)mp->b_rptr)->dl_level) {
		case DL_PROMISC_PHYS:
			flag = SLALLPHYS;
			break;

		case DL_PROMISC_SAP:
			flag = SLALLSAP;
			break;

		case DL_PROMISC_MULTI:
			flag = SLALLMULTI;
			break;

		default:
			ae_dlerrorack(wq, mp, DL_PROMISCOFF_REQ,
				DL_NOTSUPPORTED, 0);
			return;
	}

	if ((slp->sl_flags & flag) == 0) {
		ae_dlerrorack(wq, mp, DL_PROMISCOFF_REQ, DL_NOTENAB, 0);
		return;
	}

	slp->sl_flags &= ~flag;
	if (slp->sl_aep)
		(void) aeinit(slp->sl_aep);

	if (slp->sl_aep)
		aesetipq(slp->sl_aep);

	ae_dlokack(wq, mp, DL_PROMISCOFF_REQ);
}

static void
aeemreq(queue_t *wq, mblk_t *mp)
{
	struct	aestr	*slp;
	union	DL_primitives	*dlp;
	struct	ether_addr	*addrp;
	int	off;
	int	len, i;

	slp = (struct aestr *)wq->q_ptr;

	if (MBLKL(mp) < DL_ENABMULTI_REQ_SIZE) {
		ae_dlerrorack(wq, mp, DL_ENABMULTI_REQ, DL_BADPRIM, 0);
		return;
	}

	if (slp->sl_state == DL_UNATTACHED) {
		ae_dlerrorack(wq, mp, DL_ENABMULTI_REQ, DL_OUTSTATE, 0);
		return;
	}

	dlp = (union DL_primitives *)mp->b_rptr;
	len = dlp->enabmulti_req.dl_addr_length;
	off = dlp->enabmulti_req.dl_addr_offset;
	addrp = (struct ether_addr *)(mp->b_rptr + off);

	if ((len != ETHERADDRL) ||
		!MBLKIN(mp, off, len) ||
		((addrp->ether_addr_octet[0] & 01) == 0)) {
		ae_dlerrorack(wq, mp, DL_ENABMULTI_REQ, DL_BADADDR, 0);
		return;
	}

	if ((slp->sl_mccount + 1) >= AEMAXMC) {
		ae_dlerrorack(wq, mp, DL_ENABMULTI_REQ, DL_TOOMANY, 0);
		return;
	}

	/*
	 * Allocate table on first request.
	 */
	if (slp->sl_mctab == NULL)
		slp->sl_mctab = kmem_alloc(AEMCALLOC, KM_SLEEP);

	/*
	 * Check to see if the address is already in the table.
	 */
	for (i = 0; i < slp->sl_mccount; i++) {
		if (ether_cmp(&slp->sl_mctab[i], addrp) == 0) {
			ae_dlokack(wq, mp, DL_ENABMULTI_REQ);
			return;
		}
	}

	slp->sl_mctab[slp->sl_mccount++] = *addrp;

	(void) aeinit(slp->sl_aep);
	ae_dlokack(wq, mp, DL_ENABMULTI_REQ);
}

static void
aedmreq(queue_t *wq, mblk_t *mp)
{
	struct	aestr	*slp;
	union	DL_primitives	*dlp;
	struct	ether_addr	*addrp;
	int	off;
	int	len;
	int	i;

	slp = (struct aestr *)wq->q_ptr;

	if (MBLKL(mp) < DL_DISABMULTI_REQ_SIZE) {
		ae_dlerrorack(wq, mp, DL_DISABMULTI_REQ, DL_BADPRIM, 0);
		return;
	}

	if (slp->sl_state == DL_UNATTACHED) {
		ae_dlerrorack(wq, mp, DL_DISABMULTI_REQ, DL_OUTSTATE, 0);
		return;
	}

	dlp = (union DL_primitives *)mp->b_rptr;
	len = dlp->disabmulti_req.dl_addr_length;
	off = dlp->disabmulti_req.dl_addr_offset;
	addrp = (struct ether_addr *)(mp->b_rptr + off);

	if ((len != ETHERADDRL) || !MBLKIN(mp, off, len)) {
		ae_dlerrorack(wq, mp, DL_DISABMULTI_REQ, DL_BADADDR, 0);
		return;
	}

	/*
	 * Find the address in the multicast table for this Stream
	 * and delete it by shifting all subsequent multicast
	 * table entries over one.
	 */
	for (i = 0; i < slp->sl_mccount; i++)
		if (ether_cmp(addrp, &slp->sl_mctab[i]) == 0) {
			bcopy((caddr_t)&slp->sl_mctab[i+1],
				(caddr_t)&slp->sl_mctab[i],
				((slp->sl_mccount - i) *
				sizeof (struct ether_addr)));
			slp->sl_mccount--;
			(void) aeinit(slp->sl_aep);
			ae_dlokack(wq, mp, DL_DISABMULTI_REQ);
			return;
		}
	ae_dlerrorack(wq, mp, DL_DISABMULTI_REQ, DL_NOTENAB, 0);
}

static void
aepareq(queue_t *wq, mblk_t *mp)
{
	struct	aestr	*slp;
	union	DL_primitives	*dlp;
	int	type;
	struct	ae	*aep;
	struct	ether_addr	addr;

	slp = (struct aestr *)wq->q_ptr;

	if (MBLKL(mp) < DL_PHYS_ADDR_REQ_SIZE) {
		ae_dlerrorack(wq, mp, DL_PHYS_ADDR_REQ, DL_BADPRIM, 0);
		return;
	}

	dlp = (union DL_primitives *)mp->b_rptr;
	type = dlp->physaddr_req.dl_addr_type;
	aep = slp->sl_aep;

	if (aep == NULL) {
		ae_dlerrorack(wq, mp, DL_PHYS_ADDR_REQ, DL_OUTSTATE, 0);
		return;
	}

	switch (type) {
		case	DL_FACT_PHYS_ADDR:
			ae_etheraddr((struct ether_addr *)NULL, &addr);
			break;

		case	DL_CURR_PHYS_ADDR:
			ether_copy(&aep->ae_ouraddr, &addr);
			break;

		default:
			ae_dlerrorack(wq, mp, DL_PHYS_ADDR_REQ,
				DL_NOTSUPPORTED, 0);
			return;
	}

	ae_dlphysaddrack(wq, mp, (caddr_t)&addr, ETHERADDRL);
}

static void
aespareq(queue_t *wq, mblk_t *mp)
{
	struct	aestr	*slp;
	union	DL_primitives	*dlp;
	int	off;
	int	len;
	struct	ether_addr	*addrp;
	struct	ae	*aep;

	slp = (struct aestr *)wq->q_ptr;

	if (MBLKL(mp) < DL_SET_PHYS_ADDR_REQ_SIZE) {
		ae_dlerrorack(wq, mp, DL_SET_PHYS_ADDR_REQ, DL_BADPRIM, 0);
		return;
	}

	dlp = (union DL_primitives *)mp->b_rptr;
	len = dlp->set_physaddr_req.dl_addr_length;
	off = dlp->set_physaddr_req.dl_addr_offset;

	if (!MBLKIN(mp, off, len)) {
		ae_dlerrorack(wq, mp, DL_SET_PHYS_ADDR_REQ, DL_BADPRIM, 0);
		return;
	}

	addrp = (struct ether_addr *)(mp->b_rptr + off);

	/*
	 * Error if length of address isn't right or the address
	 * specified is a multicast or broadcast address.
	 */
	if ((len != ETHERADDRL) ||
		((addrp->ether_addr_octet[0] & 01) == 1) ||
		(ether_cmp(addrp, &etherbroadcastaddr) == 0)) {
		ae_dlerrorack(wq, mp, DL_SET_PHYS_ADDR_REQ, DL_BADADDR, 0);
		return;
	}

	/*
	 * Error if this stream is not attached to a device.
	 */
	if ((aep = slp->sl_aep) == NULL) {
		ae_dlerrorack(wq, mp, DL_SET_PHYS_ADDR_REQ, DL_OUTSTATE, 0);
		return;
	}

	/*
	 * Set new interface local address and re-init device.
	 * This is destructive to any other streams attached
	 * to this device.
	 */
	ether_copy(addrp, &aep->ae_ouraddr);
	(void) aeinit(slp->sl_aep);

	ae_dlokack(wq, mp, DL_SET_PHYS_ADDR_REQ);
}

static void
aeudreq(queue_t *wq, mblk_t *mp)
{
	struct	aestr	*slp;
	register	struct	ae	*aep;
	register	dl_unitdata_req_t	*dludp;
	mblk_t	*nmp;
	struct	aedladdr	*dlap;
	struct	ether_header	*headerp;
	uint32_t	off, len;
	uint32_t	sap;

	slp = (struct aestr *)wq->q_ptr;
	aep = slp->sl_aep;

	if (slp->sl_state != DL_IDLE) {
		ae_dlerrorack(wq, mp, DL_UNITDATA_REQ, DL_OUTSTATE, 0);
		return;
	}

	dludp = (dl_unitdata_req_t *)mp->b_rptr;

	off = dludp->dl_dest_addr_offset;
	len = dludp->dl_dest_addr_length;

	/*
	 * Validate destination address format.
	 */
	if (!MBLKIN(mp, off, len) || (len != AEADDRL)) {
		ae_dlerrorack(wq, mp, DL_UNITDATA_REQ, DL_BADADDR, 0);
		return;
	}

	/*
	 * Error if no M_DATA follows.
	 */
	nmp = mp->b_cont;
	if (nmp == NULL) {
		ae_dluderrorind(wq, mp, mp->b_rptr + off, len, DL_BADDATA, 0);
		return;
	}

	dlap = (struct aedladdr *)(mp->b_rptr + off);

	/*
	 * Create ethernet header by either prepending it onto the
	 * next mblk if possible, or reusing the M_PROTO block if not.
	 */
	if ((DB_REF(nmp) == 1) &&
		(MBLKHEAD(nmp) >= sizeof (struct ether_header)) &&
		(((uint32_t)nmp->b_rptr & 0x1) == 0)) {
		nmp->b_rptr -= sizeof (struct ether_header);
		headerp = (struct ether_header *)nmp->b_rptr;
		ether_copy(&dlap->dl_phys, &headerp->ether_dhost);
		ether_copy(&aep->ae_ouraddr, &headerp->ether_shost);
		sap = dlap->dl_sap;
		sap = slp->sl_sap; /* DBC */
		freeb(mp);
		mp = nmp;
	} else {
		DB_TYPE(mp) = M_DATA;
		headerp = (struct ether_header *)mp->b_rptr;
		mp->b_wptr = mp->b_rptr + sizeof (struct ether_header);
		ether_copy(&dlap->dl_phys, &headerp->ether_dhost);
		ether_copy(&aep->ae_ouraddr, &headerp->ether_shost);
		sap = dlap->dl_sap;
		sap = slp->sl_sap; /* DBC */
	}

	/*
	 * sap value in the range [0-1500] are treated equivalent
	 * and represent a desire to use 802.3 mode. Therefore compute
	 * the length and put it in the type field in header
	 */
	if ((sap <= ETHERMTU) || (slp->sl_sap == 0)) {
		sap = ae_msgsize(mp) - sizeof (struct ether_header);
		put_ether_type(headerp, sap);
	} else {
		put_ether_type(headerp, sap);
	}
	(void) aestart(wq, mp, aep);
}

/*
 * Start transmission.
 * Return zero on success,
 * otherwise put msg on wq, set 'want' flag and return nonzero.
 */
static
aestart(queue_t *wq, register mblk_t *mp, register struct ae *aep)
{
	volatile	struct	lmd	*tmdp, *ntmdp;
	struct	aebuf	*lbp;
	mblk_t	*nmp = NULL;
	int	len;
	uint32_t	a;
	u_int	sbound, soff;
	int	i;
	int	flags;

	TRACE_1(TR_FAC_AE, TR_AE_START_START, "aestart start:  wq %X", wq);

	flags = aep->ae_flags;

	if ((flags & (AERUNNING|AEPROMISC)) != AERUNNING) {
		if (!(flags & AERUNNING)) {
			(void) putbq(wq, mp);
			return (1);
		}
		if (flags & AEPROMISC) {
			if ((nmp = copymsg(mp)) == NULL)
				aep->ae_allocbfail++;
		}
	}

	len = ae_msgsize(mp);
	if (len > ETHERMAX) {
		aeerror(aep->ae_dip, "msg too big:  %d", len);
		aep->ae_oerrors++;
		freemsg(mp);
		if (nmp)
			freemsg(nmp);
		TRACE_1(TR_FAC_AE, TR_AE_START_END,
			"aestart end:  wq %X", wq);
		return (0);
	}

	/*
	 * Only copy msg to local tbuf under certain conditions.
	 */

	/*
	 * Get a local tbuf.
	 */
	if ((lbp = aegetbuf(aep, 1)) == NULL) {
		aep->ae_notbufs++;
		(void) putbq(wq, mp);
		aep->ae_wantw = 1;
		if (nmp)
			freemsg(nmp);
		TRACE_1(TR_FAC_AE, TR_AE_START_END,
			"aestart end:  wq %X", wq);
		return (1);
	}

	/*
	 * Copy msg into tbuf and free the original msg.
	 */
	a = (uint32_t)lbp->lb_buf;
	if (mp->b_cont == NULL) {
		sbound = (u_int)mp->b_rptr & ~AEBURSTMASK;
		soff = (u_int)mp->b_rptr - sbound;
		bcopy((caddr_t)sbound, (caddr_t)a,
			AEROUNDUP(len + soff, AEBURSTSIZE));
		freemsg(mp);
	} else if (mp->b_cont->b_cont == NULL) {
		/* two streams buffers */
		sbound = (u_int)mp->b_cont->b_rptr & ~AEBURSTMASK;
		soff = (u_int)mp->b_cont->b_rptr - sbound;
		a = AEROUNDUP(a + MBLKL(mp), AEBURSTSIZE);
		bcopy((caddr_t)sbound, (caddr_t)a,
		    AEROUNDUP(MBLKL(mp->b_cont) + soff, AEBURSTSIZE));
		a = a + soff - MBLKL(mp);
		bcopy((caddr_t)mp->b_rptr, (caddr_t)a, MBLKL(mp));
		/* Yuk! */
		soff = (a - (uint32_t)lbp->lb_buf);
		freemsg(mp);
	} else {
		/* more than two streams buffers */
		(void) ae_mcopymsg(mp, (uint8_t *)a);
		soff = 0;
	}
	mp = NULL;
	a = lbp->phys_addr + soff;

	/*
	 * Pad out to ETHERMIN.
	 */
	if (len < ETHERMIN)
		len = ETHERMIN;

	/*
	 * Acquire xmit lock now and hold it until done.
	 */
	mutex_enter(&aep->ae_xmitlock);

	/*
	 * Allocate a tmd and increment pointer to next tmd.
	 * Never allow "next" pointer to point to an in-use tmd.
	 */
	tmdp = aep->ae_tnextp;
	ntmdp = NEXTTMD(aep, tmdp);
	if (ntmdp == aep->ae_tcurp) {	/* out of tmds */
		aep->ae_notmds++;
		mutex_exit(&aep->ae_xmitlock);
		if (nmp)
			freemsg(nmp);
		aefreebuf(lbp);
		TRACE_1(TR_FAC_AE, TR_AE_START_END,
			"aestart end:  wq %X", wq);
		return (0);
	}

	/*
	 * Save tbuf to free later.
	 */
	i = tmdp - aep->ae_tmdp;
	aep->ae_tbufp[i] = lbp;

	/*
	 * Sync the buffer so the device sees it.
	 */
	(void) ddi_dma_sync(lbp->lb_buf_hdl, soff, len, DDI_DMA_SYNC_FORDEV);

	/*
	 * Write out the new TMD.
	 */

	ddi_put32(aep->ae_acchdl, (uint32_t *)&tmdp->lmd_addr, a);
	ddi_put16(aep->ae_acchdl, (uint16_t *)&tmdp->lmd_bcnt, -len);
	ddi_put16(aep->ae_acchdl, (uint16_t *)&tmdp->lmd_flags3, 0);
	ddi_put8(aep->ae_acchdl,
		(uchar_t *)&tmdp->lmd_flags,  LMD_STP | LMD_ENP | LMD_OWN);

	/*
	 * Update our TMD ring pointer.
	 */
	aep->ae_tnextp = ntmdp;

	/*
	 * Sync the TMD.
	 */
	AESYNCIOPB(aep, tmdp, sizeof (struct lmd), DDI_DMA_SYNC_FORDEV);

	/*
	 * Bang the chip.
	 */
	LANCE_PUT_CSR0(LANCE_TDMD | LANCE_INEA);

	/*
	 * Now it's ok to release the lock.
	 */
	mutex_exit(&aep->ae_xmitlock);

	/*
	 * Loopback a broadcast packet.
	 */
	if ((flags & AEPROMISC) && nmp)
		aesendup(aep, nmp, aepaccept);

	TRACE_1(TR_FAC_AE, TR_AE_START_END, "aestart end:  wq %X", wq);
	return (0);
}

static u_int
aeintr(register	struct ae *aep)
{
	volatile	struct	lmd	*rmdp;
	volatile	uint16_t	csr0;
	int	serviced = 0;

	mutex_enter(&aep->ae_intrlock);

	csr0 = (uint16_t)LANCE_GET_CSR0();

	TRACE_2(TR_FAC_AE, TR_AE_INTR_START,
		"aeintr start:  aep %X csr %X", aep, csr0);

	if ((csr0 & LANCE_INTR) == 0) {
		mutex_exit(&aep->ae_intrlock);
		TRACE_2(TR_FAC_AE, TR_AE_INTR_END,
			"aeintr end:  aep %X serviced %d",
			aep, serviced);
		return (serviced);
	}

	/*
	 * Clear RINT/TINT .
	 */
	LANCE_PUT_CSR0((csr0 & (LANCE_RINT | LANCE_TINT)) | LANCE_INEA);
	serviced = 1;

	/*
	 * Check for receive activity.
	 * One packet per RMD.
	 */
	if (csr0 & LANCE_RINT) {
		rmdp = aep->ae_rnextp;

		/*
		 * Sync RMD before looking at it.
		 */
		AESYNCIOPB(aep, rmdp, sizeof (struct lmd),
			DDI_DMA_SYNC_FORCPU);

		/*
		 * Loop through each RMD.
		 */
		while ((ddi_get8(aep->ae_acchdl,
				(uchar_t *)&rmdp->lmd_flags) & LMD_OWN) == 0) {
			aeread(aep, rmdp);

			/*
			 * Give the descriptor and associated
			 * buffer back to the chip.
			 */
			ddi_put16(aep->ae_acchdl,
				(uint16_t *)&rmdp->lmd_mcnt, 0);
			ddi_put8(aep->ae_acchdl,
				(uchar_t *)&rmdp->lmd_flags, LMD_OWN);

			/*
			 * Sync the RMD after writing it.
			 */
			AESYNCIOPB(aep, rmdp, sizeof (struct lmd),
				DDI_DMA_SYNC_FORDEV);

			/*
			 * Increment to next RMD.
			 */
			aep->ae_rnextp = rmdp = NEXTRMD(aep, rmdp);

			/*
			 * Sync the next RMD before looking at it.
			 */
			AESYNCIOPB(aep, rmdp, sizeof (struct lmd),
				DDI_DMA_SYNC_FORCPU);
		}
	}

	/*
	 * Check for transmit activity.
	 * One packet per TMD.
	 */
	if (csr0 & LANCE_TINT)
		aereclaim(aep);

	/*
	 * Check for errors not specifically related
	 * to transmission or reception.
	 */
	if ((csr0 & (LANCE_BABL|LANCE_MERR|LANCE_MISS|LANCE_TXON|LANCE_RXON))
		!= (LANCE_TXON|LANCE_RXON)) {
		(void) ae_chip_error(aep);
	}

done:

	/*
	 * Read back register to flush write buffers.
	 */
	csr0 = (uint16_t)LANCE_GET_CSR0();

	mutex_exit(&aep->ae_intrlock);
	TRACE_2(TR_FAC_AE, TR_AE_INTR_END,
		"aeintr end:  aep %X serviced %d", aep, serviced);
	return (serviced);
}

/*
 * Transmit completion reclaiming.
 */
static void
aereclaim(struct ae *aep)
{
	volatile	struct	lmd	*tmdp;
	int	flags;
	int	i;

	tmdp = aep->ae_tcurp;

	/*
	 * Sync TMD before looking at it.
	 */
	AESYNCIOPB(aep, tmdp, sizeof (struct lmd), DDI_DMA_SYNC_FORCPU);

	/*
	 * Loop through each TMD.
	 */
	while ((((flags = (int)ddi_get8(aep->ae_acchdl,
			(uchar_t *)&tmdp->lmd_flags)) &
			(LMD_OWN | TMD_INUSE)) == 0) &&
			(tmdp != aep->ae_tnextp)) {

		/*
		 * Keep defer/retry statistics.
		 */
		if (flags & TMD_DEF)
			aep->ae_defer++;
		if (flags & TMD_ONE)
			aep->ae_collisions++;
		else if (flags & TMD_MORE)
			aep->ae_collisions += 2;

		/*
		 * Check for transmit errors and keep output stats.
		 */
		if (ddi_get16(aep->ae_acchdl, (uint16_t *)&tmdp->lmd_flags3)
					& TMD_ANYERROR) {
			aep->ae_oerrors++;
			if (ae_xmit_error(aep, (struct lmd *)tmdp))
				return;
		} else
			aep->ae_opackets++;

		/*
		 * Free msg or buffer.
		 */
		i = tmdp - aep->ae_tmdp;
		if (aep->ae_tmblkp[i]) {
			freemsg(aep->ae_tmblkp[i]);
			aep->ae_tmblkp[i] = NULL;
		} else if (aep->ae_tbufp[i]) {
			aefreebuf(aep->ae_tbufp[i]);
			aep->ae_tbufp[i] = NULL;
		}

		tmdp = NEXTTMD(aep, tmdp);

		/*
		 * Sync TMD before looking at it.
		 */
		AESYNCIOPB(aep, tmdp, sizeof (struct lmd),
			DDI_DMA_SYNC_FORCPU);
	}

	aep->ae_tcurp = tmdp;

	/*
	 * Check for any msgs that were queued
	 * due to out-of-tmd condition.
	 */
	if (aep->ae_wantw)
		aewenable(aep);
}

/*
 * Start xmit on any msgs previously enqueued on any write queues.
 */
static void
aewenable(struct ae *aep)
{
	struct	aestr	*slp;
	queue_t	*wq;

	/*
	 * Order of wantw accesses is important.
	 */
	do {
		aep->ae_wantw = 0;
		for (slp = aestrup; slp; slp = slp->sl_nextp)
			if ((slp->sl_aep == aep) &&
				((wq = WR(slp->sl_rq))->q_first))
				qenable(wq);
	} while (aep->ae_wantw);
}

/*
 * Test upstream destination sap and address match.
 */
static struct aestr *
aeaccept(register struct aestr *slp, register struct ae	*aep, int type,
	struct ether_addr *addrp)
{
	int	sap;
	int	flags;

	for (; slp; slp = slp->sl_nextp) {
		sap = slp->sl_sap;
		flags = slp->sl_flags;

		if ((slp->sl_aep == aep) && AESAPMATCH(sap, type, flags))
			if ((ether_cmp(addrp, &aep->ae_ouraddr) == 0) ||
				(ether_cmp(addrp, &etherbroadcastaddr) == 0) ||
				(flags & SLALLPHYS) ||
				aemcmatch(slp, addrp))
				return (slp);
	}

	return (NULL);
}

/*
 * Test upstream destination sap and address match for SLALLPHYS only.
 */
/* ARGSUSED3 */
static struct aestr *
aepaccept(register struct aestr *slp, register struct ae	*aep, int type,
	struct ether_addr *addrp)
{
	int	sap;
	int	flags;

	for (; slp; slp = slp->sl_nextp) {
		sap = slp->sl_sap;
		flags = slp->sl_flags;

		if ((slp->sl_aep == aep) &&
			AESAPMATCH(sap, type, flags) &&
			(flags & SLALLPHYS))
			return (slp);
	}

	return (NULL);
}

/*
 * Set or clear the device ipq pointer.
 * XXX Assumes IP is SLFAST.
 */
static void
aesetipq(struct ae *aep)
{
	struct	aestr	*slp;
	int	ok = 1;
	queue_t	*ipq = NULL;

	rw_enter(&aestruplock, RW_READER);

	for (slp = aestrup; slp; slp = slp->sl_nextp)
		if (slp->sl_aep == aep) {
			if (slp->sl_flags & (SLALLPHYS|SLALLSAP))
				ok = 0;
			if (slp->sl_sap == ETHERTYPE_IP)
				if (ipq == NULL)
					ipq = slp->sl_rq;
				else
					ok = 0;
		}

	rw_exit(&aestruplock);

	if (ok)
		aep->ae_ipq = ipq;
	else
		aep->ae_ipq = NULL;
}

static void
aeread(struct ae *aep, volatile	struct lmd *rmdp)
{
	register	int	i;
	register	mblk_t	*mp;
	register	struct	aebuf	*lbp, *nlbp;
	struct	ether_header	*ehp;
	queue_t	*ipq;
	uint32_t	bufp;
	int	len;
	u_int	sbound, soff;
	uint32_t	a;
	u_int flags;

	TRACE_0(TR_FAC_AE, TR_AE_READ_START, "aeread start");

	aep->ae_ipackets++;

	/*
	 * Check for packet errors.
	 */
	flags = (u_int)ddi_get8(aep->ae_acchdl,
				(uchar_t *)&rmdp->lmd_flags);

	if ((flags & ~RMD_OFLO) != (LMD_STP | LMD_ENP)) {
		ae_rcv_error(aep, (struct lmd *)rmdp);
		aep->ae_ierrors++;
		TRACE_0(TR_FAC_AE, TR_AE_READ_END, "aeread end");
		return;
	}

	i = rmdp - aep->ae_rmdp;
	lbp = aep->ae_rbufp[i];

	bufp = (uint32_t)lbp->lb_buf + AEHEADROOM;
	len = (int)ddi_get16(aep->ae_acchdl,
			(uint16_t *)&rmdp->lmd_mcnt) - ETHERFCSL;

	if (len < ETHERMIN) {
		TRACE_0(TR_FAC_AE, TR_AE_READ_END, "aeread end");
		return;
	}

	/*
	 * Sync the received buffer before looking at it.
	 */

	(void) ddi_dma_sync(lbp->lb_buf_hdl, AEHEADROOM, len,
		DDI_DMA_SYNC_FORKERNEL);

	/*
	 * If a (MASTER or sun4c) and buffers are available,
	 * then "loan up".  Otherwise allocb a new mblk and copy.
	 */
	if ((NOCOPYBUFF) && (nlbp = aegetbuf(aep, 0))) {
		if ((mp = esballoc((uint8_t *)bufp, len, BPRI_LO,
			&lbp->lb_frtn)) == NULL) {
			aep->ae_allocbfail++;
			aep->ae_ierrors++;
			if (aedebug)
				aeerror(aep->ae_dip, "esballoc failed");
			aefreebuf(nlbp);
			TRACE_0(TR_FAC_AE, TR_AE_READ_END, "aeread end");
			return;
		}
		mp->b_wptr += len;
		/* leave the new rbuf with the current rmd. */
		a = nlbp->phys_addr + AEHEADROOM;
		ddi_put32(aep->ae_acchdl, (uint32_t *)&rmdp->lmd_addr, a);
		aep->ae_rbufp[i] = nlbp;

	} else {
		aep->ae_norbufs++;

		/* allocate and aligned-copy */
		if ((mp = allocb(len + (2 * AEBURSTSIZE), BPRI_LO))
			== NULL) {
			aep->ae_ierrors++;
			aep->ae_allocbfail++;
			if (aedebug)
				aeerror(aep->ae_dip, "allocb fail");
			TRACE_0(TR_FAC_AE, TR_AE_READ_END, "aeread end");
			return;
		}

		mp->b_rptr = (uint8_t *)
			AEROUNDUP((u_int)mp->b_rptr, AEBURSTSIZE);
		sbound = bufp & ~AEBURSTMASK;
		soff = bufp - sbound;
		bcopy((caddr_t)sbound, (caddr_t)mp->b_rptr,
			AEROUNDUP(len + soff, AEBURSTSIZE));
		mp->b_rptr += soff;
		mp->b_wptr = mp->b_rptr + len;
	}

	ehp = (struct ether_header *)bufp;
	ipq = aep->ae_ipq;

	/*
	 * IP shortcut
	 */
	if ((get_ether_type(ehp) == ETHERTYPE_IP) &&
		((ehp->ether_dhost.ether_addr_octet[0] & 01) == 0) &&
		(ipq) &&
		canput(ipq->q_next)) {
		mp->b_rptr += sizeof (struct ether_header);
		(void) putnext(ipq, mp);
	} else {

		/* Strip the PADs for 802.3 */
		if (get_ether_type(ehp) +
			sizeof (struct ether_header) < ETHERMIN)
			mp->b_wptr = mp->b_rptr
					+ sizeof (struct ether_header)
					+ get_ether_type(ehp);

		aesendup(aep, mp, aeaccept);
	    }

	TRACE_0(TR_FAC_AE, TR_AE_READ_END, "aeread end");
}

/*
 * Send packet upstream.
 * Assume mp->b_rptr points to ether_header.
 */
static void
aesendup(struct	ae *aep, mblk_t	*mp, struct aestr *(*acceptfunc)())
{
	struct	ether_addr	*dhostp, *shostp;
	struct	aestr	*slp, *nslp;
	mblk_t	*nmp;
	uint32_t	isgroupaddr;
	int	type;

	TRACE_0(TR_FAC_AE, TR_AE_SENDUP_START, "aesendup start");

	dhostp = &((struct ether_header *)mp->b_rptr)->ether_dhost;
	shostp = &((struct ether_header *)mp->b_rptr)->ether_shost;
	type = get_ether_type(mp->b_rptr);

	isgroupaddr = dhostp->ether_addr_octet[0] & 01;

	/*
	 * While holding a reader lock on the linked list of streams
	 * structures, attempt to match the address criteria for each stream
	 * and pass up the raw M_DATA ("fastpath") or a DL_UNITDATA_IND.
	 */

	rw_enter(&aestruplock, RW_READER);

	if ((slp = (*acceptfunc)(aestrup, aep, type, dhostp)) == NULL) {
		rw_exit(&aestruplock);
		freemsg(mp);
		TRACE_0(TR_FAC_AE, TR_AE_SENDUP_END, "aesendup end");
		return;
	}

	/*
	 * Loop on matching open streams until (*acceptfunc)() returns NULL.
	 */
	for (; nslp = (*acceptfunc)(slp->sl_nextp, aep, type, dhostp);
		slp = nslp)
		if (canput(slp->sl_rq->q_next))
			if (nmp = dupmsg(mp)) {
				if ((slp->sl_flags & SLFAST) && !isgroupaddr) {
					nmp->b_rptr +=
						sizeof (struct ether_header);
					(void) putnext(slp->sl_rq, nmp);
				} else if (slp->sl_flags & SLRAW)
					(void) putnext(slp->sl_rq, nmp);
				else if ((nmp = aeaddudind(aep, nmp, shostp,
						dhostp, type, isgroupaddr)))
						(void) putnext(slp->sl_rq, nmp);
			}
			else
				aep->ae_allocbfail++;
		else
			aep->ae_nocanput++;


	/*
	 * Do the last one.
	 */
	if (canput(slp->sl_rq->q_next)) {
		if ((slp->sl_flags & SLFAST) && !isgroupaddr) {
			mp->b_rptr += sizeof (struct ether_header);
			(void) putnext(slp->sl_rq, mp);
		} else if (slp->sl_flags & SLRAW)
			(void) putnext(slp->sl_rq, mp);
		else if ((mp = aeaddudind(aep, mp, shostp, dhostp,
			type, isgroupaddr)))
			(void) putnext(slp->sl_rq, mp);
	} else {
		freemsg(mp);
		aep->ae_nocanput++;
	}

	rw_exit(&aestruplock);
	TRACE_0(TR_FAC_AE, TR_AE_SENDUP_END, "aesendup end");
}

/*
 * Prefix msg with a DL_UNITDATA_IND mblk and return the new msg.
 */
static mblk_t *
aeaddudind(aep, mp, shostp, dhostp, type, isgroupaddr)
	struct	ae	*aep;
	mblk_t	*mp;
	struct	ether_addr	*shostp, *dhostp;
	int	type;
	uint32_t	isgroupaddr;
{
	dl_unitdata_ind_t	*dludindp;
	struct	aedladdr	*dlap;
	mblk_t	*nmp;
	int	size;

	TRACE_0(TR_FAC_AE, TR_AE_ADDUDIND_START, "aeaddudind start");

	mp->b_rptr += sizeof (struct ether_header);

	/*
	 * Allocate an M_PROTO mblk for the DL_UNITDATA_IND.
	 */
	size = sizeof (dl_unitdata_ind_t) + AEADDRL + AEADDRL;
#ifdef old
	nmp = allocb(AEROUNDUP(AEHEADROOM + size, sizeof (double)), BPRI_LO);
#else
	nmp = allocb((AEHEADROOM + size), BPRI_LO);
#endif
	if (nmp == NULL) {
		aep->ae_allocbfail++;
		aep->ae_ierrors++;
		if (aedebug)
			aeerror(aep->ae_dip, "allocb failed");
		freemsg(mp);
		TRACE_0(TR_FAC_AE, TR_AE_ADDUDIND_END, "aeaddudind end");
		return (NULL);
	}
	DB_TYPE(nmp) = (uint8_t)M_PROTO;
	nmp->b_wptr = nmp->b_datap->db_lim;
	nmp->b_rptr = nmp->b_wptr - size;

	/*
	 * Construct a DL_UNITDATA_IND primitive.
	 */
	dludindp = (dl_unitdata_ind_t *)nmp->b_rptr;
	dludindp->dl_primitive = DL_UNITDATA_IND;
	dludindp->dl_dest_addr_length = AEADDRL;
	dludindp->dl_dest_addr_offset = sizeof (dl_unitdata_ind_t);
	dludindp->dl_src_addr_length = AEADDRL;
	dludindp->dl_src_addr_offset = sizeof (dl_unitdata_ind_t) + AEADDRL;
	dludindp->dl_group_address = isgroupaddr;

	dlap = (struct aedladdr *)(nmp->b_rptr + sizeof (dl_unitdata_ind_t));
	ether_copy(dhostp, &dlap->dl_phys);
	dlap->dl_sap = (uint16_t)type;

	dlap = (struct aedladdr *)(nmp->b_rptr + sizeof (dl_unitdata_ind_t)
		+ AEADDRL);
	ether_copy(shostp, &dlap->dl_phys);
	dlap->dl_sap = (uint16_t)type;

	/*
	 * Link the M_PROTO and M_DATA together.
	 */
	nmp->b_cont = mp;
	TRACE_0(TR_FAC_AE, TR_AE_ADDUDIND_END, "aeaddudind end");
	return (nmp);
}

/*
 * Return TRUE if the given multicast address is one
 * of those that this particular Stream is interested in.
 */
static
aemcmatch(slp, addrp)
register	struct	aestr	*slp;
register	struct	ether_addr	*addrp;
{
	register	struct	ether_addr	*mctab;
	register	int	mccount;
	register	int	i;

	/*
	 * Return FALSE if not a multicast address.
	 */
	if (!(addrp->ether_addr_octet[0] & 01))
		return (0);

	/*
	 * Check if all multicasts have been enabled for this Stream
	 */
	if (slp->sl_flags & SLALLMULTI)
		return (1);

	/*
	 * Return FALSE if no multicast addresses enabled for this Stream.
	 */
	if (slp->sl_mccount == 0)
		return (0);

	/*
	 * Otherwise, find it in the table.
	 */

	mccount = slp->sl_mccount;
	mctab = slp->sl_mctab;

	for (i = 0; i < mccount; i++)
		if (!ether_cmp(addrp, &mctab[i]))
			return (1);

	return (0);
}

/*
 * Initialize chip and driver.
 * Return 0 on success, nonzero on error.
 */
static int
aeinit(register struct ae *aep)
{
	volatile	struct	lance_init_block	*ibp;
	struct	aebuf	*lbp;
	struct	aestr	*slp;
	uint32_t	a;
	uint32_t  busmaster;
	int	i;

	TRACE_1(TR_FAC_AE, TR_AE_INIT_START,
		"aeinit start:  aep %X", aep);

	if (aep->ae_flags & AESUSPENDED)
		ddi_dev_is_needed(aep->ae_dip, 0, 1);

	mutex_enter(&aep->ae_intrlock);
	mutex_enter(&aep->ae_xmitlock);
	rw_enter(&aestruplock, RW_WRITER);

	aep->ae_flags = 0;
	aep->ae_wantw = 0;
	aep->ae_inits++;

	/*
	 * Stop the chip.
	 */
	LANCE_CSR_WRITE(LANCE_CSR0, LANCE_STOP);

	/*
	 * Allocate DMA resources
	 */
	if (!aep->ae_ibp) {
		if (aealloc_dma_mem(aep) != DDI_SUCCESS)
			goto done;
	} else {
		/*
		 * Free any pending buffers or mgs.
		 */
		for (i = 0; i < 128; i++) {
			if (aep->ae_tbufp[i]) {
				aefreebuf(aep->ae_tbufp[i]);
				aep->ae_tbufp[i] = NULL;
			}
			if (aep->ae_tmblkp[i]) {
				freemsg(aep->ae_tmblkp[i]);
				aep->ae_tmblkp[i] = NULL;
			}
			if (aep->ae_rbufp[i]) {
				aefreebuf(aep->ae_rbufp[i]);
				aep->ae_rbufp[i] = NULL;
			}
		}
	}

	/*
	 * Reset RMD and TMD 'walking' pointers.
	 */
	aep->ae_rnextp = aep->ae_rmdp;
	aep->ae_tcurp = aep->ae_tmdp;
	aep->ae_tnextp = aep->ae_tmdp;

	/*
	 * Construct the LANCE initialization block.
	 */
	ibp = aep->ae_ibp;

	bzero((caddr_t)ibp, sizeof (struct lance_init_block));

	/*
	 * Mode word 0 should be all zeros except
	 * possibly for the promiscuous mode bit.
	 */

	ddi_put16(aep->ae_acchdl, (uint16_t *)&ibp->mode, 0);

	for (slp = aestrup; slp; slp = slp->sl_nextp)
		if ((slp->sl_aep == aep) && (slp->sl_flags & SLALLPHYS)) {
			ddi_put16(aep->ae_acchdl,
				(uint16_t *)&ibp->mode, 1);
			aep->ae_flags |= AEPROMISC;
			break;
		}

	/*
	 * Set our local individual ethernet address.
	 */
	ibp->ib_padr[0] = aep->ae_ouraddr.ether_addr_octet[0];
	ibp->ib_padr[1] = aep->ae_ouraddr.ether_addr_octet[1];
	ibp->ib_padr[2] = aep->ae_ouraddr.ether_addr_octet[2];
	ibp->ib_padr[3] = aep->ae_ouraddr.ether_addr_octet[3];
	ibp->ib_padr[4] = aep->ae_ouraddr.ether_addr_octet[4];
	ibp->ib_padr[5] = aep->ae_ouraddr.ether_addr_octet[5];

	/*
	 * Set up multicast address filter by passing all multicast
	 * addresses through a crc generator, and then using the
	 * high order 6 bits as a index into the 64 bit logical
	 * address filter. The high order two bits select the word,
	 * while the rest of the bits select the bit within the word.
	 */
	for (slp = aestrup; slp; slp = slp->sl_nextp)
		if (slp->sl_aep == aep) {
			if ((slp->sl_mccount == 0) &&
				!(slp->sl_flags & SLALLMULTI))
				continue;

			if (slp->sl_flags & SLALLMULTI) {
				for (i = 0; i < 4; i++) {
					ddi_put16(aep->ae_acchdl,
						(uint16_t *)&ibp->ib_ladrf[i],
							0xffff);
				}
				break;	/* All bits are already on */
			}

			for (i = 0; i < slp->sl_mccount; i++) {
				register uint8_t *cp;
				register uint32_t crc;
				register uint32_t c;
				register int len;
				int	j;

				cp = (uint8_t *)&slp->sl_mctab[i];
				c = *cp;
				crc = (uint32_t)0xffffffff;
				len = 6;
				while (len-- > 0) {
					c = *cp;
					for (j = 0; j < 8; j++) {
						if ((c & 0x01) ^ (crc & 0x01)) {
							crc >>= 1;
							/* polynomial */
							crc = crc ^ 0xedb88320;
						} else
							crc >>= 1;
						c >>= 1;
					}
					cp++;
				}
				/* Just want the 6 most significant bits. */
				crc = crc >> 26;

				/*
				 * Turn on the corresponding bit
				 * in the address filter.
				 */
				ddi_put16(aep->ae_acchdl,
					(uint16_t *)&ibp->ib_ladrf[crc >> 4],
					(1 << (crc & 0xf)) |
					ddi_get16(aep->ae_acchdl,
					(uint16_t *)&ibp->ib_ladrf[crc >> 4]));
			}
		}

	/* tell hardware where the receive ring is located */
	a = AEIOPBIOADDR(aep, aep->ae_rmdp);
	ddi_put32(aep->ae_acchdl, (uint32_t *)&ibp->ib_rdrp.lr_addr, a);
	ddi_put8(aep->ae_acchdl, (uchar_t *)&ibp->ib_rlen,
					aep->ae_nrmdp2 << 4);

	/* tell hardware where the transmit ring is located */
	a = AEIOPBIOADDR(aep, aep->ae_tmdp);
	ddi_put32(aep->ae_acchdl, (uint32_t *)&ibp->ib_tdrp.lr_addr, a);
	ddi_put8(aep->ae_acchdl, (uchar_t *)&ibp->ib_tlen,
					aep->ae_ntmdp2 << 4);

	/*
	 * Clear all descriptors.
	 */
	bzero((caddr_t)aep->ae_rmdp,
			(u_int)(aep->ae_nrmds * sizeof (struct lmd)));
	bzero((caddr_t)aep->ae_tmdp,
			(u_int)(aep->ae_ntmds * sizeof (struct lmd)));

	/*
	 * Hang out receive buffers.
	 */
	for (i = 0; i < aep->ae_nrmds; i++) {
		if ((lbp = aegetbuf(aep, 1)) == NULL) {
			aeerror(aep->ae_dip, "aeinit failed:  out of buffers");
			goto done;
		}
		aermdinit(aep, &aep->ae_rmdp[i], lbp);
		aep->ae_rbufp[i] = lbp;	/* save for later use */
	}

	/*
	 * Set CSR1, CSR2, and CSR3.
	 */

	a = AEIOPBIOADDR(aep, aep->ae_ibp);

	/* set software style (32bit mode) before writing any pointers */
	LANCE_CSR_WRITE(LANCE_CSR58, LANCE_PCNETPCI);

	/* Tell the chip where the initialization block is */
	LANCE_CSR_WRITE(LANCE_CSR1, (a & 0xffff)); /* low address register */
	LANCE_CSR_WRITE(LANCE_CSR2, ((a >> 16) & 0xffff)); /* hi address */

	/* Set bus master control register. Probably zero. */
	/* used to include LANCE_BSWP | LANCE_ACON | LANCE_BCON */
	busmaster = ddi_prop_get_int(DDI_DEV_T_ANY, aep->ae_dip, 0,
				    "busmaster-regval", 0);
	LANCE_CSR_WRITE(LANCE_CSR3, busmaster);

	/*
	 * Sync the init block and descriptors.
	 */
	AESYNCIOPB(aep, aep->ae_iopbkbase,
		AEROUNDUP((sizeof (struct lance_init_block)), DESCRING_ALIGN)
		+ (aep->ae_nrmds * sizeof (struct lmd))
		+ (aep->ae_ntmds * sizeof (struct lmd)),
		DDI_DMA_SYNC_FORDEV);

	/*
	 * Chip init.
	 */
	LANCE_CSR_WRITE(LANCE_CSR0, LANCE_INIT);

	/*
	 * Allow 10 ms for the chip to complete initialization.
	 */
	for (i = 10000; i > 0; i--) {
		if (LANCE_GET_CSR0() & LANCE_IDON)
			break;
		drv_usecwait(1);
	}

	if (i == 0)
		aeerror(aep->ae_dip, "LANCE chip didn't initialize!");
	else {

		/* Clear all status bits */
		LANCE_PUT_CSR0(LANCE_GET_CSR0() & LANCE_STAT);

		/*
		 * Chip start.
		 */
		LANCE_CSR_WRITE(LANCE_CSR0, LANCE_STRT | LANCE_INEA);

		aep->ae_flags |= AERUNNING;

		aewenable(aep);
	}

done:
	rw_exit(&aestruplock);
	mutex_exit(&aep->ae_xmitlock);
	mutex_exit(&aep->ae_intrlock);

	TRACE_1(TR_FAC_AE, TR_AE_INIT_END,
		"aeinit end:  aep %X", aep);

	return (!(aep->ae_flags & AERUNNING));
}

/*
 * Un-initialize (STOP) LANCE
 */
static void
aeuninit(struct ae *aep)
{
	register int i = AEDRAINTIME;

	/*
	 * Allow up to 'aedraintime' for pending xmit's to complete.
	 */
	while (--i > 0) {
		if (aep->ae_tcurp == aep->ae_tnextp)
			break;
		drv_usecwait(1);
	}

	mutex_enter(&aep->ae_intrlock);
	mutex_enter(&aep->ae_xmitlock);
	mutex_enter(&aep->ae_buflock);

	aep->ae_flags &= ~AERUNNING;

	/*
	 * Stop the chip.
	 */
	LANCE_CSR_WRITE(LANCE_CSR0, LANCE_STOP);

	mutex_exit(&aep->ae_buflock);
	mutex_exit(&aep->ae_xmitlock);
	mutex_exit(&aep->ae_intrlock);
}

static struct aebuf *
aegetbuf(register struct ae *aep, int pri)
{
	struct	aebuf	*lbp;
	register int	i;

	TRACE_1(TR_FAC_AE, TR_AE_GETBUF_START,
		"aegetbuf start:  aep %X", aep);

	mutex_enter(&aep->ae_buflock);

	i = aep->ae_bufi;

	if ((i == 0) || ((pri == 0) && (i < AEBUFSRESERVED))) {
		mutex_exit(&aep->ae_buflock);
		TRACE_1(TR_FAC_AE, TR_AE_GETBUF_END,
			"aegetbuf end:  aep %X", aep);
		return (NULL);
	}

	i--;
	lbp = aep->ae_buftab[i];
	aep->ae_buftab[i] = 0;
	aep->ae_bufi = i;

	mutex_exit(&aep->ae_buflock);

	TRACE_1(TR_FAC_AE, TR_AE_GETBUF_END,
		"aegetbuf end:  aep %X", aep);
	return (lbp);
}

static void
aefreebuf(register struct aebuf	*lbp)
{
	register struct	ae *aep = lbp->lb_aep;

	TRACE_1(TR_FAC_AE, TR_AE_FREEBUF_START,
		"aefreebuf start:  aep %X", aep);

	mutex_enter(&aep->ae_buflock);

	aep->ae_buftab[aep->ae_bufi++] = lbp;

	mutex_exit(&aep->ae_buflock);

	if (aep->ae_wantw)
		aewenable(aep);
	TRACE_1(TR_FAC_AE, TR_AE_FREEBUF_END,
		"aefreebuf end:  aep %X", aep);
}

/*
 * Initialize RMD.
 */
static void
aermdinit(aep, rmdp, lbp)
struct	ae	*aep;
volatile	struct	lmd	*rmdp;
struct	aebuf	*lbp;
{
	uint32_t	a;

	a = lbp->phys_addr + AEHEADROOM;

	ddi_put32(aep->ae_acchdl, (uint32_t *)&rmdp->lmd_addr, a);
	ddi_put16(aep->ae_acchdl, (uint16_t *)&rmdp->lmd_bcnt,
		(uint16_t)(-(AEBUFSIZE - AEHEADROOM)));
	ddi_put16(aep->ae_acchdl, (uint16_t *)&rmdp->lmd_mcnt, 0);
	ddi_put8(aep->ae_acchdl, (uchar_t *)&rmdp->lmd_flags, LMD_OWN);
}

/*
 * Report Receive errors.
 */
static void
ae_rcv_error(struct ae *aep, struct lmd	*rmdp)
{
	u_int flags;

	flags = (u_int)ddi_get32(aep->ae_acchdl,
					(uint32_t *)&rmdp->lmd_flags);

	if ((flags & RMD_FRAM) && !(flags & RMD_OFLO)) {
		if (aedebug)
			aeerror(aep->ae_dip, "Receive: framming error");
		aep->ae_fram++;
	}
	if ((flags & RMD_CRC) && !(flags & RMD_OFLO)) {
		if (aedebug)
			aeerror(aep->ae_dip, "Receive: crc error");
		aep->ae_crc++;
	}
	if ((flags & RMD_OFLO) && !(flags & LMD_ENP)) {
		if (aedebug)
			aeerror(aep->ae_dip, "Receive: overflow error");
		aep->ae_oflo++;
	}
	if (flags & RMD_BUFF)
		aeerror(aep->ae_dip, "Receive: BUFF set in rmd");
	/*
	 * If an OFLO error occurred, the chip may not set STP or ENP,
	 * so we ignore a missing ENP bit in these cases.
	 */
	if (!(flags & LMD_STP) && !(flags & RMD_OFLO)) {
		if (aedebug)
			aeerror(aep->ae_dip, "Receive: STP in rmd cleared");
	} else if (!(flags & LMD_ENP) && !(flags & RMD_OFLO))
		if (aedebug)
			aeerror(aep->ae_dip,
				"Receive: giant packet");
}


/*
 * Report on transmission errors paying very close attention
 * to the Rev B (October 1986) Am7990 programming manual.
 * Return 1 if aeinit() is called, 0 otherwise.
 */
static int
ae_xmit_error(struct ae	*aep, struct lmd *tmdp)
{
	u_int flags;
	static char *aenocar1 =
		"No carrier - transceiver cable problem?";

	flags = (u_int)ddi_get16(aep->ae_acchdl,
				(uint16_t *)&tmdp->lmd_flags3);

	/*
	 * BUFF is not valid if either RTRY or LCOL is set.
	 * We assume here that BUFFs are always caused by UFLO's
	 * and not driver bugs.
	 */
	if ((flags & (TMD_BUFF | TMD_RTRY | TMD_LCOL)) == TMD_BUFF) {
		if (aedebug)
			aeerror(aep->ae_dip, "Transmit: BUFF set in tmd");
	}
	if (flags & TMD_UFLO) {
		if (aedebug)
			aeerror(aep->ae_dip, "Transmit underflow");
		aep->ae_uflo++;
	}
	if (flags & TMD_LCOL) {
		if (aedebug)
			aeerror(aep->ae_dip,
				"Transmit late collision - net problem?");
		aep->ae_tlcol++;
	}

	if ((flags & (TMD_LCAR|TMD_RTRY)) == TMD_LCAR) {
		aeerror(aep->ae_dip, "%s", aenocar1);
		aep->ae_tnocar++;
	}

	if (flags & TMD_RTRY) {
		if (aedebug)
			aeerror(aep->ae_dip,
		"Transmit retried more than 16 times - net  jammed");
		aep->ae_collisions += 16;
		aep->ae_trtry++;
	}
	return (0);
}

/*
 * Handle errors reported in LANCE CSR0.
 */
static
ae_chip_error(struct ae *aep)
{

	int restart = 0;
	uint16_t csr0;

	csr0 = (uint16_t)LANCE_GET_CSR0();

	if (csr0 & LANCE_MISS) {
		if (aedebug)
			aeerror(aep->ae_dip, "missed packet");
		aep->ae_ierrors++;
		aep->ae_missed++;
		LANCE_PUT_CSR0(LANCE_MISS | LANCE_INEA);
	}

	if (csr0 & LANCE_BABL) {
	    aeerror(aep->ae_dip,
		"Babble error - packet longer than 1518 bytes");
		aep->ae_oerrors++;
	    LANCE_PUT_CSR0(LANCE_BABL | LANCE_INEA);
	}

	/*
	 * If a memory error has occurred, both the transmitter
	 * and the receiver will have shut down.
	 * Display the Reception Stopped message if it
	 * wasn't caused by MERR (should never happen) OR aedebug.
	 * Display the Transmission Stopped message if it
	 * wasn't caused by MERR (was caused by UFLO) AND aedebug
	 * since we will have already displayed a UFLO msg.
	 */
	if (csr0 & LANCE_MERR) {
		aeerror(aep->ae_dip, "Memory error!");
		aep->ae_ierrors++;
		LANCE_PUT_CSR0(LANCE_MERR | LANCE_INEA);
		restart++;
	}

	if (!(csr0 & LANCE_RXON)) {
		if (!(csr0 & LANCE_MERR) || aedebug)
			aeerror(aep->ae_dip, "Reception stopped");
		restart++;
	}

	if (!(csr0 & LANCE_TXON)) {
		if (!(csr0 & LANCE_MERR) && aedebug)
			aeerror(aep->ae_dip, "Transmission stopped");
		restart++;
	}

	if (restart) {
		mutex_exit(&aep->ae_intrlock);
		(void) aeinit(aep);
		mutex_enter(&aep->ae_intrlock);
	}
	return (restart);
}

/*VARARGS*/
static void
aeerror(dip, fmt, a1, a2, a3, a4, a5, a6)
dev_info_t	*dip;
char	*fmt, *a1, *a2, *a3, *a4, *a5, *a6;
{
	static	uint32_t last = 0;
	static	char	*lastfmt;
	uint32_t sec;

	mutex_enter(&aelock);

	/*
	 * Don't print same error message too often.
	 */
	(void) drv_getparm(TIME, &sec);
	if ((last == (sec & ~1)) && (lastfmt == fmt)) {
		mutex_exit(&aelock);
		return;
	}
	last = sec & ~1;
	lastfmt = fmt;

	cmn_err(CE_CONT, "%s%d:  ", ddi_get_name(dip), ddi_get_instance(dip));
	cmn_err(CE_CONT, fmt, a1, a2, a3, a4, a5, a6);
	cmn_err(CE_CONT, "\n");

	mutex_exit(&aelock);
}

/*
 * Store and retrieve local individual ethernet address.
 * This is typically initialized (called with 'hint' nonnull)
 * in the attach routine
 */

static void
ae_etheraddr(struct ether_addr *hint, struct ether_addr *result)
{
	static int found = 0;
	static struct ether_addr addr;

	if (!found) {
		found = 1;
		if (hint == NULL)
			return;
		addr = *hint;
		cmn_err(CE_CONT, "?Ethernet address = %s\n",
		    ether_sprintf(&addr));
	}
	if (result != NULL)
		*result = addr;
}

/*
 *  Some STREAMS routines.
 *
 */

static void
ae_merror(queue_t	*wq, mblk_t	*mp, int	error)
{
	if ((mp = (mblk_t *)ae_mexchange(wq, mp, 1, M_ERROR, -1)) == NULL)
		return;
	*mp->b_rptr = (uint8_t)error;
	qreply(wq, mp);
}

/*
 * Convert an M_IOCTL into an M_IOCACK.
 * Assumption:  mp points to an M_IOCTL msg.
 */
static void
ae_miocack(queue_t	*wq, mblk_t	*mp, int	count, int	error)
{
	struct	iocblk	*iocp;

	mp->b_datap->db_type = M_IOCACK;
	iocp = (struct iocblk *)mp->b_rptr;
	iocp->ioc_count = count;
	iocp->ioc_error = error;
	qreply(wq, mp);
}

/*
 * Convert an M_IOCTL into an M_IOCNAK.
 * Assumption:  mp points to an M_IOCTL msg.
 */
static void
ae_miocnak(queue_t	*wq, mblk_t	*mp, int	count, int	error)
{
	struct	iocblk	*iocp;

	mp->b_datap->db_type = M_IOCNAK;
	iocp = (struct iocblk *)mp->b_rptr;
	iocp->ioc_count = count;
	iocp->ioc_error = error;
	qreply(wq, mp);
}

/*
 * Exchange one msg for another.  Free old msg and allocate
 * a new one if either (1) mp is NULL, (2), requested size
 * is larger than current size, or (3) reference count of
 * the current msg is greater than one.
 * Set db_type and b_rptr/b_wptr appropriately.
 * Set the first longword of the msg to 'primtype' if
 * 'primtype' is not -1.
 *
 * On allocb() failure, return NULL after sending an
 * M_ERROR msg w/ENOSR error upstream.
 */

#define	ENOSR	63

static mblk_t *
ae_mexchange(queue_t	*wq, mblk_t	*mp, int	size,
uint8_t	type, int32_t	primtype)
{
	if ((mp == NULL) || (MBLKSIZE(mp) < size) || (DB_REF(mp) > 1)) {
		freemsg(mp);
		if ((mp = allocb(size, BPRI_LO)) == NULL) {
			if (mp = allocb(1, BPRI_HI))
				ae_merror(wq, mp, ENOSR);
			return (NULL);
		}
	}
	mp->b_datap->db_type = type;
	mp->b_rptr = mp->b_datap->db_base;
	mp->b_wptr = mp->b_rptr + size;
	if (primtype >= 0)
		*(int32_t *)mp->b_rptr = primtype;
	return (mp);
}

int
ae_msgsize(register mblk_t *mp)
{
	register int n = 0;

	for (; mp; mp = mp->b_cont)
		n += MBLKL(mp);

	return (n);
}


void
ae_mcopymsg(mblk_t *mp, uint8_t *bufp)
{
	mblk_t *bp;
	int n;

	for (bp = mp; bp; bp = bp->b_cont) {
		n = MBLKL(bp);
		bcopy((caddr_t)bp->b_rptr, (caddr_t)bufp, n);
		bufp += n;
	}

	freemsg(mp);
}

/*
 *  Common DLPI routines.
 */

static void
ae_dlbindack(queue_t	*wq, mblk_t	*mp, uint32_t	sap, caddr_t	addrp,
int	addrlen, int	maxconind, uint32_t	xidtest)
{
	union	DL_primitives	*dlp;
	int	size;

	size = sizeof (dl_bind_ack_t) + addrlen;
	if ((mp = ae_mexchange(wq, mp, size, M_PCPROTO, DL_BIND_ACK))
		== NULL)
		return;

	dlp = (union DL_primitives *)mp->b_rptr;
	dlp->bind_ack.dl_sap = sap;
	dlp->bind_ack.dl_addr_length = addrlen;
	dlp->bind_ack.dl_addr_offset = sizeof (dl_bind_ack_t);
	dlp->bind_ack.dl_max_conind = maxconind;
	dlp->bind_ack.dl_xidtest_flg = xidtest;
	bcopy((caddr_t)addrp,
		(caddr_t)(mp->b_rptr + sizeof (dl_bind_ack_t)),
		addrlen);

	qreply(wq, mp);
}

static void
ae_dlokack(queue_t	*wq, mblk_t	*mp, uint32_t	correct_primitive)
{
	union	DL_primitives	*dlp;

	if ((mp = ae_mexchange(wq, mp, sizeof (dl_ok_ack_t), M_PCPROTO,
			DL_OK_ACK)) == NULL)
		return;
	dlp = (union DL_primitives *)mp->b_rptr;
	dlp->ok_ack.dl_correct_primitive = correct_primitive;
	qreply(wq, mp);
}

static void
ae_dlerrorack(queue_t	*wq, mblk_t	*mp, uint32_t	error_primitive,
uint32_t	errno, uint32_t	unix_errno)
{
	register	union	DL_primitives	*dlp;

	if ((mp = ae_mexchange(wq, mp, sizeof (dl_error_ack_t), M_PCPROTO,
		DL_ERROR_ACK)) == NULL)
		return;
	dlp = (union DL_primitives *)mp->b_rptr;
	dlp->error_ack.dl_error_primitive = error_primitive;
	dlp->error_ack.dl_errno = errno;
	dlp->error_ack.dl_unix_errno = unix_errno;
	qreply(wq, mp);
}

#define	DLADDRL	(80)

static void
ae_dluderrorind(queue_t	*wq, mblk_t	*mp, uint8_t	*addrp,
uint32_t	addrlen, uint32_t	errno, uint32_t	unix_errno)
{
	union	DL_primitives	*dlp;
	char	buf[DLADDRL];
	int	size;

	if (addrlen > DLADDRL)
		addrlen = DLADDRL;

	bcopy((caddr_t)addrp, (caddr_t)buf, addrlen);

	size = sizeof (dl_uderror_ind_t) + addrlen;

	if ((mp = ae_mexchange(wq, mp, size, M_PCPROTO, DL_UDERROR_IND))
	    == NULL)
		return;

	dlp = (union DL_primitives *)mp->b_rptr;
	dlp->uderror_ind.dl_dest_addr_length = addrlen;
	dlp->uderror_ind.dl_dest_addr_offset = sizeof (dl_uderror_ind_t);
	dlp->uderror_ind.dl_unix_errno = unix_errno;
	dlp->uderror_ind.dl_errno = errno;
	bcopy((caddr_t)buf,
		(caddr_t)(mp->b_rptr + sizeof (dl_uderror_ind_t)),
		addrlen);
	qreply(wq, mp);
}

static void
ae_dlphysaddrack(queue_t *wq, mblk_t *mp, caddr_t addrp, int len)
{
	union	DL_primitives	*dlp;
	int	size;

	size = sizeof (dl_phys_addr_ack_t) + len;
	if ((mp = ae_mexchange(wq, mp, size, M_PCPROTO, DL_PHYS_ADDR_ACK)) ==
									NULL)
		return;
	dlp = (union DL_primitives *)mp->b_rptr;
	dlp->physaddr_ack.dl_addr_length = len;
	dlp->physaddr_ack.dl_addr_offset = sizeof (dl_phys_addr_ack_t);
	bcopy(addrp, (caddr_t)(mp->b_rptr + sizeof (dl_phys_addr_ack_t)), len);
	qreply(wq, mp);
}

/*
 * Convert Ethernet address to printable (loggable) representation.
 */
static char *
ether_sprintf(struct ether_addr *addr)
{
	register uint8_t *ap = (uint8_t *)addr;
	register int i;
	static char etherbuf[18];
	register char *cp = etherbuf;
	static char digits[] = "0123456789abcdef";

	for (i = 0; i < 6; i++) {
		if (*ap > 0x0f)
			*cp++ = digits[*ap >> 4];
		*cp++ = digits[*ap++ & 0xf];
		*cp++ = ':';
	}
	*--cp = 0;
	return (etherbuf);
}
