/*
 * Copyright (c) 1992-1994,1997, by Sun Microsystems, Inc.
 * All Rights Reserved
 */

#ifndef	_SYS_AE_H
#define	_SYS_AE_H

#pragma ident	"@(#)ae.h	1.11	97/04/04 SMI"

/*
 * ae.h header for PCnet (Am79C970) Ethernet Driver.
 */

#ifdef	__cplusplus
extern "C" {
#endif

/*
 * Definitions for module_info.
 */
#define		AEIDNUM		(68)		/* module ID number */
#define		AENAME		"ae"		/* module name */
#define		AEMINPSZ	(0)		/* min packet size */
#define		AEMAXPSZ	ETHERMTU	/* max packet size */
#define		AEHIWAT		(32 * 1024)	/* hi-water mark */
#define		AELOWAT		(1)		/* lo-water mark */

/*
 * Per-Stream instance state information.
 *
 * Each instance is dynamically allocated at open() and free'd
 * at close().  Each per-Stream instance points to at most one
 * per-device structure using the sl_aep field.  All instances
 * are threaded together into one list of active instances
 * ordered on minor device number.
 */
struct	aestr {
	struct	aestr	*sl_nextp;	/* next in list */
	queue_t	*sl_rq;			/* pointer to our rq */
	struct	ae *sl_aep;		/* attached device */
	u_long	sl_state;		/* current DL state */
	u_long	sl_sap;			/* bound sap */
	u_long	sl_flags;		/* misc. flags */
	u_int	sl_mccount;		/* # enabled multicast addrs */
	struct	ether_addr *sl_mctab;	/* table of multicast addrs */
	u_long	sl_minor;		/* minor device number */
	kmutex_t	sl_lock;	/* protect this structure */
};

/* per-stream flags */
#define	SLFAST		0x01	/* "M_DATA fastpath" mode */
#define	SLRAW		0x02	/* M_DATA plain raw mode */
#define	SLALLPHYS	0x04	/* "promiscuous mode" */
#define	SLALLMULTI	0x08	/* enable all multicast addresses */
#define	SLALLSAP	0x10	/* enable all ether type values */

/*
 * Overload LANCE tmd flags reserved field with a flag
 * to indicate that the tmd has been allocated (in use).
 */
#define	TMD_INUSE	TMD_RES

/*
 * Maximum # of multicast addresses per Stream.
 */
#define	AEMAXMC	64
#define	AEMCALLOC	(AEMAXMC * sizeof (struct ether_addr))

/*
 * Full DLSAP address length (in struct dladdr format).
 */
#define	AEADDRL	(sizeof (u_short) + ETHERADDRL)

/*
 * Return the address of an adjacent descriptor in the given ring.
 */
#define	NEXTRMD(aep, rmdp)	(((rmdp) + 1) == (aep)->ae_rmdlimp	\
	? (aep)->ae_rmdp : ((rmdp) + 1))
#define	NEXTTMD(aep, tmdp)	(((tmdp) + 1) == (aep)->ae_tmdlimp	\
	? (aep)->ae_tmdp : ((tmdp) + 1))
#define	PREVTMD(aep, tmdp)	((tmdp) == (aep)->ae_tmdp		\
	? ((aep)->ae_tmdlimp - 1) : ((tmdp) - 1))

/*
 * Per-Device instance state information.
 *
 * Each instance is dynamically allocated on first attach.
 */
struct	ae {
	struct	ae		*ae_nextp;	/* next in a linked list */
	volatile	struct	lanceregs	*ae_regsp;	/* chip regs */
	ddi_acc_handle_t	ae_handle;	/* PCnet-PCI handle to regs */
	u_int	ae_flags;			/* misc. flags */
	dev_info_t	*ae_dip;	/* associated dev_info */
	struct	ether_addr	ae_ouraddr;	/* individual address */
	ddi_iblock_cookie_t	ae_cookie;	/* cookie from ddi_add_intr */

	kmutex_t	ae_xmitlock;		/* protect xmit-side fields */
	kmutex_t	ae_intrlock;		/* protect intr-side fields */
	kmutex_t	ae_buflock;		/* protect private buffers */

	/*
	 * Initialization related structures
	 */
	struct	lance_init_block *ae_ibp;	/* chip init block */

	struct	lmd	*ae_rmdp;	/* receive descriptor ring start */
	struct	lmd	*ae_rmdlimp;	/* receive descriptor ring end */
	int	ae_nrmdp2;		/* log(2) # receive descriptors */
	int	ae_nrmds;		/* # receive descriptors */
	struct	lmd	*ae_tmdp;	/* transmit descriptor ring start */
	struct	lmd	*ae_tmdlimp;	/* transmit descriptor ring end */
	int	ae_ntmdp2;		/* log(2) # transmit descriptors */
	int	ae_ntmds;		/* # transmit descriptors */
	volatile	struct	lmd	*ae_rnextp;	/* next chip rmd */
	volatile	struct	lmd	*ae_tnextp;	/* next free tmd */
	volatile	struct	lmd	*ae_tcurp;	/* next reclaim tmd */
	u_int	ae_wantw;			/* xmit: out of resources */

	struct	aebuf	*ae_tbufp[128];	/* xmit completion tbufs */
	mblk_t	*ae_tmblkp[128];	/* xmit completion msgs */
	struct	aebuf	*ae_rbufp[128];	/* aebuf associated with RMD */

	int	ae_nbufs;		/* # buffers */
	struct	aebuf	**ae_buftab;	/* buffer pointer stack (fifo) */
	int	ae_bufi;		/* index of buffer ptr stack top */

	int	ae_tpe;			/* =1 => tpe selected */
	int	ae_autosel;		/* =1 => auto-selection enabled */
	int	ae_oopkts;		/* old ae_opackets */

	/*
	 * DDI dma handle, kernel virtual base,
	 * and io virtual base of IOPB area.
	 */
	ddi_dma_handle_t	ae_iopb_hdl;
	ddi_acc_handle_t	ae_acchdl;
	u_long	ae_iopbkbase;		/* kernel virtual address of iopb */
	u_long	ae_iopbiobase;		/* DMA-able address of iopb */

	queue_t	*ae_ipq;		/* ip read queue */

	/*
	 * DDI dma handle, kernel virtual base,
	 * and io virtual base addresses of buffer area.
	 */
	kstat_t	*ae_ksp;		/* kstat pointer */
	ulong_t	ae_ipackets;		/* # packets received */
	ulong_t	ae_ierrors;		/* # total input errors */
	ulong_t	ae_opackets;		/* # packets sent */
	ulong_t	ae_oerrors;		/* # total output errors */
	ulong_t	ae_collisions;		/* # collisions */
	ulong_t	ae_defer;		/* # defers */
	ulong_t	ae_fram;		/* # receive framming errors */
	ulong_t	ae_crc;			/* # receive crc errors */
	ulong_t	ae_oflo;		/* # receiver overflows */
	ulong_t	ae_uflo;		/* # transmit underflows */
	ulong_t	ae_missed;		/* # receive missed */
	ulong_t	ae_tlcol;		/* # transmit late collisions */
	ulong_t	ae_trtry;		/* # transmit retry failures */
	ulong_t	ae_tnocar;		/* # loss of carrier errors */
	ulong_t	ae_inits;		/* # driver inits */
	ulong_t	ae_notmds;		/* # out of tmds occurences */
	ulong_t	ae_notbufs;		/* # out of buffers for xmit */
	ulong_t	ae_norbufs;		/* # out of buffers for receive */
	ulong_t	ae_nocanput;		/* # input canput() returned false */
	ulong_t	ae_allocbfail;		/* # esballoc/allocb failed */
};

/* flags */
#define	AERUNNING	0x01	/* chip is initialized */
#define	AEPROMISC	0x04	/* promiscuous mode enabled */
#define	AESUSPENDED	0x10	/* suspended interface */

/*
 * Fast aligned copy requires both the source and destination
 * addresses have the same offset from some N-byte boundary.
 */
#define	AEBURSTSIZE	(64)
#define	AEBURSTMASK	(AEBURSTSIZE-1)

#define	AEDRAINTIME	(200000)	/* # microseconds xmit drain */

#define	AEROUNDUP(a, n)	(((a) + ((n) - 1)) & ~((n) - 1))

/*
 * Xmit/receive buffer structure.
 * The allocated structure has to meet the following requirements:
 * - lb_buf[] starts on an AEBURSTSIZE boundary.
 * - aebuf is an even multiple of AEBURSTSIZE
 * - lb_buf[] is large enough to contain max frame < ETHERMAX >
 *	plus < 3 x AEBURSTSIZE > rounded up to the next AEBURSTSIZE
 */

#define	AEBUFSIZE	(1728)

struct	aebuf {
	u_char	lb_buf[AEBUFSIZE];	/* raw buffer */
	struct	ae	*lb_aep;	/* link to device structure */
	frtn_t	lb_frtn;		/* for esballoc() */
	u_long	phys_addr;		/* physical address of buffer */
	ddi_dma_handle_t lb_buf_hdl;
	ddi_acc_handle_t lb_acchdl;
};

/*
 * Define offset from start of lb_buf[] to point receive descriptor.
 * Requirements:
 * - must be 14 bytes back of a 4-byte boundary so the start of
 *   the network packet is 4-byte aligned.
 * - leave some headroom for others
 */
#define		AEHEADROOM	34

/*
 * Private DLPI full dlsap address format.
 */
struct	aedladdr {
	struct	ether_addr	dl_phys;
	u_short	dl_sap;
};

/*
 * "Export" a few of the error counters via the kstats mechanism.
 */
struct	aestat {
	struct	kstat_named	aes_ipackets;
	struct	kstat_named	aes_ierrors;
	struct	kstat_named	aes_opackets;
	struct	kstat_named	aes_oerrors;
	struct	kstat_named	aes_collisions;
	struct	kstat_named	aes_defer;
	struct	kstat_named	aes_fram;
	struct	kstat_named	aes_crc;
	struct	kstat_named	aes_oflo;
	struct	kstat_named	aes_uflo;
	struct	kstat_named	aes_missed;
	struct	kstat_named	aes_tlcol;
	struct	kstat_named	aes_trtry;
	struct	kstat_named	aes_tnocar;
	struct	kstat_named	aes_inits;
	struct	kstat_named	aes_nocanput;
	struct	kstat_named	aes_allocbfail;
};

#ifdef	__cplusplus
}
#endif

#endif	/* _SYS_AE_H */
