/*
 * Copyright (c) 1989-1997, by Sun Microsytems, Inc.
 * All rights reserved.
 * Defines and structures used only within the driver, bst.c
 */

#ifndef	_BST_DEF_H
#define	_BST_DEF_H

#pragma ident	"@(#)bst_def.h	1.19	97/09/12 SMI"

#ifdef	__cplusplus
extern "C" {
#endif

#if	defined(_KERNEL) || defined(_KMEMUSER)

/*
 * This driver does not reset the SCSI bus. Instead we just give up
 * and complain if the target hangs. If the bus is really stuck, one
 * of the Sun drivers (e.g. sd) will reset it. If you decide that
 * your device is critical and you should in fact reset the bus in
 * this driver, turn on this define:
#define	SCSI_BUS_RESET
 */

/*
 * Local definitions, for clarity of code
 */
#if defined(_SUNOS_VTOC_16)
#define	BST_SLICE(d)	(getminor(d) & 0x1f)	/* dev_t -> partition */
#define	BST_MINOR(i, s)	((i) << 5 | (s))	/* instance/slice -> minor */
#define	BST_INST(d)	(getminor(d) >> 5)	/* dev_t -> instance */
#endif

#if defined(_SUNOS_VTOC_8)
#define	BST_SLICE(d)	(getminor(d) & 0x7)	/* dev_t -> partition */
#define	BST_MINOR(i, s)	((i) << 3 | (s))	/* instance/slice -> minor */
#define	BST_INST(d)	(getminor(d) >> 3)	/* dev_t -> instance */
#endif

#define	BST_MUTEX(targ)	(&(targ)->devp->sd_mutex)
#define	ROUTE(targ)	(&(targ)->devp->sd_address)
#define	SCSIDEV(targ)	((targ)->devp)
#define	DEVINFO(targ)	(((targ)->devp)->sd_dev)
#define	INQUIRY(targ)	(((targ)->devp)->sd_inq)

#define	BST_UNLOCK	0		/* unlock media in drive */
#define	BST_LOCK	1		/* lock media in drive */
#define	BST_STOP	0		/* stop disk */
#define	BST_START	1		/* start disk */
#define	BST_EJECT	2		/* eject disk */

#define	SCBP(pkt)	((struct scsi_status *)(pkt)->pkt_scbp)
#define	SCBP_C(pkt)	((*(pkt)->pkt_scbp) & STATUS_MASK)
#define	CDBP(pkt)	((union scsi_cdb *)(pkt)->pkt_cdbp)
#define	BST_BP_PKT(bp)	((struct scsi_pkt *)bp->av_back)
#define	ARQP(pktp) 	((struct scsi_arq_status *)((pktp)->pkt_scbp))

#define	ISCD(targ)	((targ)->iscd)
#define	REMOVABLE(targ)	(ISCD(targ) || SCSIDEV(targ)->sd_inq->inq_rmb)

#define	BST_CE_DEBUG1	((1 << 8) | CE_CONT)
#define	BST_CE_DEBUG2	((2 << 8) | CE_CONT)
#define	BST_CE_DEBUG3	((3 << 8) | CE_CONT)
#define	BST_CE_DEBUG4	((4 << 8) | CE_CONT)
#define	BST_LOG		if (bst_debug) bst_log
#define	BST_DEBUG_ENTER	if (bst_debug) debug_enter

#if defined(_SUNOS_VTOC_8)
#define	BST_NMAP		NDKMAP
#endif

#if defined(_SUNOS_VTOC_16)
#define	BST_NMAP		(NDKMAP+FD_NUMPART+1)
#define	FDISK_MAP_ALL		(NDKMAP)
#define	FDISK_MAP0		(NDKMAP+1)
#define	FDISK_MAP1		(NDKMAP+2)
#define	FDISK_MAP2		(NDKMAP+3)
#define	FDISK_MAP3		(NDKMAP+4)
#endif


/*
 * Private info for scsi targets.
 *
 * Pointed to by the un_private pointer
 * of one of the scsi_device structures.
 */
struct bst_soft {
	struct scsi_device	*devp;		/* scsi_device */
	struct dk_geom		geom;		/* disk geometry */
	struct dk_vtoc		vtoc;		/* disk vtoc */
	struct scsi_pkt		*rqs;		/* request sense pkt */
	struct diskhd		iorq;		/* i/o request queue */
	struct buf		*sbufp;		/* for use in special io */
	kcondvar_t		sbuf_cv;	/* cv for sbufp */
	ksema_t			semaphore;	/* open/close serialization */
	daddr_t			capacity;	/* total disk capacity */
	int			secsize;	/* sector size */
	int			secdiv;		/* log2 of sector size */
	int			blkshf;		/* block shift */

	struct partition	map[BST_NMAP];	/* partition table */
	struct partition	fmap[FD_NUMPART]; /* fdisk partitions map */
	u_int			offset[BST_NMAP]; /* offset to each part */
	int			active_part;	/* active fdisk partition */

	int			sbuf_busy;	/* Wait Variable */
	int			excl;		/* exclusively opened */
	int			geom_ok;	/* disk geometry is valid */
	int			fdisk_ok;	/* fdisk is valid */
	int			part_type;	/* active partition type */
	u_int			solaris_offset;	/* offset to solaris part */
	u_int			solaris_size;	/* size of solaris part */
	int			retry_ct;	/* retry count */
	int			state;		/* current state */
	int			raw_map;	/* character opens map */
	int			block_map;	/* block opens map */
	u_int			arq;		/* ARQ mode on this tgt */
	struct buf		*rqbp;		/* ARQ buf pointer */
	int			iscd;		/* disk is a cd-rom */
	char			asciilabel[LEN_DKL_ASCII]; /* ascii label */
	char			suspended;	/* system power management */
	char			pm_suspended;	/* device power management */
	char			dumping;	/* currently dumping */
	char 			power_level;	/* current power level */
	kcondvar_t		disk_busy_cv;	/* cv for blocking while */
						/* disk is busy */
	kcondvar_t		suspend_cv;	/* cv for blocking while */
						/* suspended */
	int			timeout_id;	/* timeout(9F) id */
};

/*
 * lock_lint directives.
 */
_NOTE(MUTEX_PROTECTS_DATA(scsi_device::sd_mutex, bst_soft))

_NOTE(DATA_READABLE_WITHOUT_LOCK( bst_soft::{devp rqs sbufp geom_ok
		raw_map iscd block_map geom block_map state iorq
		disk_busy_cv suspended dumping offset}))

_NOTE(SCHEME_PROTECTS_DATA("semaphore",
	bst_soft::{excl map raw_map block_map}))

_NOTE(SCHEME_PROTECTS_DATA("stable data",
	scsi_device
	partition))

_NOTE(SCHEME_PROTECTS_DATA("Unshared data",
	dk_cinfo
	uio
        buf
        scsi_pkt
	scsi_cdb
	scsi_status
	scsi_arq_status
        uscsi_cmd
	bst_soft::arq))

#endif	/* defined(_KERNEL) || defined(_KMEMUSER) */

/*
 * Driver states
 */
#define	BST_STATE_NIL		0
#define	BST_STATE_RWAIT		1	/* waiting for resources */
#define	BST_STATE_NORESP	2	/* device is not responding */

/*
 * Per-packet private data area
 */
struct bst_pkt_private {
	struct buf	*bp;
	int		dma_resid;
	int		cdblen;
};

_NOTE(SCHEME_PROTECTS_DATA("Unshared data", bst_pkt_private))
/*
 * Parameters
 */

#define	BST_IO_TIME	30	/* default command timeout, 30sec */

/*
 * 5 seconds is what we'll wait if we get a Busy Status back
 */
#define	BST_BSY_TIMEOUT		(drv_usectohz(5 * 1000000))

/*
 * Number of times we'll retry a normal operation.
 * This includes retries due to transport failure
 */
#define	BST_RETRY_COUNT		30

/*
 * bst_callback action codes
 */
#define	COMMAND_DONE		0
#define	COMMAND_DONE_ERROR	1
#define	QUE_COMMAND		2
#define	QUE_SENSE		3
#define	JUST_RETURN		4

/*
 * Ioctl commands
 */
#define	BSTIOC		('S' << 8)
#define	BSTIOC_READY	(BSTIOC|0)	/* Send a Test Unit Ready command */
#define	BSTIOC_ERRLEV	(BSTIOC|1)	/* Set Error Reporting level */

#define	BUFLEN	256	/* useful buffer length */


#ifdef	__cplusplus
}
#endif

#endif	/* _BST_DEF_H */
