/*
 * Copyright (c) 1995,1997, by Sun Microsystems, Inc.
 * All rights reserved.
 */
#pragma ident	"@(#)isp.c	1.150	97/09/11 SMI"

/*
 * isp - QLogics Intelligent SCSI Processor driver
 */

#if defined(lint) && !defined(DEBUG)
#define	DEBUG	1
#endif

#ifdef DEBUG
#define	ISPDEBUG
#endif

#include <sys/scsi/scsi.h>
#include <sys/ddi.h>
#include <sys/sunddi.h>

#include "ispmail.h"
#include "ispvar.h"
#include "ispreg.h"
#include "ispcmd.h"


#ifdef	ISPDEBUG
static int ispdebug		= 0;
static int isp_cdb_debug	= 0;
#include <sys/debug.h>
#endif


/*
 * patch in case of problems
 */
static int isp_burst_sizes_limit	= 0xff;
static int isp_scsi_tag_age_limit	= 2;
static int isp_scsi_reset_delay		= 3000;		/* milliseconds */
static int isp_scsi_initiator_id	= 7;


/*
 * dev_ops functions prototypes
 */
static int isp_info(dev_info_t *dip, ddi_info_cmd_t infocmd,
    void *arg, void **result);
static int isp_attach(dev_info_t *dip, ddi_attach_cmd_t cmd);
static int isp_detach(dev_info_t *dip, ddi_detach_cmd_t cmd);

/*
 * Function prototypes
 *
 * SCSA functions exported by means of the transport table
 */
static int isp_tran_tgt_init(dev_info_t *hba_dip, dev_info_t *tgt_dip,
	scsi_hba_tran_t *tran, struct scsi_device *sd);
static int isp_scsi_start(struct scsi_address *ap, struct scsi_pkt *pkt);
static int isp_scsi_abort(struct scsi_address *ap, struct scsi_pkt *pkt);
static int isp_scsi_reset(struct scsi_address *ap, int level);
static int isp_scsi_getcap(struct scsi_address *ap, char *cap, int whom);
static int isp_scsi_setcap(struct scsi_address *ap, char *cap, int value,
    int whom);
static struct scsi_pkt *isp_scsi_init_pkt(struct scsi_address *ap,
    struct scsi_pkt *pkt, struct buf *bp, int cmdlen, int statuslen,
    int tgtlen, int flags, int (*callback)(), caddr_t arg);
static void isp_scsi_destroy_pkt(struct scsi_address *ap, struct scsi_pkt *pkt);
static void isp_scsi_dmafree(struct scsi_address *ap, struct scsi_pkt *pkt);
static void isp_scsi_sync_pkt(struct scsi_address *ap, struct scsi_pkt *pkt);
static int isp_scsi_reset_notify(struct scsi_address *ap, int flag,
    void (*callback)(caddr_t), caddr_t arg);
static void isp_reset_notify_callback(struct isp *isp);

/*
 * isp interrupt handler
 */
static u_int isp_intr(caddr_t arg);


/*
 * internal functions
 */
static int isp_i_updatecap(struct isp *isp, int tgt);
static void isp_i_update_props(struct isp *isp, int tgt);
static void isp_i_update_this_prop(struct isp *isp, char *property,
    int value);
static void isp_i_update_sync_prop(struct isp *isp, struct isp_cmd *sp);
static void isp_i_initcap(struct isp *isp);

static int isp_i_dma_alloc(struct isp *isp, struct scsi_pkt *pkt,
	struct buf *bp, int flags, int (*callback)());
static int isp_i_dma_move(struct isp *isp, struct scsi_pkt *pkt,
	struct buf *bp);

static void isp_i_watch(caddr_t);
static void isp_i_fatal_error(struct isp *isp);

static int isp_i_start_cmd(struct isp *isp, struct isp_cmd *sp);
static void isp_i_write_request(struct isp *isp, struct isp_cmd *sp);
static struct isp_cmd *isp_i_read_response(struct isp *isp);
static int isp_i_find_freeslot(struct isp *isp);
static int isp_i_polled_cmd_start(struct isp *isp, struct isp_cmd *sp);
static void isp_i_call_pkt_comp(register struct isp_cmd *head);
static void isp_i_handle_arq(struct isp *isp, struct isp_cmd *sp);
static int isp_i_handle_mbox_cmd(struct isp *isp);

static void isp_i_qflush(register struct isp *isp,
    uint16_t start_tgt, uint16_t end_tgt);
static int isp_i_reset_interface(register struct isp *isp, int action);
static int isp_i_set_marker(register struct isp *isp,
    int16_t mod, int16_t tgt, int16_t lun);

static void isp_i_log(struct isp *isp, int level, char *fmt, ...);
static void isp_i_print_state(struct isp *isp);

static int isp_i_async_event(register struct isp *isp, int16_t event);
static int isp_i_response_error(struct isp *isp, struct isp_response *resp);

static void isp_i_mbox_cmd_init(struct isp *isp,
    struct isp_mbox_cmd *mbox_cmdp, uint8_t n_mbox_out, uint8_t n_mbox_in,
    uint16_t reg0, uint16_t reg1, uint16_t reg2,
    uint16_t reg3, uint16_t reg4, uint16_t reg5);
static int isp_i_mbox_cmd_start(struct isp *isp,
    struct isp_mbox_cmd *mbox_cmdp);
static void isp_i_mbox_cmd_complete(struct isp *isp);

#ifdef ISPDEBUG
static void isp_debug_dump_cdb(struct scsi_address *ap,
		struct scsi_pkt *pkt);
#endif


#ifdef	_DDICT
static int	ddi_in_panic(void);
static int	ddi_in_panic() { return (0); }
#ifndef	SCSI_CAP_RESET_NOTIFICATION
#define	SCSI_CAP_RESET_NOTIFICATION		14
#endif
#ifndef	SCSI_RESET_NOTIFY
#define	SCSI_RESET_NOTIFY			0x01
#endif
#ifndef	SCSI_RESET_CANCEL
#define	SCSI_RESET_CANCEL			0x02
#endif
#endif


_NOTE(SCHEME_PROTECTS_DATA("unshared data", buf scsi_pkt isp_cmd))
_NOTE(SCHEME_PROTECTS_DATA("unshared data", scsi_arq_status))
_NOTE(SCHEME_PROTECTS_DATA("unshared data", isp_response))
_NOTE(SCHEME_PROTECTS_DATA("unshared data", isp_request))
_NOTE(DATA_READABLE_WITHOUT_LOCK(isp::isp_shutdown))
_NOTE(DATA_READABLE_WITHOUT_LOCK(isp::isp_mbox.mbox_flags))
_NOTE(DATA_READABLE_WITHOUT_LOCK(isp::isp_cap))

/*
 * Local static data
 */
static void		*isp_state = NULL;
static int		isp_watchdog_tick = 60; /* watch interval in sec */
static int		isp_tick;		/* watch interval in HZ */
static int		isp_selection_timeout = 250;


#ifdef	ISP1000
/*
 * default isp dma attr structure describes device
 * and DMA engine specific attributes/constraints necessary
 * to allocate DMA resources for ISP device.
 */
static ddi_dma_attr_t isp_dma_attr = {
	DMA_ATTR_V0,				/* dma_attr_version */
	(unsigned long long)0,			/* dma_attr_addr_lo */
	(unsigned long long)0xffffffff,		/* dma_attr_addr_hi */
	(unsigned long long)0x00ffffff,		/* dma_attr_count_max */
	(unsigned long long)1,			/* dma_attr_align */
	DEFAULT_BURSTSIZE | BURST32 | BURST64,	/* dma_attr_burstsizes */
	1,					/* dma_attr_minxfer */
	(unsigned long long)0xffffffff,		/* dma_attr_maxxfer */
	(unsigned long long)(1<<24)-1,		/* dma_attr_seg */
	1,					/* dma_attr_sgllen */
	1,					/* dma_attr_granular */
	0					/* dma_attr_flags */
};
#endif	/* ISP1000 */

#ifdef	ISP1020
static ddi_dma_attr_t isp_dma_attr = {
	DMA_ATTR_V0,				/* dma_attr_version */
	(unsigned long long)0,			/* dma_attr_addr_lo */
	(unsigned long long)0xffffffff,		/* dma_attr_addr_hi */
	(unsigned long long)0x00ffffff,		/* dma_attr_count_max */
	4,					/* dma_attr_align */
	DEFAULT_BURSTSIZE | BURST32 | BURST64,	/* dma_attr_burstsizes */
	1,					/* dma_attr_minxfer */
	(unsigned long long)0x00ffffff,		/* dma_attr_maxxfer */
	(unsigned long long)0x0000ffff,		/* dma_attr_seg */
	ISP_NDATASEGS,				/* dma_attr_sgllen */
	1,					/* dma_attr_granular */
	0					/* dma_attr_flags */
};
#endif	/* ISP1020 */


/*
 * autoconfiguration routines.
 */
static struct cb_ops isp_cb_ops = {
	nodev,			/* open */
	nodev,			/* close */
	nodev,			/* strategy */
	nodev,			/* print */
	nodev,			/* dump */
	nodev,			/* read */
	nodev,			/* write */
	nodev,			/* ioctl */
	nodev,			/* devmap */
	nodev,			/* mmap */
	nodev,			/* segmap */
	nochpoll,		/* poll */
	nodev,			/* cb_prop_op */
	0,			/* streamtab  */
	D_64BIT | D_MP | D_NEW | D_HOTPLUG, /* Driver compatibility flag */
	CB_REV, 		/* cb_rev */
	nodev,			/* async I/O read entry point */
	nodev			/* async I/O write entry point */
};

static struct dev_ops isp_ops = {
	DEVO_REV,			/* rev, */
	0,				/* refcnt */
	isp_info,			/* getinfo */
	nulldev,			/* identify */
	nulldev,			/* probe */
	isp_attach,			/* attach */
	isp_detach,			/* detach */
	nodev,				/* reset */
	&isp_cb_ops,			/* char/block ops */
	NULL				/* bus ops */
};

char _depends_on[] = "misc/scsi";

static struct modldrv modldrv = {
	&mod_driverops,			/* module type - driver */
	"ISP SCSI Host Bus Adapter",	/* module name */
	&isp_ops,			/* driver ops */
};

static struct modlinkage modlinkage = {
	MODREV_1,			/* ml_rev - must be MODREV_1 */
	&modldrv,			/* ml_linkage */
	NULL				/* end of driver linkage */
};

_init(void)
{
	int ret;

	ret = ddi_soft_state_init(&isp_state, sizeof (struct isp),
	    ISP_INITIAL_SOFT_SPACE);
	if (ret != 0)
		return (ret);

	isp_tick = drv_usectohz((clock_t)isp_watchdog_tick * 1000000);

	if ((ret = scsi_hba_init(&modlinkage)) != 0) {
		ddi_soft_state_fini(&isp_state);
		return (ret);
	}

	ret = mod_install(&modlinkage);
	if (ret != 0) {
		scsi_hba_fini(&modlinkage);
		ddi_soft_state_fini(&isp_state);
	}

	return (ret);
}


_fini(void)
{
	int ret;

	if ((ret = mod_remove(&modlinkage)) != 0)
		return (ret);

	scsi_hba_fini(&modlinkage);

	ddi_soft_state_fini(&isp_state);

	return (ret);
}


_info(struct modinfo *modinfop)
{
	return (mod_info(&modlinkage, modinfop));
}


/*
 * Given the device number return the devinfo pointer
 * from the scsi_device structure.
 */
/*ARGSUSED*/
static int
isp_info(dev_info_t *dip, ddi_info_cmd_t infocmd, void *arg, void **result)
{
	return (DDI_FAILURE);
}

/*
 * Attach an instance of an isp host adapter.  Allocate data structures,
 * initialize the isp and we're on the air.
 */
/*ARGSUSED*/
static int
isp_attach(dev_info_t *dip, ddi_attach_cmd_t cmd)
{
	char buf[64];
	register int id;
	int mutex_initted = 0;
	int interrupt_added = 0;
	int bound_handle = 0;
	register struct isp *isp;
	int instance;
	struct ispregs *ip;
	scsi_hba_tran_t *tran = NULL;
	ddi_device_acc_attr_t dev_attr;
	u_int count;
	size_t rlen;
	ddi_dma_attr_t tmp_dma_attr;
	char prop_str[32];
	int i;

	instance = ddi_get_instance(dip);

	switch (cmd) {
	case DDI_ATTACH:
		dev_attr.devacc_attr_version = DDI_DEVICE_ATTR_V0;
		dev_attr.devacc_attr_dataorder = DDI_STRICTORDER_ACC;
#ifdef	ISP1000
		dev_attr.devacc_attr_endian_flags = DDI_STRUCTURE_BE_ACC;
#elif	ISP1020
		dev_attr.devacc_attr_endian_flags = DDI_STRUCTURE_LE_ACC;
#endif
		break;

	case DDI_RESUME:
		tran = (scsi_hba_tran_t *)ddi_get_driver_private(dip);
		if (!tran) {
			return (DDI_FAILURE);
		}
		isp = TRAN2ISP(tran);

		ISP_MUTEX_ENTER(isp);

		/*
		 * a power cycle may have erased the isp f/w in
		 * RAM. reinitialize the chip
		 */
		if (isp_i_reset_interface(isp, ISP_RESET_BUS_IF_BUSY)) {
			ISP_MUTEX_EXIT(isp);
			return (DDI_FAILURE);
		}

		/* restart the watch timeout */
		isp->isp_timeout_id =
			timeout(isp_i_watch, (caddr_t)isp, isp_tick);

		ISP_MUTEX_EXIT(isp);

		return (DDI_SUCCESS);

	default:
		isp_i_log(NULL, CE_WARN,
		    "isp%d: Cmd != DDI_ATTACH/DDI_RESUME", instance);
		return (DDI_FAILURE);
	}

	/*
	 * Since we know that some instantiations of this device can
	 * be plugged into slave-only SBus slots, check to see whether
	 * this is one such.
	 */
	if (ddi_slaveonly(dip) == DDI_SUCCESS) {
		isp_i_log(NULL, CE_WARN,
		    "isp%d: Device in slave-only slot, unused",
		    instance);
		return (DDI_FAILURE);
	}

	/*
	 * We do not support adapter drivers with high-level interrupts
	 */
	if (ddi_intr_hilevel(dip, 0) != 0) {
		isp_i_log(NULL, CE_WARN,
		    "isp%d: high-level interrupt not supported",
		    instance);
		return (DDI_FAILURE);
	}

	/*
	 * Allocate isp data structure.
	 */
	if (ddi_soft_state_zalloc(isp_state, instance) != DDI_SUCCESS) {
		isp_i_log(NULL, CE_WARN, "isp%d: Failed to alloc soft state",
		    instance);
		return (DDI_FAILURE);
	}

	isp = (struct isp *)ddi_get_soft_state(isp_state, instance);
	if (isp == (struct isp *)NULL) {
		isp_i_log(NULL, CE_WARN, "isp%d: Bad soft state", instance);
		ddi_soft_state_free(isp_state, instance);
		return (DDI_FAILURE);
	}

#ifdef	ISP1000
	/*
	 * map in device registers
	 */

	if (ddi_regs_map_setup(dip, 0, (caddr_t *)&ip,
		0, sizeof (struct ispregs),
		&dev_attr, &isp->isp_reg_handle) != DDI_SUCCESS) {

		isp_i_log(NULL, CE_WARN, "isp%d: Unable to map registers",
		    instance);
		ddi_soft_state_free(isp_state, instance);
		return (DDI_FAILURE);

	}
#endif	/* ISP1000 */

#ifdef	ISP1020
	/*
	 * map in device registers
	 */

	if (ddi_regs_map_setup(dip, 1, (caddr_t *)&ip,
		0, sizeof (struct ispregs),
		&dev_attr, &isp->isp_reg_handle) != DDI_SUCCESS) {

		isp_i_log(NULL, CE_WARN, "isp%d: Unable to map registers",
		    instance);
		ddi_soft_state_free(isp_state, instance);
		return (DDI_FAILURE);

	}
	/*
	 * map in the risc registers
	 */
	if (ddi_regs_map_setup(dip, 1, (caddr_t *)&isp->isp_risc_reg,
		ISP_REG_RISC_OFFSET, sizeof (struct isp_risc_regs),
		&dev_attr, &isp->isp_risc_handle) != DDI_SUCCESS) {

		isp_i_log(NULL, CE_WARN, "isp%d: Unable to map "
		    "risc registers", instance);
		ddi_regs_map_free(&isp->isp_reg_handle);
		ddi_soft_state_free(isp_state, instance);
		return (DDI_FAILURE);

	}

	/*
	 * Put register set in risc mode
	 */
	ISP_CLR_REG_BITS((isp), &ip->isp_sbus_conf1, ISP_REG_SXP_SELECT);

#endif	/* ISP1020 */

	isp->isp_cmdarea = NULL;
	tmp_dma_attr = isp_dma_attr;

	/*
	 * Allocate the isp command area
	 */
	if (ddi_dma_alloc_handle(dip, &tmp_dma_attr,
	    DDI_DMA_SLEEP, NULL, &isp->isp_dmahandle) != DDI_SUCCESS) {
		cmn_err(CE_WARN, "isp%d: cannot alloc dma handle",
		    instance);
		goto fail;
	}

	if (ddi_dma_mem_alloc(isp->isp_dmahandle,
		(u_int) ISP_QUEUE_SIZE,
		&dev_attr, DDI_DMA_CONSISTENT, DDI_DMA_SLEEP,
		NULL, (caddr_t *)&isp->isp_cmdarea, &rlen,
		&isp->isp_queue_handle) != DDI_SUCCESS) {
			cmn_err(CE_WARN, "isp%d: cannot alloc cmd area",
			    instance);
			goto fail;
	}

	if (ddi_dma_addr_bind_handle(isp->isp_dmahandle,
		NULL, (caddr_t)isp->isp_cmdarea, (u_int) ISP_QUEUE_SIZE,
		DDI_DMA_RDWR|DDI_DMA_CONSISTENT,
		DDI_DMA_SLEEP, NULL, &isp->isp_dmacookie,
		&count) != DDI_DMA_MAPPED) {
			cmn_err(CE_WARN, "isp%d: cannot bind cmd area",
			    instance);
			goto fail;
	}
	bound_handle++;
	bzero(isp->isp_cmdarea, ISP_QUEUE_SIZE);
	isp->isp_request_dvma = isp->isp_dmacookie.dmac_address;
	isp->isp_request_base = (struct isp_request *)isp->isp_cmdarea;

	isp->isp_response_dvma = isp->isp_request_dvma +
	    (ISP_MAX_REQUESTS * sizeof (struct isp_request));
	isp->isp_response_base = (struct isp_response *)
	    ((caddr_t)isp->isp_request_base +
	    (ISP_MAX_REQUESTS * sizeof (struct isp_request)));
	isp->isp_request_in = 1;
	isp->isp_request_out = 1;
	isp->isp_response_in = 1;
	isp->isp_response_out = 1;

	/*
	 * Begin setting up isp softc structure.
	 */
	isp->isp_reg = ip;

	/*
	 * get cookie so we can initialize the mutexes
	 */
	if (ddi_get_iblock_cookie(dip, (u_int)0, &isp->isp_iblock)
	    != DDI_SUCCESS) {
		goto fail;
	}

	/*
	 * Allocate a transport structure
	 */
	tran = scsi_hba_tran_alloc(dip, SCSI_HBA_CANSLEEP);
	if (tran == NULL) {
		cmn_err(CE_WARN, "isp: scsi_hba_tran_alloc failed\n");
		goto fail;
	}

	isp->isp_tran		= tran;
	isp->isp_dip		= dip;

	tran->tran_hba_private	= isp;
	tran->tran_tgt_private	= NULL;
	tran->tran_tgt_init	= isp_tran_tgt_init;
	tran->tran_tgt_probe	= (int (*)())scsi_hba_probe;
	tran->tran_tgt_free	= (void (*)())NULL;

	tran->tran_start	= isp_scsi_start;
	tran->tran_abort	= isp_scsi_abort;
	tran->tran_reset	= isp_scsi_reset;
	tran->tran_getcap	= isp_scsi_getcap;
	tran->tran_setcap	= isp_scsi_setcap;
	tran->tran_init_pkt	= isp_scsi_init_pkt;
	tran->tran_destroy_pkt	= isp_scsi_destroy_pkt;
	tran->tran_dmafree	= isp_scsi_dmafree;
	tran->tran_sync_pkt	= isp_scsi_sync_pkt;
	tran->tran_reset_notify = isp_scsi_reset_notify;


	/*
	 * find the burstsize and reduce ours if necessary
	 * If no burst size found, select a reasonable default.
	 */
	tmp_dma_attr.dma_attr_burstsizes &=
		(ddi_dma_burstsizes(isp->isp_dmahandle) &
		isp_burst_sizes_limit);
	isp->isp_burst_size = tmp_dma_attr.dma_attr_burstsizes;


	ISP_DEBUG(isp, SCSI_DEBUG, "ispattr burstsize=%x",
		isp->isp_burst_size);

	if (isp->isp_burst_size == -1) {
		isp->isp_burst_size = DEFAULT_BURSTSIZE | BURST32 | BURST64;
		ISP_DEBUG(isp, SCSI_DEBUG, "Using default burst sizes, 0x%x",
		    isp->isp_burst_size);
	} else {
		isp->isp_burst_size &= BURSTSIZE_MASK;
		ISP_DEBUG(isp, SCSI_DEBUG, "burst sizes= 0x%x",
			isp->isp_burst_size);
	}

#ifdef	ISP1000
	if (isp->isp_burst_size & BURST64) {
		isp->isp_conf1_burst_flag =
		    ISP_SBUS_CONF1_BURST_ENABLE | ISP_SBUS_CONF1_BURST64;
	} else if (isp->isp_burst_size & BURST32) {
		isp->isp_conf1_burst_flag =
		    ISP_SBUS_CONF1_BURST_ENABLE | ISP_SBUS_CONF1_BURST32;
	} else if (isp->isp_burst_size & BURST16) {
		isp->isp_conf1_burst_flag =
		    ISP_SBUS_CONF1_BURST_ENABLE | ISP_SBUS_CONF1_BURST16;
	} else if (isp->isp_burst_size & BURST8) {
		isp->isp_conf1_burst_flag =
		    ISP_SBUS_CONF1_BURST_ENABLE | ISP_SBUS_CONF1_BURST8;
	}
#endif	/* ISP1000 */

#ifdef	ISP1020
	/*
	 * Enable 8-byte bursting:
	 *	0x0:	8-byte bursting
	 *	0x1:	16-byte bursting
	 *	0x2:	32-byte bursting
	 */
	isp->isp_conf1_burst_flag =
		ISP_PCI_CONF1_BURST_ENABLE | ISP_PCI_CONF1_BURST8;
#endif	/* ISP1020 */

	/*
	 * Attach this instance of the hba
	 */
	if (scsi_hba_attach_setup(dip, &tmp_dma_attr, tran,
	    0) != DDI_SUCCESS) {
		cmn_err(CE_WARN, "isp: scsi_hba_attach failed\n");
		goto fail;
	}

	/*
	 * find scsi host id property
	 */
	id = ddi_prop_get_int(DDI_DEV_T_ANY, dip, 0,
		"scsi-initiator-id", -1);
	if (id != isp_scsi_initiator_id &&
			(id >= 0 && id < N_ISP_TARGETS_WIDE)) {
		isp_i_log(isp, CE_NOTE, "?initiator SCSI ID now %d", id);
		isp->isp_initiator_id = (uint8_t)id;
	} else {
		isp->isp_initiator_id = (uint8_t)isp_scsi_initiator_id;
	}

	/*
	 * Look up the scsi-options property
	 */
	isp->isp_scsi_options =
		ddi_prop_get_int(DDI_DEV_T_ANY, dip, 0, "scsi-options",
		    ISP_DEFAULT_SCSI_OPTIONS);
	ISP_DEBUG(isp, SCSI_DEBUG, "isp scsi-options=%x",
	    isp->isp_scsi_options);

	/*
	 * if target<n>-scsi-options property exists, use it;
	 * otherwise use the isp_scsi_options
	 */
	for (i = 0; i < N_ISP_TARGETS_WIDE; i++) {
		(void) sprintf(prop_str, "target%x-scsi-options", i);
		isp->isp_target_scsi_options[i] = ddi_prop_get_int(
			DDI_DEV_T_ANY, dip, 0, prop_str, isp->isp_scsi_options);
		ISP_DEBUG(isp, SCSI_DEBUG, "isp target%d-scsi-options=%x, "
			"isp scsi-options=%x", i,
			isp->isp_target_scsi_options[i],
			isp->isp_scsi_options);
		if (isp->isp_target_scsi_options[i] != isp->isp_scsi_options) {
			isp_i_log(isp, CE_NOTE,
				"?isp target%x-scsi-options=0x%x",
				i, isp->isp_target_scsi_options[i]);
		}
	}

	isp->isp_scsi_tag_age_limit =
		ddi_prop_get_int(DDI_DEV_T_ANY, dip, 0,
			"scsi-tag-age-limit", isp_scsi_tag_age_limit);
	ISP_DEBUG(isp, SCSI_DEBUG, "isp scsi_tage_age_limit=%d, global=%d",
	    isp->isp_scsi_tag_age_limit, isp_scsi_tag_age_limit);
	if (isp->isp_scsi_tag_age_limit != isp_scsi_tag_age_limit) {
		isp_i_log(isp, CE_NOTE, "?scsi-tag-age-limit=%d",
		    isp->isp_scsi_tag_age_limit);
	}

	isp->isp_scsi_reset_delay =
		ddi_prop_get_int(DDI_DEV_T_ANY, dip, 0,
			"scsi-reset-delay", isp_scsi_reset_delay);
	ISP_DEBUG(isp, SCSI_DEBUG, "isp scsi_reset_delay=%d, global=%d",
	    isp->isp_scsi_reset_delay, isp_scsi_reset_delay);
	if (isp->isp_scsi_reset_delay != isp_scsi_reset_delay) {
		isp_i_log(isp, CE_NOTE, "?scsi-reset-delay=%d",
		    isp->isp_scsi_reset_delay);
	}

	/*
	 * mutexes to protect the isp request and response queue
	 */
	(void) sprintf(buf, "isp%d_request_mutex", instance);
	mutex_init(ISP_REQ_MUTEX(isp), buf, MUTEX_DRIVER,
	    isp->isp_iblock);
	(void) sprintf(buf, "isp%d_response_mutex", instance);
	mutex_init(ISP_RESP_MUTEX(isp), buf, MUTEX_DRIVER,
	    isp->isp_iblock);
	mutex_initted = 1;

	if (ddi_add_intr(dip, (u_int)0, NULL,
	    (ddi_idevice_cookie_t *)0, isp_intr, (caddr_t)isp)) {
		isp_i_log(NULL, CE_WARN, "isp%d: Cannot add intr",
		    instance);
		goto fail;
	}
	interrupt_added++;

	/*
	 * set up watchdog for this isp
	 */
	ISP_DEBUG(isp, SCSI_DEBUG, "isp_watchdog_tick=%d, isp_tick=%d",
	    isp_watchdog_tick, isp_tick);
	isp->isp_timeout_id = timeout(isp_i_watch, (caddr_t)isp, isp_tick);

	ISP_MUTEX_ENTER(isp);

	/*
	 * Initialize the default Target Capabilites and Sync Rates
	 */
	isp_i_initcap(isp);

	/*
	 * reset isp and initialize capabilities
	 * Do NOT reset the bus since that will cause a reset delay
	 * which adds substantially to the boot time.
	 */
	if (isp_i_reset_interface(isp, ISP_RESET_BUS_IF_BUSY)) {
		ISP_MUTEX_EXIT(isp);
		goto fail;
	}
	ISP_MUTEX_EXIT(isp);

	isp_i_log(isp, CE_NOTE, "?Firmware Version: v%d.%02d",
	    isp->isp_major_rev, isp->isp_minor_rev);

	ddi_report_dev(dip);

	return (DDI_SUCCESS);

fail:
	isp_i_log(NULL, CE_WARN, "isp%d: Unable to attach", instance);

	if (isp->isp_cmdarea) {
		if (bound_handle) {
			if (ddi_dma_unbind_handle(isp->isp_dmahandle) !=
					DDI_SUCCESS) {
				cmn_err(CE_PANIC, "isp_attach: "
				    "ddi_dma_unbind_handle() failed\n");
				/*NOTREACHED*/
			}
		}
		ddi_dma_mem_free(&isp->isp_queue_handle);
	}
	if (isp->isp_timeout_id != 0) {
		isp->isp_shutdown = 1;
		(void) untimeout(isp->isp_timeout_id);
		isp->isp_timeout_id = 0;
	}
	if (mutex_initted) {
		mutex_destroy(ISP_REQ_MUTEX(isp));
		mutex_destroy(ISP_RESP_MUTEX(isp));
	}
	if (isp->isp_dmahandle) {
		ddi_dma_free_handle(&isp->isp_dmahandle);
	}

	if (interrupt_added) {
		ddi_remove_intr(dip, (u_int)0, isp->isp_iblock);
	}
	if (tran) {
		scsi_hba_tran_free(tran);
	}
#ifdef	ISP1020
	ddi_regs_map_free(&isp->isp_risc_handle);
#endif	/* ISP1020 */
	ddi_regs_map_free(&isp->isp_reg_handle);
	ddi_soft_state_free(isp_state, instance);
	return (DDI_FAILURE);
}

/*ARGSUSED*/
static int
isp_detach(dev_info_t *dip, ddi_detach_cmd_t cmd)
{
	struct	isp	*isp;
	struct ispregs	*ip;
	scsi_hba_tran_t	*tran;
	int instance = ddi_get_instance(dip);


	/* get transport structure pointer from the dip */
	if (!(tran = (scsi_hba_tran_t *)ddi_get_driver_private(dip))) {
		return (DDI_FAILURE);
	}

	/* get soft state from transport structure */
	isp = TRAN2ISP(tran);

	if (!isp) {
		return (DDI_FAILURE);
	}

	ISP_DEBUG(isp, SCSI_DEBUG, "isp_detach: cmd = %d", cmd);

	/* get register point from soft state structure */
	ip = isp->isp_reg;

	switch (cmd) {
	case DDI_DETACH:
		ISP_MUTEX_ENTER(isp);

		/* disable all isp interrupts first */
		ISP_WRITE_REG(isp, &ip->isp_sbus_icr,
			ISP_SBUS_ICR_DISABLE_ALL_INTS);

		/* deallocate resources */
		if (ddi_dma_unbind_handle(isp->isp_dmahandle) !=
		    DDI_SUCCESS) {
			cmn_err(CE_PANIC, "isp_attach: "
				"ddi_dma_unbind_handle() failed\n");
			/*NOTREACHED*/
		}

		ddi_dma_mem_free(&isp->isp_queue_handle);

		if (isp->isp_timeout_id != 0) {
			isp->isp_shutdown = 1;
			(void) untimeout(isp->isp_timeout_id);
			isp->isp_timeout_id = 0;
		}
		ddi_dma_free_handle(&isp->isp_dmahandle);

		ddi_remove_intr(dip, (u_int)0, isp->isp_iblock);

		scsi_hba_detach(dip);

		scsi_hba_tran_free(isp->isp_tran);

#ifdef	ISP1020
		ddi_regs_map_free(&isp->isp_risc_handle);
#endif	/* ISP1020 */

		ddi_regs_map_free(&isp->isp_reg_handle);

		ISP_MUTEX_EXIT(isp);

		mutex_destroy(ISP_REQ_MUTEX(isp));
		mutex_destroy(ISP_RESP_MUTEX(isp));


		ISP_DEBUG(isp, SCSI_DEBUG, "isp_detach: done");
		ddi_soft_state_free(isp_state, instance);

		return (DDI_SUCCESS);

	case DDI_SUSPEND:
		ISP_MUTEX_ENTER(isp);

		if (isp_i_reset_interface(isp, ISP_FORCE_RESET_BUS)) {
			ISP_MUTEX_EXIT(isp);
			return (DDI_FAILURE);
		}

		if (isp->isp_timeout_id != 0) {
			isp->isp_shutdown = 1;
			ISP_MUTEX_EXIT(isp);
			(void) untimeout(isp->isp_timeout_id);
			ISP_MUTEX_ENTER(isp);
			isp->isp_timeout_id = 0;
		}
		ISP_MUTEX_EXIT(isp);

		return (DDI_SUCCESS);

	default:
		return (DDI_FAILURE);
	}
}


/*
 * Function name : isp_tran_tgt_init
 *
 * Return Values : DDI_SUCCESS if target supported, DDI_FAILURE otherwise
 *
 */
/*ARGSUSED*/
static int
isp_tran_tgt_init(dev_info_t *hba_dip, dev_info_t *tgt_dip,
	scsi_hba_tran_t *tran, struct scsi_device *sd)
{
	return ((sd->sd_address.a_target < N_ISP_TARGETS_WIDE &&
		sd->sd_address.a_lun < 8) ? DDI_SUCCESS : DDI_FAILURE);
}


/*
 * Function name : isp_i_initcap
 *
 * Return Values : NONE
 * Description	 : Initializes the default target capabilites and
 *		   Sync Rates.
 *
 * Context	 : Called from the user thread through attach.
 *
 */
static void
isp_i_initcap(struct isp *isp)
{
	register int i;
	register uint16_t cap, synch;

	cap = 0;
	synch = 0;
	if (isp->isp_scsi_options & SCSI_OPTIONS_DR) {
		cap |= ISP_CAP_DISCONNECT;
	}
	if (isp->isp_scsi_options & SCSI_OPTIONS_PARITY) {
		cap |= ISP_CAP_PARITY;
	}
	if (isp->isp_scsi_options & SCSI_OPTIONS_SYNC) {
		cap |= ISP_CAP_SYNC;
		if (isp->isp_scsi_options & SCSI_OPTIONS_FAST) {
			synch = ISP_10M_SYNC_PARAMS;
		} else {
			synch = ISP_5M_SYNC_PARAMS;
		}
	}

	for (i = 0; i < N_ISP_TARGETS_WIDE; i++) {
		isp->isp_cap[i] = cap;
		isp->isp_synch[i] = synch;
	}
	ISP_DEBUG(isp, SCSI_DEBUG, "default cap = 0x%x", cap);
}



/*
 * Function name : isp_scsi_getcap()
 *
 * Return Values : current value of capability, if defined
 *		   -1 if capability is not defined

 * Description	 : returns current capability value
 *
 * Context	 : Can be called from different kernel process threads.
 *		   Can be called by interrupt thread.
 */
static int
isp_scsi_getcap(struct scsi_address *ap, char *cap, int whom)
{
	struct isp	*isp	= ADDR2ISP(ap);
	uint8_t		tgt	= ap->a_target;
	int		rval	= 0;

	/*
	 * We don't allow inquiring about capabilities for other targets
	 */
	if (cap == NULL || whom == 0) {
		return (-1);
	}

	ISP_MUTEX_ENTER(isp);

	switch (scsi_hba_lookup_capstr(cap)) {
	case SCSI_CAP_DMA_MAX:
		rval = 1 << 24; /* Limit to 16MB max transfer */
		break;
	case SCSI_CAP_MSG_OUT:
		rval = 1;
		break;
	case SCSI_CAP_DISCONNECT:
		if ((isp->isp_target_scsi_options[tgt] &
		    SCSI_OPTIONS_DR) == 0) {
			break;
		} else if (whom &&
		    (isp->isp_cap[tgt] & ISP_CAP_DISCONNECT) == 0) {
			break;
		}
		rval = 1;
		break;
	case SCSI_CAP_SYNCHRONOUS:
		if ((isp->isp_target_scsi_options[tgt] &
			SCSI_OPTIONS_SYNC) == 0) {
			break;
		} else if (whom &&
		    (isp->isp_cap[tgt] & ISP_CAP_SYNC) == 0) {
			break;
		}
		rval = 1;
		break;
	case SCSI_CAP_WIDE_XFER:
		if ((isp->isp_target_scsi_options[tgt] &
			SCSI_OPTIONS_WIDE) == 0) {
			break;
		} else if (whom &&
		    (isp->isp_cap[tgt] & ISP_CAP_WIDE) == 0) {
			break;
		}
		rval = 1;
		break;
	case SCSI_CAP_TAGGED_QING:
		if ((isp->isp_target_scsi_options[tgt] &
			SCSI_OPTIONS_DR) == 0 ||
		    (isp->isp_target_scsi_options[tgt] &
			SCSI_OPTIONS_TAG) == 0) {
			break;
		} else if (whom &&
		    (isp->isp_cap[tgt] & ISP_CAP_TAG) == 0) {
			break;
		}
		rval = 1;
		break;
	case SCSI_CAP_UNTAGGED_QING:
		rval = 1;
		break;
	case SCSI_CAP_PARITY:
		if (isp->isp_target_scsi_options[tgt] &
			SCSI_OPTIONS_PARITY) {
			rval = 1;
		}
		break;
	case SCSI_CAP_INITIATOR_ID:
		rval = isp->isp_initiator_id;
		break;
	case SCSI_CAP_ARQ:
		if (isp->isp_cap[tgt] & ISP_CAP_AUTOSENSE) {
			rval = 1;
		}
		break;
	case SCSI_CAP_LINKED_CMDS:
		break;
	case SCSI_CAP_RESET_NOTIFICATION:
		rval = 1;
		break;

	case SCSI_CAP_GEOMETRY:
		rval = (64 << 16) | 32;
		break;

	default:
		rval = -1;
		break;
	}

	ISP_MUTEX_EXIT(isp);

	return (rval);
}


/*
 * Function name : isp_scsi_setcap()
 *
 * Return Values : 1 - capability exists and can be set to new value
 *		   0 - capability could not be set to new value
 *		  -1 - no such capability
 *
 * Description	 : sets a capability for a target
 *
 * Context	 : Can be called from different kernel process threads.
 *		   Can be called by interrupt thread.
 */
static int
isp_scsi_setcap(struct scsi_address *ap, char *cap, int value, int whom)
{
	struct isp	*isp	= ADDR2ISP(ap);
	uint8_t		tgt	= ap->a_target;
	int		rval	= 0;

	/*
	 * We don't allow setting capabilities for other targets
	 */
	if (cap == NULL || whom == 0) {
		return (-1);
	}

	ISP_MUTEX_ENTER(isp);

	switch (scsi_hba_lookup_capstr(cap)) {
	case SCSI_CAP_DMA_MAX:
	case SCSI_CAP_MSG_OUT:
	case SCSI_CAP_PARITY:
	case SCSI_CAP_UNTAGGED_QING:
	case SCSI_CAP_LINKED_CMDS:
	case SCSI_CAP_RESET_NOTIFICATION:
		/*
		 * None of these are settable via
		 * the capability interface.
		 */
		break;
	case SCSI_CAP_DISCONNECT:
		if ((isp->isp_target_scsi_options[tgt] &
			SCSI_OPTIONS_DR) == 0) {
			break;
		} else {
			if (value) {
				isp->isp_cap[tgt] |=
				    ISP_CAP_DISCONNECT;
			} else {
				isp->isp_cap[tgt] &=
				    ~ISP_CAP_DISCONNECT;
			}
		}
		rval = 1;
		break;
	case SCSI_CAP_SYNCHRONOUS:
		if ((isp->isp_target_scsi_options[tgt] &
			SCSI_OPTIONS_SYNC) == 0) {
			break;
		} else {
			if (value) {
				isp->isp_cap[tgt] |= ISP_CAP_SYNC;
			} else {
				isp->isp_cap[tgt] &= ~ISP_CAP_SYNC;
			}
		}
		rval = 1;
		break;
	case SCSI_CAP_TAGGED_QING:
		if ((isp->isp_target_scsi_options[tgt] &
			SCSI_OPTIONS_DR) == 0 ||
		    (isp->isp_target_scsi_options[tgt] &
			SCSI_OPTIONS_TAG) == 0) {
			break;
		} else {
			if (value) {
				isp->isp_cap[tgt] |= ISP_CAP_TAG;
			} else {
				isp->isp_cap[tgt] &= ~ISP_CAP_TAG;
			}
		}
		rval = 1;
		break;
	case SCSI_CAP_WIDE_XFER:
		if ((isp->isp_target_scsi_options[tgt] &
			SCSI_OPTIONS_WIDE) == 0) {
			break;
		} else {
			if (value) {
				isp->isp_cap[tgt] |= ISP_CAP_WIDE;
			} else {
				isp->isp_cap[tgt] &= ~ISP_CAP_WIDE;
			}
		}
		rval = 1;
		break;
	case SCSI_CAP_INITIATOR_ID:
		if (value < N_ISP_TARGETS_WIDE) {
			struct isp_mbox_cmd mbox_cmd;

			isp->isp_initiator_id = (uint16_t)value;

			/*
			 * set Initiator SCSI ID
			 */
			isp_i_mbox_cmd_init(isp, &mbox_cmd, 2, 2,
			    ISP_MBOX_CMD_SET_SCSI_ID,
			    isp->isp_initiator_id,
			    0, 0, 0, 0);
			if (isp_i_mbox_cmd_start(isp, &mbox_cmd) == 0) {
				rval = 1;
			}
		}
		break;
	case SCSI_CAP_ARQ:
		if (value) {
			isp->isp_cap[tgt] |= ISP_CAP_AUTOSENSE;
		} else {
			isp->isp_cap[tgt] &= ~ISP_CAP_AUTOSENSE;
		}
		rval = 1;
		break;

	default:
		rval = -1;
		break;
	}

	/*
	 * now update the isp, if necessary
	 */
	if (rval == 1) {
		if (isp_i_updatecap(isp, tgt) == -1) {
			/*
			 * if we can't update the capabilities
			 * in the isp, we are hosed
			 */
			isp->isp_shutdown = 1;
			rval = 0;
		}
		isp->isp_need_prop_update = 1;
		isp->isp_update_targets[tgt] = 1;
	}

	ISP_MUTEX_EXIT(isp);

	return (rval);
}


/*
 * Function name : isp_i_updatecap()
 *
 * Return Values : -1	failed.
 *		    0	success
 *
 * Description	 : sync's the isp target parameters with the desired
 *		   isp_caps for the specified target
 *
 * Context	 : Can be called from different kernel process threads.
 *		   Can be called by interrupt thread.
 */
static int
isp_i_updatecap(struct isp *isp, int tgt)
{
	register uint16_t cap, synch;
	struct isp_mbox_cmd mbox_cmd;

	isp_i_mbox_cmd_init(isp, &mbox_cmd, 2, 4,
	    ISP_MBOX_CMD_GET_TARGET_CAP,
	    (uint16_t)(tgt << 8), 0, 0, 0, 0);
	if (isp_i_mbox_cmd_start(isp, &mbox_cmd)) {
	    return (-1);
	}

	cap   = mbox_cmd.mbox_in[2];
	synch = mbox_cmd.mbox_in[3];

	ISP_DEBUG2(isp, SCSI_DEBUG,
	"updatecap:tgt=%d:cap=0x%x,isp_cap=0x%x,synch=0x%x,isp_synch=0x%x",
	    tgt, cap, isp->isp_cap[tgt], synch, isp->isp_synch[tgt]);

	/*
	 * enable or disable ERRSYNC
	 */
	if (isp->isp_cap[tgt] & (ISP_CAP_WIDE | ISP_CAP_SYNC)) {
		isp->isp_cap[tgt] |= ISP_CAP_ERRSYNC;
	} else {
		isp->isp_cap[tgt] &= ~ISP_CAP_ERRSYNC;
	}

	/*
	 * Set isp cap if different from ours.
	 */
	if (isp->isp_cap[tgt] != cap ||
	    isp->isp_synch[tgt] != synch) {
		ISP_DEBUG(isp, SCSI_DEBUG,
	    "Setting Target %d, new cap=0x%x (was 0x%x), synch=0x%x",
		    tgt, isp->isp_cap[tgt], cap, isp->isp_synch[tgt]);
		isp_i_mbox_cmd_init(isp, &mbox_cmd, 4, 4,
		    ISP_MBOX_CMD_SET_TARGET_CAP, (uint16_t)(tgt << 8),
		    isp->isp_cap[tgt], isp->isp_synch[tgt], 0, 0);
		if (isp_i_mbox_cmd_start(isp, &mbox_cmd)) {
			return (-1);
		}
	}

	return (0);
}


/*
 * Function name : isp_i_update_sync_prop()
 *
 * Description	 : called  when isp reports renegotiation
 *
 * Context	 : Can be called from different kernel process threads.
 *		   Can be called by interrupt thread.
 */
static void
isp_i_update_sync_prop(struct isp *isp, struct isp_cmd *sp)
{
	register uint16_t cap, synch;
	struct isp_mbox_cmd mbox_cmd;
	int target = TGT(sp);

	ISP_DEBUG(isp, SCSI_DEBUG,
	    "tgt %d.%d: Negotiated new rate", TGT(sp), LUN(sp));

	/*
	 * Get new rate from ISP and save for later
	 * chip resets or scsi bus resets.
	 */
	isp_i_mbox_cmd_init(isp, &mbox_cmd, 2, 4,
	    ISP_MBOX_CMD_GET_TARGET_CAP,
	    (uint16_t)(target << 8), 0, 0, 0, 0);
	if (isp_i_mbox_cmd_start(isp, &mbox_cmd)) {
		return;
	}

	cap   = mbox_cmd.mbox_in[2];
	synch = mbox_cmd.mbox_in[3];

	ISP_DEBUG(isp, SCSI_DEBUG,
	"tgt=%d: cap=0x%x, isp_cap=0x%x, synch=0x%x, isp_synch=0x%x",
	    target, cap, isp->isp_cap[target], synch,
	    isp->isp_synch[target]);

	isp->isp_cap[target]   = cap;
	isp->isp_synch[target] = synch;

	isp->isp_need_prop_update = 1;
	isp->isp_update_targets[target] = 1;
}


/*
 * Function name : isp_i_update_props()
 *
 * Description	 : Creates/modifies/removes a target sync mode speed,
 *		   wide, and TQ properties
 *		   If offset is 0 then asynchronous mode is assumed and the
 *		   property is removed, if it existed.
 *
 * Context	 : Can be called from different kernel process threads.
 *		   Can not be called by interrupt thread
*/
static void
isp_i_update_props(struct isp *isp, int tgt)
{
	static char	property[32];
	int		xfer_speed = 0;
	int		wide_enabled;
	int		tq_enabled;
	int		offset;
	uint16_t	cap;
	uint16_t	synch;

	cap = isp->isp_cap[tgt];
	synch = isp->isp_synch[tgt];
	offset = ((int)synch >> 8) & 0xff;

	tq_enabled = cap & ISP_CAP_TAG;
	wide_enabled = cap & ISP_CAP_WIDE;

	if (offset && synch) {
		if (wide_enabled) {
			/* double xfer speed if wide has been enabled */
			xfer_speed = (1024 * 1024)/(((int)synch & 0xff) * 2);
		} else {
			xfer_speed = (1024 * 1024)/(((int)synch & 0xff) * 4);
		}
	}

	(void) sprintf(property, "target%x-sync-speed", tgt);
	isp_i_update_this_prop(isp, property, xfer_speed);

	(void) sprintf(property, "target%x-wide", tgt);
	isp_i_update_this_prop(isp, property, wide_enabled);

	(void) sprintf(property, "target%x-TQ", tgt);
	isp_i_update_this_prop(isp, property, tq_enabled);
}

/*
 * Update a property's value
 */
static void
isp_i_update_this_prop(struct isp *isp, char *property, int value)
{
	dev_info_t *dip = isp->isp_dip;

	ISP_DEBUG2(isp, SCSI_DEBUG,
		"isp_i_update_this_prop: %s=0x%x", property, value);

	if (ddi_prop_update_int(DDI_DEV_T_NONE, dip,
		property, value) != DDI_PROP_SUCCESS) {
		    ISP_DEBUG(isp, SCSI_DEBUG,
			"cannot update %s property to 0x%x\n",
			property, value);
	}
}



/*
 * Function name : isp_scsi_init_pkt
 *
 * Return Values : pointer to scsi_pkt, or NULL
 * Description	 : Called by kernel on behalf of a target driver
 *		   calling scsi_init_pkt(9F).
 *		   Refer to tran_init_pkt(9E) man page
 *
 * Context	 : Can be called from different kernel process threads.
 *		   Can be called by interrupt thread.
 */
static struct scsi_pkt *
isp_scsi_init_pkt(struct scsi_address *ap, register struct scsi_pkt *pkt,
	struct buf *bp, int cmdlen, int statuslen, int tgtlen,
	int flags, int (*callback)(), caddr_t arg)
{
	struct isp_cmd	*sp;
	struct isp	*isp		= ADDR2ISP(ap);
	struct scsi_pkt	*new_pkt;

	ASSERT(callback == NULL_FUNC || callback == SLEEP_FUNC);

	/*
	 * First step of isp_scsi_init_pkt:  pkt allocation
	 */
	if (pkt == NULL) {
		pkt = scsi_hba_pkt_alloc(isp->isp_dip, ap, cmdlen, statuslen,
			tgtlen, sizeof (struct isp_cmd), callback, arg);
		if (pkt == NULL) {
			return (NULL);
		}

		sp = PKT2CMD(pkt);

		/*
		 * Initialize the new pkt - we redundantly initialize
		 * all the fields for illustrative purposes.
		 */
		sp->cmd_pkt		= pkt;
		sp->cmd_flags		= 0;
		sp->cmd_scblen		= statuslen;
		sp->cmd_cdblen		= cmdlen;
		sp->cmd_dmahandle	= NULL;
		sp->cmd_ncookies	= 0;
		sp->cmd_cookie		= 0;
		sp->cmd_cookiecnt	= 0;
		sp->cmd_nwin		= 0;
		pkt->pkt_address	= *ap;
		pkt->pkt_comp		= (void (*)())NULL;
		pkt->pkt_flags		= 0;
		pkt->pkt_time		= 0;
		pkt->pkt_resid		= 0;
		pkt->pkt_statistics	= 0;
		pkt->pkt_reason		= 0;

		new_pkt = pkt;
	} else {
		sp = PKT2CMD(pkt);
		new_pkt = NULL;
	}

	/*
	 * Second step of isp_scsi_init_pkt:  dma allocation/move
	 */
	if (bp && bp->b_bcount != 0) {
		if (sp->cmd_dmahandle == NULL) {
			ASSERT((sp->cmd_flags & CFLAG_DMAVALID) == 0);
			if (isp_i_dma_alloc(isp, pkt, bp,
			    flags, callback) == -1) {
				if (new_pkt) {
					scsi_hba_pkt_free(ap, new_pkt);
				}
				return ((struct scsi_pkt *)NULL);
			}
		} else {
			ASSERT(new_pkt == NULL);
			ASSERT(sp->cmd_flags & CFLAG_DMAVALID);
			if (isp_i_dma_move(isp, pkt, bp) == -1) {
				return ((struct scsi_pkt *)NULL);
			}
		}
		ASSERT(sp->cmd_flags & CFLAG_DMAVALID);
	}

	return (pkt);
}


/*
 * Function name : isp_i_dma_alloc
 *
 * Return Values : 0 if successful, -1 if failure
 * Description	 : allocate DMA resources
 *
 * Context	 : Can only be called from isp_scsi_init_pkt()
 */
static int
isp_i_dma_alloc(struct isp *isp, struct scsi_pkt *pkt, struct buf *bp,
	int flags, int (*callback)())
{
	struct isp_cmd	*sp		= PKT2CMD(pkt);
	int		dma_flags;
	ddi_dma_attr_t	tmp_dma_attr;
	int		(*cb)(caddr_t);
	int		i;

	ASSERT(callback == NULL_FUNC || callback == SLEEP_FUNC);

	if (bp->b_flags & B_READ) {
		sp->cmd_flags &= ~CFLAG_DMASEND;
		dma_flags = DDI_DMA_READ;
	} else {
		sp->cmd_flags |= CFLAG_DMASEND;
		dma_flags = DDI_DMA_WRITE;
	}
	if (flags & PKT_CONSISTENT) {
		sp->cmd_flags |= CFLAG_CMDIOPB;
		dma_flags |= DDI_DMA_CONSISTENT;
	}
	if (flags & PKT_DMA_PARTIAL) {
		dma_flags |= DDI_DMA_PARTIAL;
	}
	dma_flags |= DDI_DMA_REDZONE;

	tmp_dma_attr = isp_dma_attr;
	tmp_dma_attr.dma_attr_burstsizes = isp->isp_burst_size;

	cb = (callback == NULL_FUNC) ? DDI_DMA_DONTWAIT : DDI_DMA_SLEEP;

	if ((i = ddi_dma_alloc_handle(isp->isp_dip, &tmp_dma_attr,
		cb, 0, &sp->cmd_dmahandle)) != DDI_SUCCESS) {

		switch (i) {
		case DDI_DMA_BADATTR:
			bioerror(bp, EFAULT);
			return (-1);

		case DDI_DMA_NORESOURCES:
			bioerror(bp, 0);
			return (-1);

		default:
			cmn_err(CE_PANIC, "ddi_dma_alloc_handle:"
				" 0x%x impossible\n", i);
			/*NOTREACHED*/
		}
	}

	i = ddi_dma_buf_bind_handle(sp->cmd_dmahandle, bp, dma_flags,
		cb, 0, &sp->cmd_dmacookies[0], &sp->cmd_ncookies);

	switch (i) {
	case DDI_DMA_PARTIAL_MAP:
		if ((dma_flags & DDI_DMA_PARTIAL) == 0) {
			cmn_err(CE_PANIC, "ddi_dma_buf_bind_handle:"
				" DDI_DMA_PARTIAL_MAP impossible\n");
			/*NOTREACHED*/
		}

		if (ddi_dma_numwin(sp->cmd_dmahandle, &sp->cmd_nwin) ==
		    DDI_FAILURE) {
			cmn_err(CE_PANIC, "ddi_dma_numwin() failed\n");
				/*NOTREACHED*/
		}

		if (ddi_dma_getwin(sp->cmd_dmahandle, sp->cmd_curwin,
		    &sp->cmd_dma_offset, &sp->cmd_dma_len,
		    &sp->cmd_dmacookies[0], &sp->cmd_ncookies) ==
		    DDI_FAILURE) {
			cmn_err(CE_PANIC, "ddi_dma_getwin() failed\n");
				/*NOTREACHED*/
		}
		goto get_dma_cookies;

	case DDI_DMA_MAPPED:
		sp->cmd_nwin = 1;
		sp->cmd_dma_len = 0;
		sp->cmd_dma_offset = 0;

get_dma_cookies:
		i = 0;
		sp->cmd_dmacount = 0;
		for (;;) {
			sp->cmd_dmacount += sp->cmd_dmacookies[i++].dmac_size;

			if (i == ISP_NDATASEGS || i == sp->cmd_ncookies)
				break;
			ddi_dma_nextcookie(sp->cmd_dmahandle,
				&sp->cmd_dmacookies[i]);
		}
		sp->cmd_cookie = i;
		sp->cmd_cookiecnt = i;

		sp->cmd_flags |= CFLAG_DMAVALID;
		pkt->pkt_resid = bp->b_bcount - sp->cmd_dmacount;
		return (0);

	case DDI_DMA_NORESOURCES:
		bioerror(bp, 0);
		break;

	case DDI_DMA_NOMAPPING:
		bioerror(bp, EFAULT);
		break;

	case DDI_DMA_TOOBIG:
		bioerror(bp, EINVAL);
		break;

	case DDI_DMA_INUSE:
		cmn_err(CE_PANIC, "ddi_dma_buf_bind_handle:"
			" DDI_DMA_INUSE impossible\n");
		/*NOTREACHED*/

	default:
		cmn_err(CE_PANIC, "ddi_dma_buf_bind_handle:"
			" 0x%x impossible\n", i);
		/*NOTREACHED*/
	}

	ddi_dma_free_handle(&sp->cmd_dmahandle);
	sp->cmd_dmahandle = NULL;
	sp->cmd_flags &= ~CFLAG_DMAVALID;
	return (-1);
}


/*
 * Function name : isp_i_dma_move
 *
 * Return Values : 0 if successful, -1 if failure
 * Description	 : move DMA resources to next DMA window
 *
 * Context	 : Can only be called from isp_scsi_init_pkt()
 */
/*ARGSUSED*/
static int
isp_i_dma_move(struct isp *isp, struct scsi_pkt *pkt, struct buf *bp)
{
	struct isp_cmd	*sp		= PKT2CMD(pkt);
	int		i;

	ASSERT(sp->cmd_flags & CFLAG_COMPLETED);
	sp->cmd_flags &= ~CFLAG_COMPLETED;

	/*
	 * If there are no more cookies remaining in this window,
	 * must move to the next window first.
	 */
	if (sp->cmd_cookie == sp->cmd_ncookies) {
		/*
		 * 1217340: cmdk reuses pkts incorrectly
		 */
		if (sp->cmd_curwin == sp->cmd_nwin && sp->cmd_nwin == 1)
			return (0);

		/*
		 * At last window, cannot move
		 */
		if (++sp->cmd_curwin >= sp->cmd_nwin)
			return (-1);

		if (ddi_dma_getwin(sp->cmd_dmahandle, sp->cmd_curwin,
		    &sp->cmd_dma_offset, &sp->cmd_dma_len,
		    &sp->cmd_dmacookies[0], &sp->cmd_ncookies) == DDI_FAILURE)
			return (-1);

		sp->cmd_cookie = 0;
	} else {
		/*
		 * Still more cookies in this window - get the next one
		 */
		ddi_dma_nextcookie(sp->cmd_dmahandle, &sp->cmd_dmacookies[0]);
	}

	/*
	 * Get remaining cookies in this window, up to our maximum
	 */
	i = 0;
	for (;;) {
		sp->cmd_dmacount += sp->cmd_dmacookies[i++].dmac_size;
		sp->cmd_cookie++;
		if (i == ISP_NDATASEGS ||
		    sp->cmd_cookie == sp->cmd_ncookies)
			break;
		ddi_dma_nextcookie(sp->cmd_dmahandle,
			&sp->cmd_dmacookies[i]);
	}
	sp->cmd_cookiecnt = i;

	pkt->pkt_resid = bp->b_bcount - sp->cmd_dmacount;
	return (0);
}


/*
 * Function name : isp_scsi_destroy_pkt
 *
 * Return Values : none
 * Description	 : Called by kernel on behalf of a target driver
 *		   calling scsi_destroy_pkt(9F).
 *		   Refer to tran_destroy_pkt(9E) man page
 *
 * Context	 : Can be called from different kernel process threads.
 *		   Can be called by interrupt thread.
 */
static void
isp_scsi_destroy_pkt(struct scsi_address *ap, struct scsi_pkt *pkt)
{
	struct isp_cmd *sp = PKT2CMD(pkt);

	/*
	 * isp_scsi_dmafree inline to make things faster
	 */
	if (sp->cmd_flags & CFLAG_DMAVALID) {
		/*
		 * Free the mapping.
		 */
		sp->cmd_flags &= ~CFLAG_DMAVALID;
		if (ddi_dma_unbind_handle(sp->cmd_dmahandle) != DDI_SUCCESS) {
			cmn_err(CE_PANIC, "isp_scsi_destroy_pkt: "
				"ddi_dma_unbind_handle() failed\n");
			/*NOTREACHED*/
		}
		ddi_dma_free_handle(&sp->cmd_dmahandle);
		sp->cmd_dmahandle = NULL;
	}

	/*
	 * Free the pkt
	 */
	scsi_hba_pkt_free(ap, pkt);
}


/*
 * Function name : isp_scsi_dmafree()
 *
 * Return Values : none
 * Description	 : free dvma resources
 *
 * Context	 : Can be called from different kernel process threads.
 *		   Can be called by interrupt thread.
 */
/*ARGSUSED*/
static void
isp_scsi_dmafree(struct scsi_address *ap, struct scsi_pkt *pkt)
{
	register struct isp_cmd *sp = PKT2CMD(pkt);

	if (sp->cmd_flags & CFLAG_DMAVALID) {
		/*
		 * Free the mapping.
		 */
		sp->cmd_flags &= ~CFLAG_DMAVALID;
		if (ddi_dma_unbind_handle(sp->cmd_dmahandle) != DDI_SUCCESS) {
			cmn_err(CE_PANIC, "isp_scsi_dmafree: "
				"ddi_dma_unbind_handle() failed\n");
			/*NOTREACHED*/
		}
		ddi_dma_free_handle(&sp->cmd_dmahandle);
		sp->cmd_dmahandle = NULL;
	}
}


/*
 * Function name : isp_scsi_sync_pkt()
 *
 * Return Values : none
 * Description	 : sync dma
 *
 * Context	 : Can be called from different kernel process threads.
 *		   Can be called by interrupt thread.
 */
/*ARGSUSED*/
static void
isp_scsi_sync_pkt(struct scsi_address *ap, struct scsi_pkt *pkt)
{
	register int i;
	register struct isp_cmd *sp = PKT2CMD(pkt);

	if (sp->cmd_flags & CFLAG_DMAVALID) {
		i = ddi_dma_sync(sp->cmd_dmahandle, sp->cmd_dma_offset,
			sp->cmd_dma_len,
			(sp->cmd_flags & CFLAG_DMASEND) ?
			DDI_DMA_SYNC_FORDEV : DDI_DMA_SYNC_FORCPU);
		if (i != DDI_SUCCESS) {
			cmn_err(CE_WARN, "isp: sync pkt failed");
		}
	}
}

/*
 * routine for reset notification setup, to register or cancel.
 */
static int
isp_scsi_reset_notify(struct scsi_address *ap, int flag,
void (*callback)(caddr_t), caddr_t arg)
{
	struct isp			*isp = ADDR2ISP(ap);
	struct isp_reset_notify_entry	*p, *beforep;
	int				rval = DDI_FAILURE;

	mutex_enter(ISP_REQ_MUTEX(isp));

	p = isp->isp_reset_notify_listf;
	beforep = NULL;

	while (p) {
		if (p->ap == ap)
			break;	/* An entry exists for this target */
		beforep = p;
		p = p->next;
	}

	if ((flag & SCSI_RESET_CANCEL) && (p != NULL)) {
		if (beforep == NULL) {
			isp->isp_reset_notify_listf = p->next;
		} else {
			beforep->next = p->next;
		}
		kmem_free((caddr_t)p, sizeof (struct isp_reset_notify_entry));
		rval = DDI_SUCCESS;

	} else if ((flag & SCSI_RESET_NOTIFY) && (p == NULL)) {
		p = kmem_zalloc(sizeof (struct isp_reset_notify_entry),
			KM_SLEEP);
		p->ap = ap;
		p->callback = callback;
		p->arg = arg;
		p->next = isp->isp_reset_notify_listf;
		isp->isp_reset_notify_listf = p;
		rval = DDI_SUCCESS;
	}

	mutex_exit(ISP_REQ_MUTEX(isp));

	return (rval);
}


/*
 * routine to perform the notification callbacks after a reset.
 * The function is entered with adapter driver mutex being held.
 */
static void
isp_reset_notify_callback(struct isp *isp)
{
	int	i, count;
	struct	isp_reset_notify_entry	*p;
	struct	notify_entry {
		void	(*callback)(caddr_t);
		caddr_t	arg;
	} *list;

	ASSERT(mutex_owned(ISP_RESP_MUTEX(isp)));
	ASSERT(mutex_owned(ISP_REQ_MUTEX(isp)) == 0 || ddi_in_panic());

	mutex_enter(ISP_REQ_MUTEX(isp));

	if ((p = isp->isp_reset_notify_listf) == NULL) {
		mutex_exit(ISP_REQ_MUTEX(isp));
		return;
	}

	count = 0;
	while (p != NULL) {
		count++;
		p = p->next;
	}

	list = kmem_alloc((size_t)(count * sizeof (struct notify_entry)),
		KM_NOSLEEP);
	if (list == NULL) {
		mutex_exit(ISP_REQ_MUTEX(isp));
		cmn_err(CE_WARN, "scsi_reset_notify: kmem_alloc failed");
		return;
	}

	p = isp->isp_reset_notify_listf;
	for (i = 0; i < count; i++, p = p->next) {
		list[i].callback = p->callback;
		list[i].arg = p->arg;
	}

	mutex_exit(ISP_REQ_MUTEX(isp));
	mutex_exit(ISP_RESP_MUTEX(isp));

	_NOTE(SCHEME_PROTECTS_DATA("No Mutex Needed", notify_entry::arg));
	_NOTE(SCHEME_PROTECTS_DATA("No Mutex Needed", notify_entry::callback));

	for (i = 0; i < count; i++) {
		if (list[i].callback != (void (*)(caddr_t))NULL)
			(void) (*list[i].callback)(list[i].arg);
	}
	kmem_free((caddr_t)list,
	    (size_t)(count * sizeof (struct notify_entry)));
	mutex_enter(ISP_RESP_MUTEX(isp));
}



/*
 * Function name : isp_scsi_start()
 *
 * Return Values : TRAN_FATAL_ERROR	- isp has been shutdown
 *		   TRAN_BUSY		- request queue is full
 *		   TRAN_ACCEPT		- pkt has been submitted to isp
 *
 * Description	 : init pkt, start the request
 *
 * Context	 : Can be called from different kernel process threads.
 *		   Can be called by interrupt thread.
 */
static int
isp_scsi_start(struct scsi_address *ap, register struct scsi_pkt *pkt)
{
	struct isp_cmd		*sp	= PKT2CMD(pkt);
	int			rval	= TRAN_ACCEPT;
	struct isp		*isp	= ADDR2ISP(ap);
	struct isp_request	*req;
	uint32_t		cur_lbolt;
	int			xfercount;
	uint16_t		cdblen;
	int			i;

	ASSERT(mutex_owned(ISP_REQ_MUTEX(isp)) == 0 || ddi_in_panic());
	ASSERT(mutex_owned(ISP_RESP_MUTEX(isp)) == 0 || ddi_in_panic());

	ISP_DEBUG2(isp, SCSI_DEBUG, "isp_scsi_start %x", sp);

	/*
	 * if we have a shutdown, return packet
	 */
	if (isp->isp_shutdown) {
		return (TRAN_FATAL_ERROR);
	}


	ASSERT(!(sp->cmd_flags & CFLAG_IN_TRANSPORT));
	sp->cmd_flags = (sp->cmd_flags & ~CFLAG_TRANFLAG) |
			    CFLAG_IN_TRANSPORT;
	pkt->pkt_reason = CMD_CMPLT;

#ifdef	ISPDEBUG
	if (isp_cdb_debug) {
		isp_debug_dump_cdb(ap, pkt);
	}
#endif	/* ISPDEBUG */

	/*
	 * set up request in cmd_isp_request area so it is ready to
	 * go once we have the request mutex
	 */
	req = &sp->cmd_isp_request;


	req->req_header.cq_count_type = (uint16_t)
		CQ_ENTRY_COUNT_TYPE(CQ_TYPE_REQUEST, 1);
	req->req_header.cq_flags_seq = CQ_FLAGS_SEQ(0, 0);

	req->req_reserved = 0;
	req->req_token = sp;
	req->req_tgt_lun = REQ_TGT_LUN(TGT(sp), LUN(sp));
	req->req_time = pkt->pkt_time;
	ISP_SET_PKT_FLAGS(pkt->pkt_flags, req->req_flags);

	/*
	 * Setup dma transfers data segments.
	 */
	if (sp->cmd_flags & CFLAG_DMAVALID) {

		if (sp->cmd_flags & CFLAG_CMDIOPB) {
			(void) ddi_dma_sync(sp->cmd_dmahandle,
			    sp->cmd_dma_offset, sp->cmd_dma_len,
			    DDI_DMA_SYNC_FORDEV);
		}

		ASSERT(sp->cmd_cookiecnt > 0 &&
		    sp->cmd_cookiecnt <= ISP_NDATASEGS);

		xfercount = 0;
		req->req_seg_count = sp->cmd_cookiecnt;
		for (i = 0; i < sp->cmd_cookiecnt; i++) {
			req->req_dataseg[i].d_count =
				sp->cmd_dmacookies[i].dmac_size;
			req->req_dataseg[i].d_base =
				sp->cmd_dmacookies[i].dmac_address;
			xfercount +=
				sp->cmd_dmacookies[i].dmac_size;
		}

		for (; i < ISP_NDATASEGS; i++) {
			req->req_dataseg[i].d_count = 0;
			req->req_dataseg[i].d_base = 0;
		}

		pkt->pkt_resid = xfercount;

		if (sp->cmd_flags & CFLAG_DMASEND) {
			req->req_flags |= ISP_REQ_FLAG_DATA_WRITE;
		} else {
			req->req_flags |= ISP_REQ_FLAG_DATA_READ;
		}
	} else {
		req->req_seg_count = 0;
		req->req_dataseg[0].d_count = 0;
	}

	/*
	 * Set up cdb in the request
	 */
#ifdef	BUG_CDB_LENGTH
	/*
	 * Work-around for a bug in the 2.5 sparc sd driver.  It allocates
	 * a pkt with cdb length of 10, then uses this pkt to issue
	 * 6-byte commands.  Work-around is to extract the scsi group
	 * id and figure out the corresponding command length.
	 * We intend to fix this in a subsequent release, so this
	 * hack should only be necessary for 2.5 and earlier.
	 */
	switch ((pkt->pkt_cdbp[0] >> 5) & 0x7) {
	case 0:
		cdblen = 6;
		break;
	case 1:
	case 2:
		cdblen = 10;
		break;
	case 5:
		cdblen = 12;
		break;
	default:
		cdblen = sp->cmd_cdblen;
		break;
	}
#ifdef	ISPDEBUG
	if (isp_cdb_debug && cdblen != sp->cmd_cdblen) {
		cmn_err(CE_CONT, "isp: cdb length %d -> %d\n",
			sp->cmd_cdblen, cdblen);
	}
#endif
#else
	cdblen = sp->cmd_cdblen;
#endif	/* BUG_CDB_LENGTH */
	req->req_cdblen = cdblen;
	bcopy((caddr_t)pkt->pkt_cdbp, (caddr_t)req->req_cdb, cdblen);

	/*
	 * calculate deadline from pkt_time
	 * Instead of multiplying by 100 (ie. HZ), we multiply by 128 so
	 * we can shift and at the same time have a 28% grace period
	 * we ignore the rare case of pkt_time == 0 and deal with it
	 * in isp_i_watch()
	 */
	if (drv_getparm(LBOLT, &cur_lbolt) != 0) {
		cmn_err(CE_PANIC, "isp: drv_getparm(LBOLT) failed\n");
	}
	sp->cmd_deadline = cur_lbolt + (pkt->pkt_time * 128);

	/*
	 * Start the cmd.  If NO_INTR, must poll for cmd completion.
	 */
	if ((pkt->pkt_flags & FLAG_NOINTR) == 0) {
		mutex_enter(ISP_REQ_MUTEX(isp));
		rval = isp_i_start_cmd(isp, sp);
		mutex_exit(ISP_REQ_MUTEX(isp));
	} else {
		rval = isp_i_polled_cmd_start(isp, sp);
	}

done:
	ASSERT(mutex_owned(ISP_REQ_MUTEX(isp)) == 0 || ddi_in_panic());
	ASSERT(mutex_owned(ISP_RESP_MUTEX(isp)) == 0 || ddi_in_panic());

	return (rval);
}


/*
 * Function name : isp_i_start_cmd()
 *
 * Return Values : TRAN_ACCEPT	- request is in the isp request queue
 *		   TRAN_BUSY	- request queue is full
 *
 * Description	 : if there is space in the request queue, copy over request
 *		   enter normal requests in the isp_slots list
 *
 * Context	 : Can be called from different kernel process threads.
 *		   Can be called by interrupt thread.
 */
static int
isp_i_start_cmd(register struct isp *isp, register struct isp_cmd *sp)
{
	struct scsi_pkt		*pkt = CMD2PKT(sp);
	int16_t			slot;

	ASSERT(mutex_owned(ISP_REQ_MUTEX(isp)));

	ISP_DEBUG2(isp, SCSI_DEBUG, "isp_i_start_cmd: sp=%x, pkt_time=%x",
	    sp, pkt->pkt_time);

	/*
	 * Check to see how much space is available in the
	 * Request Queue, save this so we do not have to do
	 * a lot of PIOs
	 */
	if (isp->isp_queue_space == 0) {
		ISP_UPDATE_QUEUE_SPACE(isp);

		/*
		 * Check now to see if the queue is still full
		 * Report TRAN_BUSY if we are full
		 */
		if (isp->isp_queue_space == 0) {
			return (TRAN_BUSY);
		}
	}

#ifdef	ISP1000
	/*
	 * The ability to disable auto request sense per packet
	 */
	if ((sp->cmd_scblen < sizeof (struct scsi_arq_status)) &&
	    (isp->isp_cap[TGT(sp)] & ISP_CAP_AUTOSENSE)) {
		sp->cmd_isp_request.req_flags |= ISP_REQ_FLAG_DISARQ;
	}
#endif

	/*
	 * Put I/O request in isp request queue to run.
	 */
	isp_i_write_request(isp, sp);

	/*
	 * Find a free slot to store the pkt in for crash protection for
	 * non-polled commands. Polled commands do not have to be kept
	 * track of since the busy wait loops keep track of them.
	 *
	 * We should *ALWAYS* be able to find a free slot; or we're broke!
	 */
	if ((pkt->pkt_flags & FLAG_NOINTR) == 0) {
		slot = isp->isp_free_slot++;
		if (isp->isp_free_slot >= (uint16_t)ISP_MAX_SLOTS) {
			isp->isp_free_slot = 0;
		}
		if (isp->isp_slots[slot].slot_cmd) {
			slot = isp_i_find_freeslot(isp);
		}

		/*
		 * Update ISP deadline time for new cmd sent to ISP
		 * There is a race here with isp_i_watch(); if the compiler
		 * rearranges this code, then this may cause some false
		 * timeouts
		 */
		isp->isp_slots[slot].slot_deadline = sp->cmd_deadline;

		sp->cmd_slot = slot;
		isp->isp_slots[slot].slot_cmd = sp;
	}

	/*
	 * Tell isp it's got a new I/O request...
	 */
	ISP_SET_REQUEST_IN(isp);
	ASSERT(isp->isp_queue_space > 0);
	isp->isp_queue_space--;

	ASSERT(mutex_owned(ISP_REQ_MUTEX(isp)));
	return (TRAN_ACCEPT);
}


/*
 * Function name : isp_i_write_request
 *
 * Usage:	   called by isp_i_start_cmd().
 *
 * Context	 : Can be called from different kernel process threads.
 *		   Can be called by interrupt thread.
 */
static void
isp_i_write_request(
	struct isp		*isp,
	struct isp_cmd		*sp)
{
	int			i;
	struct isp_request	*req;
	struct isp_request	*src	= &sp->cmd_isp_request;
	ddi_acc_handle_t	*qh	= isp->isp_queue_handle;

	/*
	 * Get the next request in pointer.
	 */
	ISP_GET_NEXT_REQUEST_IN(isp, req);

	/*
	 * Copy the request into device memory
	 */
	ddi_put16(qh, &req->req_header.cq_count_type,
		src->req_header.cq_count_type);
	ddi_put16(qh, &req->req_header.cq_flags_seq,
		src->req_header.cq_flags_seq);

	ddi_put32(qh, (uint32_t *)&req->req_token, (uint32_t)src->req_token);

	ddi_put16(qh, &req->req_tgt_lun, src->req_tgt_lun);

	ddi_put16(qh, &req->req_cdblen, src->req_cdblen);
	ddi_put16(qh, &req->req_flags, src->req_flags);
	ddi_put16(qh, &req->req_reserved, src->req_reserved);
	ddi_put16(qh, &req->req_time, src->req_time);
	ddi_put16(qh, &req->req_seg_count, src->req_seg_count);

	ddi_rep_put8(qh, src->req_cdb, req->req_cdb, ISP_MAX_CDB,
		DDI_DEV_AUTOINCR);

	for (i = 0; i < ISP_NDATASEGS; i++) {
		ddi_put32(qh, (uint32_t *)&req->req_dataseg[i].d_base,
			src->req_dataseg[i].d_base);
		ddi_put32(qh, (uint32_t *)&req->req_dataseg[i].d_count,
			src->req_dataseg[i].d_count);
	}

	/*
	 * Use correct offset and size for syncing
	 */
	(void) ddi_dma_sync(isp->isp_dmahandle,
	    (off_t)(isp->isp_request_in * sizeof (struct isp_request)),
	    (u_int)sizeof (struct isp_request),
	    DDI_DMA_SYNC_FORDEV);
}

/*
 * Function name : isp_i_read_response()
 *
 * Return Values : ptr to completed isp_cmd
 *
 * Usage:	   called by isp_intr().
 *
 * Context:	   called by interrupt thread.
 */
static struct isp_cmd *
isp_i_read_response(
	struct isp		*isp)
{
	struct isp_cmd		*sp;
	struct isp_response	*resp;
	struct isp_response	*dst;
	ddi_acc_handle_t	*qh = isp->isp_queue_handle;
	int			i;

	ISP_GET_NEXT_RESPONSE_OUT(isp, resp);

	/*
	 * Use correct offset and size for syncing
	 */
	(void) ddi_dma_sync(isp->isp_dmahandle,
	    (off_t)((ISP_MAX_REQUESTS * sizeof (struct isp_request)) +
	    (isp->isp_response_out * sizeof (struct isp_response))),
	    (u_int)sizeof (struct isp_response),
	    DDI_DMA_SYNC_FORKERNEL);

	/*
	 * Read the response from device memory
	 */
	sp = (struct isp_cmd *)ddi_get32(qh, (uint32_t *)&resp->resp_token);

#ifdef ISPDEBUG
	ASSERT(sp != NULL);
	ASSERT((sp->cmd_flags & CFLAG_COMPLETED) == 0);
	ASSERT((sp->cmd_flags & CFLAG_FINISHED) == 0);
	sp->cmd_flags |= CFLAG_FINISHED;
	ISP_DEBUG2(isp, SCSI_DEBUG, "isp_intr %x done, pkt_time=%x",
	    sp, CMD2PKT(sp)->pkt_time);
#endif

	dst = &sp->cmd_isp_response;

	dst->resp_header.cq_count_type =
		ddi_get16(qh, &resp->resp_header.cq_count_type);
	dst->resp_header.cq_flags_seq =
		ddi_get16(qh, &resp->resp_header.cq_flags_seq);

	dst->resp_token = sp;

	dst->resp_scb = ddi_get16(qh, &resp->resp_scb);
	dst->resp_reason = ddi_get16(qh, &resp->resp_reason);
	dst->resp_state = ddi_get16(qh, &resp->resp_state);
	dst->resp_status_flags = ddi_get16(qh, &resp->resp_status_flags);
	dst->resp_time = ddi_get16(qh, &resp->resp_time);
	dst->resp_rqs_count = ddi_get16(qh, &resp->resp_rqs_count);

	dst->resp_resid = ddi_get32(qh, &resp->resp_resid);

	for (i = 0; i < 2; i++) {
		dst->resp_reserved[i] = ddi_get32(qh,
			&resp->resp_reserved[i]);
	}

	for (i = 0; i < 8; i++) {
		dst->resp_request_sense[i] = ddi_get32(qh,
			&resp->resp_request_sense[i]);
	}

	return (sp);
}


/*
 * Function name : isp_intr()
 *
 * Return Values : DDI_INTR_CLAIMED, DDI_INTR_UNCLAIMED
 *
 * Context:	   called by interrupt thread.
 */

static u_int
isp_intr(caddr_t arg)
{
	register struct isp_cmd *sp;
	register struct isp_cmd *head, *tail;
	register uint16_t response_in;
	register struct isp_response *resp;
	register struct isp *isp = (struct isp *)arg;
	register struct isp_slot *isp_slot;
	register int n;

	ISP_DEBUG2(isp, SCSI_DEBUG, "isp_intr entry");

	if (ISP_INT_PENDING(isp) == 0) {
		return (DDI_INTR_UNCLAIMED);
	}

	do {
again:
		/*
		 * head list collects completed packets for callback later
		 */
		head = tail = NULL;

		/*
		 * Assume no mailbox events (e.g. mailbox cmds, asynch
		 * events, and isp dma errors) as common case.
		 */
		if (ISP_CHECK_SEMAPHORE_LOCK(isp) == 0) {
			mutex_enter(ISP_RESP_MUTEX(isp));

			/*
			 * Loop through completion response queue and post
			 * completed pkts.  Check response queue again
			 * afterwards to keep interrupt response time
			 * low under heavy load.
			 */
			isp->isp_response_in =
			    response_in = ISP_GET_RESPONSE_IN(isp);
			ASSERT(!(response_in >= ISP_MAX_RESPONSES));

			/*
			 * Calculate number of requests in the queue
			 * If this is < 0, then we are wrapping around
			 */
			n = response_in - isp->isp_response_out;

			if (n == 0) {
				goto update;
			} else if (n < 0) {
				n = ISP_MAX_REQUESTS -
				    isp->isp_response_out + response_in;
			}

			ISP_DEBUG2(isp, SCSI_DEBUG,
			    "intr: n=%d, in=%d, out=%d\n",
			    n, response_in, isp->isp_response_out);

			while (n-- > 0) {

				sp = isp_i_read_response(isp);
				resp = &sp->cmd_isp_response;

				/*
				 * Paranoia:  This should never happen.
				 */
				if (ISP_IS_RESPONSE_INVALID(resp)) {
					ISP_DEBUG(isp, SCSI_DEBUG,
				"invalid response:in=%x, out=%x, mbx5=%x",
					isp->isp_response_in,
					isp->isp_response_out,
					ISP_READ_REG(isp,
					    &isp->isp_reg->isp_mailbox5));
					continue;
				}

				/*
				 * Check response header flags.
				 */
				if (RESP_CQ_FLAGS(resp) & CQ_FLAG_ERR_MASK) {
					ISP_DEBUG(isp, SCSI_DEBUG,
					    "flag error");
					if (isp_i_response_error(isp, resp) ==
					    ACTION_IGNORE) {
						continue;
					}
				}

				/*
				 * Update isp deadman timer list before
				 * doing the callback and while we are
				 * holding the response mutex.
				 * note that polled cmds are not in
				 * isp_slots list
				 */
				isp_slot = &isp->isp_slots[sp->cmd_slot];
				if (isp_slot->slot_cmd == sp) {
					isp_slot->slot_cmd = NULL;
					isp_slot->slot_deadline = 0;
				}

				if (head) {
					tail->cmd_forw = sp;
					tail = sp;
					tail->cmd_forw = NULL;
				} else {
					tail = head = sp;
					sp->cmd_forw = NULL;
				}
			}
update:
			ISP_SET_RESPONSE_OUT(isp);
			ISP_CLEAR_RISC_INT(isp);
			mutex_exit(ISP_RESP_MUTEX(isp));

			if (head) {
				isp_i_call_pkt_comp(head);
			}
		} else {
			if (isp_i_handle_mbox_cmd(isp) != ISP_AEN_SUCCESS) {
				return (DDI_INTR_CLAIMED);
			}
			/*
			 * if there was a reset then check the response
			 * queue again
			 */
			goto again;
		}

	} while (ISP_INT_PENDING(isp));

	return (DDI_INTR_CLAIMED);
}

/*
 * Function name : isp_i_call_pkt_comp()
 *
 * Return Values : none
 *
 * Description	 :
 *		   callback into target driver
 *		   argument is a  NULL-terminated list of packets
 *		   copy over stuff from response packet
 *
 * Context	 : Can be called by interrupt thread.
 */
static void
isp_i_call_pkt_comp(register struct isp_cmd *head)
{
	register struct isp *isp;
	register struct isp_cmd *sp;
	register struct scsi_pkt *pkt;
	register struct isp_response *resp;
	register uint8_t status;

	isp = CMD2ISP(head);
	ASSERT(mutex_owned(ISP_REQ_MUTEX(isp)) == 0 || ddi_in_panic());
	ASSERT(mutex_owned(ISP_RESP_MUTEX(isp)) == 0 || ddi_in_panic());

	while (head) {
		sp = head;
		pkt = CMD2PKT(sp);
		head = sp->cmd_forw;

		ASSERT(sp->cmd_flags & CFLAG_FINISHED);

		resp = &sp->cmd_isp_response;

		status = pkt->pkt_scbp[0] = (uint8_t)resp->resp_scb;
		if (pkt->pkt_reason == CMD_CMPLT) {
			/*
			 * Bug 1212918:  firmware has defined
			 * CMD_DATA_UNDRUN as 21 for some reason
			 * and this is causing underruns to be
			 * incorrectly reported as an error.
			 */
			if (resp->resp_reason == 21)
				resp->resp_reason = CMD_CMPLT;

			pkt->pkt_reason = (uint8_t)resp->resp_reason;
			if (pkt->pkt_reason > CMD_UNX_BUS_FREE) {
				if (pkt->pkt_reason == TAG_REJECT) {
					pkt->pkt_reason = CMD_TAG_REJECT;
				} else {
					pkt->pkt_reason = CMD_TRAN_ERR;
				}
			}
		}
		pkt->pkt_state = ISP_GET_PKT_STATE(resp->resp_state);
		pkt->pkt_statistics = (uint32_t)
		    (ISP_GET_PKT_STATS(resp->resp_status_flags));
		pkt->pkt_resid = (long)resp->resp_resid;

		/*
		 * Check to see if the ISP has negotiated a new sync
		 * rate with the device and store that information
		 * for a later date
		 */
		if (pkt->pkt_statistics & ISP_STAT_NEGOTIATE) {
			ISP_MUTEX_ENTER(isp);
			isp_i_update_sync_prop(isp, sp);
			pkt->pkt_statistics &= ~ISP_STAT_NEGOTIATE;
			ISP_MUTEX_EXIT(isp);
		}

		/*
		 * check for parity errors
		 */
		if (pkt->pkt_statistics & STAT_PERR) {
			isp_i_log(isp, CE_WARN, "Parity Error");
		}

		/*
		 * was there a check condition and auto request sense?
		 */
		if (status && ((pkt->pkt_state &
		    (STATE_GOT_STATUS | STATE_ARQ_DONE)) ==
		    (STATE_GOT_STATUS | STATE_ARQ_DONE))) {
			isp_i_handle_arq(isp, sp);
		}


		/*
		 * if data was xferred and this was an IOPB, we need
		 * to do a dma sync
		 */
		if ((sp->cmd_flags & CFLAG_CMDIOPB) &&
		    (pkt->pkt_state & STATE_XFERRED_DATA)) {

			(void) ddi_dma_sync(sp->cmd_dmahandle,
			    sp->cmd_dma_offset, sp->cmd_dma_len,
			    DDI_DMA_SYNC_FORCPU);
		}


		ASSERT(sp->cmd_flags & CFLAG_IN_TRANSPORT);
		ASSERT(sp->cmd_flags & CFLAG_FINISHED);
		ASSERT((sp->cmd_flags & CFLAG_COMPLETED) == 0);

		sp->cmd_flags = (sp->cmd_flags & ~CFLAG_IN_TRANSPORT) |
				CFLAG_COMPLETED;

		/*
		 * Call packet completion routine if FLAG_NOINTR is not set.
		 * If FLAG_NOINTR is set turning on CFLAG_COMPLETED in line
		 * above will cause busy wait loop in
		 * isp_i_polled_cmd_start() to exit.
		 */
		if (((pkt->pkt_flags & FLAG_NOINTR) == 0) &&
		    pkt->pkt_comp) {
			(*pkt->pkt_comp)(pkt);
		}
	}

	ASSERT(mutex_owned(ISP_REQ_MUTEX(isp)) == 0 || ddi_in_panic());
	ASSERT(mutex_owned(ISP_RESP_MUTEX(isp)) == 0 || ddi_in_panic());
}


/*
 * Function name : isp_i_handle_mbox_cmd()
 *
 * Description	 : called from isp_intr() to handle a mbox or async event
 *
 * Context	 : is called by interrupt thread.
 */
static int
isp_i_handle_mbox_cmd(struct isp *isp)
{
	register int aen = ISP_AEN_FAILURE;
	register uint16_t event =
		ISP_READ_REG(isp, &isp->isp_reg->isp_mailbox0);

	/*
	 * process a mailbox event
	 */
	ISP_DEBUG2(isp, SCSI_DEBUG, "isp_intr: event= 0x%x", event);
	if (event & ISP_MBOX_EVENT_ASYNCH) {
		ISP_MUTEX_ENTER(isp);
		aen = isp_i_async_event(isp, event);
		ISP_MUTEX_EXIT(isp);
	} else {
		ISP_MUTEX_ENTER(isp);
		isp_i_mbox_cmd_complete(isp);
		ISP_MUTEX_EXIT(isp);
	}
	return (aen);
}

/*
 * Function name : isp_i_handle_arq()
 *
 * Description	 : called on an autorequest sense condition, sets up arqstat
 *
 * Context	 : Can be called from different kernel process threads.
 *		   Can be called by interrupt thread.
 */
static void
isp_i_handle_arq(struct isp *isp, struct isp_cmd *sp)
{
	register struct isp_response *resp = &sp->cmd_isp_response;
	char status;
	register struct scsi_pkt *pkt = CMD2PKT(sp);

	if (sp->cmd_scblen >= sizeof (struct scsi_arq_status)) {
		register struct scsi_arq_status *arqstat;

		ISP_DEBUG(isp, SCSI_DEBUG,
		    "tgt %d.%d: auto request sense", TGT(sp), LUN(sp));

		arqstat = (struct scsi_arq_status *)(pkt->pkt_scbp);
		status = pkt->pkt_scbp[0];
		bzero((caddr_t)arqstat, sizeof (struct scsi_arq_status));

		/*
		 * ISP does not provide statistics for request sense,
		 * so use same statistics as the original cmd.
		 */
		arqstat->sts_rqpkt_statistics = pkt->pkt_statistics;
		arqstat->sts_rqpkt_state =
		    (STATE_GOT_BUS | STATE_GOT_TARGET |
		    STATE_SENT_CMD | STATE_XFERRED_DATA | STATE_GOT_STATUS);
		if (resp->resp_rqs_count <
		    sizeof (struct scsi_extended_sense)) {
			arqstat->sts_rqpkt_resid =
				sizeof (struct scsi_extended_sense) -
				resp->resp_rqs_count;
		}
		bcopy((caddr_t)resp->resp_request_sense,
		    (caddr_t)&arqstat->sts_sensedata,
		    sizeof (struct scsi_extended_sense));
		/*
		 * restore status which was wiped out by bzero
		 */
		pkt->pkt_scbp[0] = status;
	} else {
		/*
		 * bad packet; can't copy over ARQ data
		 */
		ISP_SET_REASON(sp, CMD_TRAN_ERR);
	}
}


/*
 * Function name : isp_i_find_free_slot()
 *
 * Return Values : empty slot  number
 *
 * Description	 : find an empty slot in the isp_slots list
 *
 * Context	 : Can be called from different kernel process threads.
 *		   Can be called by interrupt thread.
 */
static int
isp_i_find_freeslot(struct isp *isp)
{
	register char found = 0;
	int slot = isp->isp_free_slot;
	int i;

	/*
	 * If slot in use, scan for a free one. Walk thru
	 * isp_slots, starting at current tag
	 * this should rarely happen.
	 */
	ISP_DEBUG(isp, SCSI_DEBUG, "found in use slot %d", slot);
	for (i = 0; i < (ISP_MAX_SLOTS - 1); i++) {
		slot = isp->isp_free_slot++;
		if (isp->isp_free_slot >=
		    (uint16_t)ISP_MAX_SLOTS) {
			isp->isp_free_slot = 0;
		}
		if (isp->isp_slots[slot].slot_cmd == NULL) {
			found = 1;
			break;
		}
		ISP_DEBUG2(isp, SCSI_DEBUG, "found in use slot %d", slot);
	}
	/*
	 * We should always be able to find a free slot
	 */
	if (!found) {
		isp_i_log(0, CE_PANIC, "isp: no free slots!!!");
	}
	ISP_DEBUG(isp, SCSI_DEBUG, "found free slot %d", slot);
	return (slot);
}


/*
 * Function name : isp_i_response_error()
 *
 * Return Values : ACTION_CONTINUE
 *		   ACTION_IGNORE
 * Description	 : handle isp flag event
 *
 * Context	 : Can be called by interrupt thread.
 */
static int
isp_i_response_error(struct isp *isp, struct isp_response *resp)
{
	register struct isp_cmd *sp = (struct isp_cmd *)resp->resp_token;
	int rval;

	ASSERT(mutex_owned(ISP_RESP_MUTEX(isp)));
	ASSERT(mutex_owned(ISP_REQ_MUTEX(isp)) == 0 || ddi_in_panic());

	/*
	 * isp unable to run request because of queue full condition.
	 * Take it out of the slot struct and then call isp_scsi_start()
	 * (This approach of resubmitting a command without a max retry
	 * check or some other type of throttle should be investigated.)
	 */
	if (RESP_CQ_FLAGS(resp) & CQ_FLAG_FULL) {
		struct isp_mbox_cmd mbox_cmd;

		ISP_DEBUG(isp, SCSI_DEBUG,
		    "isp_i_response_error: queue full");

		mutex_enter(ISP_REQ_MUTEX(isp));
		isp_i_mbox_cmd_init(isp, &mbox_cmd, 2, 4,
		    ISP_MBOX_CMD_GET_DEVICE_QUEUE_STATE,
		    (uint16_t)TGT(sp), 0, 0, 0, 0);
		(void) isp_i_mbox_cmd_start(isp, &mbox_cmd);

		ISP_DEBUG(isp, SCSI_DEBUG,
		    "Q State = %x, Exec cmd count = 0x%x, Total cmds = 0x%x",
		    mbox_cmd.mbox_in[1],
		    mbox_cmd.mbox_in[2],
		    mbox_cmd.mbox_in[3]);

		if (isp->isp_slots[sp->cmd_slot].slot_cmd == sp) {
			isp->isp_slots[sp->cmd_slot].slot_cmd = NULL;
			sp->cmd_slot = 0;
		}
		sp->cmd_flags &= ~CFLAG_IN_TRANSPORT;
		mutex_exit(ISP_REQ_MUTEX(isp));
		mutex_exit(ISP_RESP_MUTEX(isp));
		(void) isp_scsi_start(CMD2ADDR(sp), CMD2PKT(sp));
		mutex_enter(ISP_RESP_MUTEX(isp));
		rval = ACTION_IGNORE;
	} else {

		/*
		 * For bad request pkts, flag error and try again.
		 * This should *NEVER* happen.
		 */
		ISP_SET_REASON(sp, CMD_TRAN_ERR);
		if (RESP_CQ_FLAGS(resp) & CQ_FLAG_BADPACKET) {
			isp_i_log(isp, CE_WARN, "Bad request pkt");
		} else if (RESP_CQ_FLAGS(resp) & CQ_FLAG_BADHEADER) {
			isp_i_log(isp, CE_WARN, "Bad request pkt header");
		}
		rval = ACTION_CONTINUE;
	}

	ASSERT(mutex_owned(ISP_RESP_MUTEX(isp)));
	ASSERT(mutex_owned(ISP_REQ_MUTEX(isp)) == 0 || ddi_in_panic());

	return (rval);
}


/*
 * Function name : isp_i_polled_cmd_start()
 *
 * Return Values : TRAN_ACCEPT	if transaction was accepted
 *		   TRAN_BUSY	if I/O could not be started
 *		   TRAN_ACCEPT	if I/O timed out, pkt fields indicate error
 *
 * Description	 : Starts up I/O in normal fashion by calling isp_i_start_cmd().
 *		   Busy waits for I/O to complete or timeout.
 *
 * Context	 : Can be called from different kernel process threads.
 *		   Can be called by interrupt thread.
 */
static int
isp_i_polled_cmd_start(struct isp *isp, struct isp_cmd *sp)
{
	register int delay_loops;
	register int rval;
	register struct scsi_pkt *pkt = CMD2PKT(sp);

	ASSERT(mutex_owned(ISP_RESP_MUTEX(isp)) == 0 || ddi_in_panic());
	ASSERT(mutex_owned(ISP_REQ_MUTEX(isp)) == 0 || ddi_in_panic());

	/*
	 * set timeout to SCSI_POLL_TIMEOUT for non-polling
	 * commands that do not have this field set
	 */
	if (pkt->pkt_time == 0) {
		pkt->pkt_time = SCSI_POLL_TIMEOUT;
	}


	/*
	 * try and start up command
	 */
	mutex_enter(ISP_REQ_MUTEX(isp));
	rval = isp_i_start_cmd(isp, sp);
	mutex_exit(ISP_REQ_MUTEX(isp));
	if (rval != TRAN_ACCEPT) {
		goto done;
	}

	/*
	 * busy wait for command to finish ie. till CFLAG_COMPLETED is set
	 */
retry:
	delay_loops = ISP_TIMEOUT_DELAY(
	    (pkt->pkt_time + (2 * ISP_GRACE)),
	    ISP_NOINTR_POLL_DELAY_TIME);

	ISP_DEBUG2(isp, SCSI_DEBUG,
	"delay_loops=%d, delay=%d, pkt_time=%x, cdb[0]=%x\n", delay_loops,
	ISP_NOINTR_POLL_DELAY_TIME, pkt->pkt_time,
	*(sp->cmd_pkt->pkt_cdbp));

	while ((sp->cmd_flags & CFLAG_COMPLETED) == 0) {
		drv_usecwait(ISP_NOINTR_POLL_DELAY_TIME);


		if (--delay_loops <= 0) {
			register struct isp_response *resp;

			/*
			 * Call isp_scsi_reset() to reset the device,
			 * and if this fails then blow everything away
			 */
			if ((isp_scsi_reset(&pkt->pkt_address,
			    RESET_TARGET)) == 0) {
				mutex_enter(ISP_RESP_MUTEX(isp));
				isp_i_fatal_error(isp);
				mutex_exit(ISP_RESP_MUTEX(isp));
			}

			resp = &sp->cmd_isp_response;
			bzero((caddr_t)resp, sizeof (struct isp_response));

			/*
			 * update stats in resp_status_flags
			 * isp_i_call_pkt_compl() copies this
			 * over to pkt_statistics
			 */
			resp->resp_status_flags |=
			    STAT_BUS_RESET | STAT_TIMEOUT;
			resp->resp_reason = CMD_TIMEOUT;
#ifdef ISPDEBUG
			sp->cmd_flags |= CFLAG_FINISHED;
#endif
			isp_i_call_pkt_comp(sp);
			break;
		}
	}
	ISP_DEBUG2(isp, SCSI_DEBUG, "polled cmd done\n");

done:

	ASSERT(mutex_owned(ISP_RESP_MUTEX(isp)) == 0 || ddi_in_panic());
	ASSERT(mutex_owned(ISP_REQ_MUTEX(isp)) == 0 || ddi_in_panic());

	return (rval);
}


/*
 * Function name : isp_i_async_event
 *
 * Return Values : -1	Fatal error occurred
 *		    0	normal return
 * Description	 :
 * An Event of 8002 is a Sys Err in the ISP.  This would require
 *	Chip reset.
 *
 * An Event of 8001 is a external SCSI Bus Reset
 *
 * Context	 : Can be called from different kernel process threads.
 *		   Can be called by interrupt thread.
 */
static int
isp_i_async_event(register struct isp *isp, int16_t event)
{
	struct ispregs *ip = isp->isp_reg;
	int rval = ISP_AEN_SUCCESS;

	ASSERT(mutex_owned(ISP_RESP_MUTEX(isp)));
	ASSERT(mutex_owned(ISP_REQ_MUTEX(isp)));

	switch (ISP_GET_MBOX_EVENT(event)) {
	case ISP_MBOX_ASYNC_ERR:
		/*
		 * Force the current commands to timeout after
		 * resetting the chip.
		 */
		isp_i_log(isp, CE_WARN, "Firmware error");
		mutex_exit(ISP_REQ_MUTEX(isp));
		isp_i_fatal_error(isp);
		mutex_enter(ISP_REQ_MUTEX(isp));
		rval = ISP_AEN_RESET;
		break;

	case ISP_MBOX_ASYNC_REQ_DMA_ERR:
	case ISP_MBOX_ASYNC_RESP_DMA_ERR:
		/*
		 *  DMA failed in the ISP chip force a Reset
		 */
		isp_i_log(isp, CE_WARN, "DMA Failure (%x)", event);
		ISP_CLEAR_RISC_INT(isp);
		ISP_CLEAR_SEMAPHORE_LOCK(isp);
		mutex_exit(ISP_REQ_MUTEX(isp));
		isp_i_fatal_error(isp);
		mutex_enter(ISP_REQ_MUTEX(isp));
		rval = ISP_AEN_RESET;
		break;

	case ISP_MBOX_ASYNC_RESET:
		isp_i_log(isp, CE_WARN, "Received unexpected SCSI Reset");

		/* FALLTHROUGH */
	case ISP_MBOX_ASYNC_INT_RESET:
		/*
		 * Set a marker to for a internal SCSI BUS reset done
		 * to recover from a timeout.
		 */
		ISP_DEBUG(isp, SCSI_DEBUG,
		    "ISP initiated SCSI BUS Reset or external SCSI Reset");
		ISP_MUTEX_EXIT(isp);
		if (isp_i_set_marker(isp, SYNCHRONIZE_ALL, 0, 0)) {
			isp_i_log(isp, CE_WARN, "cannot set marker");
		}
		ISP_MUTEX_ENTER(isp);
		ISP_CLEAR_RISC_INT(isp);
		ISP_CLEAR_SEMAPHORE_LOCK(isp);

		mutex_exit(ISP_REQ_MUTEX(isp));
		isp_reset_notify_callback(isp);
		mutex_enter(ISP_REQ_MUTEX(isp));
		break;

	default:
		isp_i_log(isp, CE_NOTE,
		    "?mailbox regs(0-5): 0x%x 0x%x 0x%x 0x%x 0x%x 0x%x",
		    event,
		    ISP_READ_REG(isp, &ip->isp_mailbox1),
		    ISP_READ_REG(isp, &ip->isp_mailbox2),
		    ISP_READ_REG(isp, &ip->isp_mailbox3),
		    ISP_READ_REG(isp, &ip->isp_mailbox4),
		    ISP_READ_REG(isp, &ip->isp_mailbox5));
		ISP_CLEAR_RISC_INT(isp);
		ISP_CLEAR_SEMAPHORE_LOCK(isp);
		break;
	}

	ASSERT(mutex_owned(ISP_RESP_MUTEX(isp)));
	ASSERT(mutex_owned(ISP_REQ_MUTEX(isp)));

	return (rval);
}

/*
 * Function name : isp_i_mbox_cmd_complete ()
 *
 * Return Values : None.
 *
 * Description	 : Sets ISP_MBOX_CMD_FLAGS_COMPLETE flag so busy wait loop
 *		   in isp_i_mbox_cmd_start() exits.
 *
 * Context	 : Can be called by interrupt thread only.
 */
static void
isp_i_mbox_cmd_complete(struct isp *isp)
{
	struct ispregs		*ip = isp->isp_reg;
	uint16_t		*mbox_regp = (uint16_t *)&ip->isp_mailbox0;
	int			delay_loops;
	uint8_t			i;

	ISP_DEBUG2(isp, SCSI_DEBUG,
	    "isp_i_mbox_cmd_complete_start(cmd = 0x%x)",
	    isp->isp_mbox.mbox_cmd.mbox_out[0]);

	/*
	 * Check for completions that are caused by mailbox events
	 * but that do not set the mailbox status bit ie. 0x4xxx
	 * For now only the busy condition is checked, the others
	 * will automatically time out and error.
	 */
	delay_loops = ISP_TIMEOUT_DELAY(ISP_MBOX_CMD_BUSY_WAIT_TIME,
	    ISP_MBOX_CMD_BUSY_POLL_DELAY_TIME);

	while (ISP_READ_REG(isp, &ip->isp_mailbox0) == ISP_MBOX_BUSY) {
		drv_usecwait(ISP_MBOX_CMD_BUSY_POLL_DELAY_TIME);
		if (--delay_loops < 0) {
			isp->isp_mbox.mbox_cmd.mbox_in[0] =
			    ISP_MBOX_STATUS_FIRMWARE_ERR;
			goto fail;
		}
	}

	ISP_CLEAR_RISC_INT(isp);

	/*
	 * save away status registers
	 */
	for (i = 0; i < ISP_MAX_MBOX_REGS; i++, mbox_regp++) {
		isp->isp_mbox.mbox_cmd.mbox_in[i] =
			ISP_READ_REG(isp, mbox_regp);
	}

fail:
	/*
	 * set flag so that busy wait loop will detect this and return
	 */
	isp->isp_mbox.mbox_flags |= ISP_MBOX_CMD_FLAGS_COMPLETE;

	/*
	 * clear the semaphore lock
	 */
	ISP_CLEAR_SEMAPHORE_LOCK(isp);

	ISP_DEBUG2(isp, SCSI_DEBUG,
	    "isp_i_mbox_cmd_complete_end (cmd = 0x%x)",
	    isp->isp_mbox.mbox_cmd.mbox_out[0]);
}


/*
 * Function name : isp_i_mbox_cmd_init()
 *
 * Return Values : none
 *
 * Description	 : initializes mailbox command
 *
 * Context	 : Can be called from different kernel process threads.
 *		   Can be called by interrupt thread.
 */
static void
isp_i_mbox_cmd_init(struct isp *isp, struct isp_mbox_cmd *mbox_cmdp,
    uint8_t n_mbox_out, uint8_t n_mbox_in,
    uint16_t reg0, uint16_t reg1, uint16_t reg2,
    uint16_t reg3, uint16_t reg4, uint16_t reg5)
{

	ISP_DEBUG2(isp, SCSI_DEBUG,
	    "isp_i_mbox_cmd_init r0 = 0x%x r1 = 0x%x r2 = 0x%x",
	    reg0, reg1, reg2);
	ISP_DEBUG2(isp, SCSI_DEBUG,
	    "			 r3 = 0x%x r4 = 0x%x r5 = 0x%x",
	    reg3, reg4, reg5);

	mbox_cmdp->timeout	= ISP_MBOX_CMD_TIMEOUT;
	mbox_cmdp->retry_cnt	= ISP_MBOX_CMD_RETRY_CNT;
	mbox_cmdp->n_mbox_out	= n_mbox_out;
	mbox_cmdp->n_mbox_in	= n_mbox_in;
	mbox_cmdp->mbox_out[0]	= n_mbox_out >= 1 ? reg0 : 0;
	mbox_cmdp->mbox_out[1]	= n_mbox_out >= 2 ? reg1 : 0;
	mbox_cmdp->mbox_out[2]	= n_mbox_out >= 3 ? reg2 : 0;
	mbox_cmdp->mbox_out[3]	= n_mbox_out >= 4 ? reg3 : 0;
	mbox_cmdp->mbox_out[4]	= n_mbox_out >= 5 ? reg4 : 0;
	mbox_cmdp->mbox_out[5]	= n_mbox_out >= 6 ? reg5 : 0;
}


/*
 * Function name : isp_i_mbox_cmd_start()
 *
 * Return Values : 0   if no error
 *		   -1  on error
 *		   Status registers are returned in structure that is passed in.
 *
 * Description	 : Sends mailbox cmd to ISP and busy waits for completion.
 *		   Serializes accesses to the mailboxes.
 *		   Mailbox cmds are used to initialize the ISP, modify default
 *			parameters, and load new firmware.
 *
 * Context	 : Can be called from different kernel process threads.
 *		   Can be called by interrupt thread.
 */
static int isp_debug_mbox = 0;

static int
isp_i_mbox_cmd_start(struct isp *isp, struct isp_mbox_cmd *mbox_cmdp)
{
	struct ispregs		*ip = isp->isp_reg;
	uint16_t		*mbox_regp;
	char			retry_cnt = (char)mbox_cmdp->retry_cnt;
	int			delay_loops, rval = 0;
	uint8_t			i;

	ISP_DEBUG2(isp, SCSI_DEBUG, "isp_i_mbox_cmd_start_start(cmd = 0x%x)",
	    mbox_cmdp->mbox_out[0]);

	ASSERT(mutex_owned(ISP_RESP_MUTEX(isp)));
	ASSERT(mutex_owned(ISP_REQ_MUTEX(isp)));

	if (isp->isp_shutdown) {
		return (-1);
	}

	/* save away mailbox command */
	bcopy((caddr_t)mbox_cmdp, (caddr_t)&isp->isp_mbox.mbox_cmd,
	    sizeof (struct isp_mbox_cmd));

retry:

	/* write outgoing registers */
	mbox_regp = &ip->isp_mailbox0;
	for (i = 0; i < isp->isp_mbox.mbox_cmd.n_mbox_out; i++, mbox_regp++) {
		ISP_WRITE_REG(isp, mbox_regp,
			isp->isp_mbox.mbox_cmd.mbox_out[i]);
	}

	/*
	 * Turn completed flag off indicating mbox command was issued.
	 * Interrupt handler will set flag when done.
	 */
	isp->isp_mbox.mbox_flags &= ~ISP_MBOX_CMD_FLAGS_COMPLETE;

	/* signal isp that mailbox cmd was loaded */
	ISP_REG_SET_HOST_INT(isp);

	/* busy wait for mailbox cmd to be processed. */
	delay_loops = ISP_TIMEOUT_DELAY(mbox_cmdp->timeout,
	    ISP_MBOX_CMD_BUSY_POLL_DELAY_TIME);

	ISP_MUTEX_EXIT(isp);
	while ((isp->isp_mbox.mbox_flags &
			ISP_MBOX_CMD_FLAGS_COMPLETE) == 0) {
		drv_usecwait(ISP_MBOX_CMD_BUSY_POLL_DELAY_TIME);
		/* if cmd does not complete retry or return error */
		if (--delay_loops <= 0) {
			if (--retry_cnt <= 0) {
				rval = -1;
				ISP_MUTEX_ENTER(isp);
				goto done;
			} else {
				ISP_MUTEX_ENTER(isp);
				goto retry;
			}
		}
		ISP_MUTEX_ENTER(isp);
		if (ISP_CHECK_SEMAPHORE_LOCK(isp)) {
			uint16_t event = ISP_READ_REG(isp,
				&isp->isp_reg->isp_mailbox0);

			if (event & ISP_MBOX_EVENT_ASYNCH) {
				/*
				 * if an async event occurs, add
				 * some code here to decode the
				 * event and start an error recovery
				 */
				rval = -1;
				goto done;
			} else {
				isp_i_mbox_cmd_complete(isp);
			}
		}
		ISP_MUTEX_EXIT(isp);
	}
	ISP_MUTEX_ENTER(isp);

	/*
	 * copy registers saved by isp_i_mbox_cmd_complete()
	 * to mbox_cmdp
	 */
	for (i = 0; i < ISP_MAX_MBOX_REGS; i++) {
	    mbox_cmdp->mbox_in[i] = isp->isp_mbox.mbox_cmd.mbox_in[i];
	}

done:
	if (rval || isp_debug_mbox) {
		ISP_DEBUG(isp, SCSI_DEBUG, "mbox cmd %s:",
		    (rval ? "failed" : "succeeded"));
		ISP_DEBUG(isp, SCSI_DEBUG,
		    "cmd= 0x%x; 0x%x, 0x%x, 0x%x, 0x%x, 0x%x",
		    mbox_cmdp->mbox_out[0], mbox_cmdp->mbox_out[1],
		    mbox_cmdp->mbox_out[2], mbox_cmdp->mbox_out[3],
		    mbox_cmdp->mbox_out[4], mbox_cmdp->mbox_out[5]);
		ISP_DEBUG(isp, SCSI_DEBUG,
		    "status= 0x%x; 0x%x, 0x%x, 0x%x, 0x%x, 0x%x",
		    mbox_cmdp->mbox_in[0], mbox_cmdp->mbox_in[1],
		    mbox_cmdp->mbox_in[2], mbox_cmdp->mbox_in[3],
		    mbox_cmdp->mbox_in[4], mbox_cmdp->mbox_in[5]);
	} else {
		ISP_DEBUG2(isp, SCSI_DEBUG, "mbox cmd succeeded:");
		ISP_DEBUG2(isp, SCSI_DEBUG,
		    "cmd= 0x%x; 0x%x, 0x%x, 0x%x, 0x%x, 0x%x",
		    mbox_cmdp->mbox_out[0], mbox_cmdp->mbox_out[1],
		    mbox_cmdp->mbox_out[2], mbox_cmdp->mbox_out[3],
		    mbox_cmdp->mbox_out[4], mbox_cmdp->mbox_out[5]);
		ISP_DEBUG2(isp, SCSI_DEBUG,
		    "status= 0x%x; 0x%x, 0x%x, 0x%x, 0x%x, 0x%x",
		    mbox_cmdp->mbox_in[0], mbox_cmdp->mbox_in[1],
		    mbox_cmdp->mbox_in[2], mbox_cmdp->mbox_in[3],
		    mbox_cmdp->mbox_in[4], mbox_cmdp->mbox_in[5]);
	}

	ISP_DEBUG2(isp, SCSI_DEBUG, "isp_i_mbox_cmd_start_end (cmd = 0x%x)",
	    mbox_cmdp->mbox_out[0]);

	ASSERT(mutex_owned(ISP_RESP_MUTEX(isp)));
	ASSERT(mutex_owned(ISP_REQ_MUTEX(isp)));

	return (rval);
}


/*
 * Function name : isp_i_watch()
 *
 * Return Values : none
 * Description	 : Isp deadman timer routine.
 *
 * A hung isp controller is detected by failure to complete
 * cmds within a timeout interval (including grace period for
 * isp error recovery).	 All target error recovery is handled
 * directly by the isp.
 *
 * If isp hung, restart by resetting the isp and flushing the
 * crash protection queue (isp_slots) via isp_i_qflush.
 *
 * if lbolt wraps around then those packets will never timeout; there
 * is small risk of a hang in this short time frame. It is cheaper though
 * to ignore this problem since this is an extremely unlikely event
 *
 * Tagged queueing gives us other headaches since we cannot know
 * exactly when a command starts.  For example, a command with a
 * 2-hour timeout, will cause a second command with a 60-second timeout
 * to be timed-out.
 *
 * Context	 : Can be called by timeout thread.
 */

static void
isp_i_watch(caddr_t arg)
{
	struct isp	*isp		= (struct isp *)arg;
	int16_t		slot;
	uint32_t	cur_lbolt;
	uint32_t	deadline;
	struct isp_cmd	*sp;
	int		tgt;

	if (isp->isp_shutdown) {
		return;
	}

	if (drv_getparm(LBOLT, &cur_lbolt) != 0) {
		cmn_err(CE_PANIC, "isp: drv_getparm(LBOLT) failed\n");
	}

	mutex_enter(ISP_RESP_MUTEX(isp));

	ISP_DEBUG2(isp, SCSI_DEBUG,
	    "isp_i_watch: cur_lbolt=%d\n", cur_lbolt);

	for (slot = 0; slot < ISP_MAX_SLOTS; slot++) {

		mutex_enter(ISP_REQ_MUTEX(isp));
		sp = isp->isp_slots[slot].slot_cmd;
		deadline = isp->isp_slots[slot].slot_deadline;
		mutex_exit(ISP_REQ_MUTEX(isp));

		if (sp != NULL && deadline <= cur_lbolt) {
			if (sp->cmd_pkt->pkt_time) {
				isp_i_log(isp, CE_WARN,
				    "Fatal timeout on target %d.%d",
				    TGT(sp), LUN(sp));
				isp_i_fatal_error(isp);
				break;
			}
		}
	}

	/*
	 * Perform delayed property updates per target
	 */
	if (isp->isp_need_prop_update) {
		isp->isp_need_prop_update = 0;
		for (tgt = 0; tgt < N_ISP_TARGETS_WIDE; tgt++) {
			if (isp->isp_update_targets[tgt]) {
				isp_i_update_props(isp, tgt);
				isp->isp_update_targets[tgt] = 0;
			}
		}
	}

	/*
	 * Set up next timeout
	 */
	isp->isp_timeout_id = timeout(isp_i_watch, (caddr_t)isp, isp_tick);

	mutex_exit(ISP_RESP_MUTEX(isp));
}



/*
 * Function name : isp_i_fatal_error()
 *
 * Return Values :  none
 *
 * Description	 :
 * Isp fatal error recovery:
 * Reset the isp and flush the active queues and attempt restart.
 * This should only happen in case of a firmware bug or hardware
 * death.  Flushing is from backup queue as ISP cannot be trusted.
 *
 * Context	 : Can be called from different kernel process threads.
 *		   Can be called by interrupt thread.
 */
static void
isp_i_fatal_error(struct isp *isp)
{
	ASSERT(mutex_owned(ISP_RESP_MUTEX(isp)));
	ASSERT(mutex_owned(ISP_REQ_MUTEX(isp)) == 0 || ddi_in_panic());

	/*
	 * If shutdown flag is set than no need to do
	 * fatal error recovery.
	 */
	if (isp->isp_shutdown) {
		return;
	}

	isp_i_log(isp, CE_WARN, "Fatal error, resetting interface");

	/*
	 * hold off starting new requests by grabbing the request mutex
	 */
	mutex_enter(ISP_REQ_MUTEX(isp));

	isp_i_print_state(isp);

	(void) isp_i_reset_interface(isp, ISP_FORCE_RESET_BUS);

	isp_i_qflush(isp, (uint16_t)0, (uint16_t)N_ISP_TARGETS_WIDE);

	mutex_exit(ISP_REQ_MUTEX(isp));
	isp_reset_notify_callback(isp);

	ASSERT(mutex_owned(ISP_RESP_MUTEX(isp)));
	ASSERT(mutex_owned(ISP_REQ_MUTEX(isp)) == 0 || ddi_in_panic());
}


/*
 * Function name : isp_i_qflush()
 *
 * Return Values : none
 * Description	 :
 *	Flush isp queues  over range specified
 *	from start_tgt to end_tgt.  Flushing goes from oldest to newest
 *	to preserve some cmd ordering.
 *	This is used for isp crash recovery as normally isp takes
 *	care of target or bus problems.
 *
 * Context	 : Can be called from different kernel process threads.
 *		   Can be called by interrupt thread.
 */
static void
isp_i_qflush(register struct isp *isp, uint16_t start_tgt, uint16_t end_tgt)
{
	register struct isp_cmd *sp;
	register struct isp_cmd *head, *tail;
	register int16_t slot;
	register int i, n = 0;
	register struct isp_response *resp;

	ASSERT(start_tgt <= end_tgt);
	ASSERT(mutex_owned(ISP_RESP_MUTEX(isp)));
	ASSERT(mutex_owned(ISP_REQ_MUTEX(isp)));

	ISP_DEBUG(isp, SCSI_DEBUG,
	    "isp_i_qflush: range= %d-%d", start_tgt, end_tgt);

	/*
	 * Flush specified range of active queue entries
	 * (e.g. target nexus).
	 * we allow for just flushing one target, ie start_tgt == end_tgt
	 */
	head = tail = NULL;

	/*
	 * If flushing active queue, start with current free slot
	 * ie. oldest request, to preserve some order.
	 */
	slot = isp->isp_free_slot;

	for (i = 0; i < ISP_MAX_SLOTS; i++) {

		sp = isp->isp_slots[slot].slot_cmd;
		if (sp &&
		    (TGT(sp) >= start_tgt) &&
		    (TGT(sp) <= end_tgt)) {
			isp->isp_slots[slot].slot_cmd = NULL;
			resp = &sp->cmd_isp_response;
			bzero((caddr_t)resp,
			    sizeof (struct isp_response));

			/*
			 * update stats in resp_status_flags
			 * isp_i_call_pkt_compl() copies this
			 * over to pkt_statistics
			 */
			resp->resp_status_flags = STAT_BUS_RESET | STAT_ABORTED;
			resp->resp_reason = CMD_RESET;
#ifdef ISPDEBUG
			sp->cmd_flags |= CFLAG_FINISHED;
#endif
			/*
			 * queue up sp
			 * we don't want to do a callback yet
			 * until we have flushed everything and
			 * can release the mutexes
			 */
			n++;
			if (head) {
				tail->cmd_forw = sp;
				tail = sp;
				tail->cmd_forw = NULL;
			} else {
				tail = head = sp;
				sp->cmd_forw = NULL;
			}
		}

		/*
		 * Wraparound
		 */
		if (++slot >= ISP_MAX_SLOTS) {
			slot = 0;
		}
	}

	if (head) {
		/*
		 * if we would	hold the REQ mutex and	the target driver
		 * decides to do a scsi_reset(), we will get a recursive
		 * mutex failure in isp_i_set_marker
		 */
		ISP_DEBUG(isp, SCSI_DEBUG, "isp_i_qflush: %d flushed", n);
		ISP_MUTEX_EXIT(isp);
		isp_i_call_pkt_comp(head);
		ISP_MUTEX_ENTER(isp);
	}


	ASSERT(mutex_owned(ISP_RESP_MUTEX(isp)));
	ASSERT(mutex_owned(ISP_REQ_MUTEX(isp)));
}


/*
 * Function name : isp_i_set_marker()
 *
 * Return Values : none
 * Description	 :
 * Send marker request to unlock the request queue for a given target/lun
 * nexus.
 *
 * Context	 : Can be called from different kernel process threads.
 *		   Can be called by interrupt thread.
 */
/*ARGSUSED*/
static int
isp_i_set_marker(register struct isp *isp, int16_t mod, int16_t tgt,
	int16_t lun)
{
	register struct isp_request	*req;
	ddi_acc_handle_t		*qh	= isp->isp_queue_handle;

	ASSERT(mutex_owned(ISP_REQ_MUTEX(isp)) == 0 || ddi_in_panic());

	mutex_enter(ISP_REQ_MUTEX(isp));

	/*
	 * Check to see how much space is available in the
	 * Request Queue, save this so we do not have to do
	 * a lot of PIOs
	 */
	if (isp->isp_queue_space == 0) {
		ISP_UPDATE_QUEUE_SPACE(isp);

		/*
		 * Check now to see if the queue is still full
		 */
		if (isp->isp_queue_space == 0) {
			mutex_exit(ISP_REQ_MUTEX(isp));
			return (-1);
		}
	}

	ISP_GET_NEXT_REQUEST_IN(isp, req);

	/*
	 * Copy the request into device memory
	 */
	ddi_put16(qh, &req->req_header.cq_count_type,
		CQ_ENTRY_COUNT_TYPE(CQ_TYPE_MARKER, 0));
	ddi_put16(qh, &req->req_tgt_lun, REQ_TGT_LUN(tgt, lun));
	ddi_put16(qh, &req->req_modifier, mod);

	/*
	 * Use correct offset and size for syncing
	 */
	(void) ddi_dma_sync(isp->isp_dmahandle,
	    (off_t)(isp->isp_request_in * sizeof (struct isp_request)),
	    (u_int)sizeof (struct isp_request),
	    DDI_DMA_SYNC_FORDEV);

	/*
	 * Tell isp it's got a new I/O request...
	 */
	ISP_SET_REQUEST_IN(isp);
	mutex_exit(ISP_REQ_MUTEX(isp));

	ASSERT(mutex_owned(ISP_REQ_MUTEX(isp)) == 0 || ddi_in_panic());
	return (0);
}


/*
 * Function name : isp_scsi_abort()
 *
 * Return Values : 0	- abort failed
 *		   1	- abort succeeded
 * Description	 :
 * SCSA interface routine to abort pkt(s) in progress.
 * Abort the pkt specified.  If NULL pkt, abort ALL pkts.
 *
 * Context	 : Can be called from different kernel process threads.
 *		   Can be called by interrupt thread.
 */
static int
isp_scsi_abort(struct scsi_address *ap, struct scsi_pkt *pkt)
{
	struct isp		*isp = ADDR2ISP(ap);
	struct isp_mbox_cmd	mbox_cmd;
	uint16_t		 arg;

	ASSERT(mutex_owned(ISP_REQ_MUTEX(isp)) == 0 || ddi_in_panic());
	ASSERT(mutex_owned(ISP_RESP_MUTEX(isp)) == 0 || ddi_in_panic());

	/*
	 * hold off new requests, we need the req mutex anyway for mbox cmds.
	 */
	ISP_MUTEX_ENTER(isp);

	/*
	 * If no space in request queue, return error
	 */
	if (isp->isp_queue_space == 0) {
		ISP_UPDATE_QUEUE_SPACE(isp);

		/*
		 * Check now to see if the queue is still full
		 */
		if (isp->isp_queue_space == 0) {
			ISP_DEBUG(isp, SCSI_DEBUG,
			    "isp_scsi_abort: No space in Queue for Marker");
			goto fail;
		}
	}

	if (pkt) {
		struct isp_cmd *sp = PKT2CMD(pkt);

		ISP_DEBUG(isp, SCSI_DEBUG, "aborting pkt 0x%x", (int)pkt);

		arg = ((uint16_t)ap->a_target << 8) | ((uint16_t)ap->a_lun);
		isp_i_mbox_cmd_init(isp, &mbox_cmd, 4, 4,
		    ISP_MBOX_CMD_ABORT_IOCB, arg, MSW(sp), LSW(sp), 0, 0);
		if (isp_i_mbox_cmd_start(isp, &mbox_cmd)) {
			goto fail;
		}

		ISP_MUTEX_EXIT(isp);
		if (isp_i_set_marker(isp, SYNCHRONIZE_NEXUS,
		    (int16_t)ap->a_target, (int16_t)ap->a_lun)) {
			mutex_enter(ISP_RESP_MUTEX(isp));
			isp_i_fatal_error(isp);
			mutex_enter(ISP_REQ_MUTEX(isp));
			goto fail;
		}
	} else {
		ISP_DEBUG(isp, SCSI_DEBUG, "aborting all pkts");

		arg = ((uint16_t)ap->a_target << 8) | ((uint16_t)ap->a_lun);
		isp_i_mbox_cmd_init(isp, &mbox_cmd, 2, 2,
		    ISP_MBOX_CMD_ABORT_DEVICE, arg, 0, 0, 0, 0);
		if (isp_i_mbox_cmd_start(isp, &mbox_cmd)) {
			goto fail;
		}

		ISP_MUTEX_EXIT(isp);
		if (isp_i_set_marker(isp, SYNCHRONIZE_NEXUS,
		    (int16_t)ap->a_target, (int16_t)ap->a_lun)) {
			mutex_enter(ISP_RESP_MUTEX(isp));
			isp_i_fatal_error(isp);
			mutex_enter(ISP_REQ_MUTEX(isp));
			goto fail;
		}
	}

	return (1);

fail:

	ISP_MUTEX_EXIT(isp);
	return (0);
}


/*
 * Function name : isp_scsi_reset()
 *
 * Return Values : 0 - reset failed
 *		   1 - reset succeeded
 * Description	 :
 * SCSA interface routine to perform scsi resets on either
 * a specified target or the bus (default).
 *
 * Context	 : Can be called from different kernel process threads.
 *		   Can be called by interrupt thread.
 */
static int
isp_scsi_reset(struct scsi_address *ap, int level)
{
	struct isp *isp = ADDR2ISP(ap);
	struct isp_mbox_cmd mbox_cmd;
	uint16_t arg;
	int	rval = 0;

	ASSERT(mutex_owned(ISP_REQ_MUTEX(isp)) == 0 || ddi_in_panic());
	ASSERT(mutex_owned(ISP_RESP_MUTEX(isp)) == 0 || ddi_in_panic());

	/*
	 * hold off new requests, we need the req mutex anyway for mbox cmds.
	 */
	ISP_MUTEX_ENTER(isp);

	/*
	 * If no space in request queue, return error
	 */
	if (isp->isp_queue_space == 0) {
		ISP_UPDATE_QUEUE_SPACE(isp);

		/*
		 * Check now to see if the queue is still full
		 */
		if (isp->isp_queue_space == 0) {
			ISP_DEBUG(isp, SCSI_DEBUG,
			    "isp_scsi_abort: No space in Queue for Marker");
			goto fail;
		}
	}

	if (level == RESET_TARGET) {
		ISP_DEBUG(isp, SCSI_DEBUG, "isp_scsi_reset: reset target %d",
		    ap->a_target);

		arg = ((uint16_t)ap->a_target << 8) | ((uint16_t)ap->a_lun);

		isp_i_mbox_cmd_init(isp, &mbox_cmd, 2, 2,
		    ISP_MBOX_CMD_ABORT_TARGET, arg, 0, 0, 0, 0);
		if (isp_i_mbox_cmd_start(isp, &mbox_cmd)) {
			goto fail;
		}

		ISP_MUTEX_EXIT(isp);
		if (isp_i_set_marker(isp, SYNCHRONIZE_TARGET,
		    (int16_t)ap->a_target, (int16_t)ap->a_lun)) {
			mutex_enter(ISP_RESP_MUTEX(isp));
			isp_i_fatal_error(isp);
			mutex_enter(ISP_REQ_MUTEX(isp));
			goto fail;
		}
		ISP_MUTEX_ENTER(isp);
	} else {
		ISP_DEBUG(isp, SCSI_DEBUG, "isp_scsi_reset: reset bus");

		isp_i_mbox_cmd_init(isp, &mbox_cmd, 2, 2,
		    ISP_MBOX_CMD_BUS_RESET,
		    (uint16_t)((int)isp->isp_scsi_reset_delay),
		    0, 0, 0, 0);
		if (isp_i_mbox_cmd_start(isp, &mbox_cmd)) {
			goto fail;
		}

		mutex_exit(ISP_REQ_MUTEX(isp));
		isp_reset_notify_callback(isp);
		mutex_enter(ISP_REQ_MUTEX(isp));

		ISP_MUTEX_EXIT(isp);
		if (isp_i_set_marker(isp, SYNCHRONIZE_ALL, 0, 0)) {
			mutex_enter(ISP_RESP_MUTEX(isp));
			isp_i_fatal_error(isp);
			mutex_enter(ISP_REQ_MUTEX(isp));
			goto fail;
		}
		ISP_MUTEX_ENTER(isp);
	}

	rval = 1;

fail:

	ISP_MUTEX_EXIT(isp);

	return (rval);
}


/*
 * Function name : isp_i_reset_interface()
 *
 * Return Values : 0 - success
 *		  -1 - hw failure
 *
 * Description	 :
 * Master reset routine for hardware/software.	Resets softc struct,
 * isp chip, and scsi bus.  The works!
 * This function is called from isp_attach with no mutexes held or
 * from isp_i_fatal_error with response and request mutex held
 *
 * NOTE: it is up to the caller to flush the response queue and isp_slots
 *	 list
 *
 * Context	 : Can be called from different kernel process threads.
 *		   Can be called by interrupt thread.
 */
static int
isp_i_reset_interface(register struct isp *isp, int action)
{
	struct ispregs		*ip;
	register int		i;
	struct isp_mbox_cmd	mbox_cmd;
	int			rval = -1;

	ip = isp->isp_reg;

	/*
	 * Handle reset recovery; reset the bus before we reset the
	 * isp chip
	 */
	/*
	 * To speed up kernel boot, check if bus free.
	 * If it's free, skip scsi bus reset.
	 */
	ISP_WRITE_RISC_HCCR(isp, ISP_HCCR_CMD_PAUSE);

#ifdef	ISP1020
	/*
	 * Set register set to sxp registers
	 */
	ISP_SET_REG_BITS(isp, &ip->isp_sbus_conf1, ISP_REG_SXP_SELECT);

#endif	/* ISP1020 */

	if ((action & ISP_FORCE_RESET_BUS) ||
	    (ISP_READ_REG(isp, &ip->isp_sxp_pins_control) &
			ISP_SXP_BUS_BUSY)) {
		ISP_DEBUG(isp, SCSI_DEBUG, "Resetting SCSI bus");

		ISP_WRITE_REG(isp, &ip->isp_sxp_override,
			ISP_SXP_ORIDE_RESET_MODULE);

		ISP_WRITE_REG(isp, &ip->isp_sxp_command,
			ISP_SXP_RESET_BUS_CMD);

		drv_usecwait((clock_t)10000);

		/*
		 * After 10ms if the bus is not free then reset interface
		 * will fail.
		 */
		if (ISP_READ_REG(isp, &ip->isp_sxp_pins_control) &
				ISP_SXP_BUS_BUSY) {
			goto fail;
		}

		drv_usecwait((clock_t)isp->isp_scsi_reset_delay * 1000);
	}

#ifdef	ISP1020
	/*
	 * Back to risc registers for the rest of the driver
	 */
	ISP_CLR_REG_BITS(isp, &ip->isp_sbus_conf1, ISP_REG_SXP_SELECT);

#endif	/* ISP1020 */

	/*
	 * Handle resetting isp chip.
	 */
	ISP_DEBUG(isp, SCSI_DEBUG, "Resetting isp");

	/*
	 * Reset the isp.
	 */
	ISP_WRITE_REG(isp, &ip->isp_sbus_icr, ISP_SBUS_ICR_SOFT_RESET);
	ISP_WRITE_REG(isp, &ip->isp_cdma_control,
	    ISP_DMA_CON_RESET_INT | ISP_DMA_CON_CLEAR_CHAN);
	ISP_WRITE_REG(isp, &ip->isp_dma_control,
	    ISP_DMA_CON_RESET_INT | ISP_DMA_CON_CLEAR_CHAN);

	/*
	 * Wait for isp to fire up.
	 */
	for (i = 0; i < ISP_RESET_WAIT; i++) {
		if (ISP_GET_MBOX_STATUS(
		    ISP_READ_REG(isp, &ip->isp_mailbox0)) != ISP_MBOX_BUSY) {
			break;
		}
		drv_usecwait(1000);
	}
	if (i == ISP_RESET_WAIT) {
		ISP_DEBUG(isp, SCSI_DEBUG, "timeout on isp to fire up");
		goto fail;
	}

#ifdef	ISP1020
	ISP_CLR_REG_BITS(isp, &ip->isp_sbus_conf1, ISP_REG_SXP_SELECT);
#endif	/* ISP1020 */

	/*
	 * Initialize the SBus Control Parameter configuration.
	 */
	ISP_WRITE_RISC_HCCR(isp, ISP_HCCR_CMD_PAUSE);
	ISP_WRITE_REG(isp, &ip->isp_sbus_conf1, isp->isp_conf1_burst_flag);
	ISP_WRITE_REG(isp, &ip->isp_sbus_icr,
	    ISP_SBUS_ICR_ENABLE_RISC_INT | ISP_SBUS_ICR_ENABLE_ALL_INTS);

	ISP_WRITE_RISC_MTR(isp,
	    ISP_RISC_MTR_PAGE0_DEFAULT | ISP_RISC_MTR_PAGE1_DEFAULT);

	ISP_WRITE_RISC_HCCR(isp, ISP_HCCR_CMD_RELEASE);

#ifdef	ISP1020
	ISP_WRITE_RISC_HCCR(isp, 0x9000);
#endif	/* ISP1020 */

	/*
	 * Start ISP firmware up.
	 */
	isp_i_mbox_cmd_init(isp, &mbox_cmd, 2, 6,
	    ISP_MBOX_CMD_START_FW, ISP_RISC_CODE_ADDR,
	    0, 0, 0, 0);
	if (isp_i_mbox_cmd_start(isp, &mbox_cmd)) {
		goto fail;
	}

	/*
	 * set Initiator SCSI ID
	 */
	isp_i_mbox_cmd_init(isp, &mbox_cmd, 2, 2,
	    ISP_MBOX_CMD_SET_SCSI_ID, isp->isp_initiator_id,
	    0, 0, 0, 0);
	if (isp_i_mbox_cmd_start(isp, &mbox_cmd)) {
		goto fail;
	}

	ISP_DEBUG(isp, SCSI_DEBUG, "Resetting queues");

	/*
	 * Initialize request and response queue indexes.
	 */
	isp->isp_request_in  = isp->isp_request_out = 0;
	isp->isp_request_ptr = isp->isp_request_base;

	isp->isp_response_in  = isp->isp_response_out = 0;
	isp->isp_response_ptr = isp->isp_response_base;

	isp_i_mbox_cmd_init(isp, &mbox_cmd, 5, 5,
	    ISP_MBOX_CMD_INIT_REQUEST_QUEUE, ISP_MAX_REQUESTS,
	    MSW(isp->isp_request_dvma), LSW(isp->isp_request_dvma),
	    isp->isp_request_in, 0);
	if (isp_i_mbox_cmd_start(isp, &mbox_cmd)) {
		goto fail;
	}

	isp_i_mbox_cmd_init(isp, &mbox_cmd, 6, 6,
	    ISP_MBOX_CMD_INIT_RESPONSE_QUEUE, ISP_MAX_RESPONSES,
	    MSW(isp->isp_response_dvma), LSW(isp->isp_response_dvma),
	    0, isp->isp_response_out);
	if (isp_i_mbox_cmd_start(isp, &mbox_cmd)) {
		goto fail;
	}

	/*
	 * Get ISP Ram firmware version numbers
	 */
	isp_i_mbox_cmd_init(isp, &mbox_cmd, 1, 6,
	    ISP_MBOX_CMD_ABOUT_PROM, 0, 0, 0, 0, 0);
	if (isp_i_mbox_cmd_start(isp, &mbox_cmd)) {
		goto fail;
	}

	isp->isp_major_rev = mbox_cmd.mbox_in[1];
	isp->isp_minor_rev = mbox_cmd.mbox_in[2];
	ISP_DEBUG2(isp, SCSI_DEBUG, "Firmware Version: major = %d, minor = %d",
	    isp->isp_major_rev, isp->isp_minor_rev);

#ifdef	ISP1000
	/*
	 * Warn user about older firmware versions on sbus cards.
	 * This driver is unlikely to work on any firmware
	 * older than 1.16 due to firmware bugs.
	 */
	if (isp->isp_major_rev < 1 ||
	    (isp->isp_major_rev == 1 && isp->isp_minor_rev < 16)) {
		cmn_err(CE_WARN, "%s%d: firmware version %d.%d"
			" may be out-of-date",
			ddi_get_name(isp->isp_dip),
			ddi_get_instance(isp->isp_dip),
			isp->isp_major_rev, isp->isp_minor_rev);
	}
#endif

	/*
	 * Handle isp capabilities adjustments.
	 */
	ISP_DEBUG(isp, SCSI_DEBUG, "Initializing isp capabilities");

	/*
	 * Check and adjust "host id" as required.
	 */
	isp_i_mbox_cmd_init(isp, &mbox_cmd, 1, 2,
	    ISP_MBOX_CMD_GET_SCSI_ID, 0, 0, 0, 0, 0);
	if (isp_i_mbox_cmd_start(isp, &mbox_cmd)) {
		goto fail;
	}
	if (mbox_cmd.mbox_in[1] != isp->isp_initiator_id) {
		ISP_DEBUG(isp, SCSI_DEBUG, "id = %d(%d)",
		    isp->isp_initiator_id,
		    mbox_cmd.mbox_in[1]);
		isp_i_mbox_cmd_init(isp, &mbox_cmd, 2, 2,
		    ISP_MBOX_CMD_SET_SCSI_ID, isp->isp_initiator_id,
		    0, 0, 0, 0);
		if (isp_i_mbox_cmd_start(isp, &mbox_cmd)) {
			goto fail;
		}
	}

	/*
	 * Check and adjust "retries" as required.
	 */
	isp_i_mbox_cmd_init(isp, &mbox_cmd, 1, 3,
	    ISP_MBOX_CMD_GET_RETRY_ATTEMPTS, 0, 0, 0, 0, 0);
	if (isp_i_mbox_cmd_start(isp, &mbox_cmd)) {
		goto fail;
	}
	if (mbox_cmd.mbox_in[1] != ISP_RETRIES ||
	    mbox_cmd.mbox_in[2] != ISP_RETRY_DELAY) {
		isp_i_mbox_cmd_init(isp, &mbox_cmd, 3, 3,
		    ISP_MBOX_CMD_SET_RETRY_ATTEMPTS, ISP_RETRIES,
		    ISP_RETRY_DELAY, 0, 0, 0);
		if (isp_i_mbox_cmd_start(isp, &mbox_cmd)) {
			goto fail;
		}
		ISP_DEBUG(isp, SCSI_DEBUG, "retries = %d(%d), delay = %d(%d)",
		    ISP_RETRIES, mbox_cmd.mbox_in[1],
		    ISP_RETRY_DELAY, mbox_cmd.mbox_in[2]);
	}

	isp_i_mbox_cmd_init(isp, &mbox_cmd, 2, 2,
	    ISP_MBOX_CMD_SET_SEL_TIMEOUT, isp_selection_timeout, 0, 0, 0, 0);
	if (isp_i_mbox_cmd_start(isp, &mbox_cmd)) {
		goto fail;
	}

	/*
	 * Check and adjust "tag age limit" as required.
	 */
	isp_i_mbox_cmd_init(isp, &mbox_cmd, 1, 2,
	    ISP_MBOX_CMD_GET_AGE_LIMIT, 0, 0, 0, 0, 0);
	if (isp_i_mbox_cmd_start(isp, &mbox_cmd)) {
		goto fail;
	}
	if (mbox_cmd.mbox_in[1] != isp->isp_scsi_tag_age_limit) {
		ISP_DEBUG(isp, SCSI_DEBUG, "tag age = %d(%d)",
		    isp->isp_scsi_tag_age_limit,
		    mbox_cmd.mbox_in[1]);
		isp_i_mbox_cmd_init(isp, &mbox_cmd, 2, 1,
		    ISP_MBOX_CMD_SET_AGE_LIMIT,
		    (uint16_t)isp->isp_scsi_tag_age_limit, 0, 0, 0, 0);
		if (isp_i_mbox_cmd_start(isp, &mbox_cmd)) {
			goto fail;
		}
	}

	/*
	 * Check and adjust isp queues as required.
	 */
	for (i = 0; i < N_ISP_TARGETS_WIDE; i++) {
		isp_i_mbox_cmd_init(isp, &mbox_cmd, 2, 4,
		    ISP_MBOX_CMD_GET_DEVICE_QUEUE_PARAMS,
		    (uint16_t)(i << 8), 0, 0, 0, 0);
		if (isp_i_mbox_cmd_start(isp, &mbox_cmd)) {
		    goto fail;
		}
		ISP_DEBUG2(isp, SCSI_DEBUG,
		    "Max Queue Depth = 0x%x, Exec Trottle = 0x%x",
		    mbox_cmd.mbox_in[2], mbox_cmd.mbox_in[3]);
	}

	isp_i_mbox_cmd_init(isp, &mbox_cmd, 2, 4,
	    ISP_MBOX_CMD_GET_FIRMWARE_STATUS, 0, 0, 0, 0, 0);
	(void) isp_i_mbox_cmd_start(isp, &mbox_cmd);

	ISP_DEBUG(isp, SCSI_DEBUG, "Max # of IOs = 0x%x",
	    mbox_cmd.mbox_in[2]);

	/*
	 * Update caps from isp.
	 */
	ISP_DEBUG(isp, SCSI_DEBUG, "Getting isp capabilities");
	for (i = 0; i < N_ISP_TARGETS_WIDE; i++) {
		if (isp_i_updatecap(isp, i) == -1)
			goto fail;
	}

	rval = 0;

fail:
	if (rval) {
		ISP_DEBUG(isp, SCSI_DEBUG, "reset interface failed");
		isp->isp_shutdown = 1;
		isp_i_log(isp, CE_WARN, "interface going offline");
		ISP_CLEAR_RISC_INT(isp);

		ISP_WRITE_REG(isp, &ip->isp_sbus_icr,
			ISP_SBUS_ICR_DISABLE_ALL_INTS);
		ISP_WRITE_RISC_HCCR(isp, ISP_HCCR_CMD_PAUSE);
		ISP_WRITE_RISC_PSR(isp, 0xc000);
		ISP_WRITE_RISC_PCR(isp, 0x1000);

		isp_i_qflush(isp, (uint16_t)0, (uint16_t)N_ISP_TARGETS_WIDE);
	}

	return (rval);
}


#include <sys/varargs.h>

/*
 * Error logging, printing, and debug print routines
 */

/*VARARGS3*/
static void
isp_i_log(struct isp *isp, int level, char *fmt, ...)
{
	char	buf[256];
	va_list	ap;

	va_start(ap, fmt);
	(void) vsprintf(buf, fmt, ap);
	va_end(ap);

	scsi_log(isp ? isp->isp_dip : NULL, "isp", level, "%s\n", buf);
}


static void
isp_i_print_state(struct isp *isp)
{
	char buf[128];
	register int i;
	char risc_paused = 0;
	struct ispregs *ip = isp->isp_reg;

	/* Put isp header in buffer for later messages. */
	isp_i_log(isp, CE_NOTE, "State dump from isp registers and driver:");

	/*
	 * Print isp registers.
	 */
	(void) sprintf(buf,
		"mailboxes(0-5): 0x%x, 0x%x, 0x%x, 0x%x, 0x%x, 0x%x",
		ISP_READ_REG(isp, &ip->isp_mailbox0),
		ISP_READ_REG(isp, &ip->isp_mailbox1),
		ISP_READ_REG(isp, &ip->isp_mailbox2),
		ISP_READ_REG(isp, &ip->isp_mailbox3),
		ISP_READ_REG(isp, &ip->isp_mailbox4),
		ISP_READ_REG(isp, &ip->isp_mailbox5));
	isp_i_log(isp, CE_NOTE, buf);

	if (ISP_READ_RISC_HCCR(isp) ||
			ISP_READ_REG(isp, &ip->isp_sbus_sema)) {
		(void) sprintf(buf,
			"hccr= 0x%x, sbus_sema= 0x%x",
			ISP_READ_RISC_HCCR(isp),
			ISP_READ_REG(isp, &ip->isp_sbus_sema));
		isp_i_log(isp, CE_NOTE, buf);
	}
	if ((ISP_READ_RISC_HCCR(isp) & ISP_HCCR_PAUSE) == 0) {
		ISP_WRITE_RISC_HCCR(isp, ISP_HCCR_CMD_PAUSE);
		risc_paused = 1;
	}

	(void) sprintf(buf,
		"sbus: isr= 0x%x, icr= 0x%x, conf0= 0x%x, conf1= 0x%x",
		ISP_READ_REG(isp, &ip->isp_sbus_isr),
		ISP_READ_REG(isp, &ip->isp_sbus_icr),
		ISP_READ_REG(isp, &ip->isp_sbus_conf0),
		ISP_READ_REG(isp, &ip->isp_sbus_conf1));
	isp_i_log(isp, CE_NOTE, buf);

	(void) sprintf(buf,
		"cdma: count= %d, addr= 0x%x, status= 0x%x, conf= 0x%x",
		ISP_READ_REG(isp, &ip->isp_cdma_count),
		(ISP_READ_REG(isp, &ip->isp_cdma_addr_hi) << 16) |
		ISP_READ_REG(isp, &ip->isp_cdma_addr_lo),
		ISP_READ_REG(isp, &ip->isp_cdma_status),
		ISP_READ_REG(isp, &ip->isp_cdma_conf));

	if ((i = ISP_READ_REG(isp, &ip->isp_cdma_control)) != 0) {
		(void) sprintf(&buf[strlen(buf)], ", control= 0x%x",
			(uint16_t)i);
	}
	if ((i = ISP_READ_REG(isp, &ip->isp_cdma_fifo_status)) != 0) {
		(void) sprintf(&buf[strlen(buf)], ", fifo_status= 0x%x",
			(uint16_t)i);
	}
	isp_i_log(isp, CE_NOTE, buf);

	(void) sprintf(buf,
		"dma: count= %d, addr= 0x%x, status= 0x%x, conf= 0x%x",
		(ISP_READ_REG(isp, &ip->isp_dma_count_hi) << 16) |
		ISP_READ_REG(isp, &ip->isp_dma_count_lo),
		(ISP_READ_REG(isp, &ip->isp_dma_addr_hi)  << 16) |
		ISP_READ_REG(isp, &ip->isp_dma_addr_lo),
		ISP_READ_REG(isp, &ip->isp_dma_status),
		ISP_READ_REG(isp, &ip->isp_dma_conf));

	if ((i = ISP_READ_REG(isp, &ip->isp_dma_control)) != 0) {
		(void) sprintf(&buf[strlen(buf)], ", control= 0x%x",
			(uint16_t)i);
	}
	if ((i = ISP_READ_REG(isp, &ip->isp_dma_fifo_status)) != 0) {
		(void) sprintf(&buf[strlen(buf)], ", fifo_status= 0x%x",
			(uint16_t)i);
	}
	isp_i_log(isp, CE_NOTE, buf);

	/*
	 * If the risc isn't already paused, pause it now.
	 */
	if ((ISP_READ_RISC_HCCR(isp) & ISP_HCCR_PAUSE) == 0) {
		ISP_WRITE_RISC_HCCR(isp, ISP_HCCR_CMD_PAUSE);
		risc_paused = 1;
	}

	(void) sprintf(buf,
	    "risc: R0-R7= 0x%x, 0x%x, 0x%x, 0x%x, 0x%x, 0x%x, 0x%x 0x%x",
	    ISP_READ_RISC_ACC(isp),
	    ISP_READ_RISC_R1(isp),
	    ISP_READ_RISC_R2(isp),
	    ISP_READ_RISC_R3(isp),
	    ISP_READ_RISC_R4(isp),
	    ISP_READ_RISC_R5(isp),
	    ISP_READ_RISC_R6(isp),
	    ISP_READ_RISC_R7(isp));
	isp_i_log(isp, CE_NOTE, buf);

	(void) sprintf(buf,
	    "risc: R8-R15= 0x%x, 0x%x, 0x%x, 0x%x, 0x%x, 0x%x, 0x%x 0x%x",
	    ISP_READ_RISC_R8(isp),
	    ISP_READ_RISC_R9(isp),
	    ISP_READ_RISC_R10(isp),
	    ISP_READ_RISC_R11(isp),
	    ISP_READ_RISC_R12(isp),
	    ISP_READ_RISC_R13(isp),
	    ISP_READ_RISC_R14(isp),
	    ISP_READ_RISC_R15(isp));
	isp_i_log(isp, CE_NOTE, buf);

	(void) sprintf(buf,
	    "risc: PSR= 0x%x, IVR= 0x%x, PCR=0x%x, RAR0=0x%x, RAR1=0x%x",
	    ISP_READ_RISC_PSR(isp),
	    ISP_READ_RISC_IVR(isp),
	    ISP_READ_RISC_PCR(isp),
	    ISP_READ_RISC_RAR0(isp),
	    ISP_READ_RISC_RAR1(isp));
	isp_i_log(isp, CE_NOTE, buf);

	(void) sprintf(buf,
	    "risc: LCR= 0x%x, PC= 0x%x, MTR=0x%x, EMB=0x%x, SP=0x%x",
	    ISP_READ_RISC_LCR(isp),
	    ISP_READ_RISC_PC(isp),
	    ISP_READ_RISC_MTR(isp),
	    ISP_READ_RISC_EMB(isp),
	    ISP_READ_RISC_SP(isp));
	isp_i_log(isp, CE_NOTE, buf);

	/*
	 * If we paused the risc, restart it.
	 */
	if (risc_paused) {
		ISP_WRITE_RISC_HCCR(isp, ISP_HCCR_CMD_RELEASE);
	}

	/*
	 * Print isp queue settings out.
	 */
	isp->isp_request_out = ISP_READ_REG(isp, &ip->isp_mailbox4);
	(void) sprintf(buf,
	    "request(in/out)= %d/%d, response(in/out)= %d/%d",
	    isp->isp_request_in, isp->isp_request_out,
	    isp->isp_response_in, isp->isp_response_out);
	isp_i_log(isp, CE_NOTE, buf);

	(void) sprintf(buf,
	    "request_ptr(current, base)=  0x%x (0x%x)",
	    (int)isp->isp_request_ptr, (int)isp->isp_request_base);
	isp_i_log(isp, CE_NOTE, buf);

	(void) sprintf(buf,
	    "response_ptr(current, base)= 0x%x (0x%x)",
	    (int)isp->isp_response_ptr, (int)isp->isp_response_base);
	isp_i_log(isp, CE_NOTE, buf);

	if (ISP_READ_REG(isp, &ip->isp_cdma_addr_hi) ||
			ISP_READ_REG(isp, &ip->isp_cdma_addr_lo)) {
		(void) sprintf(buf,
		    "dvma request_ptr= 0x%x - 0x%x",
		    (int)isp->isp_request_dvma,
		    (int)isp->isp_response_dvma);
		isp_i_log(isp, CE_NOTE, buf);

		(void) sprintf(buf,
		    "dvma response_ptr= 0x%x - 0x%x",
		    (int)isp->isp_response_dvma,
		    (int)isp->isp_request_dvma + ISP_QUEUE_SIZE);
		isp_i_log(isp, CE_NOTE, buf);
	}


	/*
	 * Print out sync scsi info and suppress trailing zero
	 * period and offset entries.
	 */
	if (isp->isp_scsi_options & SCSI_OPTIONS_SYNC) {
		(void) sprintf(buf, "period/offset:");
		for (i = 0; i < N_ISP_TARGETS; i++) {
			(void) sprintf(&buf[strlen(buf)], " %d/%d",
			    PERIOD_MASK(isp->isp_synch[i]),
			    OFFSET_MASK(isp->isp_synch[i]));
		}
		isp_i_log(isp, CE_NOTE, buf);
		(void) sprintf(buf, "period/offset:");
		for (i = N_ISP_TARGETS; i < N_ISP_TARGETS_WIDE; i++) {
			(void) sprintf(&buf[strlen(buf)], " %d/%d",
			    PERIOD_MASK(isp->isp_synch[i]),
			    OFFSET_MASK(isp->isp_synch[i]));
		}
		isp_i_log(isp, CE_NOTE, buf);
	}
}



#ifdef ISPDEBUG

static void
isp_debug_dump_cdb(struct scsi_address *ap, struct scsi_pkt *pkt)
{
	static char	hex[]	= "0123456789abcdef";
	struct isp	*isp	= ADDR2ISP(ap);
	struct isp_cmd	*sp	= PKT2CMD(pkt);
	uint8_t		*cdb	= pkt->pkt_cdbp;
	char		buf [256];
	char		*p;
	int		i;

	(void) sprintf(buf, "isp%d: <%d,%d> ",
		ddi_get_instance(isp->isp_dip),
		ap->a_target, ap->a_lun);

	p = buf + strlen(buf);

	*p++ = '[';
	for (i = 0; i < sp->cmd_cdblen; i++, cdb++) {
		if (i != 0)
			*p++ = ' ';
		*p++ = hex[(*cdb >> 4) & 0x0f];
		*p++ = hex[*cdb & 0x0f];
	}
	*p++ = ']';
	*p++ = '\n';
	*p = 0;

	cmn_err(CE_CONT, buf);
}
#endif	/* ISPDEBUG */
