/*
 * Copyright (c) 1995,1997, by Sun Microsystems, Inc.
 * All rights reserved.
 */

#ifndef _SYS_SCSI_ADAPTERS_ISPVAR_H
#define	_SYS_SCSI_ADAPTERS_ISPVAR_H

#pragma ident	"@(#)ispvar.h	1.77	97/09/02 SMI"

#include <sys/note.h>

#ifdef	__cplusplus
extern "C" {
#endif

/*
 * Convenient short hand defines
 */
#define	TRUE			 1
#define	FALSE			 0
#define	UNDEFINED		-1

#define	CNUM(isp)		(ddi_get_instance(isp->isp_tran.tran_dev))

#define	ISP_RETRY_DELAY		5
#define	ISP_RETRIES		0	/* retry of selections */
#define	ISP_INITIAL_SOFT_SPACE	5	/* Used for the softstate_init func */

#define	MSW(x)			(int16_t)(((int32_t)x >> 16) & 0xFFFF)
#define	LSW(x)			(int16_t)((int32_t)x & 0xFFFF)

#define	TGT(sp)			(CMD2PKT(sp)->pkt_address.a_target)
#define	LUN(sp)			(CMD2PKT(sp)->pkt_address.a_lun)

#define	HW_REV(val)		(((val) >>8) & 0xff)
#define	FW_REV(val)		((val) & 0xff)

/*
 * Targets supported
 */
#define	N_ISP_TARGETS		8
#define	N_ISP_TARGETS_WIDE	16


/*
 * Default isp scsi-options
 */
#define	ISP_DEFAULT_SCSI_OPTIONS					\
					SCSI_OPTIONS_PARITY	|	\
					SCSI_OPTIONS_DR		|	\
					SCSI_OPTIONS_SYNC	|	\
					SCSI_OPTIONS_TAG	|	\
					SCSI_OPTIONS_FAST	|	\
					SCSI_OPTIONS_WIDE

/*
 *	Tag reject
 */
#define	TAG_REJECT	28
/*
 * Interrupt actions returned by isp_i_flag_event()
 */
#define	ACTION_CONTINUE		0	/* Continue */
#define	ACTION_RETURN		1	/* Exit */
#define	ACTION_IGNORE		2	/* Ignore */

/*
 * Reset actions for isp_i_reset_interface()
 */
#define	ISP_RESET_BUS_IF_BUSY	0x01	/* reset scsi bus if it is busy */
#define	ISP_FORCE_RESET_BUS	0x02	/* reset scsi bus on error reco */


/*
 * extracting period and offset from isp_synch
 */
#define	PERIOD_MASK(val)	((val) & 0xff)
#define	OFFSET_MASK(val)	(((val) >>8) & 0xff)

/*
 * timeout values
 */
#define	ISP_GRACE		10	/* Timeout margin (sec.) */
#define	ISP_TIMEOUT_DELAY(secs, delay)	(secs * (1000000 / delay))

/*
 * delay time for polling loops
 */
#define	ISP_NOINTR_POLL_DELAY_TIME		1000	/* usecs */

/*
 * busy wait delay time after chip reset
 */
#define	ISP_CHIP_RESET_BUSY_WAIT_TIME		100	/* usecs */

/*
 * timeout for ISP coming out of reset
 */
#define	ISP_RESET_WAIT				1000	/* ms */
#define	ISP_SOFT_RESET_TIME			1	/* second */


/*
 * Debugging macros
 */
#ifdef ISPDEBUG
#define	ISP_DEBUG	if (ispdebug) isp_i_log
#define	ISP_DEBUG2	if (ispdebug > 1) isp_i_log
#else	/* ISPDEBUG */
#define	ispdebug	(0)
#define	INFORMATIVE	(0)
#define	DEBUGGING	(0)
#define	DEBUGGING_ALL	(0)

#define	ISP_DEBUG	if (0) isp_i_log
#define	ISP_DEBUG2	if (0) isp_i_log
#endif /* ISPDEBUG */

/*
 * Size definitions for request and response queues.
 */
#ifdef	ISP1000
#define	ISP_MAX_REQUESTS	256
#define	ISP_MAX_RESPONSES	256
#endif

#ifdef	ISP1020
#define	ISP_MAX_REQUESTS	16
#define	ISP_MAX_RESPONSES	16
#endif

/*
 * Maximum cdb length
 */
#define	ISP_MAX_CDB		12


#define	ISP_QUEUE_SIZE							\
	(ISP_MAX_REQUESTS  * sizeof (struct isp_request) +		\
	    ISP_MAX_RESPONSES * sizeof (struct isp_response))

/*
 * ISP request packet as defined by the Firmware Interface
 */
struct isp_dataseg {
	int32_t	d_base;
	int32_t	d_count;
};

/*
 * Request/response queue header
 *
 * These are read/written via the endian-neutral data access functions
 * as words, which swaps bytes as necessary if the host and device differ.
 */
struct cq_header {
	uint16_t	cq_count_type;
	uint16_t	cq_flags_seq;
};


#define	CQ_ENTRY_COUNT(count_type)		((count_type >> 8) & 0xff)
#define	CQ_ENTRY_TYPE(count_type)		(count_type & 0xff)
#define	CQ_SEQNO(flags_seq)			((flags_seq >> 8) & 0xff)
#define	CQ_FLAGS(flags_seq)			(flags_seq & 0xff)

#define	CQ_ENTRY_COUNT_TYPE(count, type)	((count << 8) | (type))
#define	CQ_FLAGS_SEQ(flags, seq)		((seq << 8) | (flags))

#define	REQ_TGT_LUN(tgt, lun)			(((tgt) << 8) | (lun))


#define	RESP_CQ_FLAGS(resp)	((resp->resp_header.cq_flags_seq) & 0xff)


#define	ISP_NDATASEGS		4

struct isp_request {
	struct cq_header	req_header;
	void			*req_token;

	uint16_t		req_tgt_lun;

	uint16_t		req_cdblen;
#define	req_modifier		req_cdblen	/* marker packet */
	uint16_t		req_flags;
	uint16_t		req_reserved;
	uint16_t		req_time;
	uint16_t		req_seg_count;

	uint8_t			req_cdb[ISP_MAX_CDB];
	struct isp_dataseg	req_dataseg[ISP_NDATASEGS];
};

#define	ISP_UPDATE_QUEUE_SPACE(isp)					\
	isp->isp_request_out = ISP_GET_REQUEST_OUT(isp);		\
	if (isp->isp_request_in == isp->isp_request_out) {		\
		isp->isp_queue_space = ISP_MAX_REQUESTS - 1;		\
	} else if (isp->isp_request_in > isp->isp_request_out) {	\
		isp->isp_queue_space = ((ISP_MAX_REQUESTS - 1) -	\
		    (isp->isp_request_in - isp->isp_request_out));	\
	} else {							\
		isp->isp_queue_space = isp->isp_request_out -		\
		    isp->isp_request_in - 1;				\
	}

/*
 * Header flags definitions
 */
#define	CQ_FLAG_CONTINUATION	0x01
#define	CQ_FLAG_FULL		0x02
#define	CQ_FLAG_BADHEADER	0x04
#define	CQ_FLAG_BADPACKET	0x08
#define	CQ_FLAG_ERR_MASK	\
	(CQ_FLAG_FULL | CQ_FLAG_BADHEADER | CQ_FLAG_BADPACKET)

/*
 * Header entry_type definitions
 */
#define	CQ_TYPE_REQUEST		1
#define	CQ_TYPE_DATASEG		2
#define	CQ_TYPE_RESPONSE	3
#define	CQ_TYPE_MARKER		4
#define	CQ_TYPE_CMDONLY		5


/*
 * marker packet (req_modifier) values
 */
#define	SYNCHRONIZE_NEXUS	0
#define	SYNCHRONIZE_TARGET	1
#define	SYNCHRONIZE_ALL		2

/*
 * request flag values
 */
#define	ISP_REQ_FLAG_NODISCON		0x0001
#define	ISP_REQ_FLAG_HEAD_TAG		0x0002
#define	ISP_REQ_FLAG_ORDERED_TAG	0x0004
#define	ISP_REQ_FLAG_SIMPLE_TAG		0x0008
#define	ISP_REQ_FLAG_USE_TRN		0x0010
#define	ISP_REQ_FLAG_DATA_READ		0x0020
#define	ISP_REQ_FLAG_DATA_WRITE		0x0040
#define	ISP_REQ_FLAG_DATA_WRITE		0x0040
#define	ISP_REQ_FLAG_DISARQ		0x0080
#define	ISP_REQ_FLAG_MASK		0x00FF

/*
 * translate scsi_pkt flags into ISP request packet flags
 * It would be illegal if two flags are set; the driver does not
 * check for this. Setting NODISCON and a tag flag is harmless.
 */
#define	ISP_SET_PKT_FLAGS(scsa_flags, isp_flags) {			\
	isp_flags = (scsa_flags >> 11) & 0xe; /* tags */		\
	isp_flags |= (scsa_flags >> 1) & 0x1; /* no disconnect */	\
}

/*
 * isp_request size minus header.
 */
#define	ISP_PAYLOAD		\
	(sizeof (struct isp_request) - sizeof (struct cq_header))

/*
 * throttle values for ISP request queue
 */
#define	SHUTDOWN_THROTTLE	-1	/* do not submit any requests */
#define	CLEAR_THROTTLE		(ISP_MAX_REQUESTS -1)

/*
 * XXX: Note, this request queue macro *ASSUMES* that queue full cannot
 *	occur.
 */
#define	ISP_GET_NEXT_REQUEST_IN(isp, ptr) {				\
	(ptr) = (isp)->isp_request_ptr;					\
	if ((isp)->isp_request_in == (ISP_MAX_REQUESTS - 1)) {		\
		(isp)->isp_request_in = 0;				\
		(isp)->isp_request_ptr = (isp)->isp_request_base;	\
	} else {							\
		(isp)->isp_request_in++;				\
		(isp)->isp_request_ptr++;				\
	}								\
}

/*
 * slots queue for isp timeout handling
 * Must be a multiple of 8
 */
#define	ISP_DISK_QUEUE_DEPTH	100
#define	ISP_MAX_SLOTS		((N_ISP_TARGETS_WIDE * ISP_DISK_QUEUE_DEPTH) + \
				ISP_MAX_REQUESTS)

/*
 * ISP response packet as defined by the Firmware Interface
 */
struct isp_response {
	struct cq_header	resp_header;
	void			*resp_token;

	uint16_t		resp_scb;
	uint16_t		resp_reason;
	uint16_t		resp_state;
	uint16_t		resp_status_flags;
	uint16_t		resp_time;
	uint16_t		resp_rqs_count;

	uint32_t		resp_resid;
	uint32_t		resp_reserved[2];
	uint32_t		resp_request_sense[8];
};

#define	ISP_GET_NEXT_RESPONSE_OUT(isp, ptr) {				\
	(ptr) = (isp)->isp_response_ptr;				\
	if ((isp)->isp_response_out == (ISP_MAX_RESPONSES - 1)) {	\
		(isp)->isp_response_out = 0;				\
		(isp)->isp_response_ptr = (isp)->isp_response_base;	\
	} else {							\
		(isp)->isp_response_out++;				\
		(isp)->isp_response_ptr++;				\
	}								\
}

#define	ISP_IS_RESPONSE_INVALID(resp) \
	(CQ_ENTRY_TYPE(((resp)->resp_header.cq_count_type)) != CQ_TYPE_RESPONSE)


#define	ISP_GET_PKT_STATE(state)	((uint32_t)(state >> 8))
#define	ISP_GET_PKT_STATS(stats)	((uint32_t)(stats))

#define	ISP_STAT_NEGOTIATE	0x0080

#define	ISP_SET_REASON(sp, reason) { \
	if ((sp) && CMD2PKT(sp)->pkt_reason == CMD_CMPLT) \
		CMD2PKT(sp)->pkt_reason = (reason); \
}

/*
 * mutex short hands
 */
#define	ISP_REQ_MUTEX(isp)	(&isp->isp_request_mutex)
#define	ISP_RESP_MUTEX(isp)	(&isp->isp_response_mutex)


#define	ISP_MUTEX_ENTER(isp)	mutex_enter(ISP_RESP_MUTEX(isp)),	\
				mutex_enter(ISP_REQ_MUTEX(isp))
#define	ISP_MUTEX_EXIT(isp)	mutex_exit(ISP_RESP_MUTEX(isp)),	\
				mutex_exit(ISP_REQ_MUTEX(isp))


/*
 * HBA interface macros
 */
#define	SDEV2TRAN(sd)		((sd)->sd_address.a_hba_tran)
#define	SDEV2ADDR(sd)		(&((sd)->sd_address))
#define	PKT2TRAN(pkt)		((pkt)->pkt_address.a_hba_tran)
#define	ADDR2TRAN(ap)		((ap)->a_hba_tran)

#define	TRAN2ISP(tran)		((struct isp *)(tran)->tran_hba_private)
#define	SDEV2ISP(sd)		(TRAN2ISP(SDEV2TRAN(sd)))
#define	PKT2ISP(pkt)		(TRAN2ISP(PKT2TRAN(pkt)))
#define	ADDR2ISP(ap)		(TRAN2ISP(ADDR2TRAN(ap)))

#define	CMD2ADDR(cmd)		(&CMD2PKT(cmd)->pkt_address)
#define	CMD2TRAN(cmd)		(CMD2PKT(cmd)->pkt_address.a_hba_tran)
#define	CMD2ISP(cmd)		(TRAN2ISP(CMD2TRAN(cmd)))


/*
 * isp softstate structure
 */

/*
 * deadline slot structure for timeout handling
 */
struct isp_slot {
	struct isp_cmd	*slot_cmd;
	clock_t		slot_deadline;
};


/*
 * Record the reset notification requests from target drivers.
 */
struct isp_reset_notify_entry {
	struct scsi_address		*ap;
	void				(*callback)(caddr_t);
	caddr_t				arg;
	struct isp_reset_notify_entry	*next;
};


struct isp {

	/*
	 * Transport structure for this instance of the hba
	 */
	scsi_hba_tran_t		*isp_tran;

	/*
	 * dev_info_t reference can be found in the transport structure
	 */
	dev_info_t		*isp_dip;

	/*
	 * Interrupt block cookie
	 */
	ddi_iblock_cookie_t	isp_iblock;

	/*
	 * Firmware revision number
	 */
	uint16_t		isp_major_rev;
	uint16_t		isp_minor_rev;

	/*
	 * timeout id
	 */
	int			isp_timeout_id;

	/*
	 * scsi options, scsi_tag_age_limit  per isp
	 */
	int			isp_scsi_options;
	int			isp_target_scsi_options[N_ISP_TARGETS_WIDE];
	int			isp_scsi_tag_age_limit;

	/*
	 * scsi_reset_delay per isp
	 */
	u_int			isp_scsi_reset_delay;

	/*
	 * current host ID
	 */
	uint8_t			isp_initiator_id;

	/*
	 * suspended flag for power management
	 */
	uint8_t			isp_suspended;

	/*
	 * Host adapter capabilities and offset/period values per target
	 */
	uint16_t		isp_cap[N_ISP_TARGETS_WIDE];
	int16_t			isp_synch[N_ISP_TARGETS_WIDE];

	/*
	 * ISP Hardware register pointer.
	 */
	struct ispregs		*isp_reg;

#ifdef	ISP1020
	struct isp_risc_regs	*isp_risc_reg;
#endif

	/*
	 * mbox values are stored here before and after the mbox cmd
	 */
	struct isp_mbox		isp_mbox;

	/*
	 * shutdown flag if things get really confused
	 */
	uint8_t			isp_shutdown;

	/*
	 * Update properties from watchdog
	 */
	uint8_t			isp_need_prop_update;
	uint8_t			isp_update_targets[N_ISP_TARGETS_WIDE];

	/*
	 * request and response queue dvma space
	 */
	caddr_t			isp_cmdarea;
	ddi_dma_cookie_t	isp_dmacookie;
	ddi_dma_handle_t	isp_dmahandle;
	uint32_t		isp_request_dvma,
				isp_response_dvma;
	ddi_acc_handle_t	isp_queue_handle;

	/*
	 * Register access handles
	 */
	ddi_acc_handle_t	isp_reg_handle;
#ifdef	ISP1020
	ddi_acc_handle_t	isp_risc_handle;
#endif

	/*
	 * ISP input request and output response queue pointers
	 * and mutexes protecting request and response queue
	 */
	u_int			isp_queue_space;
	kmutex_t		isp_request_mutex;
	kmutex_t		isp_response_mutex;
	uint16_t		isp_request_in,
				isp_request_out;
	uint16_t		isp_response_in,
				isp_response_out;

	struct isp_request	*isp_request_ptr,
				*isp_request_base;
	struct isp_response	*isp_response_ptr,
				*isp_response_base;

	int			isp_burst_size;
	uint16_t		isp_conf1_burst_flag;

	struct	isp_slot	isp_slots[ISP_MAX_SLOTS];
	uint16_t		isp_free_slot;

	struct isp_reset_notify_entry	*isp_reset_notify_listf;
};

_NOTE(MUTEX_PROTECTS_DATA(isp::isp_request_mutex, isp::isp_queue_space))
_NOTE(MUTEX_PROTECTS_DATA(isp::isp_request_mutex, isp::isp_request_in))
_NOTE(MUTEX_PROTECTS_DATA(isp::isp_request_mutex, isp::isp_request_out))
_NOTE(MUTEX_PROTECTS_DATA(isp::isp_request_mutex, isp::isp_request_ptr))
_NOTE(MUTEX_PROTECTS_DATA(isp::isp_request_mutex, isp::isp_mbox))
_NOTE(MUTEX_PROTECTS_DATA(isp::isp_request_mutex, isp::isp_slots))

_NOTE(MUTEX_PROTECTS_DATA(isp::isp_response_mutex, isp::isp_response_in))
_NOTE(MUTEX_PROTECTS_DATA(isp::isp_response_mutex, isp::isp_response_out))
_NOTE(MUTEX_PROTECTS_DATA(isp::isp_response_mutex, isp::isp_response_ptr))


/*
 * Convenient short-hand macros for reading/writing isp registers
 */
#define	ISP_WRITE_REG(isp, regp, value)					\
	ddi_put16((isp)->isp_reg_handle, (uint16_t *)(regp), (value))

#define	ISP_READ_REG(isp, regp)						\
	ddi_get16((isp)->isp_reg_handle, (uint16_t *)(regp))

/*
 * Convenient short-hand macros for setting/clearing register bits
 */
#define	ISP_SET_REG_BITS(isp, regp, value)				\
	ISP_WRITE_REG((isp), (regp),					\
		(ISP_READ_REG((isp), (regp)) | (value)))

#define	ISP_CLR_REG_BITS(isp, regp, value)				\
	ISP_WRITE_REG((isp), (regp),					\
		(ISP_READ_REG((isp), (regp)) & ~(value)))




#ifdef	ISP1000
/*
 * Hardware access definitions for the ISP 1000 chip
 *
 */

#define	ISP_READ_RISC_ACC(isp)						\
	ISP_READ_REG((isp), &(isp)->isp_reg->isp_risc_acc)

#define	ISP_READ_RISC_R1(isp)						\
	ISP_READ_REG((isp), &(isp)->isp_reg->isp_risc_r1)

#define	ISP_READ_RISC_R2(isp)						\
	ISP_READ_REG((isp), &(isp)->isp_reg->isp_risc_r2)

#define	ISP_READ_RISC_R3(isp)						\
	ISP_READ_REG((isp), &(isp)->isp_reg->isp_risc_r3)

#define	ISP_READ_RISC_R4(isp)						\
	ISP_READ_REG((isp), &(isp)->isp_reg->isp_risc_r4)

#define	ISP_READ_RISC_R5(isp)						\
	ISP_READ_REG((isp), &(isp)->isp_reg->isp_risc_r5)

#define	ISP_READ_RISC_R6(isp)						\
	ISP_READ_REG((isp), &(isp)->isp_reg->isp_risc_r6)

#define	ISP_READ_RISC_R7(isp)						\
	ISP_READ_REG((isp), &(isp)->isp_reg->isp_risc_r7)

#define	ISP_READ_RISC_R8(isp)						\
	ISP_READ_REG((isp), &(isp)->isp_reg->isp_risc_r8)

#define	ISP_READ_RISC_R9(isp)						\
	ISP_READ_REG((isp), &(isp)->isp_reg->isp_risc_r9)

#define	ISP_READ_RISC_R10(isp)						\
	ISP_READ_REG((isp), &(isp)->isp_reg->isp_risc_r10)

#define	ISP_READ_RISC_R11(isp)						\
	ISP_READ_REG((isp), &(isp)->isp_reg->isp_risc_r11)

#define	ISP_READ_RISC_R12(isp)						\
	ISP_READ_REG((isp), &(isp)->isp_reg->isp_risc_r12)

#define	ISP_READ_RISC_R13(isp)						\
	ISP_READ_REG((isp), &(isp)->isp_reg->isp_risc_r13)

#define	ISP_READ_RISC_R14(isp)						\
	ISP_READ_REG((isp), &(isp)->isp_reg->isp_risc_r14)

#define	ISP_READ_RISC_R15(isp)						\
	ISP_READ_REG((isp), &(isp)->isp_reg->isp_risc_r15)

#define	ISP_READ_RISC_PSR(isp)						\
	ISP_READ_REG((isp), &(isp)->isp_reg->isp_risc_psr)

#define	ISP_WRITE_RISC_PSR(isp, value)					\
	ISP_WRITE_REG((isp), &(isp)->isp_reg->isp_risc_psr,		\
		(value))

#define	ISP_READ_RISC_IVR(isp)						\
	ISP_READ_REG((isp), &(isp)->isp_reg->isp_risc_ivr)

#define	ISP_READ_RISC_PCR(isp)						\
	ISP_READ_REG((isp), &(isp)->isp_reg->isp_risc_pcr)

#define	ISP_WRITE_RISC_PCR(isp, value)					\
	ISP_WRITE_REG((isp), &(isp)->isp_reg->isp_risc_pcr,		\
		(value))

#define	ISP_READ_RISC_RAR0(isp)						\
	ISP_READ_REG((isp), &(isp)->isp_reg->isp_risc_rar0)

#define	ISP_READ_RISC_RAR1(isp)						\
	ISP_READ_REG((isp), &(isp)->isp_reg->isp_risc_rar1)

#define	ISP_READ_RISC_LCR(isp)						\
	ISP_READ_REG((isp), &(isp)->isp_reg->isp_risc_lcr)

#define	ISP_READ_RISC_PC(isp)						\
	ISP_READ_REG((isp), &(isp)->isp_reg->isp_risc_pc)

#define	ISP_WRITE_RISC_PC(isp, value)					\
	ISP_WRITE_REG((isp), &(isp)->isp_reg->isp_risc_pc,		\
		(value))

#define	ISP_READ_RISC_MTR(isp)						\
	ISP_READ_REG((isp), &(isp)->isp_reg->isp_risc_mtr)

#define	ISP_WRITE_RISC_MTR(isp, value)					\
	ISP_WRITE_REG((isp), &(isp)->isp_reg->isp_risc_mtr,		\
		(value))

#define	ISP_READ_RISC_EMB(isp)						\
	ISP_READ_REG((isp), &(isp)->isp_reg->isp_risc_emb)

#define	ISP_WRITE_RISC_EMB(isp, value)					\
	ISP_WRITE_REG((isp), &(isp)->isp_reg->isp_risc_emb,		\
		(value))

#define	ISP_READ_RISC_SP(isp)						\
	ISP_READ_REG((isp), &(isp)->isp_reg->isp_risc_sp)

#define	ISP_WRITE_RISC_SP(isp, value)					\
	ISP_WRITE_REG((isp), &(isp)->isp_reg->isp_risc_sp,		\
		(value))

#define	ISP_READ_RISC_HCCR(isp)						\
	ISP_READ_REG((isp), &(isp)->isp_reg->isp_hccr)

#define	ISP_WRITE_RISC_HCCR(isp, value)					\
	ISP_WRITE_REG((isp), &(isp)->isp_reg->isp_hccr,			\
		(value))


#define	ISP_REG_SET_HOST_INT(isp)					\
	ISP_WRITE_REG((isp), &(isp)->isp_reg->isp_hccr,			\
		ISP_HCCR_CMD_SET_HOST_INT)

#define	ISP_REG_GET_HOST_INT(isp)					\
	(ISP_READ_REG((isp), &(isp)->isp_reg->isp_hccr) &		\
		ISP_HCCR_HOST_INT)

#define	ISP_CLEAR_RISC_INT(isp)						\
	ISP_WRITE_REG((isp), &(isp)->isp_reg->isp_hccr,			\
		ISP_HCCR_CMD_CLEAR_RISC_INT)

#define	ISP_REG_GET_RISC_INT(isp)					\
	(ISP_READ_REG((isp), &(isp)->isp_reg->isp_sbus_isr) &		\
		ISP_SBUS_ISR_RISC_INT)

#define	ISP_CLEAR_SEMAPHORE_LOCK(isp)					\
	ISP_CLR_REG_BITS((isp), &(isp)->isp_reg->isp_sbus_sema,		\
		ISP_SBUS_SEMA_LOCK)

#define	ISP_SET_REQUEST_IN(isp)						\
	ISP_WRITE_REG((isp), &(isp)->isp_reg->isp_mailbox4,		\
		(isp)->isp_request_in)

#define	ISP_SET_RESPONSE_OUT(isp)					\
	ISP_WRITE_REG((isp), &(isp)->isp_reg->isp_mailbox5,		\
		isp->isp_response_out)

#define	ISP_GET_REQUEST_OUT(isp)					\
	ISP_READ_REG((isp), &(isp)->isp_reg->isp_mailbox4)

#define	ISP_GET_RESPONSE_IN(isp)					\
	ISP_READ_REG((isp), &(isp)->isp_reg->isp_mailbox5)

#define	ISP_INT_PENDING(isp)						\
	(ISP_READ_REG((isp), &(isp)->isp_reg->isp_sbus_isr) &		\
		ISP_SBUS_ISR_RISC_INT)

#define	ISP_CHECK_SEMAPHORE_LOCK(isp)					\
	(ISP_READ_REG((isp), &(isp)->isp_reg->isp_sbus_sema) &		\
		ISP_SBUS_SEMA_LOCK)

#endif /* ISP1000 */


#ifdef ISP1020

/*
 * Hardware access definitions for the ISP 1020 chip
 *
 */


#define	ISP_WRITE_RISC_REG(isp, regp, value)				\
	ddi_put16((isp)->isp_risc_handle, (uint16_t *)(regp), (value))

#define	ISP_READ_RISC_REG(isp, regp)					\
	ddi_get16((isp)->isp_risc_handle, (uint16_t *)(regp))

#define	ISP_READ_RISC_ACC(isp)						\
	ISP_READ_RISC_REG((isp), &(isp)->isp_risc_reg->isp_risc_acc)

#define	ISP_READ_RISC_R1(isp)						\
	ISP_READ_RISC_REG((isp), &(isp)->isp_risc_reg->isp_risc_r1)

#define	ISP_READ_RISC_R2(isp)						\
	ISP_READ_RISC_REG((isp), &(isp)->isp_risc_reg->isp_risc_r2)

#define	ISP_READ_RISC_R3(isp)						\
	ISP_READ_RISC_REG((isp), &(isp)->isp_risc_reg->isp_risc_r3)

#define	ISP_READ_RISC_R4(isp)						\
	ISP_READ_RISC_REG((isp), &(isp)->isp_risc_reg->isp_risc_r4)

#define	ISP_READ_RISC_R5(isp)						\
	ISP_READ_RISC_REG((isp), &(isp)->isp_risc_reg->isp_risc_r5)

#define	ISP_READ_RISC_R6(isp)						\
	ISP_READ_RISC_REG((isp), &(isp)->isp_risc_reg->isp_risc_r6)

#define	ISP_READ_RISC_R7(isp)						\
	ISP_READ_RISC_REG((isp), &(isp)->isp_risc_reg->isp_risc_r7)

#define	ISP_READ_RISC_R8(isp)						\
	ISP_READ_RISC_REG((isp), &(isp)->isp_risc_reg->isp_risc_r8)

#define	ISP_READ_RISC_R9(isp)						\
	ISP_READ_RISC_REG((isp), &(isp)->isp_risc_reg->isp_risc_r9)

#define	ISP_READ_RISC_R10(isp)						\
	ISP_READ_RISC_REG((isp), &(isp)->isp_risc_reg->isp_risc_r10)

#define	ISP_READ_RISC_R11(isp)						\
	ISP_READ_RISC_REG((isp), &(isp)->isp_risc_reg->isp_risc_r11)

#define	ISP_READ_RISC_R12(isp)						\
	ISP_READ_RISC_REG((isp), &(isp)->isp_risc_reg->isp_risc_r12)

#define	ISP_READ_RISC_R13(isp)						\
	ISP_READ_RISC_REG((isp), &(isp)->isp_risc_reg->isp_risc_r13)

#define	ISP_READ_RISC_R14(isp)						\
	ISP_READ_RISC_REG((isp), &(isp)->isp_risc_reg->isp_risc_r14)

#define	ISP_READ_RISC_R15(isp)						\
	ISP_READ_RISC_REG((isp), &(isp)->isp_risc_reg->isp_risc_r15)

#define	ISP_READ_RISC_PSR(isp)						\
	ISP_READ_RISC_REG((isp), &(isp)->isp_risc_reg->isp_risc_psr)

#define	ISP_WRITE_RISC_PSR(isp, value)					\
	ISP_WRITE_RISC_REG((isp), &(isp)->isp_risc_reg->isp_risc_psr,	\
		(value))

#define	ISP_READ_RISC_IVR(isp)						\
	ISP_READ_RISC_REG((isp), &(isp)->isp_risc_reg->isp_risc_ivr)

#define	ISP_READ_RISC_PCR(isp)						\
	ISP_READ_RISC_REG((isp), &(isp)->isp_risc_reg->isp_risc_pcr)

#define	ISP_WRITE_RISC_PCR(isp, value)					\
	ISP_WRITE_RISC_REG((isp), &(isp)->isp_risc_reg->isp_risc_pcr,	\
		(value))

#define	ISP_READ_RISC_RAR0(isp)						\
	ISP_READ_RISC_REG((isp), &(isp)->isp_risc_reg->isp_risc_rar0)

#define	ISP_READ_RISC_RAR1(isp)						\
	ISP_READ_RISC_REG((isp), &(isp)->isp_risc_reg->isp_risc_rar1)

#define	ISP_READ_RISC_LCR(isp)						\
	ISP_READ_RISC_REG((isp), &(isp)->isp_risc_reg->isp_risc_lcr)

#define	ISP_READ_RISC_PC(isp)						\
	ISP_READ_RISC_REG((isp), &(isp)->isp_risc_reg->isp_risc_pc)

#define	ISP_WRITE_RISC_PC(isp, value)					\
	ISP_WRITE_RISC_REG((isp), &(isp)->isp_risc_reg->isp_risc_pc,	\
		(value))

#define	ISP_READ_RISC_MTR(isp)						\
	ISP_READ_RISC_REG((isp), &(isp)->isp_risc_reg->isp_risc_mtr)

#define	ISP_WRITE_RISC_MTR(isp, value)					\
	ISP_WRITE_RISC_REG((isp), &(isp)->isp_risc_reg->isp_risc_mtr,	\
		(value))

#define	ISP_READ_RISC_EMB(isp)						\
	ISP_READ_RISC_REG((isp), &(isp)->isp_risc_reg->isp_risc_emb)

#define	ISP_WRITE_RISC_EMB(isp, value)					\
	ISP_WRITE_RISC_REG((isp), &(isp)->isp_risc_reg->isp_risc_emb,	\
		(value))

#define	ISP_READ_RISC_SP(isp)						\
	ISP_READ_RISC_REG((isp), &(isp)->isp_risc_reg->isp_risc_sp)

#define	ISP_WRITE_RISC_SP(isp, value)					\
	ISP_WRITE_RISC_REG((isp), &(isp)->isp_risc_reg->isp_risc_sp,	\
		(value))

#define	ISP_READ_RISC_HCCR(isp)						\
	ISP_READ_RISC_REG((isp), &(isp)->isp_risc_reg->isp_hccr)

#define	ISP_WRITE_RISC_HCCR(isp, value)					\
	ISP_WRITE_RISC_REG((isp), &(isp)->isp_risc_reg->isp_hccr,	\
		(value))


#define	ISP_REG_SET_HOST_INT(isp)					\
	ISP_WRITE_RISC_REG((isp), &(isp)->isp_risc_reg->isp_hccr,	\
		ISP_HCCR_CMD_SET_HOST_INT)

#define	ISP_REG_GET_HOST_INT(isp)					\
	(ISP_READ_RISC_REG((isp), &(isp)->isp_risc_reg->isp_hccr) &	\
		ISP_HCCR_HOST_INT)

#define	ISP_CLEAR_RISC_INT(isp)						\
	ISP_WRITE_RISC_REG((isp), &(isp)->isp_risc_reg->isp_hccr,	\
		ISP_HCCR_CMD_CLEAR_RISC_INT)

#define	ISP_REG_GET_RISC_INT(isp)					\
	(ISP_READ_REG((isp), &(isp)->isp_reg->isp_sbus_isr) &		\
		ISP_SBUS_ISR_RISC_INT)

#define	ISP_CLEAR_SEMAPHORE_LOCK(isp)					\
	ISP_CLR_REG_BITS((isp), &(isp)->isp_reg->isp_sbus_sema,		\
		ISP_SBUS_SEMA_LOCK)

#define	ISP_SET_REQUEST_IN(isp)						\
	ISP_WRITE_REG((isp), &(isp)->isp_reg->isp_mailbox4,		\
		(isp)->isp_request_in)

#define	ISP_SET_RESPONSE_OUT(isp)					\
	ISP_WRITE_REG((isp), &(isp)->isp_reg->isp_mailbox5,		\
		isp->isp_response_out)

#define	ISP_GET_REQUEST_OUT(isp)					\
	ISP_READ_REG((isp), &(isp)->isp_reg->isp_mailbox4)

#define	ISP_GET_RESPONSE_IN(isp)					\
	ISP_READ_REG((isp), &(isp)->isp_reg->isp_mailbox5)

#define	ISP_INT_PENDING(isp)						\
	(ISP_READ_REG((isp), &(isp)->isp_reg->isp_sbus_isr) &		\
		ISP_SBUS_ISR_RISC_INT)

#define	ISP_CHECK_SEMAPHORE_LOCK(isp)					\
	(ISP_READ_REG((isp), &(isp)->isp_reg->isp_sbus_sema) &		\
		ISP_SBUS_SEMA_LOCK)

#endif /* ISP1020 */



#ifdef	__cplusplus
}
#endif

#endif	/* _SYS_SCSI_ADAPTERS_ISPVAR_H */
