/*
 * Copyright (c) 1996,1997 by Sun Microsystems, Inc.
 * All rights reserved.
 */

#ifndef	_PCEPP_H
#define	_PCEPP_H

#pragma ident   "@(#)pcepp.h	1.19    97/09/09 SMI"

#ifdef	__cplusplus
extern "C" {
#endif

/*
 *  Solaris PCMCIA Parallel Port Driver
 *
 */


/*
 * Values for CS_EVENT_CLIENT_INFO event handler
 */
#define	PCEPP_CLIENT_DESCRIPTION	"PCMCIA DDK parallel driver"
					/*
					 * This should be replaced with
					 *	your company name
					 */
#define	PCEPP_VENDOR_DESCRIPTION	"DDK Sample Driver, Sun Microsystems, Inc. (c) 1996"
#define	PCEPP_REV_LEVEL			0x100
#define	PCEPP_REV_DAY			15
#define	PCEPP_REV_MONTH			8
#define	PCEPP_REV_YEAR			16
#define	PCEPP_REV_DATE			CS_CLIENT_INFO_MAKE_DATE( \
						PCEPP_REV_DAY, \
						PCEPP_REV_MONTH, \
						PCEPP_REV_YEAR)


/* Driver name */
#define	PCEPP_NAME		"pcepp"

/* Driver ID string */
#define	PCEPP_DRIVERID		"DDK PCMCIA Parallel Driver"

/* Return socket number information */
#define	PCEPP_SOCKET(dev)	(getminor(dev) & 0x3f)

/* Ready timeout */
#define	PCEPP_READY_TIMEOUT	10


/*
 * Device type define which are used by the 'node_type'
 *	element of the ddi_minor_data structure
 */
#define	PCEPP_NT_PARALLEL	"ddi_parallel"

/*
 * Driver state flags
 */
				/* RegisterCLient is OK */
#define	PCEPP_REGCLIENT		0x00000001
				/* added to interrupt chain */
#define	PCEPP_SOFTINTROK	0x00000002
				/* Done init mutex */
#define	PCEPP_DID_INITMUTEX	0x00000004
				/* RequestSocketMask is OK */
#define	PCEPP_REQSOCKMASK	0x00000008
				/* did RequestIO */
#define	PCEPP_REQUESTIO		0x00000010
				/* did RequestIRQ */
#define	PCEPP_REQUESTIRQ	0x00000020
				/* did RequestConfiguration */
#define	PCEPP_REQUESTCONFIG	0x00000040
				/* did MakeDeviceNode */
#define	PCEPP_MAKEDEVICENODE	0x00000080
				/* open is complete */
#define	PCEPP_ISOPEN		0x00000100

/*
 * Card event flags
 */
#define	PCEPP_CARD_READY	0x01	/* Card Ready and initialized */
#define	PCEPP_CARD_WAIT_READY	0x02	/* Waiting for Card Ready event */
#define	PCEPP_CARD_BUSY		0x04	/* card is busy */
#define	PCEPP_CARD_INSERTED	0x08	/* card is inserted */
#define	PCEPP_CARD_ERROR	0x10	/* error initializing card */

/*
 * PC Card present macro
 */
#define	PCEPP_CARD_IS_READY(pm)						\
	(((pm)->card_event & (PCEPP_CARD_READY | PCEPP_CARD_INSERTED |	\
	PCEPP_CARD_ERROR)) == (PCEPP_CARD_READY | PCEPP_CARD_INSERTED))

/*
 * Transmit Status
 */
#define	PCEPP_XMIT_BUSY		0x00000001
#define	PCEPP_XMIT_SUCCESS	0x00000002

/*
 * Soft interrupt level
 *	The driver uses a two-level interrupt scheme;
 *	the hardware interrupts at a high level, and the
 *	driver schedules a softint on a lower level to deal
 *	with the STREAMS processing.
 */
#define	PCEPP_SOFT_PREF	DDI_SOFTINT_MED


/*
 * Extended Control Register base
 */
#define	PCEPP_IOBASE_ECR	0x402


/*
 * STREAMS Misc.
 */
#define	PCEPP_IDNUM	0	/* should be a unique id; zero works */
#define	PCEPP_HIWAT	8192	/* driver flow control high water */
#define	PCEPP_LOWAT	1024	/* driver flow control low water */


/*
 * Mutex related macros
 */
#define	PCEPP_HIMUTEX_ENTER(pps)			\
		mutex_enter(&(pps)->event_hi_mutex);	\
		mutex_enter(&(pps)->irq_mutex);

#define	PCEPP_HIMUTEX_EXIT(pps)				\
		mutex_exit(&(pps)->irq_mutex);		\
		mutex_exit(&(pps)->event_hi_mutex);



/*
 * Device Control Register defines
 */
#define	PCEPP_ECR_BI_DIRECT	0x20
#define	PCEPP_ECR_INTR_ENB	0x10
#define	PCEPP_ECR_SEL_PRT	0x08
#define	PCEPP_ECR_INIT_PRT	0x04
#define	PCEPP_ECR_AUTOFD_ENB	0x02
#define	PCEPP_ECR_STROBE_ENB	0x01


/*
 * Device Status Register defines
 */
#define	PCEPP_ESR_PRN_READY	0x80	/* Printer is not BUSY */
#define	PCEPP_ESR_ACK		0x40	/* READY */
#define	PCEPP_ESR_PERROR	0x20	/* NOPAPER */
#define	PCEPP_ESR_SELECT	0x10	/* Printer is ONLINE */
#define	PCEPP_ESR_FAULT		0x08	/* no ERROR detected */
#define	PCEPP_ESR_STATUS_3	0x04
#define	PCEPP_ESR_STATUS_2	0x02
#define	PCEPP_ESR_EPP_TO	0x01


/*
 * Register offset definitions
 */
#define	PCEPP_REGS_DATA		0x00	/* Data Register */
#define	PCEPP_REGS_DSR		0x01	/* Device Status Register */
#define	PCEPP_REGS_DCR		0x02	/* Device Control Register */
#define	PCEPP_REGS_EAR		0x03	/* EPP Address Register */
#define	PCEPP_REGS_EDR0		0x04	/* EPP Data Register 0 */
#define	PCEPP_REGS_EDR1		0x05	/* EPP Data Register 1 */
#define	PCEPP_REGS_EDR2		0x06	/* EPP Data Register 2 */
#define	PCEPP_REGS_EDR3		0x07	/* EPP Data Register 3 */


/*
 * pcepp_cftable_t and pcepp_cftable_params_t structures are used
 *	to store values from the CISTPL_CFTABLE_ENTRY tuples.
 */
typedef struct pcepp_cftable_params_t {
	u_char		config_index;
	u_long		addr_lines;	/* IO addr lines decoded */
	u_long		pin;		/* PRR bits valid mask */
	unsigned	card_vcc;
	unsigned	card_vpp1;
	unsigned	card_vpp2;

	u_long		card_base1;	/* base of I/O register */
	u_long		card_base2;	/* base of I/O register */
	u_long		length1;	/* length of IO range */
	u_long		length2;	/* length of IO range */

} pcepp_cftable_params_t;

typedef struct pcepp_cftable_t {
				/* desireability factor */
	u_long			desireability;
	pcepp_cftable_params_t	p;
	struct pcepp_cftable_t	*prev;
	struct pcepp_cftable_t	*next;
} pcepp_cftable_t;

/*
 * pcepp_cis_addr_tran_t
 *	This structure is used to provide a mapping
 *	between a base address and a desierability factor
 *	for the CISTPL_CFTABLE_ENTRY tuple processing.
 */
typedef struct pcepp_cis_addr_tran_t {
			/* how desireable this entry is */
	u_long		desireability;
			/* base offset of PC card register */
	u_long		card_base1;
} pcepp_cis_addr_tran_t;

/*
 * We generally don't want to use the standard MS-DOS/x86
 *	COMx or LPTx addresses if we don't have to -
 *	this hopefully prevents conflicts between PCMCIA
 *	I/O devices and build-in I/O devices.
 *
 *	This is the desrireability order since we would
 *	like to configure the LPT port with Enhanced
 *	Parallel Port (EPP) mode.
 *
 *	Desireability              Mode
 *	--------------------------------------------
 *	              SPP	Bi-direct	EPP
 *	0x278            x          x            x
 *	0x378            x          x            x
 *	0x000            x
 */
pcepp_cis_addr_tran_t pcepp_cis_addr_tran[] = {
	/*   desireability    card_base1 */
	{	0x0fffe,	0x278	},	/* LPT2 */
	{	0x0ffff,	0x378	},	/* LPT1 */
	{	0x0fffd,	0x000	},
};


/*
 * pcepp_cis_vars_t structure
 *	used to save interesting information from the CIS
 */
#define	PCEPP_PS_MAX	CISTPL_VERS_1_MAX_PROD_STRINGS

typedef struct pcepp_cis_vars_t {
	u_long		flags;		/* general capability flags */

			/* CISTPL_CONFIG */
	u_long		config_base;	/* base offset of config reg */
	u_long		present;	/* config reg present flags */

			/* CISTPL_VERS_1 */
	u_long		major_revision;
	u_long		minor_revision;
	int		nstring;
	char		prod_strings[PCEPP_PS_MAX][CIS_MAX_TUPLE_DATA_LEN];

			/* CISTPL_MANFID */
	u_long		manufacturer_id;
	u_long		card_id;

			/* CISTPL_CFTABLE_ENTRY */
	u_char		config_index;
	u_long		addr_lines;	/* IO addr lines decoded */
	u_long		pin;		/* PRR bits valid mask */
	unsigned	card_vcc;
	unsigned	card_vpp1;
	unsigned	card_vpp2;

			/*
			 * To support BasePort1/NumPorts1
			 *	and BasePort2/NumPorts2
			 */
	u_long		card_base1;	/* base of I/O register */
	u_long		card_base2;	/* base of I/O register */
	u_long		length1;	/* length of IO range */
	u_long		length2;	/* length of IO range */

	pcepp_cftable_t	cftable;	/* CFTABLE_ENTRY table */
	int		n_cistpl_cftable_entry;

			/* CISTPL_FUNCID and CISTPL_FUNCE */
	int		n_cistpl_funcid;

} pcepp_cis_vars_t;


/*
 * Driver state structure
 */
typedef struct pcepp_state_t {
	kmutex_t		event_hi_mutex;	/* hi-priority mutex */
	kmutex_t		event_mutex;	/* low-priority mutex */
	kmutex_t		irq_mutex;

	ddi_iblock_cookie_t	softint_cookie;	/* soft intr cookie */
	ddi_softintr_t		softint_id;

				/* CS client handle */
	client_handle_t		client_handle;
				/* various state flags */
	unsigned		flags;
				/* Device dev_info_t */
	dev_info_t		*dip;
				/* card ready timeout ID */
	int			ready_timeout_id;
				/* dip instance */
	int			instance;
				/* Logical socket number */
	int			sn;
				/* recent card event */
	u_long			card_event;
				/* save information from the CIS */
	pcepp_cis_vars_t	cis_vars;

				/* card register handle */
	acc_handle_t		card_handle;
	acc_handle_t		card_handle_ecr;

				/* STREAMS */
				/* back pointer to write queue */
	queue_t			*rdq_ptr;
				/* back pointer to write queue */
	queue_t			*wrq_ptr;
				/* current message being output */
	mblk_t			*msg;

				/* soft tx buffer */
	u_char			pcepp_txdata;

				/* tty struct */
	struct strtty		pcepp_tty;

				/* ready wait */
	kcondvar_t		readywait_cv;
} pcepp_state_t;

#ifdef	__cplusplus
}
#endif

#endif /* _PCEPP_H */
