/*
 * Copyright (c) 1996,1997 by Sun Microsystems, Inc.
 * All rights reserved.
 */

#ifndef _PCSRAMIO_H
#define	_PCSRAMIO_H

#pragma ident	"@(#)pcsram.h	1.12	97/04/07 SMI"

#ifdef	__cplusplus
extern "C" {
#endif

/*
 * Debug message printing
 */
#ifdef	DEBUG
static int pcsram_debug = 0;
#define	PCSRAM_DEBUG(args)	if (pcsram_debug) cmn_err args
#else
#define	PCSRAM_DEBUG(args)
#endif

/*
 * Values for CS_EVENT_CLIENT_INFO event handler
 */
#define	PCSRAM_CLIENT_DESCRIPTION	"PCMCIA DDK memory card driver"
					/*
					 * This should be replaced with
					 *	your company name
					 */
#define	PCSRAM_VENDOR_DESCRIPTION	"DDK Sample Driver, Sun Microsystems, Inc. (c) 1996"
#define	PCSRAM_REV_LEVEL		0x100
#define	PCSRAM_REV_DAY			15
#define	PCSRAM_REV_MONTH		8
#define	PCSRAM_REV_YEAR			16
#define	PCSRAM_REV_DATE			CS_CLIENT_INFO_MAKE_DATE( \
						PCSRAM_REV_DAY, \
						PCSRAM_REV_MONTH, \
						PCSRAM_REV_YEAR)

/*
 * PC Card present macro
 */
#define	PCSRAM_CARD_PRESENT(rs)		((rs)->card_event & \
						PCSRAM_CARD_IS_READY)

/*
 * driver ID string
 */
#define	PCSRAM_DRIVERID		"DDK PCMCIA Memory Driver"
#define	PCSRAM_NAME		"pcsram"

/*
 *  Mutex and condition variables
 */
#define	MUTEX_CONTROLER_NAME	"pcsram_state mutex"
#define	CV_SOFT_NAME		"pcsram_state condvar"


/*
 * Minor device number encoding:
 *
 *      s s s s | s s p p | p p p p | p p p p
 *
 *      s - socket number of this device
 *      p - partition number of this device
 *
 */
#define	PCSRAM_SOCKET(dev)		((getminor(dev) >> 10) & 0x3f)
#define	PCSRAM_PARTTION(dev)		(getminor(dev) & 0x3ff)
#define	PCSRAM_SETMINOR(skt, part)	((skt<<10) | (part))

/*
 * physical I/O
 */
#define	DEFAULT_MAXPHYS		(63 * 1024)	/* '126b' */

/*
 * flags in pcsram_state.flags field
 */
				/* added to interrupt chain */
#define	PCSRAM_SOFTINTROK	0x00000001
				/* RegisterCLient is OK */
#define	PCSRAM_REGCLIENT	0x00000002
				/* cv/mutex_init in attach */
#define	PCSRAM_DID_INITMUTEX	0x00000004
				/* RequestSocketMask is OK */
#define	PCSRAM_REQSOCKMASK	0x00000008
				/* we have a valid window */
#define	PCSRAM_HAS_WINDOW	0x00000010
				/* MakeDeviceNode is OK */
#define	PCSRAM_MAKEDEVICENODE	0x00000020

/*
 * Flags used for pcsram_state.card_event
 */
				/* card is fully functional */
#define	PCSRAM_CARD_IS_READY	0x00000001

/*
 * Drive characteristic conversion macros
 *	cs   - total card size
 *	hd   - number of head
 *	ss   - sector size
 *	st   - sector per track
 */
#define	PCSRAM_GET_NCYL(cs, hd, ss, st)		(cs/(hd*ss*st))

/*
 * PCMCIA SRAM memory card characteristic structure
 */
struct hd_char {
	int drv_ncyl;		/* number of cylinders */
	int drv_nhead;		/* number of heads */
	int drv_secptrack;	/* sectors per track */
	int drv_sec_size;	/* sector size */
};


/*
 * Configure number of cylinder so we
 * can get even total sectors on memory card
 *
 *	ncyl -  number_of_cylinder
 *	nhd  -  number_of_head
 *	spt  -  sector_per_track
 *	bps  -  byte_per_sector
 *
 *      card_size = ncyl * nhd * spt * bps
 *
 *      card_size       ncyl   nhd     spt      byte/sect
 *      --------------------------------------------------
 *      524288          64      2       8          512
 *      1048576         128     2       8          512
 *      2097152         256     2       8          512
 *         -             -      -       -           -
 *      67108864        8192    2       8          512
 *
 */
static struct hd_char hdtypes = {
		/* setup for maximum 64MB card */
	8192,	/* number of cylinders (64 to 8192) */
	2,	/* number of heads (fixed) */
	8,	/* sectors per track (fixed) */
	512	/* sector size (fixed) */
};



/*
 * This state structure contains:
 *      a pointer to the dev_info node for this instance
 *      mutexes
 *      condition variables
 */
typedef struct pcsram_state_t {
				/* various board-level flags */
	unsigned		flags;
				/* soft interrupt cookie */
	ddi_iblock_cookie_t	soft_blk_cookie;
				/* softint identifier */
	ddi_softintr_t		softint_id;
				/* Device dev_info_t */
	dev_info_t		*dip;
				/* dip instance */
	int			instance;

				/* CardServices stuff */
				/* CS client handle */
	client_handle_t		client_handle;
				/* protects hilevel events */
	kmutex_t		event_hilock;
				/* window stuff */
	window_handle_t		window_handle;
				/*  window size */
	u_long			win_size;

				/* low-priority mutex */
	kmutex_t		mutex;

				/* block/character device open */
	int			blk_open;
	int			chr_open;
				/* count of layered opens */
	int			nlayered;
				/*
				 * device active, cannot be unloaded
				 * (xx_strategy)
				 */
	int			busy;
				/*
				 * device active, cannot be unloaded
				 * (xx_write)
				 */
	int			busy_wr;
	int			busy_rd;

				/* recent card event */
	u_long			card_event;

				/* transfer list */
	struct	buf		*blist;
				/* Packed label for this unit */
	struct	dk_label	un_label;

				/* posting battery condition */
	int			battery_dead_posted;
	int			battery_low_posted;
				/*  Card Eject Posted Flag  */
	int			card_eject_posted;
				/*  ejected while mouting Flag  */
	int			ejected_while_mounting;

				/* pointer to hard drive */
				/* characteristics */
	struct	hd_char		*hdrv_chars;
				/* Memory card size */
	int			card_size;
				/* access handle for base of window */
	acc_handle_t		access_handle;

	int			sn;	/* Logical socket number */
				/* Got card size flag */
	int			got_cardsize_flag;

				/* ready wait */
	kcondvar_t		readywait_cv;

				/* physical I/O transfer size */
	int			maxphys;
} pcsram_state_t;

#ifdef	__cplusplus
}
#endif

#endif /* _PCSRAMIO_H */
