/*
 * Copyright 1993,1997 by Sun Microsystems, Inc.
 * All rights reserved.
 */

#ifndef	_PVIPREG_H
#define	_PVIPREG_H

#pragma ident	"@(#)pvipreg.h	1.15	97/04/07 SMI"

/*
 * The following describes the structure of the Weitek Power 9100
 * User Interface Controller.
 *
 * Some of the addresses of the P9100 depend on the byte and bit
 * ordering of the host.  The values _LITTLE_ENDIAN and _BIG_ENDIAN
 * from isa_defs.h should set the right addresses.  The endianess
 * can be forced by setting PVIP_LITTLE_ENDIAN to 0 or 1.  The
 * values PVIP_ENDIAN* can be set to a constant if they are wrong
 * for your machine.  If its desired to determine this at runtime,
 * set the #define	PVIP_ENDIAN* to the runtime variable containing
 * a 0-7 for the value that is in bits 16-18 of the address
 * (PVIP_ADDR_SWAP_*).
 */

#include <sys/types.h>
#include <sys/isa_defs.h>

#ifdef	__cplusplus
extern "C" {
#endif

#define	PVIP_NUM_PAT	8	/* Number of pattern registers */

typedef struct pvip_control_regs {

	/* System Control Registers */

	uint32_t	: 32;				/* 0x000z0000    */
	uint32_t	pvip_cr_sysconfig;		/* 0x000z0004 rw */
	uint32_t	pvip_cr_interrupt;		/* 0x000z0008 rw */
	uint32_t	pvip_cr_interrupt_en;		/* 0x000z000c rw */
	uint32_t	pvip_cr_alt_write_bank;		/* 0x000z0010 rw */
	uint32_t	pvip_cr_alt_read_bank;		/* 0x000z0014 rw */
	uint32_t	pvip_cr_filler1[0x40 - 0x6];

	/* Video Control Registers */

	uint32_t	pvip_cr_filler2;		/* 0x000z0100 hg */
	uint32_t	pvip_cr_hrzc;			/* 0x000z0104 ro */
	uint32_t	pvip_cr_hrzt;			/* 0x000z0108 rw */
	uint32_t	pvip_cr_hrzsr;			/* 0x000z010c rw */
	uint32_t	pvip_cr_hrzbr;			/* 0x000z0110 rw */
	uint32_t	pvip_cr_hrzbf;			/* 0x000z0114 rw */
	uint32_t	pvip_cr_prehrzc;		/* 0x000z0118 rw */
	uint32_t	pvip_cr_vrtc;			/* 0x000z011c ro */
	uint32_t	pvip_cr_vrtt;			/* 0x000z0120 rw */
	uint32_t	pvip_cr_vrtsr;			/* 0x000z0124 rw */
	uint32_t	pvip_cr_vrtbr;			/* 0x000z0128 rw */
	uint32_t	pvip_cr_vrtbf;			/* 0x000z012c rw */
	uint32_t	pvip_cr_prevrtc;		/* 0x000z0130 rw */
	uint32_t	pvip_cr_sraddr;			/* 0x000z0134 ro */
	uint32_t	pvip_cr_srtctl;			/* 0x000z0138 rw */
	uint32_t	pvip_cr_qsfcounter;		/* 0x000z013c ro */
	uint32_t	pvip_cr_srtctl2;		/* 0x000z0140 rw */
	uint32_t	pvip_cr_filler3[0x20 - 0x11];

	/* Vram Control Registers */

	uint32_t	: 32;				/* 0x000z0180    */
	uint32_t	pvip_cr_mem_config;		/* 0x000z0184 rw */
	uint32_t	pvip_cr_rfperiod;		/* 0x000z0188 rw */
	uint32_t	pvip_cr_rfcount;		/* 0x000z018c ro */
	uint32_t	pvip_cr_rlmax;			/* 0x000z0190 rw */
	uint32_t	pvip_cr_rlcur;			/* 0x000z0194 ro */
	uint32_t	pvip_cr_pu_config;		/* 0x000z0198 ro */
	uint32_t	pvip_cr_filler4[0x20 - 0x7];

	/* Ramdac Control Registers */

	uint32_t	pvip_cr_ramdac[16];		/* 0x000z0200 rw */

} pvip_control_regs_t;				/* 0x000s0000    */


typedef struct pvip_drawing_regs  {

	/* Drawing Control Registers */

	uint32_t	pvip_dr_status;			/* 0x000s0000 ro */
	uint32_t	pvip_dr_blit;			/* 0x000s0004 ro */
	uint32_t	pvip_dr_quad;			/* 0x000s0008 ro */
	uint32_t	pvip_dr_pixel8;			/* 0x000s000c wo */
	uint32_t	: 32;
	uint32_t	pvip_dr_next_pixels;		/* 0x000s0014 wo */
	uint32_t	pvip_dr_filler6[0x1a];
	uint32_t	pvip_dr_pixel1[32];		/* 0x000s0080 wo */
	uint32_t	pvip_dr_filler7[0x20];

	/* Parameter Engine Control Registers */

	uint32_t	: 32;				/* 0x000s0180    */
	uint32_t	pvip_dr_oor;			/* 0x000s0184 ro */
	uint32_t	: 32;				/* 0x000s0188    */
	uint32_t	pvip_dr_cindex;			/* 0x000s018c rw */
	int32_t		pvip_dr_w_off_xy;		/* 0x000s0190 rw */
	int32_t		pvip_dr_pe_w_min;		/* 0x000s0194 ro */
	int32_t		pvip_dr_pe_w_max;		/* 0x000s0198 ro */
	uint32_t	: 32;				/* 0x000s019c    */
	uint32_t	pvip_dr_yclip;			/* 0x000s01a0 ro */
	uint32_t	pvip_dr_xclip;			/* 0x000s01a4 ro */
	uint32_t	pvip_dr_xedge_lt;		/* 0x000s01a8 ro */
	uint32_t	pvip_dr_xedge_gt;		/* 0x000s01ac ro */
	uint32_t	pvip_dr_yedge_lt;		/* 0x000s01b0 ro */
	uint32_t	pvip_dr_yedge_gt;		/* 0x000s01b4 ro */
	uint32_t	pvip_dr_filler8[0x20 - 0xe];

	/* Drawing Engine Pixel Processing Registers */

	uint32_t	pvip_dr_fground;		/* 0x000s0200 rw */
	uint32_t	pvip_dr_bground;		/* 0x000s0204 rw */
	uint32_t	pvip_dr_pmask;			/* 0x000s0208 rw */
	uint32_t	pvip_dr_draw_mode;		/* 0x000s020c rw */
	int32_t		pvip_dr_pat_originx;		/* 0x000s0210 rw */
	int32_t		pvip_dr_pat_originy;		/* 0x000s0214 rw */
	uint32_t	pvip_dr_raster;			/* 0x000s0218 rw */
	uint32_t	pvip_dr_pixel8_reg;		/* 0x000s021c rw */
	int32_t		pvip_dr_w_min;			/* 0x000s0220 rw */
	int32_t		pvip_dr_w_max;			/* 0x000s0224 rw */
	uint32_t	pvip_dr_filler9[0xe - 0xa];
	uint32_t	pvip_dr_color2;			/* 0x000s0238 rw */
	uint32_t	pvip_dr_color3;			/* 0x000s023c rw */
	uint32_t	pvip_dr_filler9a[0x20 - 0x10];
	uint32_t	pvip_dr_pattern[PVIP_NUM_PAT];	/* 0x000s0280 rw */
	uint32_t	pvip_dr_byte_w_min;		/* 0x000s02a0 rw */
	uint32_t	pvip_dr_byte_w_max;		/* 0x000s02a4 rw */
	uint32_t	pvip_dr_filler10[0x380 - 0x2a];

	/* Parameter Engine Coordinate Registers */

	int32_t		: 32;				/* 0x000s1000    */
	int32_t		: 32;				/* 0x000s1004    */
	int32_t		pvip_dr_x0;			/* 0x000s1008 rw */
	int32_t		: 32;				/* 0x000s100c    */
	int32_t		pvip_dr_y0;			/* 0x000s1010 rw */
	int32_t		: 32;				/* 0x000s1014    */
	int32_t		pvip_dr_xy0;			/* 0x000s1018 rw */
	int32_t		: 32;				/* 0x000s101c    */
	int32_t		: 32;				/* 0x000s1020    */
	int32_t		: 32;				/* 0x000s1024    */
	int32_t		pvip_dr_rel_x0;			/* 0x000s1028 wo */
	int32_t		: 32;				/* 0x000s102c    */
	int32_t		pvip_dr_rel_y0;			/* 0x000s1030 wo */
	int32_t		: 32;				/* 0x000s1034    */
	int32_t		pvip_dr_rel_xy0;		/* 0x000s1038 wo */
	int32_t		: 32;				/* 0x000s103c    */
	int32_t		: 32;				/* 0x000s1040    */
	int32_t		: 32;				/* 0x000s1044    */
	int32_t		pvip_dr_x1;			/* 0x000s1048 rw */
	int32_t		: 32;				/* 0x000s104c    */
	int32_t		pvip_dr_y1;			/* 0x000s1050 rw */
	int32_t		: 32;				/* 0x000s1054    */
	int32_t		pvip_dr_xy1;			/* 0x000s1058 rw */
	int32_t		: 32;				/* 0x000s105c    */
	int32_t		: 32;				/* 0x000s1060    */
	int32_t		: 32;				/* 0x000s1064    */
	int32_t		pvip_dr_rel_x1;			/* 0x000s1068 wo */
	int32_t		: 32;				/* 0x000s106c    */
	int32_t		pvip_dr_rel_y1;			/* 0x000s1070 wo */
	int32_t		: 32;				/* 0x000s1074    */
	int32_t		pvip_dr_rel_xy1;		/* 0x000s1078 wo */
	int32_t		: 32;				/* 0x000s107c    */
	int32_t		: 32;				/* 0x000s1080    */
	int32_t		: 32;				/* 0x000s1084    */
	int32_t		pvip_dr_x2;			/* 0x000s1088 rw */
	int32_t		: 32;				/* 0x000s108c    */
	int32_t		pvip_dr_y2;			/* 0x000s1090 rw */
	int32_t		: 32;				/* 0x000s1094    */
	int32_t		pvip_dr_xy2;			/* 0x000s1098 rw */
	int32_t		: 32;				/* 0x000s109c    */
	int32_t		: 32;				/* 0x000s10a0    */
	int32_t		: 32;				/* 0x000s10a4    */
	int32_t		pvip_dr_rel_x2;			/* 0x000s10a8 wo */
	int32_t		: 32;				/* 0x000s10ac    */
	int32_t		pvip_dr_rel_y2;			/* 0x000s10b0 wo */
	int32_t		: 32;				/* 0x000s10b4    */
	int32_t		pvip_dr_rel_xy2;		/* 0x000s10b8 wo */
	int32_t		: 32;				/* 0x000s10bc    */
	int32_t		: 32;				/* 0x000s10c0    */
	int32_t		: 32;				/* 0x000s10c4    */
	int32_t		pvip_dr_x3;			/* 0x000s10c8 rw */
	int32_t		: 32;				/* 0x000s10cc    */
	int32_t		pvip_dr_y3;			/* 0x000s10d0 rw */
	int32_t		: 32;				/* 0x000s10d4    */
	int32_t		pvip_dr_xy3;			/* 0x000s10d8 rw */
	int32_t		: 32;				/* 0x000s10dc    */
	int32_t		: 32;				/* 0x000s10e0    */
	int32_t		: 32;				/* 0x000s10e4    */
	int32_t		pvip_dr_rel_x3;			/* 0x000s10e8 wo */
	int32_t		: 32;				/* 0x000s10ec    */
	int32_t		pvip_dr_rel_y3;			/* 0x000s10f0 wo */
	int32_t		: 32;				/* 0x000s10f4    */
	int32_t		pvip_dr_rel_xy3;		/* 0x000s10f8 wo */
	int32_t		: 32;				/* 0x000s10fc    */
	int32_t		pvip_dr_filler11[0x40];

	/* Load Coordinate Pseudo Registers */

	int32_t		: 32;				/* 0x000s1200    */
	int32_t		: 32;				/* 0x000s1204    */
	int32_t		pvip_dr_point_x;		/* 0x000s1208 wo */
	int32_t		: 32;				/* 0x000s120c    */
	int32_t		pvip_dr_point_y;		/* 0x000s1210 wo */
	int32_t		: 32;				/* 0x000s1214    */
	int32_t		pvip_dr_point_xy;		/* 0x000s1218 wo */
	int32_t		: 32;				/* 0x000s121c    */
	int32_t		: 32;				/* 0x000s1220    */
	int32_t		: 32;				/* 0x000s1224    */
	int32_t		pvip_dr_point_rel_x;		/* 0x000s1228 wo */
	int32_t		: 32;				/* 0x000s122c    */
	int32_t		pvip_dr_point_rel_y;		/* 0x000s1230 wo */
	int32_t		: 32;				/* 0x000s1234    */
	int32_t		pvip_dr_point_rel_xy;		/* 0x000s1238 wo */
	int32_t		: 32;				/* 0x000s123c    */
	int32_t		: 32;				/* 0x000s1240    */
	int32_t		: 32;				/* 0x000s1244    */
	int32_t		pvip_dr_line_x;			/* 0x000s1248 wo */
	int32_t		: 32;				/* 0x000s124c    */
	int32_t		pvip_dr_line_y;			/* 0x000s1250 wo */
	int32_t		: 32;				/* 0x000s1254    */
	int32_t		pvip_dr_line_xy;		/* 0x000s1258 wo */
	int32_t		: 32;				/* 0x000s125c    */
	int32_t		: 32;				/* 0x000s1260    */
	int32_t		: 32;				/* 0x000s1264    */
	int32_t		pvip_dr_line_rel_x;		/* 0x000s1268 wo */
	int32_t		: 32;				/* 0x000s126c    */
	int32_t		pvip_dr_line_rel_y;		/* 0x000s1270 wo */
	int32_t		: 32;				/* 0x000s1274    */
	int32_t		pvip_dr_line_rel_xy;		/* 0x000s1278 wo */
	int32_t		: 32;				/* 0x000s127c    */
	int32_t		: 32;				/* 0x000s1280    */
	int32_t		: 32;				/* 0x000s1284    */
	int32_t		pvip_dr_tri_x;			/* 0x000s1288 wo */
	int32_t		: 32;				/* 0x000s128c    */
	int32_t		pvip_dr_tri_y;			/* 0x000s1290 wo */
	int32_t		: 32;				/* 0x000s1294    */
	int32_t		pvip_dr_tri_xy;			/* 0x000s1298 wo */
	int32_t		: 32;				/* 0x000s129c    */
	int32_t		: 32;				/* 0x000s12a0    */
	int32_t		: 32;				/* 0x000s12a4    */
	int32_t		pvip_dr_tri_rel_x;		/* 0x000s12a8 wo */
	int32_t		: 32;				/* 0x000s12ac    */
	int32_t		pvip_dr_tri_rel_y;		/* 0x000s12b0 wo */
	int32_t		: 32;				/* 0x000s12b4    */
	int32_t		pvip_dr_tri_rel_xy;		/* 0x000s12b8 wo */
	int32_t		: 32;				/* 0x000s12bc    */
	int32_t		: 32;				/* 0x000s12c0    */
	int32_t		: 32;				/* 0x000s12c4    */
	int32_t		pvip_dr_quad_x;			/* 0x000s12c8 wo */
	int32_t		: 32;				/* 0x000s12cc    */
	int32_t		pvip_dr_quad_y;			/* 0x000s12d0 wo */
	int32_t		: 32;				/* 0x000s12d4    */
	int32_t		pvip_dr_quad_xy;		/* 0x000s12d8 wo */
	int32_t		: 32;				/* 0x000s12dc    */
	int32_t		: 32;				/* 0x000s12e0    */
	int32_t		: 32;				/* 0x000s12e4    */
	int32_t		pvip_dr_quad_rel_x;		/* 0x000s12e8 wo */
	int32_t		: 32;				/* 0x000s12ec    */
	int32_t		pvip_dr_quad_rel_y;		/* 0x000s12f0 wo */
	int32_t		: 32;				/* 0x000s12f4    */
	int32_t		pvip_dr_quad_rel_xy;		/* 0x000s12f8 wo */
	int32_t		: 32;				/* 0x000s12fc    */
	int32_t		: 32;				/* 0x000s1300    */
	int32_t		: 32;				/* 0x000s1304    */
	int32_t		pvip_dr_rect_x;			/* 0x000s1308 wo */
	int32_t		: 32;				/* 0x000s130c    */
	int32_t		pvip_dr_rect_y;			/* 0x000s1310 wo */
	int32_t		: 32;				/* 0x000s1314    */
	int32_t		pvip_dr_rect_xy;		/* 0x000s1318 wo */
	int32_t		: 32;				/* 0x000s131c    */
	int32_t		: 32;				/* 0x000s1320    */
	int32_t		: 32;				/* 0x000s1324    */
	int32_t		pvip_dr_rect_rel_x;		/* 0x000s1328 wo */
	int32_t		: 32;				/* 0x000s132c    */
	int32_t		pvip_dr_rect_rel_y;		/* 0x000s1330 wo */
	int32_t		: 32;				/* 0x000s1334    */
	int32_t		pvip_dr_rect_rel_xy;		/* 0x000s1338 wo */
	int32_t		: 32;				/* 0x000s133c    */

} pvip_drawing_regs_t;				/* 0x00200000    */

typedef struct pvip_extend_regs  {

	/* Extend Control Registers */

	uint8_t	pvip_svga_window_a;		/* 0x0000 */
	uint8_t	pvip_ex_filler0[0x100-1];
	uint8_t	pvip_svga_window_b;		/* 0x0100 */
	uint8_t	pvip_ex_filler1[0x100-1];
	uint8_t	pvip_svga_bank_b_en;		/* 0x0200 */
	uint8_t	pvip_ex_filler2[0x100-1];
	uint8_t	pvip_monitor_ctrl;		/* 0x0300 */
	uint8_t	pvip_ex_filler3[0x1c6-1];

	/* BrookTree RAMDAC registers */

	uint8_t	pvip_bt485_reg4;		/* 0x0400 */
	uint8_t	pvip_bt485_reg5;		/* 0x0401 */
	uint8_t	pvip_bt485_reg6;		/* 0x0402 */
	uint8_t	pvip_bt485_reg7;		/* 0x0403 */
	uint8_t	pvip_ex_filler4[0x400-4];
	uint8_t	pvip_bt485_reg8;		/* 0x0800 */
	uint8_t	pvip_bt485_reg9;		/* 0x0801 */
	uint8_t	pvip_bt485_rega;		/* 0x0802 */
	uint8_t	pvip_bt485_regb;		/* 0x0803 */
	uint8_t	pvip_ex_filler5[0x400-4];
	uint8_t	pvip_bt485_regc;		/* 0x0c00 */
	uint8_t	pvip_bt485_regd;		/* 0x0c01 */
	uint8_t	pvip_bt485_rege;		/* 0x0c02 */
	uint8_t	pvip_bt485_regf;		/* 0x0c03 */

} pvip_extend_regs_t;

typedef struct pvip_vga_regs  {

	uint8_t	: 8;
	uint8_t	: 8;
	uint8_t	pvip_vga_misc_out;		/* 0x0002 */
	uint8_t	: 8;
	uint8_t	pvip_vga_seq_index;		/* 0x0004 */
	uint8_t	pvip_vga_seq_data;		/* 0x0005 */
	uint8_t	pvip_bt485_reg0;		/* 0x0006 */
	uint8_t	pvip_bt485_reg1;		/* 0x0007 */
	uint8_t	pvip_bt485_reg2;		/* 0x0008 */
	uint8_t	pvip_bt485_reg3;		/* 0x0009 */
	uint8_t	: 8;
	uint8_t	: 8;
	uint8_t	pvip_vga_misc_in;		/* 0x000c */

} pvip_vga_regs_t;

typedef struct pvip {
    pvip_control_regs_t *pvip_control_regp[8];
    pvip_drawing_regs_t *pvip_drawing_regp[8];
    uchar_t *pvip_frame_bufferp;
    pvip_extend_regs_t *pvip_bt485_basep;
    pvip_vga_regs_t *pvip_vga_basep;
} pvip_t;

typedef pvip_t *pvipp_t;

	/* Address Endian Bits */

#define	PVIP_ADDR_SWAP_HALF_WORDS	4	/* 0x00040000 */
#define	PVIP_ADDR_SWAP_BYTES		2	/* 0x00020000 */
#define	PVIP_ADDR_SWAP_BITS		1	/* 0x00010000 */

#ifndef PVIP_LITTLE_ENDIAN
#if defined(_LITTLE_ENDIAN) && !defined(_BIG_ENDIAN)
#define	PVIP_LITTLE_ENDIAN 1
#elif defined(_BIG_ENDIAN) && !defined(_LITTLE_ENDIAN)
#define	PVIP_LITTLE_ENDIAN 0
#endif
#endif

#if PVIP_LITTLE_ENDIAN

#ifndef PVIP_ENDIAN_PIXEL1
#define	PVIP_ENDIAN_PIXEL1 \
	(PVIP_ADDR_SWAP_HALF_WORDS | \
	PVIP_ADDR_SWAP_BYTES | \
	PVIP_ADDR_SWAP_BITS)			/* 0x00070000 */
#endif

#ifndef PVIP_ENDIAN_PIXEL8
#define	PVIP_ENDIAN_PIXEL8 \
	(PVIP_ADDR_SWAP_HALF_WORDS | \
	PVIP_ADDR_SWAP_BYTES)			/* 0x00060000 */
#endif

#ifndef PVIP_ENDIAN_PATTERN
#define	PVIP_ENDIAN_PATTERN \
	(PVIP_ADDR_SWAP_HALF_WORDS | \
	PVIP_ADDR_SWAP_BYTES | \
	PVIP_ADDR_SWAP_BITS)			/* 0x00070000 */
#endif

#ifndef PVIP_ENDIAN_CURSOR
#define	PVIP_ENDIAN_CURSOR \
	(PVIP_ADDR_SWAP_HALF_WORDS | \
	PVIP_ADDR_SWAP_BYTES | \
	PVIP_ADDR_SWAP_BITS)			/* 0x00070000 */
#endif

#else   /* !PVIP_LITTLE_ENDIAN */

#ifndef PVIP_ENDIAN_PIXEL1
#define	PVIP_ENDIAN_PIXEL1	0
#endif

#ifndef PVIP_ENDIAN_PIXEL8
#define	PVIP_ENDIAN_PIXEL8	0
#endif

#ifndef PVIP_ENDIAN_PATTERN
#define	PVIP_ENDIAN_PATTERN	0
#endif

#ifndef PVIP_ENDIAN_CURSOR
#define	PVIP_ENDIAN_CURSOR	0
#endif

#endif

#ifndef PVIPCR
#define	PVIPCR pvip_control_regp[0]
#endif

#ifndef PVIPDR
#define	PVIPDR pvip_drawing_regp[0]
#endif

#ifndef PVIPDRP1
#define	PVIPDRP1 pvip_drawing_regp[PVIP_ENDIAN_PIXEL1]
#endif

#ifndef PVIPDRP8
#define	PVIPDRP8 pvip_drawing_regp[PVIP_ENDIAN_PIXEL8]
#endif

#ifndef PVIPDRPT
#define	PVIPDRPT pvip_drawing_regp[PVIP_ENDIAN_PATTERN]
#endif

#ifndef PVIPDRCT
#define	PVIPDRCT pvip_drawing_regp[PVIP_ENDIAN_CURSOR]
#endif

#ifndef PVIPVR
#define	PVIPVR pvip_vga_basep
#endif

#ifndef PVIPER
#define	PVIPER pvip_bt485_basep
#endif

#define	pvip_bground	PVIPDR->pvip_dr_bground		/* 0x000s0204 rw */
#define	pvip_blit	PVIPDR->pvip_dr_blit		/* 0x000s0004 ro */
#define	pvip_cindex	PVIPDR->pvip_dr_cindex		/* 0x000s018c rw */
#define	pvip_draw_mode	PVIPDR->pvip_dr_draw_mode	/* 0x000s020c rw */
#define	pvip_fground	PVIPDR->pvip_dr_fground		/* 0x000s0200 rw */
#define	pvip_hrzbf	PVIPCR->pvip_cr_hrzbf		/* 0x000z0114 rw */
#define	pvip_hrzbr	PVIPCR->pvip_cr_hrzbr		/* 0x000z0110 rw */
#define	pvip_hrzc	PVIPCR->pvip_cr_hrzc		/* 0x000z0104 ro */
#define	pvip_hrzsr	PVIPCR->pvip_cr_hrzsr		/* 0x000z010c rw */
#define	pvip_hrzt	PVIPCR->pvip_cr_hrzt		/* 0x000z0108 rw */
#define	pvip_interrupt	PVIPCR->pvip_cr_interrupt	/* 0x000z0008 rw */
#define	pvip_interrupt_en PVIPCR->pvip_cr_interrupt_en	/* 0x000z000c rw */
#define	pvip_line_rel_x	PVIPDR->pvip_dr_line_rel_x	/* 0x000s1268 wo */
#define	pvip_line_rel_xy PVIPDR->pvip_dr_line_rel_xy	/* 0x000s1278 wo */
#define	pvip_line_rel_y	PVIPDR->pvip_dr_line_rel_y	/* 0x000s1270 wo */
#define	pvip_line_x	PVIPDR->pvip_dr_line_x		/* 0x000s1248 wo */
#define	pvip_line_xy	PVIPDR->pvip_dr_line_xy		/* 0x000s1258 wo */
#define	pvip_line_y	PVIPDR->pvip_dr_line_y		/* 0x000s1250 wo */
#define	pvip_mem_config	PVIPCR->pvip_cr_mem_config	/* 0x000z0184 rw */
#define	pvip_next_pixels PVIPDR->pvip_dr_next_pixels	/* 0x000s0014 wo */
#define	pvip_oor	PVIPDR->pvip_dr_oor		/* 0x000s0184 ro */
#define	pvip_pat_originx PVIPDR->pvip_dr_pat_originx	/* 0x000s0210 rw */
#define	pvip_pat_originy PVIPDR->pvip_dr_pat_originy	/* 0x000s0214 rw */
#define	pvip_pattern	PVIPDRPT->pvip_dr_pattern	/* 0x000s0280 rw */
#define	pvip_pe_w_max	PVIPDR->pvip_dr_pe_w_max	/* 0x000s0198 ro */
#define	pvip_pe_w_min	PVIPDR->pvip_dr_pe_w_min	/* 0x000s0194 ro */
#define	pvip_pixel1	PVIPDRP1->pvip_dr_pixel1	/* 0x000s0080 wo */
#define	pvip_pixel8	PVIPDRP8->pvip_dr_pixel8	/* 0x000s000c wo */
#define	pvip_pixel8_reg	PVIPDRP8->pvip_dr_pixel8_reg	/* 0x000s021c rw */
#define	pvip_pmask	PVIPDR->pvip_dr_pmask		/* 0x000s0208 rw */
#define	pvip_point_rel_x PVIPDR->pvip_dr_point_rel_x	/* 0x000s1228 wo */
#define	pvip_point_rel_xy PVIPDR->pvip_dr_point_rel_xy	/* 0x000s1238 wo */
#define	pvip_point_rel_y PVIPDR->pvip_dr_point_rel_y	/* 0x000s1230 wo */
#define	pvip_point_x	PVIPDR->pvip_dr_point_x		/* 0x000s1208 wo */
#define	pvip_point_xy	PVIPDR->pvip_dr_point_xy	/* 0x000s1218 wo */
#define	pvip_point_y	PVIPDR->pvip_dr_point_y		/* 0x000s1210 wo */
#define	pvip_prehrzc	PVIPCR->pvip_cr_prehrzc		/* 0x000z0118 rw */
#define	pvip_prevrtc	PVIPCR->pvip_cr_prevrtc		/* 0x000z0130 rw */
#define	pvip_qsfcounter	PVIPCR->pvip_cr_qsfcounter	/* 0x000z013c ro */
#define	pvip_quad	PVIPDR->pvip_dr_quad		/* 0x000s0008 ro */
#define	pvip_quad_rel_x	PVIPDR->pvip_dr_quad_rel_x	/* 0x000s12e8 wo */
#define	pvip_quad_rel_xy PVIPDR->pvip_dr_quad_rel_xy	/* 0x000s12f8 wo */
#define	pvip_quad_rel_y	PVIPDR->pvip_dr_quad_rel_y	/* 0x000s12f0 wo */
#define	pvip_quad_x	PVIPDR->pvip_dr_quad_x		/* 0x000s12c8 wo */
#define	pvip_quad_xy	PVIPDR->pvip_dr_quad_xy		/* 0x000s12d8 wo */
#define	pvip_quad_y	PVIPDR->pvip_dr_quad_y		/* 0x000s12d0 wo */
#define	pvip_raster	PVIPDR->pvip_dr_raster		/* 0x000s0218 rw */
#define	pvip_rect_rel_x	PVIPDR->pvip_dr_rect_rel_x	/* 0x000s1328 wo */
#define	pvip_rect_rel_xy PVIPDR->pvip_dr_rect_rel_xy	/* 0x000s1338 wo */
#define	pvip_rect_rel_y	PVIPDR->pvip_dr_rect_rel_y	/* 0x000s1330 wo */
#define	pvip_rect_x	PVIPDR->pvip_dr_rect_x		/* 0x000s1308 wo */
#define	pvip_rect_xy	PVIPDR->pvip_dr_rect_xy		/* 0x000s1318 wo */
#define	pvip_rect_y	PVIPDR->pvip_dr_rect_y		/* 0x000s1310 wo */
#define	pvip_rel_x0	PVIPDR->pvip_dr_rel_x0		/* 0x000s1028 wo */
#define	pvip_rel_x1	PVIPDR->pvip_dr_rel_x1		/* 0x000s1068 wo */
#define	pvip_rel_x2	PVIPDR->pvip_dr_rel_x2		/* 0x000s10a8 wo */
#define	pvip_rel_x3	PVIPDR->pvip_dr_rel_x3		/* 0x000s10e8 wo */
#define	pvip_rel_xy0	PVIPDR->pvip_dr_rel_xy0		/* 0x000s1038 wo */
#define	pvip_rel_xy1	PVIPDR->pvip_dr_rel_xy1		/* 0x000s1078 wo */
#define	pvip_rel_xy2	PVIPDR->pvip_dr_rel_xy2		/* 0x000s10b8 wo */
#define	pvip_rel_xy3	PVIPDR->pvip_dr_rel_xy3		/* 0x000s10f8 wo */
#define	pvip_rel_y0	PVIPDR->pvip_dr_rel_y0		/* 0x000s1030 wo */
#define	pvip_rel_y1	PVIPDR->pvip_dr_rel_y1		/* 0x000s1070 wo */
#define	pvip_rel_y2	PVIPDR->pvip_dr_rel_y2		/* 0x000s10b0 wo */
#define	pvip_rel_y3	PVIPDR->pvip_dr_rel_y3		/* 0x000s10f0 wo */
#define	pvip_rfcount	PVIPCR->pvip_cr_rfcount		/* 0x000z018c ro */
#define	pvip_rfperiod	PVIPCR->pvip_cr_rfperiod	/* 0x000z0188 rw */
#define	pvip_rlcur	PVIPCR->pvip_cr_rlcur		/* 0x000z0194 ro */
#define	pvip_rlmax	PVIPCR->pvip_cr_rlmax		/* 0x000z0190 rw */
#define	pvip_pu_config	PVIPCR->pvip_cr_pu_config	/* 0x000z0198 ro */
#define	pvip_sraddr	PVIPCR->pvip_cr_sraddr		/* 0x000z0134 ro */
#define	pvip_srtctl	PVIPCR->pvip_cr_srtctl		/* 0x000z0138 rw */
#define	pvip_srtctl2	PVIPCR->pvip_cr_srtctl2		/* 0x000z0140 rw */
#define	pvip_status	PVIPDR->pvip_dr_status		/* 0x000s0000 ro */
#define	pvip_sysconfig	PVIPCR->pvip_cr_sysconfig	/* 0x000z0004 rw */
#define	pvip_tri_rel_x	PVIPDR->pvip_dr_tri_rel_x	/* 0x000s12a8 wo */
#define	pvip_tri_rel_xy	PVIPDR->pvip_dr_tri_rel_xy	/* 0x000s12b8 wo */
#define	pvip_tri_rel_y	PVIPDR->pvip_dr_tri_rel_y	/* 0x000s12b0 wo */
#define	pvip_tri_x	PVIPDR->pvip_dr_tri_x		/* 0x000s1288 wo */
#define	pvip_tri_xy	PVIPDR->pvip_dr_tri_xy		/* 0x000s1298 wo */
#define	pvip_tri_y	PVIPDR->pvip_dr_tri_y		/* 0x000s1290 wo */
#define	pvip_vrtbf	PVIPCR->pvip_cr_vrtbf		/* 0x000z012c rw */
#define	pvip_vrtbr	PVIPCR->pvip_cr_vrtbr		/* 0x000z0128 rw */
#define	pvip_vrtc	PVIPCR->pvip_cr_vrtc		/* 0x000z011c ro */
#define	pvip_vrtsr	PVIPCR->pvip_cr_vrtsr		/* 0x000z0124 rw */
#define	pvip_vrtt	PVIPCR->pvip_cr_vrtt		/* 0x000z0120 rw */
#define	pvip_w_max	PVIPDR->pvip_dr_w_max		/* 0x000s0224 rw */
#define	pvip_w_min	PVIPDR->pvip_dr_w_min		/* 0x000s0220 rw */
#define	pvip_w_off_xy	PVIPDR->pvip_dr_w_off_xy	/* 0x000s0190 rw */
#define	pvip_x0		PVIPDR->pvip_dr_x0		/* 0x000s1008 rw */
#define	pvip_x1		PVIPDR->pvip_dr_x1		/* 0x000s1048 rw */
#define	pvip_x2		PVIPDR->pvip_dr_x2		/* 0x000s1088 rw */
#define	pvip_x3		PVIPDR->pvip_dr_x3		/* 0x000s10c8 rw */
#define	pvip_xclip	PVIPDR->pvip_dr_xclip		/* 0x000s01a4 ro */
#define	pvip_xedge_gt	PVIPDR->pvip_dr_xedge_gt	/* 0x000s01ac ro */
#define	pvip_xedge_lt	PVIPDR->pvip_dr_xedge_lt	/* 0x000s01a8 ro */
#define	pvip_xy0	PVIPDR->pvip_dr_xy0		/* 0x000s1018 rw */
#define	pvip_xy1	PVIPDR->pvip_dr_xy1		/* 0x000s1058 rw */
#define	pvip_xy2	PVIPDR->pvip_dr_xy2		/* 0x000s1098 rw */
#define	pvip_xy3	PVIPDR->pvip_dr_xy3		/* 0x000s10d8 rw */
#define	pvip_y0		PVIPDR->pvip_dr_y0		/* 0x000s1010 rw */
#define	pvip_y1		PVIPDR->pvip_dr_y1		/* 0x000s1050 rw */
#define	pvip_y2		PVIPDR->pvip_dr_y2		/* 0x000s1090 rw */
#define	pvip_y3		PVIPDR->pvip_dr_y3		/* 0x000s10d0 rw */
#define	pvip_yclip	PVIPDR->pvip_dr_yclip		/* 0x000s01a0 ro */
#define	pvip_yedge_gt	PVIPDR->pvip_dr_yedge_gt	/* 0x000s01b4 ro */
#define	pvip_yedge_lt	PVIPDR->pvip_dr_yedge_lt	/* 0x000s01b0 ro */

#define	pvip_ramdac0	PVIPVR->pvip_bt485_reg2		/* 0x000z0200 rw */
#define	pvip_ramdac1	PVIPVR->pvip_bt485_reg3		/* 0x000z0204 rw */
#define	pvip_ramdac2	PVIPVR->pvip_bt485_reg0		/* 0x000z0208 rw */
#define	pvip_ramdac3	PVIPVR->pvip_bt485_reg1		/* 0x000z020c rw */

#define	pvip_ramdac4	PVIPER->pvip_bt485_reg6		/* 0x000z0210 rw */
#define	pvip_ramdac5	PVIPER->pvip_bt485_reg7		/* 0x000z0214 rw */
#define	pvip_ramdac6	PVIPER->pvip_bt485_reg4		/* 0x000z0218 rw */
#define	pvip_ramdac7	PVIPER->pvip_bt485_reg5		/* 0x000z021c rw */
#define	pvip_ramdac8	PVIPER->pvip_bt485_rega		/* 0x000z0220 rw */
#define	pvip_ramdac9	PVIPER->pvip_bt485_regb		/* 0x000z0224 rw */
#define	pvip_ramdac10	PVIPER->pvip_bt485_reg8		/* 0x000z0228 rw */
#define	pvip_ramdac11	PVIPER->pvip_bt485_reg9		/* 0x000z022c rw */
#define	pvip_ramdac12	PVIPER->pvip_bt485_rege		/* 0x000z0230 rw */
#define	pvip_ramdac13	PVIPER->pvip_bt485_regf		/* 0x000z0234 rw */
#define	pvip_ramdac14	PVIPER->pvip_bt485_regc		/* 0x000z0238 rw */
#define	pvip_ramdac15	PVIPER->pvip_bt485_regd		/* 0x000z023c rw */

	/* Coordinate Registers 0x000s1??? rw */

#define	PVIP_COORD_BITS			14
#define	PVIP_COORD_SIGN_BIT		(1 << (PVIP_COORD_BITS - 1))
#define	PVIP_COORD_MIN			(-PVIP_COORD_SIGN_BIT)
#define	PVIP_COORD_MAX			(PVIP_COORD_SIGN_BIT - 1)
#define	PVIP_COORD_MASK			((1 << PVIP_COORD_BITS) - 1)
#define	PVIP_COORD_SIGN_EXTEND(x)	(((x) & PVIP_COORD_SIGN_BIT) ? \
	((x) | PVIP_COORD_MIN) : ((x) & PVIP_COORD_MAX))

	/* Draw Mode Register 0x000s020c rw */

#define	PVIP_DRAWMODE_PICK_CTRL		0x00000008
#define	PVIP_DRAWMODE_PICK		0x00000004
#define	PVIP_DRAWMODE_DEST_BUFFER_CTRL	0x00000002
#define	PVIP_DRAWMODE_DEST_BUFFER	0x00000001

	/* Interrupt Register 0x000z0008 rw */

#define	PVIP_INT_VBLANKED_CTRL		0x00000020
#define	PVIP_INT_VBLANKED		0x00000010
#define	PVIP_INT_PICKED_CTRL		0x00000008
#define	PVIP_INT_PICKED			0x00000004
#define	PVIP_INT_DE_IDLE_CTRL		0x00000002
#define	PVIP_INT_DE_IDLE		0x00000001

	/* Interrupt Enable Register 0x000z000c rw */

#define	PVIP_INTEN_MEN_CTRL		0x00000080
#define	PVIP_INTEN_MEN			0x00000040
#define	PVIP_INTEN_VBLANKED_EN_CTRL	0x00000020
#define	PVIP_INTEN_VBLANKED_EN		0x00000010
#define	PVIP_INTEN_PICKED_EN_CTRL	0x00000008
#define	PVIP_INTEN_PICKED_EN		0x00000004
#define	PVIP_INTEN_DE_IDLE_EN_CTRL	0x00000002
#define	PVIP_INTEN_DE_IDLE_EN		0x00000001

	/* Memory Configuration Register 0x000z0184 rw */

#define	PVIP_MEMCFG_MEMORY_CONFIG_1	0x00000000 /* 1MB (two cycle) */
#define	PVIP_MEMCFG_MEMORY_CONFIG_2	0x00000001 /* 1MB (one cycle) */
#define	PVIP_MEMCFG_MEMORY_CONFIG_3	0x00000002 /* 2MB (single buf) */
#define	PVIP_MEMCFG_MEMORY_CONFIG_4	0x00000003 /* 1MB * 2 (double buf) */
#define	PVIP_MEMCFG_MEMORY_CONFIG_5	0x00000004 /* 2MB * 2 (double buf) */
#define	PVIP_MEMCFG_MEMORY_CONFIG_MASK	0x00000007

	/* Out Of Range Register 0x000s0184 ro */

#define	PVIP_OOR_X3_OOR			0x00000080
#define	PVIP_OOR_X2_OOR			0x00000040
#define	PVIP_OOR_X1_OOR			0x00000020
#define	PVIP_OOR_X0_OOR			0x00000010
#define	PVIP_OOR_Y3_OOR			0x00000008
#define	PVIP_OOR_Y2_OOR			0x00000004
#define	PVIP_OOR_Y1_OOR			0x00000002
#define	PVIP_OOR_Y0_OOR			0x00000001

	/* Raster Register 0x000s0218 rw */

#define	PVIP_RASTER_USE_PATTERN	0x00020000
#define	PVIP_RASTER_QUAD_DRAW_MODE	0x00010000
#define	PVIP_RASTER_ROP_MINTERMS	0x0000ffff
#define	PVIP_RASTER_ROP_F_MASK		0x0000ff00
#define	PVIP_RASTER_ROP_B_MASK		0x0000f0f0
#define	PVIP_RASTER_ROP_S_MASK		0x0000cccc
#define	PVIP_RASTER_ROP_D_MASK		0x0000aaaa

	/* Screen Repaint Timing Control 0x000z0138 rw */

#define	PVIP_SRTCTL_INTERNAL_VSYNC	0x00000100
#define	PVIP_SRTCTL_INTERNAL_HSYNC	0x00000080
#define	PVIP_SRTCTL_COMPOSITE		0x00000040
#define	PVIP_SRTCTL_ENABLE_VIDEO	0x00000020
#define	PVIP_SRTCTL_HBLNK_RELOAD	0x00000010
#define	PVIP_SRTCTL_DISPLAY_BUFFER	0x00000008
#define	PVIP_SRTCTL_QSFSELECT		0x00000007

	/* Screen Repaint Timing Control 0x000z0140 rw */

#define	PVIP_SRTCLT2_VSYNC_POLAR_LOW	0x00000001
#define	PVIP_SRTCLT2_VSYNC_POLAR_F	0x00000002
#define	PVIP_SRTCLT2_HSYNC_POLAR_LOW	0x00000004
#define	PVIP_SRTCLT2_HSYNC_POLAR_F	0x00000008

	/* Status Register 0x000s0000 ro */

#define	PVIP_STATUS_ISSUE_QB		0x80000000
#define	PVIP_STATUS_BUSY		0x40000000
#define	PVIP_STATUS_QUAD_OR_BUSY (PVIP_STATUS_ISSUE_QB | PVIP_STATUS_BUSY)
#define	PVIP_STATUS_PICKED		0x00000080
#define	PVIP_STATUS_PIXEL_SOFTWARE	0x00000040
#define	PVIP_STATUS_BLIT_SOFTWARE	0x00000020
#define	PVIP_STATUS_QUAD_SOFTWARE	0x00000010
#define	PVIP_STATUS_QUAD_CONCAVE	0x00000008
#define	PVIP_STATUS_QUAD_HIDDEN		0x00000004
#define	PVIP_STATUS_QUAD_VISIBLE	0x00000002
#define	PVIP_STATUS_QUAD_INTERSECTS	0x00000001

	/* System Configuration Register 0x000z0004 rw */

#define	PVIP_SYSCONF_8_BIT_PIXELS	0x08000000
#define	PVIP_SYSCONF_16_BIT_PIXELS	0x0c000000
#define	PVIP_SYSCONF_24_BIT_PIXELS	0x1c000000
#define	PVIP_SYSCONF_32_BIT_PIXELS	0x14000000
#define	PVIP_SYSCONF_DRIVELOAD2		0x01000000
#define	PVIP_SYSCONF_HRES1_MASK		0x00700000
#define	PVIP_SYSCONF_HRES1_SHIFT	20
#define	PVIP_SYSCONF_HRES2_MASK		0x000e0000
#define	PVIP_SYSCONF_HRES2_SHIFT	17
#define	PVIP_SYSCONF_HRES3_MASK		0x0001c000
#define	PVIP_SYSCONF_HRES3_SHIFT	14
#define	PVIP_SYSCONF_HRES4_MASK		0x60000000
#define	PVIP_SYSCONF_HRES4_SHIFT	29
#define	PVIP_SYSCONF_PIXEL_SWAP_HALF	0x00002000
#define	PVIP_SYSCONF_PIXEL_SWAP_BYTE	0x00001000
#define	PVIP_SYSCONF_PIXEL_SWAP_BITS	0x00000800
#define	PVIP_SYSCONF_PIXEL_BUF_READ	0x00000400
#define	PVIP_SYSCONF_PIXEL_BUF_WRITE	0x00000200
#define	PVIP_SYSCONF_VERSION_MASK	0x00000007
#define	PVIP_SYSCONF_HRES_NOADD		0x00000000
#define	PVIP_SYSCONF_HRES_ADD32		0x00000001
#define	PVIP_SYSCONF_HRES_ADD64		0x00000002
#define	PVIP_SYSCONF_HRES_ADD128	0x00000003
#define	PVIP_SYSCONF_HRES_ADD256	0x00000004
#define	PVIP_SYSCONF_HRES_ADD512	0x00000005
#define	PVIP_SYSCONF_HRES_ADD1024	0x00000006
#define	PVIP_SYSCONF_HRES_ADD2048	0x00000007

	/* Video Control Mask */

#define	PVIP_VIDCTRL_VALUE_MASK	0x00000fff

	/* Window Minimum and Maximum Mask */

#define	PVIP_WINDOW_MASK		0x00001fff
#define	PVIP_WINDOW_XY_MASK		0x1fff1fff

	/* Xclip Register 0x000s01a4 ro */

#define	PVIP_XCLIP_X3_LT_MIN		0x00000080
#define	PVIP_XCLIP_X2_LT_MIN		0x00000040
#define	PVIP_XCLIP_X1_LT_MIN		0x00000020
#define	PVIP_XCLIP_X0_LT_MIN		0x00000010
#define	PVIP_XCLIP_X3_GT_MAX		0x00000008
#define	PVIP_XCLIP_X2_GT_MAX		0x00000004
#define	PVIP_XCLIP_X1_GT_MAX		0x00000002
#define	PVIP_XCLIP_X0_GT_MAX		0x00000001

	/* Xedge_gt Register 0x000s01ac ro */

#define	PVIP_XEDGEGT_X0_LT_X2		0x00000020
#define	PVIP_XEDGEGT_X1_LT_X3		0x00000010
#define	PVIP_XEDGEGT_X3_LT_X0		0x00000008
#define	PVIP_XEDGEGT_X2_LT_X3		0x00000004
#define	PVIP_XEDGEGT_X1_LT_X2		0x00000002
#define	PVIP_XEDGEGT_X0_LT_X1		0x00000001

	/* Xedge_lt Register 0x000s01a8 ro */

#define	PVIP_XEDGELT_X0_GT_X2		0x00000020
#define	PVIP_XEDGELT_X1_GT_X3		0x00000010
#define	PVIP_XEDGELT_X3_GT_X0		0x00000008
#define	PVIP_XEDGELT_X2_GT_X3		0x00000004
#define	PVIP_XEDGELT_X1_GT_X2		0x00000002
#define	PVIP_XEDGELT_X0_GT_X1		0x00000001

	/* Yclip Register 0x000s01a0 ro */

#define	PVIP_YCLIP_Y3_LT_MIN		0x00000080
#define	PVIP_YCLIP_Y2_LT_MIN		0x00000040
#define	PVIP_YCLIP_Y1_LT_MIN		0x00000020
#define	PVIP_YCLIP_Y0_LT_MIN		0x00000010
#define	PVIP_YCLIP_Y3_GT_MAX		0x00000008
#define	PVIP_YCLIP_Y2_GT_MAX		0x00000004
#define	PVIP_YCLIP_Y1_GT_MAX		0x00000002
#define	PVIP_YCLIP_Y0_GT_MAX		0x00000001

	/* Yedge_gt Register 0x000s01b4 ro */

#define	PVIP_YEDGEGT_Y0_LT_Y2		0x00000020
#define	PVIP_YEDGEGT_Y1_LT_Y3		0x00000010
#define	PVIP_YEDGEGT_Y3_LT_Y0		0x00000008
#define	PVIP_YEDGEGT_Y2_LT_Y3		0x00000004
#define	PVIP_YEDGEGT_Y1_LT_Y2		0x00000002
#define	PVIP_YEDGEGT_Y0_LT_Y1		0x00000001

	/* Yedge_lt Register 0x000s01b0 ro */

#define	PVIP_YEDGELT_Y0_GT_Y2		0x00000020
#define	PVIP_YEDGELT_Y1_GT_Y3		0x00000010
#define	PVIP_YEDGELT_Y3_GT_Y0		0x00000008
#define	PVIP_YEDGELT_Y2_GT_Y3		0x00000004
#define	PVIP_YEDGELT_Y1_GT_Y2		0x00000002
#define	PVIP_YEDGELT_Y0_GT_Y1		0x00000001

#define	PVIP_PTR(pvip)			(PVIP_NOOP(), (pvip))

#define	PVIP_XY(x, y)		\
	(((long) (x) << 16) | (long) (y) & 0xffff)

#define	PVIP_PAT_COPY(d, s)	\
	(d)[0] = (s)[0],	\
	(d)[1] = (s)[1],	\
	(d)[2] = (s)[2],	\
	(d)[3] = (s)[3],	\
	(d)[4] = (s)[4],	\
	(d)[5] = (s)[5],	\
	(d)[6] = (s)[6],	\
	(d)[7] = (s)[7]

#define	PVIP_LINEBYTES(sysconfig)  \
	(((sysconfig) & PVIP_SYSCONF_HRES3_MASK ?	  \
	1 << ((((sysconfig) & PVIP_SYSCONF_HRES3_MASK) >>  \
	PVIP_SYSCONF_HRES3_SHIFT) + 4) : 0) +	      \
	((sysconfig) & PVIP_SYSCONF_HRES2_MASK ?	   \
	1 << ((((sysconfig) & PVIP_SYSCONF_HRES2_MASK) >>  \
	PVIP_SYSCONF_HRES2_SHIFT) + 4) : 0) +	      \
	((sysconfig) & PVIP_SYSCONF_HRES1_MASK ?	   \
	1 << ((((sysconfig) & PVIP_SYSCONF_HRES1_MASK) >>  \
	PVIP_SYSCONF_HRES1_SHIFT) + 4) : 0))

/*
 * These MARCROs are shared with the DDX driver, thus are in
 * the header file.
 */
#define	PVIP_GET_ENABLE_VIDEO(pvip)   \
	(((pvip) -> pvip_srtctl & PVIP_SRTCTL_ENABLE_VIDEO) != 0)

#define	PVIP_SET_VBLANK_INTR(pvip)    \
	(pvip) -> pvip_interrupt_en = PVIP_INTEN_MEN_CTRL | \
	PVIP_INTEN_MEN | PVIP_INTEN_VBLANKED_EN_CTRL | \
	PVIP_INTEN_VBLANKED_EN

#define	PVIP_STOP_VBLANK_INTR(pvip)   \
	(pvip) -> pvip_interrupt_en = PVIP_INTEN_VBLANKED_EN_CTRL

#define	PVIP_CLEAR_VBLANK_INTR(pvip)  \
	(pvip) -> pvip_interrupt = PVIP_INT_VBLANKED_CTRL

#define	PVIP_NOOP(pvip)   (pvip)

#define	PVIP_WAIT_BUSY(pvip)	\
	while ((pvip) -> pvip_status & PVIP_STATUS_BUSY)

#define	PVIP_DRAW_QUAD(pvip)	\
	while ((pvip) -> pvip_quad & PVIP_STATUS_ISSUE_QB)

#define	PVIP_DRAW_BLIT(pvip)	\
	while ((pvip) -> pvip_blit & PVIP_STATUS_ISSUE_QB)

#define	PVIP_WAIT_VSYNC(pvip) \
	while ((pvip) -> pvip_vrtsr & PVIP_VIDCTRL_VALUE_MASK) != \
	(pvip) -> pvip_vrtc & PVIP_VIDCTRL_VALUE_MASK))

#define	PVIP_SET_ENABLE_VIDEO(pvip) \
	(pvip) -> pvip_srtctl |= PVIP_SRTCTL_ENABLE_VIDEO

#define	PVIP_CLEAR_ENABLE_VIDEO(pvip) \
	(pvip) -> pvip_srtctl &= ~PVIP_SRTCTL_ENABLE_VIDEO

#define	PVIP_SET_BUFFER_0(pvip)  \
	pvipregs -> pvip_sysconfig &= ~(PVIP_SYSCONF_PIXEL_BUF_READ | \
	PVIP_SYSCONF_PIXEL_BUF_WRITE)

#define	PVIP_SET_BUFFER_1(pvip)  \
	pvipregs -> pvip_sysconfig |= PVIP_SYSCONF_PIXEL_BUF_READ | \
	PVIP_SYSCONF_PIXEL_BUF_WRITE

	/* These addresses hang the cpu.  I suspect its  a  hardware */
	/* bug of the Weitek p9000.				  */

#define	P9000_BAD_ADDRESSES(addr) \
	(((addr) & 0xfff81ffc) == \
	(long) &((p9000p_t)0)->pvip_control_regs[0].pvip_cr_filler2 || \
	((addr) & 0xfff81fc0) == \
	(long) &((p9000p_t)0)->pvip_control_regs[0].pvip_cr_filler3)

#define	PVIP_NUM_CONTROL_REG		8
#define	PVIP_NUM_DRAWING_REG		8

/*
 * P9000 Physical address map exported by the hardware.
 */
#define	P9000_CONTROL_BASE0		0x00100000
#define	P9000_CONTROL_BASE1		0x00110000
#define	P9000_CONTROL_BASE2		0x00120000
#define	P9000_CONTROL_BASE3		0x00130000
#define	P9000_CONTROL_BASE4		0x00140000
#define	P9000_CONTROL_BASE5		0x00150000
#define	P9000_CONTROL_BASE6		0x00160000
#define	P9000_CONTROL_BASE7		0x00170000
#define	P9000_CONTROL_SIZE		0x00002000	/* 0x800 bytes act. */

#define	P9000_DRAWING_BASE0		0x00180000
#define	P9000_DRAWING_BASE1		0x00190000
#define	P9000_DRAWING_BASE2		0x001a0000
#define	P9000_DRAWING_BASE3		0x001b0000
#define	P9000_DRAWING_BASE4		0x001c0000
#define	P9000_DRAWING_BASE5		0x001d0000
#define	P9000_DRAWING_BASE6		0x001e0000
#define	P9000_DRAWING_BASE7		0x001f0000
#define	P9000_DRAWING_SIZE		0x00002000

#define	P9000_FRAME_BUFFER_BASE		0x00200000
#define	P9000_FRAME_BUFFER_SIZE		0x00200000

/*
 * P9100 Physical address map exported by the hardware.
 */
#define	P9100_CONTROL_BASE0		0x00000000
#define	P9100_CONTROL_BASE1		0x00010000
#define	P9100_CONTROL_BASE2		0x00020000
#define	P9100_CONTROL_BASE3		0x00030000
#define	P9100_CONTROL_BASE4		0x00040000
#define	P9100_CONTROL_BASE5		0x00050000
#define	P9100_CONTROL_BASE6		0x00060000
#define	P9100_CONTROL_BASE7		0x00070000
#define	P9100_CONTROL_SIZE		0x00002000	/* 0x800 bytes act. */

#define	P9100_DRAWING_BASE0		0x00002000
#define	P9100_DRAWING_BASE1		0x00012000
#define	P9100_DRAWING_BASE2		0x00022000
#define	P9100_DRAWING_BASE3		0x00032000
#define	P9100_DRAWING_BASE4		0x00042000
#define	P9100_DRAWING_BASE5		0x00052000
#define	P9100_DRAWING_BASE6		0x00062000
#define	P9100_DRAWING_BASE7		0x00072000
#define	P9100_DRAWING_SIZE		0x00002000

#define	P9100_PIXEL8_BASE0		0x00004000
#define	P9100_PIXEL8_BASE1		0x00014000
#define	P9100_PIXEL8_BASE2		0x00024000
#define	P9100_PIXEL8_BASE3		0x00034000
#define	P9100_PIXEL8_BASE4		0x00044000
#define	P9100_PIXEL8_BASE5		0x00054000
#define	P9100_PIXEL8_BASE6		0x00064000
#define	P9100_PIXEL8_BASE7		0x00074000
#define	P9100_PIXEL8_SIZE		0x00004000

#define	P9100_IFRAME_BUFFER_BASE	0x000a0000
#define	P9100_IFRAME_BUFFER_SIZE	0x00010000

#define	P9100_FRAME_BUFFER_BASE		0x00800000
#define	P9100_FRAME_BUFFER_SIZE		0x00200000

/*
 * Logical address map exported by the driver (made to work with the stock
 * DDX libraries.
 */
#define	PVIP_C_CONTROL_BASE0		0x00100000
#define	PVIP_C_CONTROL_BASE1		0x00110000
#define	PVIP_C_CONTROL_BASE2		0x00120000
#define	PVIP_C_CONTROL_BASE3		0x00130000
#define	PVIP_C_CONTROL_BASE4		0x00140000
#define	PVIP_C_CONTROL_BASE5		0x00150000
#define	PVIP_C_CONTROL_BASE6		0x00160000
#define	PVIP_C_CONTROL_BASE7		0x00170000
#define	PVIP_C_CONTROL_SIZE		0x00002000	/* 0x800 bytes act. */

#define	PVIP_C_DRAWING_BASE0		0x00180000
#define	PVIP_C_DRAWING_BASE1		0x00190000
#define	PVIP_C_DRAWING_BASE2		0x001a0000
#define	PVIP_C_DRAWING_BASE3		0x001b0000
#define	PVIP_C_DRAWING_BASE4		0x001c0000
#define	PVIP_C_DRAWING_BASE5		0x001d0000
#define	PVIP_C_DRAWING_BASE6		0x001e0000
#define	PVIP_C_DRAWING_BASE7		0x001f0000
#define	PVIP_C_DRAWING_SIZE		0x00002000

#define	PVIP_VBASE			0x00000000
#define	PVIP_REGISTER_BASE		0x00100000
#define	PVIP_REGISTER_SIZE		0x00100000
#define	PVIP_CONTROL_BASE		0x00100000
#define	PVIP_CONTROL_SIZE		0x00080000
#define	PVIP_DRAWING_BASE		0x00180000
#define	PVIP_DRAWING_SIZE		0x00080000
#define	PVIP_FRAME_BUFFER_BASE		0x00200000
#define	PVIP_FRAME_BUFFER_SIZE		0x00200000
#define	PVIP_SIZE			0x00400000

#define	PVIP_VRT_VADDR			0x10000000
#define	PVIP_VRT_SIZE			4096

#define	PVIP_P9100			0x9100
#define	PVIP_P9000			0x9001

#define	PVIP_NONE			0xFFFFFFFF

#ifdef	__cplusplus
}
#endif

#endif	/* !_PVIPREG_H */
