#pragma ident	"@(#)tblt.c	35.5	97/04/07 SMI"

/*
 * Copyright 1993,1997 by Sun Microsystems, Inc. Mountain View, CA.
 *
 *	All Rights Reserved
 *
 * Permission  to  use,  copy,  modify,  and  distribute   this
 * software  and  its documentation for any purpose and without
 * fee is hereby granted, provided that the above copyright no-
 * tice  appear  in all copies and that both that copyright no-
 * tice and this permission notice appear in  supporting  docu-
 * mentation,  and  that the names of Sun or MIT not be used in
 * advertising or publicity pertaining to distribution  of  the
 * software  without specific prior written permission. Sun and
 * M.I.T. make no representations about the suitability of this
 * software for any purpose. It is provided "as is" without any
 * express or implied warranty.
 *
 * SUN DISCLAIMS ALL WARRANTIES WITH REGARD TO  THIS  SOFTWARE,
 * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FIT-
 * NESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL SUN BE  LI-
 * ABLE  FOR  ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
 * ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,  DATA  OR
 * PROFITS,  WHETHER  IN  AN  ACTION OF CONTRACT, NEGLIGENCE OR
 * OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION  WITH
 * THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

/*
 * Sample tablet STREAMS module.
 *
 * This driver makes use of a few kernel interfaces that are not
 * currently part of the DDI/DKI and are subject to change in future
 * releases of Solaris.  Occurrences of these interfaces are bracketed in
 * this source file with the C preprocessor directive "#ifndef _DDICT";
 * developers should exercise caution in propagating these interfaces to
 * drivers derived from this sample.
 *
 * The interfaces are:
 *
 * 	struct timeval -|
 * 	uniqtime()	|-- used by the uncommitted vuid_event.h
 *				interface
 * 	getioseqno()   -|
 *
 * and interfaces described by the following header files:
 *
 * 	sys/termios.h		general terminal interface for async ports
 * 	sys/vuid_event.h	unsupported interface for input devices
 */

#ifdef	_DDICT
#define	timeval queue
#endif

#include <sys/types.h>
#include <sys/param.h>
#include <sys/signal.h>
#include <sys/stream.h>
#include <sys/stropts.h>
#include <sys/termio.h>
#include <sys/kmem.h>
#include <sys/file.h>
#include <sys/uio.h>
#include <sys/errno.h>

#include "vuid_event.h"

#include <sys/ddi.h>
#include <sys/sunddi.h>

#include "tbltio.h"
#include "tbltreg.h"

#include <sys/cmn_err.h>

/*
 * the following #define's are for an uncommitted interface
 * (see <sys/strtty.h>
 */
#define	MC_SERVICEIMM   3
#define	MC_SERVICEDEF	4

static int tbltopen(queue_t *, dev_t *, int, int, cred_t *);
static int tbltclose(queue_t *, int, cred_t *);
static int tbltwput(queue_t *, mblk_t *);
static int tbltrput(queue_t *, mblk_t *);
static int tbltrserv(queue_t *);

static struct module_info tbltmiinfo = {
		0,
		"tblt",
		0,
		INFPSZ,
		2048,
		128
};

static struct qinit tbltrinit = {
		tbltrput,
		tbltrserv,
		tbltopen,
		tbltclose,
		(int (*)())NULL,
		&tbltmiinfo,
		NULL
};

static struct module_info tbltmoinfo = {
		0,
		"tblt",
		0,
		INFPSZ,
		2048,
		128
};

static struct qinit tbltwinit = {
		tbltwput,
		(int (*)())NULL,
		tbltopen,
		tbltclose,
		(int (*)())NULL,
		&tbltmoinfo,
		NULL
};

static struct streamtab tblt_info = {
		&tbltrinit,
		&tbltwinit,
		NULL,
		NULL
};

/*
 * This is the loadable module wrapper.
 */
#include <sys/modctl.h>

static struct fmodsw fsw = {
		"tblt",
		&tblt_info,
		(D_NEW|D_MP|D_MTPERMOD)
};

/*
 * Module linkage information for the kernel.
 */
extern struct mod_ops mod_strmodops;

static struct modlstrmod modlstrmod = {
		&mod_strmodops, "streams module for tablet", &fsw
};

static struct modlinkage modlinkage = {
		MODREV_1, &modlstrmod, NULL
};

_init()
{
	return (mod_install(&modlinkage));
}

_info(modinfop)
struct modinfo *modinfop;
{
	return (mod_info(&modlinkage, modinfop));
}

_fini(void)
{
	return (mod_remove(&modlinkage));
}

struct tbltdata {
	struct	tblt_softc tbltd_softc;
	queue_t	*tbltd_readq;		/* upstream read queue */
	mblk_t	*tbltd_iocpending;	/* "ioctl" awaiting buffer */
	int	tbltd_flags;		/* random flags */
	int	tbltd_iocid;		/* ID of "ioctl" being waited for */
	int	tbltd_iocerror;		/* error return from "ioctl" */
	short	tbltd_state;		/* state counter for input routine */
	int	tbltd_timeout_id;	/* id returned by timeout() */
	short	tbltd_pen_state;
	int	tbltd_baud_rate;
	int	tbltd_rserv_qbufcall_id;
	int	tbltd_open_callback_id;
};

#define	TBLT_PEN_OUTOFPROX	0
#define	TBLT_PEN_ENTERPROX	1
#define	TBLT_PEN_INPROX		2

#define	TBLT_IOCWAIT	0x00000002	/* "open" waiting for ioctl to finish */
#define	TBLT_IOCTOSS	0x00000004	/* Toss ioct return */

/*
 * Input routine states. See tbltinput().
 */
#define	TBLT_WAIT_BUTN		0
#define	TBLT_PWAIT_LEAD		1
#define	TBLT_PWAIT_X1		2
#define	TBLT_PWAIT_X2		3
#define	TBLT_PWAIT_Y		4
#define	TBLT_PWAIT_Y1		5
#define	TBLT_PWAIT_Y2		6
#define	TBLT_PWAIT_PRESSURE	7

/*
 * Tablet buffer size in bytes.  Place here as variable so that one could
 * massage it using adb if it turns out to be too small.
 */
static int TBLT_BUF_BYTES = 4096;

/*
 * Pen initialization string
 */
static char	tblt_pen_init_press[] = "SP\rPH1\rAS1\rSU2\rAL2\rIT2\rST\r";

static void	tbltresched(long);
static void	tbltioctl(queue_t *, mblk_t *);
static void	tbltflush(struct tbltdata *);
static void 	tbltinput(register struct tbltdata *, char);
static void	tbltincr(struct tbltdata *);
static void 	tblt_abs_pos(register struct tbltdata *);
static int 	getnextbaudrate(int);

#ifndef	_DDICT
extern void uniqtime();
#endif

static void
tbltopen_qbufcall_callback(long arg)
{
	register struct tbltdata *tbltd = (struct tbltdata *)arg;

	tbltd->tbltd_open_callback_id = 0;

	/* leaving here should wakeup the qwait_sig() in tbltopen() */
}

/*
 * Open a tablet.
 */
/*ARGSUSED*/
static int
tbltopen(queue_t *q, dev_t *devp, int oflag, int sflag, cred_t *credp)
{
	register struct tabletbuf *b;
	register struct tblt_softc *tblt;
	register struct tbltdata *tbltd;
	mblk_t *mp;
	mblk_t *set_msg;
	mblk_t *datap;
	register struct iocblk *iocb;
	register struct termios *cb;
	int lwp_error;

	if (q->q_ptr != NULL)
		return (0);		/* already attached */

	if (sflag != MODOPEN)
		return (EINVAL);

	/*
	 * Allocate an tbltdata structure.
	 */
	tbltd = (struct tbltdata *)kmem_zalloc(sizeof (struct tbltdata), 0);

	/*
	 * Set up queue pointers, so that the "put" procedure will accept
	 * the reply to the "ioctl" message we send down.
	 */
	q->q_ptr = (caddr_t)tbltd;
	WR(q)->q_ptr = (caddr_t)tbltd;
	tbltd->tbltd_readq = q;

	/*
	 * Enable xxput() and xxsrv() procedures on this queue.
	 */
	qprocson(q);

	/*
	 * Setup tty modes.
	 */
	while ((mp = allocb(sizeof (struct iocblk), BPRI_HI)) == NULL) {
		tbltd->tbltd_open_callback_id =
		    qbufcall(q, sizeof (struct iocblk), BPRI_HI,
		    tbltopen_qbufcall_callback, (long)tbltd);

		if (!qwait_sig(q)) {
			if (tbltd->tbltd_open_callback_id) {
				qunbufcall(q, tbltd->tbltd_open_callback_id);
				tbltd->tbltd_open_callback_id = 0;
			}
			qprocsoff(q);
			kmem_free((void *)tbltd, sizeof (*tbltd));
			return (EINTR);
		}
		if (tbltd->tbltd_open_callback_id) {
			qunbufcall(q, tbltd->tbltd_open_callback_id);
			tbltd->tbltd_open_callback_id = 0;
		}
	}
	while ((datap = allocb(sizeof (struct termios), BPRI_HI)) == NULL) {
		tbltd->tbltd_open_callback_id =
		    qbufcall(q, sizeof (struct iocblk), BPRI_HI,
		    tbltopen_qbufcall_callback, (long)tbltd);

		if (!qwait_sig(q)) {
			if (tbltd->tbltd_open_callback_id) {
				qunbufcall(q, tbltd->tbltd_open_callback_id);
				tbltd->tbltd_open_callback_id = 0;
			}
			freemsg(mp);
			qprocsoff(q);
			kmem_free((void *)tbltd, sizeof (*tbltd));
			return (EINTR);
		}
		if (tbltd->tbltd_open_callback_id) {
			qunbufcall(q, tbltd->tbltd_open_callback_id);
			tbltd->tbltd_open_callback_id = 0;
		}
	}
	iocb = (struct iocblk *)mp->b_wptr;
	iocb->ioc_cr = credp;
	iocb->ioc_cmd = TCSETSF;
#ifndef	_DDICT
	iocb->ioc_id = getiocseqno();
#endif /* _DDICT */
	iocb->ioc_count = sizeof (struct iocblk);
	iocb->ioc_error = 0;
	iocb->ioc_rval = 0;
	mp->b_wptr += (sizeof (*iocb))/(sizeof (*datap->b_wptr));
	mp->b_datap->db_type = M_IOCTL;
	cb = (struct termios *)datap->b_wptr;
	cb->c_iflag = 0;
	cb->c_oflag = 0;
	cb->c_cflag = CREAD|CS8|B9600;
	cb->c_lflag = 0;
	bzero((caddr_t)cb->c_cc, NCCS);
	datap->b_wptr += (sizeof (*cb))/(sizeof (*datap->b_wptr));
	datap->b_datap->db_type = M_DATA;
	mp->b_cont = datap;
	tbltd->tbltd_flags |= TBLT_IOCWAIT;	/* indicate that we're */
	tbltd->tbltd_iocid = iocb->ioc_id;	/* waiting for this response */
	(void) putnext(WR(q), mp);

	/*
	 * Now wait for it.  Let our read queue put routine wake us up
	 * when it arrives.
	 */
	while (tbltd->tbltd_flags & TBLT_IOCWAIT) {
		if (!qwait_sig(q)) {
			lwp_error = EINTR;
			goto error;
		}
	}
	if ((lwp_error = tbltd->tbltd_iocerror) != 0)
		goto error;

	/*
	 * Set up private data.
	 */
	tbltd->tbltd_state = TBLT_WAIT_BUTN;
	tbltd->tbltd_pen_state = TBLT_PEN_OUTOFPROX;
	tbltd->tbltd_iocpending = NULL;
	tbltd->tbltd_baud_rate = B9600;
	tblt = &tbltd->tbltd_softc;

	/*
	 * Allocate buffer and initialize data.
	 */
	if (tblt->tblt_buf == 0) {
		tblt->tblt_bufbytes = TBLT_BUF_BYTES;
		b = (struct tabletbuf *) kmem_zalloc((u_int)tblt->tblt_bufbytes,
		    0);
		if (b == 0) {
			lwp_error = EINVAL;
			goto error;
		}
		b->mb_size = 1
		    + (tblt->tblt_bufbytes - sizeof (struct tabletbuf))
		    / sizeof (struct tabletinfo);
		tblt->tblt_buf = b;
		tblt->tblt_vuidaddr = vuid_first(TABLET_DEVID);
		while ((set_msg = allocb(sizeof (tblt_pen_init_press),
		    BPRI_MED)) == NULL) {
			tbltd->tbltd_open_callback_id =
			    qbufcall(q, sizeof (struct iocblk), BPRI_HI,
			    tbltopen_qbufcall_callback, (long)tbltd);

			if (!qwait_sig(q)) {
				if (tbltd->tbltd_open_callback_id) {
					qunbufcall(q,
					    tbltd->tbltd_open_callback_id);
					tbltd->tbltd_open_callback_id = 0;
				}
				qprocsoff(q);
				kmem_free((void *)tbltd, sizeof (*tbltd));
				return (EINTR);
			}
			if (tbltd->tbltd_open_callback_id) {
				qunbufcall(q, tbltd->tbltd_open_callback_id);
				tbltd->tbltd_open_callback_id = 0;
			}
		}

		bcopy(tblt_pen_init_press, (char *) (set_msg->b_rptr),
		    sizeof (tblt_pen_init_press));
		set_msg->b_wptr += sizeof (tblt_pen_init_press);
		(void) putnext(WR(q), set_msg);

		tbltflush(tbltd);
	}

	/*
	 * Tell the module below us that it should return input immediately.
	 */
	(void) putnextctl1(WR(q), M_CTL, MC_SERVICEIMM);

	return (0);

error:

	/*
	 * Disable xxput() and xxsrv() procedures on this queue.
	 */
	qprocsoff(q);

	bzero((caddr_t) tbltd, sizeof (*tbltd));
	bzero((caddr_t) tblt, sizeof (*tblt));
	return (lwp_error);
}

/*
 * Close the tablet
 */
/*ARGSUSED*/
static int
tbltclose(queue_t *q, int flag, cred_t *credp)
{
	register struct tbltdata *tbltd = (struct tbltdata *)q->q_ptr;
	register struct tblt_softc *tblt;

	/*
	 * Tell the module below us that it need not return input immediately.
	 */
	(void) putnextctl1(q, M_CTL, MC_SERVICEDEF);

	/*
	 * Disable xxput() and xxsrv() procedures on this queue.
	 */
	qprocsoff(q);

	/*
	 * Since we're about to destroy our private data, turn off
	 * our open flag first, so we don't accept any more input
	 * and try to use that data.
	 */
	tbltd->tbltd_flags = 0;

	if (tbltd->tbltd_iocpending != NULL) {
		/*
		 * We were holding an "ioctl" response pending the
		 * availability of an "mblk" to hold data to be passed up;
		 * another "ioctl" came through, which means that "ioctl"
		 * must have timed out or been aborted.
		 */
		freemsg(tbltd->tbltd_iocpending);
		tbltd->tbltd_iocpending = NULL;
	}

	if (tbltd->tbltd_rserv_qbufcall_id) {
		qunbufcall(tbltd->tbltd_readq, tbltd->tbltd_rserv_qbufcall_id);
		tbltd->tbltd_rserv_qbufcall_id = 0;
	}

	tblt = &tbltd->tbltd_softc;
	/* Free tablet buffer */
	if (tblt->tblt_buf != NULL)
		kmem_free((void *)tblt->tblt_buf, (u_int)tblt->tblt_bufbytes);
	/* Free tbltdata structure */
	kmem_free((void *)tbltd, sizeof (*tbltd));
	return (0);
}

/*
 * Read queue service routine.
 * Turn buffered tablet events into stream messages.
 */
static int
tbltrserv(register queue_t *q)
{
	struct tbltdata *tbltd = (struct tbltdata *)q->q_ptr;
	register Firm_event *fep;
	register struct tblt_softc *tblt;
	register struct tabletbuf *b;
	register struct tabletinfo *mi;
	mblk_t	 *bp;

	tblt = &tbltd->tbltd_softc;
	b = tblt->tblt_buf;
	while (canputnext(q) && tblt->tblt_oldoff != b->mb_off) {
		mi = &b->mb_info[tblt->tblt_oldoff];

		bp = NULL;
		switch (tblt->tblt_eventstate) {

		case EVENT_X:
			/*
			 * Send X if changed.
			 */
			if (mi->mi_pen_x != tblt->pen_prev_x) {
				if ((bp = allocb(sizeof (Firm_event),
				    BPRI_HI)) != NULL) {
					fep = (Firm_event *)bp->b_wptr;
					fep->id
					    = vuid_id_addr(tblt->tblt_vuidaddr)
					    | vuid_id_offset(TBLT_X_ABSOLUTE);
					fep->pair_type = FE_PAIR_NONE;
					fep->pair = 0;
					fep->value = mi->mi_pen_x;
					tblt->pen_prev_x = mi->mi_pen_x;
#ifndef	_DDICT
					fep->time = mi->mi_time;
#endif
				} else {
					if ((tbltd->tbltd_rserv_qbufcall_id =
					    qbufcall(tbltd->tbltd_readq,
					    sizeof (Firm_event),
					    BPRI_HI, tbltresched, (long)tbltd)))
						return (0); /* try later */
					/*
					 * qbufcall failed; just pitch
					 * this event
					 */
					tblt->tblt_eventstate = EVENT_PROXIMITY;
				}
			}
			break;

		case EVENT_Y:
			/*
			 * Send Y if changed.
			 */
			if (mi->mi_pen_y != tblt->pen_prev_y) {
				if ((bp = allocb(sizeof (Firm_event), BPRI_HI))
				    != NULL) {
					fep = (Firm_event *)bp->b_wptr;
					fep->id
					    = vuid_id_addr(tblt->tblt_vuidaddr)
					    | vuid_id_offset(TBLT_Y_ABSOLUTE);
					fep->pair_type = FE_PAIR_NONE;
					fep->pair = 0;
					fep->value = mi->mi_pen_y;
					tblt->pen_prev_y = mi->mi_pen_y;
#ifndef	_DDICT
					fep->time = mi->mi_time;
#endif
				} else {
					if ((tbltd->tbltd_rserv_qbufcall_id =
					    qbufcall(tbltd->tbltd_readq,
					    sizeof (Firm_event),
					    BPRI_HI, tbltresched, (long)tbltd)))
						return (0); /* try later */
					/*
					 * qbufcall failed; just pitch
					 * this event
					 */
					tblt->tblt_eventstate = EVENT_PROXIMITY;
				}
			}
			break;

		case EVENT_PRESSURE:
			/*
			 * Send pressure if changed.
			 */
			if (mi->mi_pressure != tblt->tblt_prevpressure) {
				if ((bp = allocb(sizeof (Firm_event), BPRI_HI))
				    != NULL) {
					fep = (Firm_event *)bp->b_wptr;
					fep->id
					    = vuid_id_addr(tblt->tblt_vuidaddr)
					    | vuid_id_offset(TBLT_PRESSURE);
					fep->pair_type = FE_PAIR_NONE;
					fep->pair = 0;
					fep->value = mi->mi_pressure;
					tblt->tblt_prevpressure
					    = mi->mi_pressure;
#ifndef	_DDICT
					fep->time = mi->mi_time;
#endif
				} else {
					if ((tbltd->tbltd_rserv_qbufcall_id =
					    qbufcall(tbltd->tbltd_readq,
					    sizeof (Firm_event),
					    BPRI_HI, tbltresched, (long)tbltd)))
						return (0); /* try later */
					/*
					 * qbufcall failed; just pitch
					 * this event
					 */
					tblt->tblt_eventstate = EVENT_PROXIMITY;
				}
			}
			break;

		case EVENT_PROXIMITY:
			/*
			 * Send proximity if changed.
			 */
			if (mi->mi_proximity != tblt->tblt_prevproximity) {
				if ((bp = allocb(sizeof (Firm_event), BPRI_HI))
				    != NULL) {
					fep = (Firm_event *)bp->b_wptr;
					fep->id
					    = vuid_id_addr(tblt->tblt_vuidaddr)
					    | vuid_id_offset(TBLT_PROXIMITY);
					fep->pair_type = FE_PAIR_NONE;
					fep->pair = 0;
					fep->value = mi->mi_proximity;
					tblt->tblt_prevproximity
					    = mi->mi_proximity;
#ifndef	_DDICT
					fep->time = mi->mi_time;
#endif
				} else {
					if ((tbltd->tbltd_rserv_qbufcall_id =
					    qbufcall(tbltd->tbltd_readq,
					    sizeof (Firm_event),
					    BPRI_HI, tbltresched, (long)tbltd)))
						return (0); /* try later */
					/*
					 * qbufcall failed; just pitch
					 * this event
					 */
					tblt->tblt_eventstate = EVENT_PROXIMITY;
				}
			}
			break;

		} /* end of switch (tblt->tblt_eventstate) */

		if (bp != NULL) {
			bp->b_wptr += sizeof (Firm_event);
			(void) putnext(q, bp);
		}

		if (tblt->tblt_eventstate == EVENT_PROXIMITY) {
			tblt->tblt_eventstate = EVENT_X;
			tblt->tblt_oldoff++;	/* next event */
			if (tblt->tblt_oldoff >= b->mb_size)
				/* circular buffer wraparound */
				tblt->tblt_oldoff = 0;
		} else
			tblt->tblt_eventstate++;

	} /* End of while() */

	return (0);
}

static void
tbltresched(long tbltdptr)
{
	register queue_t *q;
	register struct tbltdata *tbltd = (struct tbltdata *)tbltdptr;

	if (tbltd->tbltd_rserv_qbufcall_id)
		tbltd->tbltd_rserv_qbufcall_id = 0;

	if ((q = tbltd->tbltd_readq) != 0)
		qenable(q);	/* run the service procedure */
}

/*
 * Line discipline output queue put procedure: handles M_IOCTL
 * messages.
 */
static int
tbltwput(register queue_t *q, register mblk_t *mp)
{

	/*
	 * Process M_FLUSH, and some M_IOCTL, messages here; pass
	 * everything else down.
	 */
	switch (mp->b_datap->db_type) {

	case M_FLUSH:
		if (*mp->b_rptr & FLUSHW)
			flushq(q, FLUSHDATA);
		if (*mp->b_rptr & FLUSHR)
			flushq(RD(q), FLUSHDATA);

	default:
		(void) putnext(q, mp);	/* pass it down the line */
		break;

	case M_IOCTL:
		tbltioctl(q, mp);
		break;
	}
	return (0);
}


static void
tbltioctl(register queue_t *q, register mblk_t *mp)
{
	struct tbltdata *tbltd;
	register struct tblt_softc *tblt;
	register struct iocblk *iocp;
	caddr_t data;
	Vuid_addr_probe *addr_probe;
	int err = 0;

	tbltd = (struct tbltdata *)q->q_ptr;
	if (tbltd == 0) {
		err = EINVAL;
		goto out;
	}
	tblt = &tbltd->tbltd_softc;

	iocp = (struct iocblk *)mp->b_rptr;
	if (mp->b_cont != NULL)
		data = (caddr_t)mp->b_cont->b_rptr;

	switch (iocp->ioc_cmd) {
	case VUIDSADDR:
		addr_probe = (Vuid_addr_probe *) data;
		if (addr_probe->base != VKEY_FIRST) {
			err = ENODEV;
			break;
		}
		tblt->tblt_vuidaddr = addr_probe->data.next;
		break;

	case VUIDGADDR:
		addr_probe = (Vuid_addr_probe *) data;
		if (addr_probe->base != VKEY_FIRST) {
			err = ENODEV;
			break;
		}
		addr_probe->data.current = tblt->tblt_vuidaddr;
		break;

	default:
		(void) putnext(q, mp);	/* pass it down the line */
		return;
	}

out:
	if (err != 0) {
		iocp->ioc_rval = 0;
		iocp->ioc_error = err;
		mp->b_datap->db_type = M_IOCNAK;
	} else {
		iocp->ioc_rval = 0;
		iocp->ioc_error = 0;	/* brain rot */
		mp->b_datap->db_type = M_IOCACK;
	}
	qreply(q, mp);
	return;
}

static void
tbltflush(register struct tbltdata *tbltd)
{
	register struct tblt_softc *tblt = &tbltd->tbltd_softc;
	register queue_t *q;

	tblt->tblt_oldoff = 0;
	tblt->tblt_eventstate = EVENT_X;
	if (tblt->tblt_buf)
		tblt->tblt_buf->mb_off = 0;
	tblt->tblt_prevpressure = 0;
	tblt->tblt_prevproximity = 0;
	if ((q = tbltd->tbltd_readq) != NULL && q->q_next != NULL)
		(void) putnextctl1(q, M_FLUSH, FLUSHR);
}

/*
 * Tablet read queue put procedure.
 */
static int
tbltrput(queue_t *q, mblk_t *mp)
{
	register struct tbltdata *tbltd = (struct tbltdata *)q->q_ptr;
	register mblk_t *bp;
	register mblk_t *imp;
	register mblk_t *datap;
	register struct iocblk *iocb;
	register struct termios *cb;
	register char *readp;
	struct iocblk *iocp;

	if (tbltd == 0)
		return (0);

	switch (mp->b_datap->db_type) {

	case M_FLUSH:
		if (*mp->b_rptr & FLUSHW)
			flushq(WR(q), FLUSHDATA);
		if (*mp->b_rptr & FLUSHR)
			flushq(q, FLUSHDATA);

	default:
		(void) putnext(q, mp);
		return (0);

	case M_BREAK:
		if (tbltd->tbltd_flags & TBLT_IOCTOSS) {
			freemsg(mp);
			return (0);
		}

		/*
			 * Change baud rate.
			 */
		if ((imp = allocb(sizeof (struct iocblk), BPRI_HI)) == NULL) {
			return (0);
		}
		if ((datap = allocb(sizeof (struct termios), BPRI_HI))
		    == NULL) {
			freemsg(imp);
			return (0);
		}
		tbltd->tbltd_baud_rate
		    = getnextbaudrate(tbltd->tbltd_baud_rate);
		iocb = (struct iocblk *)imp->b_wptr;
		iocb->ioc_cmd = TCSETSF;
#ifndef	_DDICT
		iocb->ioc_id = getiocseqno();
#endif
		iocb->ioc_count = sizeof (struct iocblk);
		iocb->ioc_error = 0;
		iocb->ioc_rval = 0;
		imp->b_wptr += (sizeof (*iocb))/(sizeof (*datap->b_wptr));
		imp->b_datap->db_type = M_IOCTL;
		cb = (struct termios *)datap->b_wptr;
		cb->c_iflag = 0;
		cb->c_oflag = 0;
		cb->c_cflag = CREAD|CS8|tbltd->tbltd_baud_rate;
		cb->c_lflag = 0;
		bzero((caddr_t)cb->c_cc, NCCS);
		datap->b_wptr += (sizeof (*cb))/(sizeof (*datap->b_wptr));
		datap->b_datap->db_type = M_DATA;
		imp->b_cont = datap;
		tbltd->tbltd_flags |= TBLT_IOCTOSS|TBLT_IOCWAIT;
		tbltd->tbltd_iocid = iocb->ioc_id;
		tbltflush(tbltd);
		flushq(q, FLUSHALL);
		(void) putnext(WR(q), imp);
		freemsg(mp);
		return (0);

	case M_IOCACK:
	case M_IOCNAK:
		/*
			 * If we are doing an "ioctl" ourselves, check if this
			 * is the reply to that code.  If so, wake up the
			 * "open" routine, and toss the reply, otherwise just
			 * pass it up.
			 */
		iocp = (struct iocblk *)mp->b_rptr;
		if (!(tbltd->tbltd_flags & TBLT_IOCWAIT) ||
		    iocp->ioc_id != tbltd->tbltd_iocid) {
			/*
			 * This isn't the reply we're looking for.  Move along.
			 */
			(void) putnext(q, mp);
		} else {
			tbltd->tbltd_flags &= ~TBLT_IOCWAIT;
			tbltd->tbltd_iocerror = iocp->ioc_error;
			/*
			 * If we sent down a request to change the baud rate.
			 * This is the reply.  Just ignore it.
			 */
			if (tbltd->tbltd_flags & TBLT_IOCTOSS) {
				tbltd->tbltd_flags &= ~TBLT_IOCTOSS;
				tbltflush(tbltd);
				flushq(q, FLUSHALL);
			}
			freemsg(mp);
		}
		return (0);

	case M_DATA:
		if (tbltd->tbltd_flags & TBLT_IOCTOSS) {
			freemsg(mp);
			return (0);
		}
		break;
	}

	/*
	 * A data message, consisting of bytes from the tablet.
	 * Hand each byte to our input routine.
	 */
	bp = mp;

	do {
		readp = (char *)bp->b_rptr;
		while (readp < (char *)bp->b_wptr)
			tbltinput(tbltd, *readp++);
		bp->b_rptr = (unsigned char *)readp;
	} while ((bp = bp->b_cont) != NULL);	/* next block, if any */

	freemsg(mp);
	return (0);
}



/*
 * Increment the tablet sample pointer.
 */
static void
tbltincr(struct tbltdata  *tbltd)
{
	register struct tblt_softc *tblt = &tbltd->tbltd_softc;
	register struct tabletbuf *b;
	register int    wake;

	b = tblt->tblt_buf;
	if (b == NULL)
		return;

	/* See if we need to wake up anyone waiting for input */
	wake = (b->mb_off == tblt->tblt_oldoff) &&
	    (!(tbltd->tbltd_rserv_qbufcall_id));

	/* Adjust circular buffer pointer */
	if (++b->mb_off >= b->mb_size)
		b->mb_off = 0;

	/*
	 * If over-took read index then flush buffer so that tablet state
	 * is consistent.
	 */
	if (b->mb_off == tblt->tblt_oldoff)
		tbltflush(tbltd);

	if (wake)
		qenable(tbltd->tbltd_readq);	/* run the service procedure */
}

/*
 * The input data coming from the tablet consists of a 7-byte
 * data block:
 *	Byte 1: 1, In Prox, Stylus, Pressure mode, NA, Sign-X, X15-X14
 *	Byte 2: 0, X13-X7
 *	Byte 3: 0, X6-X0
 *	Byte 4: 0, NA, NA, NA, NA, Sign-Y, Y15-Y14
 *	Byte 5: 0, Y13-Y7
 *	Byte 6: 0, Y6-Y0
 *	Byte 7: 0, Sign-pressure, P5-P0
 */

static void
tbltinput(register struct tbltdata *tbltd, char c)
{
	register struct tblt_softc *tblt;
	register struct tabletbuf *b;
	register struct tabletinfo *mi;

	tblt = &tbltd->tbltd_softc;
	b = tblt->tblt_buf;
	if (b == NULL)
		return;

	mi = &b->mb_info[b->mb_off];

	/*
	 * This will force the first packet to be thrown away.  It is garbage
	 * anyway.  Bug in hardware.
	 */
	if (tbltd->tbltd_pen_state == TBLT_PEN_OUTOFPROX) {
		tbltd->tbltd_pen_state = TBLT_PEN_ENTERPROX;
		tbltd->tbltd_state = TBLT_PWAIT_LEAD;
		return;
	}

	switch (tbltd->tbltd_state) {
	case TBLT_PWAIT_LEAD:
		/*
		 * Wait for a packet header byte.
		 */
		if (!(c & TBLT_PEN_LEADBYTE)) {
			return;
		}

		if (c & TBLT_PEN_PROX)
			mi->mi_proximity = 1;
			else
			mi->mi_proximity = 0;
		mi->mi_pen_x = (unsigned int)(c & TBLT_PEN_X) << X_SHIFT;
#ifndef	_DDICT
		uniqtime(&mi->mi_time);	/* record time when sample arrived */
#endif
		tbltd->tbltd_state = TBLT_PWAIT_X1;
		break;

	case TBLT_PWAIT_X1:
		mi->mi_pen_x |= (unsigned int)c << X1_SHIFT;
		tbltd->tbltd_state = TBLT_PWAIT_X2;
		break;

	case TBLT_PWAIT_X2:
		mi->mi_pen_x |= (unsigned int)c;
		tbltd->tbltd_state = TBLT_PWAIT_Y;
		break;

	case TBLT_PWAIT_Y:
		mi->mi_pen_y = (unsigned int)(c & TBLT_PEN_Y) << Y_SHIFT;
		tbltd->tbltd_state = TBLT_PWAIT_Y1;
		break;

	case TBLT_PWAIT_Y1:
		mi->mi_pen_y |= (unsigned int)c << Y1_SHIFT;
		tbltd->tbltd_state = TBLT_PWAIT_Y2;
		break;

	case TBLT_PWAIT_Y2:
		mi->mi_pen_y |= (unsigned int)c;
		tblt_abs_pos(tbltd);
		tbltd->tbltd_state = TBLT_PWAIT_PRESSURE;
		break;

	case TBLT_PWAIT_PRESSURE:
		tbltd->tbltd_state = TBLT_PWAIT_LEAD;

		if (!(mi->mi_proximity))
			tbltd->tbltd_pen_state = TBLT_PEN_OUTOFPROX;
			else
			tbltd->tbltd_pen_state = TBLT_PEN_INPROX;

		if (mi->mi_proximity) {
			if (c & TBLT_PEN_SIGN)
				mi->mi_pressure = (c + 0x20) & 0x3f;
				else
				mi->mi_pressure = c + 0x20;
		} else
			mi->mi_pressure = 0;

		tbltincr(tbltd);
		break;
	default:
		/*
		 * Bad state, lets wait for a lead byte
		 */
		tbltd->tbltd_state = TBLT_PWAIT_LEAD;
	}
}

static void
tblt_abs_pos(register struct tbltdata *tbltd)
{
	register struct tblt_softc *tblt;
	register struct tabletbuf *b;
	register struct tabletinfo *mi;

	tblt = &tbltd->tbltd_softc;
	b = tblt->tblt_buf;
	if (b == NULL)
		return;

	mi = &b->mb_info[b->mb_off];

	/*
	 * Scale tablet
	 */
	mi->mi_pen_x = mi->mi_pen_x * TBLT_NABS_SCALE / TBLT_DABS_SCALE;
	mi->mi_pen_y = mi->mi_pen_y * TBLT_NABS_SCALE / TBLT_DABS_SCALE;

	mi->mi_pen_x -= TBLT_PEN_XSTART;
	if (mi->mi_pen_x > TBLT_PEN_XMAX)
		mi->mi_pen_x = TBLT_PEN_XMAX;
	else if (mi->mi_pen_x < 0)
		mi->mi_pen_x = 0;

	mi->mi_pen_y -= TBLT_PEN_YSTART;
	if (mi->mi_pen_y > TBLT_PEN_YMAX)
		mi->mi_pen_y = TBLT_PEN_YMAX;
	else if (mi->mi_pen_y < 0)
		mi->mi_pen_y = 0;
}

static int
getnextbaudrate(int b)
{
	switch (b) {
	case B1200:
		return (B2400);

	case B9600:
		return (B19200);

	case B19200:
	default:
		return (B1200);

	}
}
