
/*
 * Copyright (c) 1996,1997, by Sun Microsystems, Inc.
 * All Rights Reserved
 */

#pragma ident	"@(#)tbltreg.h	35.5	97/04/07 SMI"


/*
 * Permission  to  use,  copy,  modify,  and  distribute   this
 * software  and  its documentation for any purpose and without
 * fee is hereby granted, provided that the above copyright no-
 * tice  appear  in all copies and that both that copyright no-
 * tice and this permission notice appear in  supporting  docu-
 * mentation,  and  that the names of Sun or MIT not be used in
 * advertising or publicity pertaining to distribution  of  the
 * software  without specific prior written permission. Sun and
 * M.I.T. make no representations about the suitability of this
 * software for any purpose. It is provided "as is" without any
 * express or implied warranty.
 *
 * SUN DISCLAIMS ALL WARRANTIES WITH REGARD TO  THIS  SOFTWARE,
 * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FIT-
 * NESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL SUN BE  LI-
 * ABLE  FOR  ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
 * ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,  DATA  OR
 * PROFITS,  WHETHER  IN  AN  ACTION OF CONTRACT, NEGLIGENCE OR
 * OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION  WITH
 * THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

/*
 * Software mouse registers
 */

#ifndef _SYS_TBLTREG_H
#define	_SYS_TBLTREG_H

#pragma ident	"@(#)tbltreg.h	1.1	31 Mar 1993 SMI"

#ifdef	__cplusplus
extern "C" {
#endif

/*
 * Tablet sample.
 */
struct	tabletinfo {
	short	mi_pen_x;	/* current pen absolute X coordinate */
	short	mi_pen_y;	/* current pen absolute Y coordinate */
	char	mi_pressure;	/* current pressure */
	char	mi_proximity;	/* current proximity */
#ifndef	_DDICT
	struct	timeval mi_time; /* timestamp */
#endif
};

/*
 * Circular buffer storing mouse events.
 */
struct	tabletbuf {
	short	mb_size;	/* size (in tabletinfo units) of buf */
	short	mb_off;		/* current offset in buffer */
	struct	tabletinfo mb_info[1];	/* however many samples */
};

struct	tblt_softc {
	struct	tabletbuf *tblt_buf;	/* pointer to mouse buffer */
	short	tblt_bufbytes;		/* buffer size (in bytes) */
	short	tblt_oldoff;		/* index into tabletbuf */
	short	tblt_eventstate;	/* current event being generated */
	short	tblt_vuidaddr;		/* vuid addr for TBLT_VUID_FORMAT */
	short	pen_prev_x;		/* Previous pen x position */
	short	pen_prev_y;		/* Previous pen y position */
	char	tblt_prevpressure;	/* pressure level as of last message */
	char	tblt_prevproximity;	/* proximity as of last message */
};

#define	EVENT_X		0	/* generating delta-X event */
#define	EVENT_Y		1	/* generating delta-Y event */
#define	EVENT_PRESSURE	2	/* generating pressure event */
#define	EVENT_PROXIMITY	3	/* generating proximity event */

/*
 * Pen data masks
 */
#define	TBLT_PEN_LEADBYTE	0x80 /* Indicates byte is the firt in packet */
#define	TBLT_PEN_PROX		0x40 /* Indicates pen is in proximity */
#define	TBLT_PEN_X		0x03 /* First 2 bits of X position */
#define	TBLT_PEN_Y		0x03 /* First 2 bits of Y position */

#define	TBLT_PEN_SIGN		0x40 /* Pressure pen tip sign bit */

#define	X_SHIFT			14 /* Shift to get next 7 X position bits */
#define	X1_SHIFT		7 /* Shift to get next 7 X position bits */
#define	Y_SHIFT			14 /* Shift to get next 7 Y position bits */
#define	Y1_SHIFT		7 /* Shift to get next 7 Y position bits */

#define	TBLT_NABS_SCALE	1 	/* Numerator for tablet scaling */
#define	TBLT_DABS_SCALE	3	/* Denomenator for tablet scaling */

#define	TBLT_PEN_XSTART		20 /* Ignore first 20 X pixels of tablet */
#define	TBLT_PEN_YSTART		20 /* Ignore first 20 Y pixels of tablet */

#define	TBLT_PEN_XMAX		1159 /* Ignore anything over screen X size */
#define	TBLT_PEN_YMAX		899 /* Ignore anything over screen Y size */

#ifdef _KERNEL
#define	TBLTIOGETBUF	_IOWR(m, 1, int)	/* TBLTIOGETBUF is OBSOLETE */
	/* Get mouse buffer ptr so (window system in particular) can chase */
	/* around buffer to get events. */
#endif

#ifdef	__cplusplus
}
#endif

#endif	/* _SYS_TBLTREG_H */
